@@ -18,7 +18,6 @@
 
 //BEGIN Includes
 #include "katemodemanager.h"
-#include "katemodemanager.moc"
 #include "katewildcardmatcher.h"
 
 #include "katedocument.h"
@@ -46,7 +45,6 @@
 
 #include <QtGui/QLabel>
 #include <QtGui/QLayout>
-#include <QtGui/QLineEdit>
 #include <QtGui/QPushButton>
 #include <QtGui/QToolButton>
 #include <kvbox.h>
@@ -88,6 +86,7 @@ void KateModeManager::update ()
     type->mimetypes = cg.readXdgListEntry ("Mimetypes");
     type->priority = cg.readEntry ("Priority", 0);
     type->varLine = cg.readEntry ("Variables");
+    type->indenter = cg.readEntry ("Indenter");
     
     type->hl = cg.readEntry ("Highlighting");
    
@@ -126,6 +125,7 @@ void KateModeManager::update ()
       type->mimetypes = modes[i]->mimetype.split (';', QString::SkipEmptyParts);
       type->priority = modes[i]->priority.toInt();
       type->version = modes[i]->version;
+      type->indenter = modes[i]->indenter;
       type->hl = modes[i]->name;
       type->hlGenerated = true;
     }
@@ -180,6 +180,7 @@ void KateModeManager::save (const QList<KateFileType *>& v)
     config.writeXdgListEntry ("Wildcards", type->wildcards);
     config.writeXdgListEntry ("Mimetypes", type->mimetypes);
     config.writeEntry ("Priority", type->priority);
+    config.writeEntry ("Indenter", type->indenter);
 
     QString varLine = type->varLine;
     if (QRegExp("kate:(.*)").indexIn(varLine) < 0)
@@ -228,7 +229,7 @@ QString KateModeManager::fileType (KateDocument *doc)
   // Try wildcards
   if ( ! fileName.isEmpty() )
   {
-    static QStringList commonSuffixes = QString(".orig;.new;~;.bak;.BAK").split (";");
+    static const QStringList commonSuffixes = QString(".orig;.new;~;.bak;.BAK").split (';');
 
     if (!(result = wildcardsFind(fileName)).isEmpty())
       return result;
@@ -239,7 +240,7 @@ QString KateModeManager::fileType (KateDocument *doc)
         return result;
     }
 
-    for (QStringList::Iterator it = commonSuffixes.begin(); it != commonSuffixes.end(); ++it) {
+    for (QStringList::ConstIterator it = commonSuffixes.begin(); it != commonSuffixes.end(); ++it) {
       if (*it != backupSuffix && fileName.endsWith(*it)) {
         if (!(result = wildcardsFind(fileName.left(length - (*it).length()))).isEmpty())
           return result;
@@ -289,7 +290,7 @@ QString KateModeManager::wildcardsFind (const QString &fileName)
       continue;
     }
 
-    foreach (QString wildcard, type->wildcards)
+    foreach (const QString &wildcard, type->wildcards)
     {
       if (KateWildcardMatcher::exactMatch(fileName, wildcard)) {
         match = type;
