@@ -27,6 +27,9 @@
 
 #include <kdebug.h>
 
+
+static QThreadStorage<QMap<KateSmartManager*,int> *> m_usingRevision;
+
 //Uncomment this to debug the translation of ranges. If that is enabled,
 //all ranges are first translated completely separately out of the translation process,
 //and at the end the result is compared. If the result mismatches, an assertion is triggered.
@@ -324,7 +327,7 @@ void KateSmartManager::slotTextChanged(KateEditInfo* edit)
     int currentCanExpand = endGroup ? s_maximumGroupSize - currentGroup->length() : s_defaultGroupSize - currentGroup->length();
     int expanded = 0;
 
-    if (currentCanExpand) {
+    if (currentCanExpand > 0) {
        int expandBy = qMin(edit->translate().line(), currentCanExpand);
       // Current group can expand to accommodate the extra lines
       currentGroup->setNewEndLine(currentGroup->endLine() + expandBy);
@@ -352,7 +355,9 @@ void KateSmartManager::slotTextChanged(KateEditInfo* edit)
       currentGroup->merge();
 
     // Reduce the size of the current group
-    currentGroup->setNewEndLine(currentGroup->endLine() + edit->translate().line());
+    // this leads to broken groups where end line is smaller than the line of
+    // some contained cursors, bug 226409
+    // currentGroup->setNewEndLine(currentGroup->endLine() + edit->translate().line());
   }
 
   // Shift the groups so they have their new start and end lines
@@ -402,7 +407,7 @@ void KateSmartManager::slotTextChanged(KateEditInfo* edit)
 
     if (!mostSpecific)
       mostSpecific = range;
-    range->feedbackMostSpecific(mostSpecific);
+    range->feedbackRangeContentsChanged(mostSpecific);
   }
 
 #ifdef DEBUG_TRANSLATION
@@ -482,6 +487,7 @@ void KateSmartGroup::translatedChanged(const KateEditInfo& edit)
 
 void KateSmartGroup::translatedChanged2(const KateEditInfo& edit)
 {
+  Q_UNUSED(edit)
   //Tell the affected parent smart-ranges to rebuild their child-structure, so they stay consistent
   QSet<KTextEditor::SmartRange*> rebuilt;
 
@@ -696,16 +702,18 @@ void KateSmartManager::clear( bool includingInternal )
 void KateSmartManager::useRevision(int revision)
 {
   if (!m_usingRevision.hasLocalData())
-    m_usingRevision.setLocalData(new int);
+    m_usingRevision.setLocalData(new QMap<KateSmartManager*,int>());
 
-  *m_usingRevision.localData() = revision;
+  m_usingRevision.localData()->insert(this,revision);
 }
 
-int KateSmartManager::usingRevision() const
+int KateSmartManager::usingRevision()
 {
-  if (m_usingRevision.hasLocalData())
-    return *m_usingRevision.localData();
-
+  if (m_usingRevision.hasLocalData()) {
+    QMap<KateSmartManager*,int> *m=m_usingRevision.localData();
+    if (m->contains(this))
+      return m->value(this);
+  }
   return -1;
 }
 
@@ -719,7 +727,7 @@ int KateSmartManager::currentRevision() const
   return doc()->history()->revision();
 }
 
-Cursor KateSmartManager::translateFromRevision(const Cursor& cursor, SmartCursor::InsertBehavior insertBehavior) const
+Cursor KateSmartManager::translateFromRevision(const Cursor& cursor, SmartCursor::InsertBehavior insertBehavior)
 {
   Cursor ret = cursor;
 
@@ -729,7 +737,7 @@ Cursor KateSmartManager::translateFromRevision(const Cursor& cursor, SmartCursor
   return ret;
 }
 
-Range KateSmartManager::translateFromRevision(const Range& range, KTextEditor::SmartRange::InsertBehaviors insertBehavior) const
+Range KateSmartManager::translateFromRevision(const Range& range, KTextEditor::SmartRange::InsertBehaviors insertBehavior)
 {
   Cursor start = range.start(), end = range.end();
 
