@@ -1,6 +1,6 @@
 /*
     This file is part of the KDE libraries
-    Copyright (c) 2005-2008 David Jarvie <djarvie@kde.org>
+    Copyright (c) 2005-2009 David Jarvie <djarvie@kde.org>
 
     This library is free software; you can redistribute it and/or
     modify it under the terms of the GNU Library General Public
@@ -153,6 +153,16 @@ class KDateTimeSpecPrivate;
  * chosen as the basis for KDateTime. For those who need an extended date
  * range, other classes exist.
  *
+ * @section simulation Simulation Facility
+ *
+ * This class provides a facility to simulate the local system time, which
+ * affects all functions using or returning the system time. This facility is
+ * provided for testing purposes only, and is only available if the library is
+ * compiled with debug enabled. In release mode, simulation is inoperative and
+ * the real local system time is used at all times. Use
+ * setSimulatedSystemTime() to set or clear the simulated time. To read the
+ * real (not simulated) system time, use realCurrentLocalDateTime().
+ *
  * @see KTimeZone, KSystemTimeZones, QDateTime, QDate, QTime
  * @see <a href="http://www.w3.org/TR/timezone/">W3C: Working with Time Zones</a>
  * @author David Jarvie \<djarvie@kde.org\>.
@@ -1119,7 +1129,7 @@ class KDECORE_EXPORT KDateTime //krazy:exclude=dpointer (implicitly shared)
      * expressed in UTC.
      *
      * @return current date/time
-     * @see currentLocalDateTime(), currentDateTime()
+     * @see currentLocalDateTime(), currentDateTime(), currentLocalDate(), currentLocalTime()
      */
     static KDateTime currentUtcDateTime();
 
@@ -1133,6 +1143,24 @@ class KDECORE_EXPORT KDateTime //krazy:exclude=dpointer (implicitly shared)
      */
     static KDateTime currentDateTime(const Spec &spec);
 
+    /**
+     * Returns the current date in the local time zone, as reported by the
+     * system clock.
+     *
+     * @return current date
+     * @see currentLocalDateTime(), currentLocalTime()
+     */
+    static QDate currentLocalDate();
+
+    /**
+     * Returns the current time of day in the local time zone, as reported
+     * by the system clock.
+     *
+     * @return current date
+     * @see currentLocalDateTime(), currentLocalDate()
+     */
+    static QTime currentLocalTime();
+
     /**
      * Returns the date/time as a string. The @p format parameter determines the
      * format of the result string. The @p format codes used for the date and time
@@ -1228,7 +1256,7 @@ class KDECORE_EXPORT KDateTime //krazy:exclude=dpointer (implicitly shared)
     /**
      * Returns the KDateTime represented by @p string, using the @p format given.
      *
-     * This method is the inverse of @ref toString(TimeFormat), except that it can
+     * This method is the inverse of toString(TimeFormat), except that it can
      * only return a time specification of UTC, OffsetFromUTC or ClockTime. An
      * actual named time zone cannot be returned since an offset from UTC only
      * partially specifies a time zone.
@@ -1523,9 +1551,55 @@ class KDECORE_EXPORT KDateTime //krazy:exclude=dpointer (implicitly shared)
      */
     void detach();
 
-    /** Write @p dateTime to the datastream @p out, in binary format. */
+    /**
+     * Set an adjustment to be applied when fetching the current system time.
+     * This is applied by all KDateTime methods which return the system date
+     * and/or time.
+     *
+     * The supplied date/time is used as the current simulated time and the
+     * time adjustment is set to the difference between the real current time
+     * and @p newTime. If @p newTime has a time zone, that time zone is set
+     * to be the simulated local system time zone by calling
+     * KSystemTimeZones::setLocalZone()).
+     *
+     * To cancel time simulation, supply an invalid @p newTime parameter.
+     *
+     * @warning This function is provided only for testing purposes, and should
+     *          not be used in released code. If the library is compiled without
+     *          debug enabled, setSimulatedSystemTime() has no effect.
+     *          To avoid confusion, it is recommended that calls to it should be
+     *          conditionally compiled, e.g.:
+     *          \code
+     *          #ifndef NDEBUG
+     *             KDateTime::simulateSystemTime(kdt);
+     *          #endif
+     *          \endcode
+     *
+     * @param newTime the current simulated time, or invalid to cancel simulation
+     *
+     * @see currentDateTime(), currentLocalDateTime(), currentUtcDateTime(),
+     *      currentLocalDate(), currentLocalTime()
+     */
+    static void setSimulatedSystemTime(const KDateTime& newTime);
+
+    /**
+     * Return the real (not simulated) system time.
+     *
+     * @warning This method is provided only for testing purposes, and should
+     *          not be used in released code. If the library is compiled without
+     *          debug enabled, currentLocalDateTime() and realCurrentLocalDateTime()
+     *          both return the real system time.
+     *          To avoid confusion, it is recommended that calls to
+     *          realCurrentLocalDateTime() should be conditionally compiled, e.g.:
+     *          \code
+     *          #ifndef NDEBUG
+     *             dt = KDateTime::realCurrentLocalDateTime();
+     *          #endif
+     *          \endcode
+     */
+    static KDateTime realCurrentLocalDateTime();
+
     friend QDataStream &operator<<(QDataStream &out, const KDateTime &dateTime);
-    /** Read a KDateTime object into @p dateTime from @p in, in binary format. */
     friend QDataStream &operator>>(QDataStream &in, KDateTime &dateTime);
 
   private:
@@ -1537,9 +1611,9 @@ QDataStream &operator<<(QDataStream &out, const KDateTime::Spec &spec);
 /** Read a KDateTime::Spec object into @p spec from @p in, in binary format. */
 QDataStream &operator>>(QDataStream &in, KDateTime::Spec &spec);
 
-/** @copydoc  KDateTime::operator<< */
+/** Write @p dateTime to the datastream @p out, in binary format. */
 QDataStream &operator<<(QDataStream &out, const KDateTime &dateTime);
-/** @copydoc  KDateTime::operator>> */
+/** Read a KDateTime object into @p dateTime from @p in, in binary format. */
 QDataStream &operator>>(QDataStream &in, KDateTime &dateTime);
 
 #endif
