@@ -31,6 +31,7 @@
 #endif
 
 #include "kdebug.h"
+#include <QThreadStorage>
 
 #ifdef Q_WS_WIN
 #include <fcntl.h>
@@ -81,6 +82,15 @@
 #include <kconfig.h>
 #include "kcomponentdata.h"
 
+#ifdef Q_OS_SOLARIS
+// For the purposes of KDebug Solaris has a GNU-libc-compatible 
+// backtrace() function. This isn't detected by the CMake checks
+// normally (check_function_exists fails), but we know it's there.
+// For better results, we would use walk_context(), but that's
+// a little more code -- see also the crash handler in kcrash.cpp .
+#define HAVE_BACKTRACE (1)
+#endif
+
 #ifdef HAVE_BACKTRACE
 #include <execinfo.h>
 #endif
@@ -110,14 +120,15 @@ public:
     qint64 writeData(const char *data, qint64 len)
         {
             if (len) {
-                int nPriority = *data++;
                 // not using fromRawData because we need a terminating NUL
-                QByteArray buf(data, len);
-
-                syslog(nPriority, "%s", buf.constData());
+                const QByteArray buf(data, len);
+                syslog(m_priority, "%s", buf.constData());
             }
             return len;
         }
+    void setPriority(int priority) { m_priority = priority; }
+private:
+    int m_priority;
 };
 
 class KFileDebugStream: public KNoDebugStream
@@ -127,18 +138,18 @@ public:
     qint64 writeData(const char *data, qint64 len)
         {
             if (len) {
-                QByteArray buf = QByteArray::fromRawData(data, len);
-                int pos = buf.indexOf('\0');
-                Q_ASSERT(pos != -1);
-
-                QFile aOutputFile(QFile::decodeName(data));
+                QFile aOutputFile(m_fileName);
                 aOutputFile.open(QIODevice::WriteOnly | QIODevice::Append | QIODevice::Unbuffered);
-                aOutputFile.write(data + pos + 1, len - pos - 1);
+                QByteArray buf = QByteArray::fromRawData(data, len);
+                aOutputFile.write(buf.trimmed());
                 aOutputFile.putChar('\n');
                 aOutputFile.close();
             }
             return len;
         }
+    void setFileName(const QString& fn) { m_fileName = fn; }
+private:
+    QString m_fileName;
 };
 
 class KMessageBoxDebugStream: public KNoDebugStream
@@ -149,16 +160,14 @@ public:
         {
             if (len) {
                 // Since we are in kdecore here, we cannot use KMsgBox
-                QString caption = QString::fromAscii(data, len);
-                int pos = caption.indexOf(QLatin1Char('\0'));
-                Q_ASSERT(pos != -1);
-
-                QString msg = caption.mid(pos + 1);
-                caption.truncate(pos);
-                KMessage::message(KMessage::Information, msg, caption);
+                QString msg = QString::fromAscii(data, len);
+                KMessage::message(KMessage::Information, msg, m_caption);
             }
             return len;
         }
+    void setCaption(const QString& h) { m_caption = h; }
+private:
+    QString m_caption;
 };
 
 class KLineEndStrippingDebugStream: public KNoDebugStream
@@ -181,6 +190,7 @@ struct KDebugPrivate
         QtOutput = 2,
         SyslogOutput = 3,
         NoOutput = 4,
+        DefaultOutput = QtOutput, // if you change DefaultOutput, also change the defaults in kdebugdialog!
         Unknown = 5
     };
 
@@ -201,7 +211,7 @@ struct KDebugPrivate
     typedef QHash<unsigned int, Area> Cache;
 
     KDebugPrivate()
-        : config(0), kDebugDBusIface(0)
+        : config(0), kDebugDBusIface(0), m_disableAll(false)
     {
         Q_ASSERT(int(QtDebugMsg) == 0);
         Q_ASSERT(int(QtFatalMsg) == 3);
@@ -221,6 +231,11 @@ struct KDebugPrivate
         if (kde_kdebug_enable_dbus_interface) {
             kDebugDBusIface = new KDebugDBusIface;
         }
+
+        for (int i = 0; i < 8; i++) {
+            m_nullOutputYesNoCache[i] = -1;
+        }
+
     }
 
     ~KDebugPrivate()
@@ -234,12 +249,16 @@ struct KDebugPrivate
         cache.clear();
 
         Area &areaData = cache[0];
-        areaData.clear(QtOutput);
+        areaData.clear();
 
         //AB: this is necessary here, otherwise all output with area 0 won't be
         //prefixed with anything, unless something with area != 0 is called before
         areaData.name = KGlobal::mainComponent().componentName().toUtf8();
 
+        for (int i = 0; i < 8; i++) {
+            m_nullOutputYesNoCache[i] = -1;
+        }
+
         QString filename(KStandardDirs::locate("config", QLatin1String("kdebug.areas")));
         if (filename.isEmpty()) {
             return;
@@ -289,7 +308,16 @@ struct KDebugPrivate
     inline int level(QtMsgType type)
     { return int(type) - int(QtDebugMsg); }
 
-    OutputMode areaOutputMode(QtMsgType type, unsigned int area)
+    QString groupNameForArea(unsigned int area) const
+    {
+        QString groupName = QString::number(area);
+        if (area == 0 || !config->hasGroup(groupName)) {
+            groupName = QString::fromLocal8Bit(cache.value(area).name);
+        }
+        return groupName;
+    }
+
+    OutputMode areaOutputMode(QtMsgType type, unsigned int area, bool enableByDefault)
     {
         if (!config)
             return QtOutput;
@@ -298,6 +326,8 @@ struct KDebugPrivate
         switch (type) {
         case QtDebugMsg:
             key = QLatin1String( "InfoOutput" );
+            if (m_disableAll)
+                return NoOutput;
             break;
         case QtWarningMsg:
             key = QLatin1String( "WarnOutput" );
@@ -312,8 +342,8 @@ struct KDebugPrivate
             break;
         }
 
-        KConfigGroup cg(config, QString::number(area));
-        int mode = cg.readEntry(key, 2);
+        const KConfigGroup cg(config, groupNameForArea(area));
+        const int mode = cg.readEntry(key, int(enableByDefault ? DefaultOutput : NoOutput));
         return OutputMode(mode);
     }
 
@@ -340,48 +370,75 @@ struct KDebugPrivate
             break;
         }
 
-        KConfigGroup cg(config, QString::number(area));
+        KConfigGroup cg(config, groupNameForArea(area));
         return cg.readPathEntry(aKey, QLatin1String("kdebug.dbg"));
     }
 
-    Cache::Iterator areaData(QtMsgType type, unsigned int num)
+    KConfig* configObject()
     {
         if (!config) {
             if (!KGlobal::hasMainComponent()) {
-                // we don't have a config and we can't create one...
-                Area &area = cache[0]; // create a dummy entry
-                Q_UNUSED(area);
-                return cache.find(0);
+                return NULL;
             }
 
             config = new KConfig(QLatin1String("kdebugrc"), KConfig::NoGlobals);
-            loadAreaNames();
+            m_disableAll = config->group(QString()).readEntry("DisableAll", false);
+        }
+        return config;
+    }
+
+    Cache::Iterator areaData(QtMsgType type, unsigned int num, bool enableByDefault = true)
+    {
+        if (!configObject()) {
+            // we don't have a config and we can't create one...
+            Area &area = cache[0]; // create a dummy entry
+            if (KGlobal::hasMainComponent())
+                area.name = KGlobal::mainComponent().componentName().toUtf8();
+            return cache.find(0);
         }
 
-        if (!cache.contains(num))
+        if (cache.count() <= 1) { // empty or containing only entry "0"
+            loadAreaNames(); // fills 'cache'
+        }
+
+        Cache::Iterator it = cache.find(num);
+        if (it == cache.end()) {
             // unknown area
             return cache.find(0);
+        }
 
-        int l = level(type);
-        Cache::Iterator it = cache.find(num);
-        if (it->mode[l] == Unknown)
-            it->mode[l] = areaOutputMode(type, num);
-        if (it->mode[l] == FileOutput && it->logFileName[l].isEmpty())
-            it->logFileName[l] = logFileName(type, num);
+        if (num == 0 && type == QtDebugMsg) { // area 0 is special, it becomes the named area "appname"
+            static bool s_firstDebugFromApplication = true;
+            if (s_firstDebugFromApplication) {
+                s_firstDebugFromApplication = false;
+                writeGroupForNamedArea(it->name, enableByDefault);
+            }
+        }
+
+        const int lev = level(type);
+        //qDebug() << "in cache for" << num << ":" << it->mode[lev];
+        if (it->mode[lev] == Unknown)
+            it->mode[lev] = areaOutputMode(type, num, enableByDefault);
+        if (it->mode[lev] == FileOutput && it->logFileName[lev].isEmpty())
+            it->logFileName[lev] = logFileName(type, num);
 
         return it;
     }
 
     QDebug setupFileWriter(const QString &fileName)
     {
-        QDebug result(&filewriter);
-        result.nospace() << qPrintable(fileName) << '\0';
+        if (!filewriter.hasLocalData())
+            filewriter.setLocalData(new KFileDebugStream);
+        filewriter.localData()->setFileName(fileName);
+        QDebug result(filewriter.localData());
         return result;
     }
 
     QDebug setupMessageBoxWriter(QtMsgType type, const QByteArray &areaName)
     {
-        QDebug result(&messageboxwriter);
+        if (!messageboxwriter.hasLocalData())
+            messageboxwriter.setLocalData(new KMessageBoxDebugStream);
+        QDebug result(messageboxwriter.localData());
         QByteArray header;
 
         switch (type) {
@@ -402,13 +459,15 @@ struct KDebugPrivate
 
         header += areaName;
         header += ')';
-        result.nospace() << header.constData() << '\0';
+        messageboxwriter.localData()->setCaption(QString::fromAscii(header));
         return result;
     }
 
     QDebug setupSyslogWriter(QtMsgType type)
     {
-        QDebug result(&syslogwriter);
+        if (!syslogwriter.hasLocalData())
+            syslogwriter.setLocalData(new KSyslogDebugStream);
+        QDebug result(syslogwriter.localData());
         int level = 0;
 
         switch (type) {
@@ -426,30 +485,55 @@ struct KDebugPrivate
             level = LOG_ERR;
             break;
         }
-        result.nospace() << char(level);
+        syslogwriter.localData()->setPriority(level);
         return result;
     }
 
     QDebug setupQtWriter(QtMsgType type)
     {
-        if (type != QtDebugMsg)
+        if (type != QtDebugMsg) {
+            if (type == QtWarningMsg) {
+                // KDE warnings are not the same thing as Qt warnings
+                // in Qt, warnings indicate bad code, which must be corrected before the release
+                // in KDE, it's just something that everyone sees (including users)
+                type = QtDebugMsg;
+            }
             return QDebug(type);
+        }
         return QDebug(&lineendstrippingwriter);
     }
 
-    QDebug printHeader(QDebug s, const QByteArray &areaName, const char *, int, const char *funcinfo, bool colored)
+    QDebug printHeader(QDebug s, const QByteArray &areaName, const char *, int, const char *funcinfo, QtMsgType type, bool colored)
     {
 #ifdef KDE_EXTENDED_DEBUG_OUTPUT
-        QByteArray programName = cache.value(0).name;
-        if (programName.isEmpty())
-            programName = "<unknown program name>";
-        s.nospace() << programName.constData() << "(" << unsigned(getpid()) << ")";
-        if (areaName != programName)
-            s << "/" << areaName.constData();
+        static bool printProcessInfo = (qgetenv("KDE_DEBUG_NOPROCESSINFO").isEmpty());
+        static bool printAreaName = (qgetenv("KDE_DEBUG_NOAREANAME").isEmpty());
+        static bool printMethodName = (qgetenv("KDE_DEBUG_NOMETHODNAME").isEmpty());
+        QByteArray programName;
+        s = s.nospace();
+        if (printProcessInfo) {
+            programName = cache.value(0).name;
+            if (programName.isEmpty()) {
+                if (QCoreApplication::instance())
+                    programName = QCoreApplication::instance()->applicationName().toLocal8Bit();
+                else
+                    programName = "<unknown program name>";
+            }
+            s << programName.constData() << "(" << unsigned(getpid()) << ")";
+        }
+        if (printAreaName && (!printProcessInfo || areaName != programName)) {
+            if (printProcessInfo)
+                s << "/";
+            s << areaName.constData();
+        }
 
-        if (funcinfo) {
-            if(colored)
-                s << "\033[0;34m"; //blue
+        if (funcinfo && printMethodName) {
+            if (colored) {
+                if (type <= QtDebugMsg)
+                    s << "\033[0;34m"; //blue
+                else
+                    s << "\033[0;31m"; //red
+            }
 # ifdef Q_CC_GNU
             // strip the function info down to the base function name
             // note that this throws away the template definitions,
@@ -517,7 +601,7 @@ struct KDebugPrivate
 
         if (areaName.isEmpty())
             areaName = cache.value(0).name;
-        
+
         bool colored=false;
 
         QDebug s(&devnull);
@@ -539,23 +623,39 @@ struct KDebugPrivate
             s = setupQtWriter(type);
 #ifndef Q_OS_WIN
             //only color if the debug goes to a tty.
-            colored = env_colored && isatty(fileno(stderr));  
+            colored = env_colored && isatty(fileno(stderr));
 #endif
             break;
         }
 
-        return printHeader(s, areaName, debugFile, line, funcinfo, colored);
+        return printHeader(s, areaName, debugFile, line, funcinfo, type, colored);
+    }
+
+    void writeGroupForNamedArea(const QByteArray& areaName, bool enabled)
+    {
+        // Ensure that this area name appears in kdebugrc, so that users (via kdebugdialog)
+        // can turn it off.
+        KConfig* cfgObj = configObject();
+        if (cfgObj) {
+            KConfigGroup cg(cfgObj, QString::fromUtf8(areaName));
+            const QString key = QString::fromLatin1("InfoOutput");
+            if (!cg.hasKey(key)) {
+                cg.writeEntry(key, int(enabled ? KDebugPrivate::QtOutput : KDebugPrivate::NoOutput));
+            }
+        }
     }
 
     QMutex mutex;
     KConfig *config;
     KDebugDBusIface *kDebugDBusIface;
     Cache cache;
+    bool m_disableAll;
+    int m_nullOutputYesNoCache[8];
 
     KNoDebugStream devnull;
-    KSyslogDebugStream syslogwriter;
-    KFileDebugStream filewriter;
-    KMessageBoxDebugStream messageboxwriter;
+    QThreadStorage<KSyslogDebugStream*> syslogwriter;
+    QThreadStorage<KFileDebugStream*> filewriter;
+    QThreadStorage<KMessageBoxDebugStream*> messageboxwriter;
     KLineEndStrippingDebugStream lineendstrippingwriter;
 };
 
@@ -630,12 +730,72 @@ QDebug operator<<(QDebug s, const KUrl &url)
 void kClearDebugConfig()
 {
     if (!kDebug_data) return;
-    QMutexLocker locker(&kDebug_data->mutex);
-    delete kDebug_data->config;
-    kDebug_data->config = 0;
+    KDebugPrivate* d = kDebug_data;
+    QMutexLocker locker(&d->mutex);
+    delete d->config;
+    d->config = 0;
 
-    KDebugPrivate::Cache::Iterator it = kDebug_data->cache.begin(),
-                                  end = kDebug_data->cache.end();
+    KDebugPrivate::Cache::Iterator it = d->cache.begin(),
+                                  end = d->cache.end();
     for ( ; it != end; ++it)
         it->clear();
+
+    for (int i = 0; i < 8; i++) {
+        d->m_nullOutputYesNoCache[i] = -1;
+    }
+}
+
+// static
+bool KDebug::hasNullOutput(QtMsgType type,
+                           bool condition,
+                           int area,
+                           bool enableByDefault)
+{
+    if (!condition) {
+        return true;
+    }
+    if (kDebug_data.isDestroyed()) {
+         // kDebugStream() will generate a warning anyway, so we don't.
+        return false;
+    }
+    KDebugPrivate *const d = kDebug_data;
+    QMutexLocker locker(&d->mutex);
+
+    if (type == QtDebugMsg) {
+        int *entries = d->m_nullOutputYesNoCache;
+        for (int i = 0; i < 8; i += 2) {
+            if (entries[i] == area) {
+                return entries[i + 1];
+            }
+        }
+    }
+
+    KDebugPrivate::Cache::Iterator it = d->areaData(type, area, enableByDefault);
+    const bool ret = it->mode[d->level(type)] == KDebugPrivate::NoOutput;
+
+    // cache result for next time...
+    if (type == QtDebugMsg) {
+        int *entries = d->m_nullOutputYesNoCache;
+        int idx = (qrand() % 4) * 2;
+        entries[idx] = area;
+        entries[idx + 1] = ret;
+    }
+
+    return ret;
+}
+
+int KDebug::registerArea(const QByteArray& areaName, bool enabled)
+{
+    // TODO for optimization: static int s_lastAreaNumber = 1;
+    KDebugPrivate* d = kDebug_data;
+    QMutexLocker locker(&d->mutex);
+    int areaNumber = 1;
+    while (d->cache.contains(areaNumber)) {
+        ++areaNumber;
+    }
+    KDebugPrivate::Area areaData;
+    areaData.name = areaName;
+    d->cache.insert(areaNumber, areaData);
+    d->writeGroupForNamedArea(areaName, enabled);
+    return areaNumber;
 }
