@@ -6,6 +6,7 @@
     Copyright (c) 2003 Waldo Bastian <bastian@kde.org>
     Copyright (c) 2003 Zack Rusin <zack@kde.org>
     Copyright (c) 2006 Michaël Larouche <michael.larouche@kdemail.net>
+    Copyright (c) 2008 Allen Winter <winter@kde.org>
 
     This library is free software; you can redistribute it and/or
     modify it under the terms of the GNU Library General Public
@@ -46,7 +47,7 @@ static inline std::ostream &operator<<(std::ostream &o, const QString &str)
 static void parseArgs(const QStringList &args, QString &directory, QString &file1, QString &file2)
 {
     int fileCount = 0;
-    directory = ".";
+    directory = '.';
 
     for (int i = 1; i < args.count(); ++i) {
         if (args.at(i) == "-d" ||  args.at(i) == "--directory") {
@@ -125,6 +126,7 @@ class CfgEntry
       QString name;
       QString context;
       QString label;
+      QString toolTip;
       QString whatsThis;
     };
     class Choices
@@ -150,11 +152,11 @@ class CfgEntry
 
     CfgEntry( const QString &group, const QString &type, const QString &key,
               const QString &name, const QString &context, const QString &label,
-              const QString &whatsThis, const QString &code,
+              const QString &toolTip, const QString &whatsThis, const QString &code,
               const QString &defaultValue, const Choices &choices, const QList<Signal> signalList,
               bool hidden )
       : mGroup( group ), mType( type ), mKey( key ), mName( name ),
-        mContext( context ), mLabel( label ), mWhatsThis( whatsThis ),
+        mContext( context ), mLabel( label ), mToolTip( toolTip ), mWhatsThis( whatsThis ),
         mCode( code ), mDefaultValue( defaultValue ), mChoices( choices ),
         mSignalList(signalList), mHidden( hidden )
     {
@@ -178,6 +180,9 @@ class CfgEntry
     void setLabel( const QString &label ) { mLabel = label; }
     QString label() const { return mLabel; }
 
+    void setToolTip( const QString &toolTip ) { mToolTip = toolTip; }
+    QString toolTip() const { return mToolTip; }
+
     void setWhatsThis( const QString &whatsThis ) { mWhatsThis = whatsThis; }
     QString whatsThis() const { return mWhatsThis; }
 
@@ -252,6 +257,7 @@ class CfgEntry
     QString mName;
     QString mContext;
     QString mLabel;
+    QString mToolTip;
     QString mWhatsThis;
     QString mCode;
     QString mDefaultValue;
@@ -357,6 +363,15 @@ static QString setFunction(const QString &n, const QString &className = QString(
   return result;
 }
 
+static QString getDefaultFunction(const QString &n, const QString &className = QString())
+{
+  QString result = "default"+n+"Value";
+  result[7] = result[7].toUpper();
+
+  if ( !className.isEmpty() )
+    result = className + "::" + result;
+  return result;
+}
 
 static QString getFunction(const QString &n, const QString &className = QString())
 {
@@ -378,11 +393,11 @@ static void addQuotes( QString &s )
 static QString quoteString( const QString &s )
 {
   QString r = s;
-  r.replace( "\\", "\\\\" );
-  r.replace( "\"", "\\\"" );
-  r.replace( "\r", "" );
-  r.replace( "\n", "\\n\"\n\"" );
-  return "\"" + r + "\"";
+  r.replace( '\\', "\\\\" );
+  r.replace( '\"', "\\\"" );
+  r.remove( '\r' );
+  r.replace( '\n', "\\n\"\n\"" );
+  return '\"' + r + '\"';
 }
 
 static QString literalString( const QString &s )
@@ -436,23 +451,31 @@ static void preProcessDefault( QString &defaultValue, const QString &name,
 
     } else if ( type == "Path" && !defaultValue.isEmpty() ) {
       defaultValue = literalString( defaultValue );
-
-    } else if ( (type == "StringList" || type == "PathList") && !defaultValue.isEmpty() ) {
+    } else if ( type == "Url" && !defaultValue.isEmpty() ) {
+      defaultValue = "KUrl( " + literalString(defaultValue) + ')';
+    } else if ( ( type == "UrlList" || type == "StringList" || type == "PathList") && !defaultValue.isEmpty() ) {
       QTextStream cpp( &code, QIODevice::WriteOnly | QIODevice::Append );
       if (!code.isEmpty())
          cpp << endl;
 
       cpp << "  QStringList default" << name << ";" << endl;
-      const QStringList defaults = defaultValue.split( "," );
+      const QStringList defaults = defaultValue.split( ',' );
       QStringList::ConstIterator it;
-      for( it = defaults.begin(); it != defaults.end(); ++it ) {
-        cpp << "  default" << name << ".append( QString::fromUtf8( \"" << *it << "\" ) );"
-            << endl;
+      for( it = defaults.constBegin(); it != defaults.constEnd(); ++it ) {
+        cpp << "  default" << name << ".append( ";
+        if( type == "UrlList" ) {
+          cpp << "KUrl(";       
+        }
+        cpp << "QString::fromUtf8( \"" << *it << "\" ) ";
+        if( type == "UrlList" ) {
+          cpp << ") ";
+        }
+        cpp << ");" << endl;
       }
       defaultValue = "default" + name;
 
     } else if ( type == "Color" && !defaultValue.isEmpty() ) {
-      QRegExp colorRe("\\d+,\\s*\\d+,\\s*\\d+");
+      QRegExp colorRe("\\d+,\\s*\\d+,\\s*\\d+(,\\s*\\d+)?");
       if (colorRe.exactMatch(defaultValue))
       {
         defaultValue = "QColor( " + defaultValue + " )";
@@ -464,7 +487,7 @@ static void preProcessDefault( QString &defaultValue, const QString &name,
 
     } else if ( type == "Enum" ) {
       QList<CfgEntry::Choice>::ConstIterator it;
-      for( it = choices.choices.begin(); it != choices.choices.end(); ++it ) {
+      for( it = choices.choices.constBegin(); it != choices.choices.constEnd(); ++it ) {
         if ( (*it).name == defaultValue ) {
           if ( globalEnums && choices.name().isEmpty() )
             defaultValue.prepend( choices.prefix );
@@ -482,9 +505,9 @@ static void preProcessDefault( QString &defaultValue, const QString &name,
       cpp << "  QList<int> default" << name << ";" << endl;
       if (!defaultValue.isEmpty())
       {
-        QStringList defaults = defaultValue.split( "," );
+        QStringList defaults = defaultValue.split( ',' );
         QStringList::ConstIterator it;
-        for( it = defaults.begin(); it != defaults.end(); ++it ) {
+        for( it = defaults.constBegin(); it != defaults.constEnd(); ++it ) {
           cpp << "  default" << name << ".append( " << *it << " );"
               << endl;
         }
@@ -503,6 +526,7 @@ CfgEntry *parseEntry( const QString &group, const QDomElement &element )
   QString hidden = element.attribute( "hidden" );
   QString context = element.attribute( "context" );
   QString label;
+  QString toolTip;
   QString whatsThis;
   QString defaultValue;
   QString code;
@@ -523,6 +547,10 @@ CfgEntry *parseEntry( const QString &group, const QDomElement &element )
       label = e.text();
       context = e.attribute( "context" );
     }
+    else if ( tag == "tooltip" ) {
+      toolTip = e.text();
+      context = e.attribute( "context" );
+    }
     else if ( tag == "whatsthis" ) {
       whatsThis = e.text();
       context = e.attribute( "context" );
@@ -607,6 +635,10 @@ CfgEntry *parseEntry( const QString &group, const QDomElement &element )
               choice.label = e3.text();
               choice.context = e3.attribute( "context" );
             }
+            if ( e3.tagName() == "tooltip" ) {
+              choice.toolTip = e3.text();
+              choice.context = e3.attribute( "context" );
+            }
             if ( e3.tagName() == "whatsthis" ) {
               choice.whatsThis = e3.text();
               choice.context = e3.attribute( "context" );
@@ -638,9 +670,9 @@ CfgEntry *parseEntry( const QString &group, const QDomElement &element )
 
   if ( nameIsEmpty ) {
     name = key;
-    name.replace( " ", QString() );
+    name.remove( ' ' );
   } else if ( name.contains( ' ' ) ) {
-      std::cout<<"Entry '"<<qPrintable(name)<<"' contains spaces! <name> elements can't contain spaces!"<<std::endl;
+    std::cout<<"Entry '"<<qPrintable(name)<<"' contains spaces! <name> elements can not contain spaces!"<<std::endl;
     name.remove( ' ' );
   }
 
@@ -671,7 +703,7 @@ CfgEntry *parseEntry( const QString &group, const QDomElement &element )
   {
     // Adjust name
     paramName = name;
-    name.replace("$("+param+')', QString());
+    name.remove("$("+param+')');
     // Lookup defaults for indexed entries
     for(int i = 0; i <= paramMax; i++)
     {
@@ -740,7 +772,7 @@ CfgEntry *parseEntry( const QString &group, const QDomElement &element )
     preProcessDefault(defaultValue, name, type, choices, code);
   }
 
-  CfgEntry *result = new CfgEntry( group, type, key, name, context, label, whatsThis,
+  CfgEntry *result = new CfgEntry( group, type, key, name, context, label, toolTip, whatsThis,
                                    code, defaultValue, choices, signalList,
                                    hidden == "true" );
   if (!param.isEmpty())
@@ -758,32 +790,40 @@ CfgEntry *parseEntry( const QString &group, const QDomElement &element )
   return result;
 }
 
+static bool isUnsigned(const QString& type)
+{
+    if ( type == "UInt" )        return true;
+    if ( type == "ULongLong" )   return true;
+    return false;
+}
+
 /**
   Return parameter declaration for given type.
 */
-QString param( const QString &type )
+QString param( const QString &t )
 {
-    if ( type == "String" )           return "const QString &";
-    else if ( type == "StringList" )  return "const QStringList &";
-    else if ( type == "Font" )        return "const QFont &";
-    else if ( type == "Rect" )        return "const QRect &";
-    else if ( type == "Size" )        return "const QSize &";
-    else if ( type == "Color" )       return "const QColor &";
-    else if ( type == "Point" )       return "const QPoint &";
-    else if ( type == "Int" )         return "int";
-    else if ( type == "UInt" )        return "uint";
-    else if ( type == "Bool" )        return "bool";
-    else if ( type == "Double" )      return "double";
-    else if ( type == "DateTime" )    return "const QDateTime &";
-    else if ( type == "LongLong" )    return "qint64";
-    else if ( type == "ULongLong" )   return "quint64";
-    else if ( type == "IntList" )     return "const QList<int> &";
-    else if ( type == "Enum" )        return "int";
-    else if ( type == "Path" )        return "const QString &";
-    else if ( type == "PathList" )    return "const QStringList &";
-    else if ( type == "Password" )    return "const QString &";
-    else if ( type == "Url" )         return "const KUrl &";
-    else if ( type == "UrlList" )     return "const KUrl::List &";
+    const QString type = t.toLower();
+    if ( type == "string" )           return "const QString &";
+    else if ( type == "stringlist" )  return "const QStringList &";
+    else if ( type == "font" )        return "const QFont &";
+    else if ( type == "rect" )        return "const QRect &";
+    else if ( type == "size" )        return "const QSize &";
+    else if ( type == "color" )       return "const QColor &";
+    else if ( type == "point" )       return "const QPoint &";
+    else if ( type == "int" )         return "int";
+    else if ( type == "uint" )        return "uint";
+    else if ( type == "bool" )        return "bool";
+    else if ( type == "double" )      return "double";
+    else if ( type == "datetime" )    return "const QDateTime &";
+    else if ( type == "longlong" )    return "qint64";
+    else if ( type == "ulonglong" )   return "quint64";
+    else if ( type == "intlist" )     return "const QList<int> &";
+    else if ( type == "enum" )        return "int";
+    else if ( type == "path" )        return "const QString &";
+    else if ( type == "pathlist" )    return "const QStringList &";
+    else if ( type == "password" )    return "const QString &";
+    else if ( type == "url" )         return "const KUrl &";
+    else if ( type == "urllist" )     return "const KUrl::List &";
     else {
         std::cerr <<"kconfig_compiler does not support type \""<< type <<"\""<<std::endl;
         return "QString"; //For now, but an assert would be better
@@ -793,60 +833,62 @@ QString param( const QString &type )
 /**
   Actual C++ storage type for given type.
 */
-QString cppType( const QString &type )
+QString cppType( const QString &t )
 {
-    if ( type == "String" )           return "QString";
-    else if ( type == "StringList" )  return "QStringList";
-    else if ( type == "Font" )        return "QFont";
-    else if ( type == "Rect" )        return "QRect";
-    else if ( type == "Size" )        return "QSize";
-    else if ( type == "Color" )       return "QColor";
-    else if ( type == "Point" )       return "QPoint";
-    else if ( type == "Int" )         return "int";
-    else if ( type == "UInt" )        return "uint";
-    else if ( type == "Bool" )        return "bool";
-    else if ( type == "Double" )      return "double";
-    else if ( type == "DateTime" )    return "QDateTime";
-    else if ( type == "LongLong" )    return "qint64";
-    else if ( type == "ULongLong" )   return "quint64";
-    else if ( type == "IntList" )     return "QList<int>";
-    else if ( type == "Enum" )        return "int";
-    else if ( type == "Path" )        return "QString";
-    else if ( type == "PathList" )    return "QStringList";
-    else if ( type == "Password" )    return "QString";
-    else if ( type == "Url" )         return "KUrl";
-    else if ( type == "UrlList" )     return "KUrl::List";
+    const QString type = t.toLower();
+    if ( type == "string" )           return "QString";
+    else if ( type == "stringlist" )  return "QStringList";
+    else if ( type == "font" )        return "QFont";
+    else if ( type == "rect" )        return "QRect";
+    else if ( type == "size" )        return "QSize";
+    else if ( type == "color" )       return "QColor";
+    else if ( type == "point" )       return "QPoint";
+    else if ( type == "int" )         return "int";
+    else if ( type == "uint" )        return "uint";
+    else if ( type == "bool" )        return "bool";
+    else if ( type == "double" )      return "double";
+    else if ( type == "datetime" )    return "QDateTime";
+    else if ( type == "longlong" )    return "qint64";
+    else if ( type == "ulonglong" )   return "quint64";
+    else if ( type == "intlist" )     return "QList<int>";
+    else if ( type == "enum" )        return "int";
+    else if ( type == "path" )        return "QString";
+    else if ( type == "pathlist" )    return "QStringList";
+    else if ( type == "password" )    return "QString";
+    else if ( type == "url" )         return "KUrl";
+    else if ( type == "urllist" )     return "KUrl::List";
     else {
         std::cerr<<"kconfig_compiler does not support type \""<< type <<"\""<<std::endl;
         return "QString"; //For now, but an assert would be better
     }
 }
 
-QString defaultValue( const QString &type )
+QString defaultValue( const QString &t )
 {
-    if ( type == "String" )           return "\"\""; // Use empty string, not null string!
-    else if ( type == "StringList" )  return "QStringList()";
-    else if ( type == "Font" )        return "QFont()";
-    else if ( type == "Rect" )        return "QRect()";
-    else if ( type == "Size" )        return "QSize()";
-    else if ( type == "Color" )       return "QColor(128, 128, 128)";
-    else if ( type == "Point" )       return "QPoint()";
-    else if ( type == "Int" )         return "0";
-    else if ( type == "UInt" )        return "0";
-    else if ( type == "Bool" )        return "false";
-    else if ( type == "Double" )      return "0.0";
-    else if ( type == "DateTime" )    return "QDateTime()";
-    else if ( type == "LongLong" )    return "0";
-    else if ( type == "ULongLong" )   return "0";
-    else if ( type == "IntList" )     return "QList<int>()";
-    else if ( type == "Enum" )        return "0";
-    else if ( type == "Path" )        return "\"\""; // Use empty string, not null string!
-    else if ( type == "PathList" )    return "QStringList()";
-    else if ( type == "Password" )    return "\"\""; // Use empty string, not null string!
-    else if ( type == "Url" )         return "KUrl()";
-    else if ( type == "UrlList" )     return "KUrl::List()";
+    const QString type = t.toLower();
+    if ( type == "string" )           return "\"\""; // Use empty string, not null string!
+    else if ( type == "stringlist" )  return "QStringList()";
+    else if ( type == "font" )        return "QFont()";
+    else if ( type == "rect" )        return "QRect()";
+    else if ( type == "size" )        return "QSize()";
+    else if ( type == "color" )       return "QColor(128, 128, 128)";
+    else if ( type == "point" )       return "QPoint()";
+    else if ( type == "int" )         return "0";
+    else if ( type == "uint" )        return "0";
+    else if ( type == "bool" )        return "false";
+    else if ( type == "double" )      return "0.0";
+    else if ( type == "datedime" )    return "QDateTime()";
+    else if ( type == "longlong" )    return "0";
+    else if ( type == "ulonglong" )   return "0";
+    else if ( type == "intlist" )     return "QList<int>()";
+    else if ( type == "enum" )        return "0";
+    else if ( type == "path" )        return "\"\""; // Use empty string, not null string!
+    else if ( type == "pathlist" )    return "QStringList()";
+    else if ( type == "password" )    return "\"\""; // Use empty string, not null string!
+    else if ( type == "url" )         return "KUrl()";
+    else if ( type == "urllist" )     return "KUrl::List()";
     else {
-        std::cerr<<"Error, kconfig_compiler doesn't support the \""<< type <<"\" type!"<<std::endl;
+        std::cerr<<"Error, kconfig_compiler does not support the \""<< type <<"\" type!"<<std::endl;
         return "QString"; //For now, but an assert would be better
     }
 }
@@ -955,8 +997,8 @@ QString paramString(const QString &group, const QList<Param> &parameters)
   QString paramString = group;
   QString arguments;
   int i = 1;
-  for (QList<Param>::ConstIterator it = parameters.begin();
-       it != parameters.end(); ++it)
+  for (QList<Param>::ConstIterator it = parameters.constBegin();
+       it != parameters.constEnd(); ++it)
   {
      if (paramString.contains("$("+(*it).name+')'))
      {
@@ -989,6 +1031,18 @@ QString userTextsFunctions( CfgEntry *e, QString itemVarStr=QString(), QString i
       txt+= quoteString(e->label());
     txt+= ") );\n";
   }
+  if ( !e->toolTip().isEmpty() ) {
+    txt += "  " + itemVarStr + "->setToolTip( ";
+    if ( !e->context().isEmpty() )
+      txt += "i18nc(" + quoteString(e->context()) + ", ";
+    else
+      txt += "i18n(";
+    if ( !e->param().isEmpty() )
+      txt += quoteString(e->toolTip().replace("$("+e->param()+')', i));
+    else
+      txt+= quoteString(e->toolTip());
+    txt+=") );\n";
+  }
   if ( !e->whatsThis().isEmpty() ) {
     txt += "  " + itemVarStr + "->setWhatsThis( ";
     if ( !e->context().isEmpty() )
@@ -1040,13 +1094,15 @@ QString memberMutatorBody( CfgEntry *e )
 
   if (!e->minValue().isEmpty())
   {
-    out << "if (v < " << e->minValue() << ")" << endl;
-    out << "{" << endl;
-    out << "  kDebug() << \"" << setFunction(n);
-    out << ": value \" << v << \" is less than the minimum value of ";
-    out << e->minValue()<< "\" << endl;" << endl;
-    out << "  v = " << e->minValue() << ";" << endl;
-    out << "}" << endl;
+    if (e->minValue() != "0" || !isUnsigned(t)) { // skip writing "if uint<0" (#187579)
+      out << "if (v < " << e->minValue() << ")" << endl;
+      out << "{" << endl;
+      out << "  kDebug() << \"" << setFunction(n);
+      out << ": value \" << v << \" is less than the minimum value of ";
+      out << e->minValue()<< "\";" << endl;
+      out << "  v = " << e->minValue() << ";" << endl;
+      out << "}" << endl;
+    }
   }
 
   if (!e->maxValue().isEmpty())
@@ -1055,12 +1111,12 @@ QString memberMutatorBody( CfgEntry *e )
     out << "{" << endl;
     out << "  kDebug() << \"" << setFunction(n);
     out << ": value \" << v << \" is greater than the maximum value of ";
-    out << e->maxValue()<< "\" << endl;" << endl;
+    out << e->maxValue()<< "\";" << endl;
     out << "  v = " << e->maxValue() << ";" << endl;
     out << "}" << endl << endl;
   }
 
-  out << "if (!" << This << "isImmutable( QString::fromLatin1 ( \"";
+  out << "if (!" << This << "isImmutable( QString::fromLatin1( \"";
   if (!e->param().isEmpty())
   {
     out << e->paramName().replace("$("+e->param()+")", "%1") << "\" ).arg( ";
@@ -1100,6 +1156,32 @@ QString memberMutatorBody( CfgEntry *e )
   return result;
 }
 
+// returns the member get default implementation
+// which should go in the h file if inline
+// or the cpp file if not inline
+QString memberGetDefaultBody( CfgEntry *e )
+{
+  QString result = e->code();
+  QTextStream out(&result, QIODevice::WriteOnly);
+  out << endl;
+
+  if (!e->param().isEmpty()) {
+    out << "  switch (i) {" << endl;
+    for (int i = 0; i <= e->paramMax(); ++i) {
+      if (!e->paramDefaultValue(i).isEmpty()) {
+        out << "  case " << i << ": return " << e->paramDefaultValue(i) << ';' << endl;
+      }
+    }
+    out << "  default:" << endl;
+    out << "    return " << e->defaultValue().replace("$("+e->param()+')', "i") << ';' << endl;
+    out << "  }" << endl;
+  } else {
+    out << "  return " << e->defaultValue() << ';';
+  }
+
+  return result;
+}
+
 // returns the item accesor implementation
 // which should go in the h file if inline
 // or the cpp file if not inline
@@ -1133,6 +1215,30 @@ QString indent(QString text, int spaces)
     return result;
 }
 
+// adds as many 'namespace foo {' lines to p_out as
+// there are namespaces in p_ns
+void beginNamespaces(const QString &p_ns, QTextStream &p_out) 
+{
+  if ( !p_ns.isEmpty() ) {
+    const QStringList nameSpaces = p_ns.split( "::" );
+    foreach (const QString &ns, nameSpaces )
+      p_out << "namespace " << ns << " {" << endl;
+    p_out << endl;
+  }
+}
+
+// adds as many '}' lines to p_out as
+// there are namespaces in p_ns
+void endNamespaces(const QString &p_ns, QTextStream &p_out)
+{
+  if ( !p_ns.isEmpty() ) {
+    const int namespaceCount = p_ns.count( "::" ) + 1;
+    for ( int i = 0; i < namespaceCount; ++i )
+      p_out << "}" << endl;
+    p_out << endl;
+  }
+}
+
 
 int main( int argc, char **argv )
 {
@@ -1180,7 +1286,8 @@ int main( int argc, char **argv )
      allMutators = true;
   itemAccessors = codegenConfig.value("ItemAccessors", false).toBool();
   bool setUserTexts = codegenConfig.value("SetUserTexts", false).toBool();
-
+  QStringList defaultGetters = codegenConfig.value("DefaultValueGetters", QStringList()).toStringList();
+  bool allDefaultGetters = (defaultGetters.count() == 1) && (defaultGetters.at(0).toLower() == "true");
   globalEnums = codegenConfig.value("GlobalEnums", false).toBool();
   useEnumTypes = codegenConfig.value("UseEnumTypes", false).toBool();
 
@@ -1247,7 +1354,7 @@ int main( int argc, char **argv )
         CfgEntry *entry = parseEntry( group, e2 );
         if ( entry ) entries.append( entry );
         else {
-          std::cerr << "Can't parse entry." << std::endl;
+          std::cerr << "Can not parse entry." << std::endl;
           return 1;
         }
       }
@@ -1317,7 +1424,7 @@ int main( int argc, char **argv )
 
   QFile header( baseDir + headerFileName );
   if ( !header.open( QIODevice::WriteOnly ) ) {
-    std::cerr << "Can't open '" << baseDir  << headerFileName << "for writing." << std::endl;
+    std::cerr << "Can not open '" << baseDir  << headerFileName << "for writing." << std::endl;
     return 1;
   }
 
@@ -1326,14 +1433,14 @@ int main( int argc, char **argv )
   h << "// This file is generated by kconfig_compiler from " << QFileInfo(inputFilename).fileName() << "." << endl;
   h << "// All changes you do to this file will be lost." << endl;
 
-  h << "#ifndef " << ( !nameSpace.isEmpty() ? nameSpace.toUpper() + '_' : "" )
+  h << "#ifndef " << ( !nameSpace.isEmpty() ? QString(nameSpace).replace( "::", "_" ).toUpper() + '_' : "" )
     << className.toUpper() << "_H" << endl;
-  h << "#define " << ( !nameSpace.isEmpty() ? nameSpace.toUpper() + '_' : "" )
+  h << "#define " << ( !nameSpace.isEmpty() ? QString(nameSpace).replace( "::", "_" ).toUpper() + '_' : "" )
     << className.toUpper() << "_H" << endl << endl;
 
   // Includes
   QStringList::ConstIterator it;
-  for( it = headerIncludes.begin(); it != headerIncludes.end(); ++it ) {
+  for( it = headerIncludes.constBegin(); it != headerIncludes.constEnd(); ++it ) {
     if ( (*it).startsWith('"') )
       h << "#include " << *it << endl;
     else
@@ -1349,15 +1456,14 @@ int main( int argc, char **argv )
   h << "#include <kdebug.h>" << endl << endl;
 
   // Includes
-  for( it = includes.begin(); it != includes.end(); ++it ) {
+  for( it = includes.constBegin(); it != includes.constEnd(); ++it ) {
     if ( (*it).startsWith('"') )
       h << "#include " << *it << endl;
     else
       h << "#include <" << *it << ">" << endl;
   }
 
-  if ( !nameSpace.isEmpty() )
-    h << "namespace " << nameSpace << " {" << endl << endl;
+  beginNamespaces(nameSpace, h);
 
   // Private class declaration
   if ( dpointer )
@@ -1374,13 +1480,13 @@ int main( int argc, char **argv )
 
   // enums
   QList<CfgEntry*>::ConstIterator itEntry;
-  for( itEntry = entries.begin(); itEntry != entries.end(); ++itEntry ) {
+  for( itEntry = entries.constBegin(); itEntry != entries.constEnd(); ++itEntry ) {
     const CfgEntry::Choices &choices = (*itEntry)->choices();
     QList<CfgEntry::Choice> chlist = choices.choices;
     if ( !chlist.isEmpty() ) {
       QStringList values;
       QList<CfgEntry::Choice>::ConstIterator itChoice;
-      for( itChoice = chlist.begin(); itChoice != chlist.end(); ++itChoice ) {
+      for( itChoice = chlist.constBegin(); itChoice != chlist.constEnd(); ++itChoice ) {
         values.append( choices.prefix + (*itChoice).name );
       }
       if ( choices.name().isEmpty() ) {
@@ -1451,10 +1557,10 @@ int main( int argc, char **argv )
             h << " KSharedConfig::Ptr config"
                 << (parameters.isEmpty() ? " = KGlobal::config()" : ", ");
     }
-    for (QList<Param>::ConstIterator it = parameters.begin();
-         it != parameters.end(); ++it)
+    for (QList<Param>::ConstIterator it = parameters.constBegin();
+         it != parameters.constEnd(); ++it)
     {
-       if (it != parameters.begin())
+       if (it != parameters.constBegin())
          h << ",";
        h << " " << param((*it).type) << " " << (*it).name;
     }
@@ -1476,7 +1582,7 @@ int main( int argc, char **argv )
   else
     Const = " const";
 
-  for( itEntry = entries.begin(); itEntry != entries.end(); ++itEntry ) {
+  for( itEntry = entries.constBegin(); itEntry != entries.constEnd(); ++itEntry ) {
     QString n = (*itEntry)->name();
     QString t = (*itEntry)->type();
 
@@ -1538,6 +1644,37 @@ int main( int argc, char **argv )
       h << ";" << endl;
     }
 
+    // Default value Accessor
+    if ((allDefaultGetters || defaultGetters.contains(n)) && !(*itEntry)->defaultValue().isEmpty()) {
+      h << endl;
+      h << "    /**" << endl;
+      h << "      Get " << (*itEntry)->label() << " default value" << endl;
+      h << "    */" << endl;
+      if (staticAccessors)
+        h << "    static" << endl;
+      h << "    ";
+      if (useEnumTypes && t == "Enum")
+        h << enumType(*itEntry);
+      else
+        h << cppType(t);
+      h << " " << getDefaultFunction(n) << "(";
+      if ( !(*itEntry)->param().isEmpty() )
+          h << " " << cppType( (*itEntry)->paramType() ) <<" i ";
+      h << ")" << Const << endl;
+      h << "    {" << endl;
+      h << "        return ";
+      if (useEnumTypes && t == "Enum")
+        h << "static_cast<" << enumType(*itEntry) << ">(";
+      h << getDefaultFunction(n) << "_helper(";
+      if ( !(*itEntry)->param().isEmpty() )
+          h << " i ";
+      h << ")";
+      if (useEnumTypes && t == "Enum")
+        h << ")";
+      h << ";" << endl;
+      h << "    }" << endl;
+    }
+
     // Item accessor
     if ( itemAccessors ) {
       h << endl;
@@ -1622,8 +1759,8 @@ int main( int argc, char **argv )
   }
 
   // Class Parameters
-  for (QList<Param>::ConstIterator it = parameters.begin();
-       it != parameters.end(); ++it)
+  for (QList<Param>::ConstIterator it = parameters.constBegin();
+       it != parameters.constEnd(); ++it)
   {
      h << "    " << cppType((*it).type) << " mParam" << (*it).name << ";" << endl;
   }
@@ -1631,7 +1768,7 @@ int main( int argc, char **argv )
   if ( memberVariables != "dpointer" )
   {
     QString group;
-    for( itEntry = entries.begin(); itEntry != entries.end(); ++itEntry ) {
+    for( itEntry = entries.constBegin(); itEntry != entries.constEnd(); ++itEntry ) {
       if ( (*itEntry)->group() != group ) {
         group = (*itEntry)->group();
         h << endl;
@@ -1643,11 +1780,22 @@ int main( int argc, char **argv )
         h << QString("[%1]").arg( (*itEntry)->paramMax()+1 );
       }
       h << ";" << endl;
+
+      if ( allDefaultGetters || defaultGetters.contains((*itEntry)->name()) )
+      {
+        h << "    ";
+        if (staticAccessors)
+          h << "static ";
+        h << cppType((*itEntry)->type()) << " " << getDefaultFunction((*itEntry)->name()) << "_helper(";
+        if ( !(*itEntry)->param().isEmpty() )
+            h << " " << cppType( (*itEntry)->paramType() ) <<" i ";
+        h << ")" << Const << ";" << endl;
+      }
     }
 
     h << endl << "  private:" << endl;
     if ( itemAccessors ) {
-       for( itEntry = entries.begin(); itEntry != entries.end(); ++itEntry ) {
+       for( itEntry = entries.constBegin(); itEntry != entries.constEnd(); ++itEntry ) {
         h << "    Item" << itemType( (*itEntry)->type() ) << " *" << itemVar( *itEntry );
         if ( !(*itEntry)->param().isEmpty() ) h << QString("[%1]").arg( (*itEntry)->paramMax()+1 );
         h << ";" << endl;
@@ -1661,6 +1809,17 @@ int main( int argc, char **argv )
   {
     // use a private class for both member variables and items
     h << "  private:" << endl;
+    for( itEntry = entries.constBegin(); itEntry != entries.constEnd(); ++itEntry ) {
+      if ( allDefaultGetters || defaultGetters.contains((*itEntry)->name()) ) {
+        h << "    ";
+        if (staticAccessors)
+          h << "static ";
+        h << cppType((*itEntry)->type()) << " " << getDefaultFunction((*itEntry)->name()) << "_helper(";
+        if ( !(*itEntry)->param().isEmpty() )
+            h << " " << cppType( (*itEntry)->paramType() ) <<" i ";
+        h << ")" << Const << ";" << endl;
+      }
+    }
     h << "    " + className + "Private *d;" << endl;
   }
 
@@ -1672,7 +1831,7 @@ int main( int argc, char **argv )
 
   h << "};" << endl << endl;
 
-  if ( !nameSpace.isEmpty() ) h << "}" << endl << endl;
+  endNamespaces(nameSpace, h);
 
   h << "#endif" << endl << endl;
 
@@ -1681,7 +1840,7 @@ int main( int argc, char **argv )
 
   QFile implementation( baseDir + implementationFileName );
   if ( !implementation.open( QIODevice::WriteOnly ) ) {
-    std::cerr << "Can't open '" << qPrintable(implementationFileName) << "for writing."
+    std::cerr << "Can not open '" << qPrintable(implementationFileName) << "for writing."
               << std::endl;
     return 1;
   }
@@ -1694,7 +1853,7 @@ int main( int argc, char **argv )
 
   cpp << "#include \"" << headerFileName << "\"" << endl << endl;
 
-  for( it = sourceIncludes.begin(); it != sourceIncludes.end(); ++it ) {
+  for( it = sourceIncludes.constBegin(); it != sourceIncludes.constEnd(); ++it ) {
     if ( (*it).startsWith('"') )
       cpp << "#include " << *it << endl;
     else
@@ -1719,12 +1878,11 @@ int main( int argc, char **argv )
   // private class implementation
   if ( dpointer )
   {
-    if ( !nameSpace.isEmpty() )
-      cpp << "namespace " << nameSpace << " {" << endl;
+    beginNamespaces(nameSpace, cpp);
     cpp << "class " << className << "Private" << endl;
     cpp << "{" << endl;
     cpp << "  public:" << endl;
-    for( itEntry = entries.begin(); itEntry != entries.end(); ++itEntry ) {
+    for( itEntry = entries.constBegin(); itEntry != entries.constEnd(); ++itEntry ) {
       if ( (*itEntry)->group() != group ) {
         group = (*itEntry)->group();
         cpp << endl;
@@ -1738,7 +1896,7 @@ int main( int argc, char **argv )
       cpp << ";" << endl;
     }
     cpp << endl << "    // items" << endl;
-    for( itEntry = entries.begin(); itEntry != entries.end(); ++itEntry ) {
+    for( itEntry = entries.constBegin(); itEntry != entries.constEnd(); ++itEntry ) {
       cpp << "    KConfigSkeleton::Item" << itemType( (*itEntry)->type() ) << " *" << itemVar( *itEntry );
       if ( !(*itEntry)->param().isEmpty() ) cpp << QString("[%1]").arg( (*itEntry)->paramMax()+1 );
         cpp << ";" << endl;
@@ -1748,14 +1906,12 @@ int main( int argc, char **argv )
     }
 
     cpp << "};" << endl << endl;
-    if ( !nameSpace.isEmpty() )
-      cpp << "}" << endl << endl;
+    endNamespaces(nameSpace, cpp);
   }
 
   // Singleton implementation
   if ( singleton ) {
-    if( !nameSpace.isEmpty() )
-      cpp << "namespace " << nameSpace << " {" << endl;
+    beginNamespaces(nameSpace, cpp);
     cpp << "class " << className << "Helper" << endl;
     cpp << '{' << endl;
     cpp << "  public:" << endl;
@@ -1763,8 +1919,7 @@ int main( int argc, char **argv )
     cpp << "    ~" << className << "Helper() { delete q; }" << endl;
     cpp << "    " << className << " *q;" << endl;
     cpp << "};" << endl;
-    if( !nameSpace.isEmpty() )
-      cpp << "}" << endl;
+    endNamespaces(nameSpace, cpp);
     cpp << "K_GLOBAL_STATIC(" << className << "Helper, s_global" << className << ")" << endl;
 
     cpp << className << " *" << className << "::self()" << endl;
@@ -1807,10 +1962,10 @@ int main( int argc, char **argv )
     cpp << (parameters.isEmpty() ? " " : ", ");
   }
 
-  for (QList<Param>::ConstIterator it = parameters.begin();
-       it != parameters.end(); ++it)
+  for (QList<Param>::ConstIterator it = parameters.constBegin();
+       it != parameters.constEnd(); ++it)
   {
-     if (it != parameters.begin())
+     if (it != parameters.constBegin())
        cpp << ",";
      cpp << " " << param((*it).type) << " " << (*it).name;
   }
@@ -1823,16 +1978,23 @@ int main( int argc, char **argv )
   cpp << ")" << endl;
 
   // Store parameters
-  for (QList<Param>::ConstIterator it = parameters.begin();
-       it != parameters.end(); ++it)
+  for (QList<Param>::ConstIterator it = parameters.constBegin();
+       it != parameters.constEnd(); ++it)
   {
      cpp << "  , mParam" << (*it).name << "(" << (*it).name << ")" << endl;
   }
 
+  if ( hasSignals && !dpointer )
+    cpp << "  , " << varName("settingsChanged") << "(0)" << endl;
+
   cpp << "{" << endl;
 
   if (dpointer)
+  {
     cpp << "  d = new " + className + "Private;" << endl;
+    if (hasSignals)
+      cpp << "  " << varPath("settingsChanged") << " = 0;" << endl;
+  }
   // Needed in case the singleton class is used as baseclass for
   // another singleton.
   if (singleton) {
@@ -1842,7 +2004,7 @@ int main( int argc, char **argv )
 
   group.clear();
 
-  for( itEntry = entries.begin(); itEntry != entries.end(); ++itEntry ) {
+  for( itEntry = entries.constBegin(); itEntry != entries.constEnd(); ++itEntry ) {
     if ( (*itEntry)->group() != group ) {
       if ( !group.isEmpty() ) cpp << endl;
       group = (*itEntry)->group();
@@ -1850,19 +2012,18 @@ int main( int argc, char **argv )
     }
 
     QString key = paramString( (*itEntry)->key(), parameters );
-    if ( !(*itEntry)->code().isEmpty())
-    {
+    if ( !(*itEntry)->code().isEmpty() ) {
       cpp << (*itEntry)->code() << endl;
     }
     if ( (*itEntry)->type() == "Enum" ) {
-      cpp << "  QList<KConfigSkeleton::ItemEnum::Choice> values"
+      cpp << "  QList<KConfigSkeleton::ItemEnum::Choice2> values"
           << (*itEntry)->name() << ";" << endl;
       QList<CfgEntry::Choice> choices = (*itEntry)->choices().choices;
       QList<CfgEntry::Choice>::ConstIterator it;
-      for( it = choices.begin(); it != choices.end(); ++it ) {
+      for( it = choices.constBegin(); it != choices.constEnd(); ++it ) {
         cpp << "  {" << endl;
-        cpp << "    KConfigSkeleton::ItemEnum::Choice choice;" << endl;
-        cpp << "    choice.name = QLatin1String( \"" << (*it).name << "\" );" << endl;
+        cpp << "    KConfigSkeleton::ItemEnum::Choice2 choice;" << endl;
+        cpp << "    choice.name = QLatin1String(\"" << (*it).name << "\");" << endl;
         if ( setUserTexts ) {
           if ( !(*it).label.isEmpty() ) {
             cpp << "    choice.label = ";
@@ -1872,13 +2033,22 @@ int main( int argc, char **argv )
               cpp << "i18n(";
             cpp << quoteString((*it).label) << ");" << endl;
           }
-          if ( !(*it).whatsThis.isEmpty() )
+          if ( !(*it).toolTip.isEmpty() ) {
+            cpp << "    choice.toolTip = ";
+            if ( !(*it).context.isEmpty() )
+              cpp << "i18nc(" + quoteString((*it).context) + ", ";
+            else
+              cpp << "i18n(";
+            cpp << quoteString((*it).toolTip) << ");" << endl;
+          }
+          if ( !(*it).whatsThis.isEmpty() ) {
             cpp << "    choice.whatsThis = ";
             if ( !(*it).context.isEmpty() )
               cpp << "i18nc(" + quoteString((*it).context) + ", ";
             else
               cpp << "i18n(";
             cpp << quoteString((*it).whatsThis) << ");" << endl;
+          }
         }
         cpp << "    values" << (*itEntry)->name() << ".append( choice );" << endl;
         cpp << "  }" << endl;
@@ -1949,7 +2119,7 @@ int main( int argc, char **argv )
   if (dpointer)
   {
     // setters and getters go in Cpp if in dpointer mode
-    for( itEntry = entries.begin(); itEntry != entries.end(); ++itEntry ) {
+    for( itEntry = entries.constBegin(); itEntry != entries.constEnd(); ++itEntry ) {
       QString n = (*itEntry)->name();
       QString t = (*itEntry)->type();
 
@@ -1963,7 +2133,7 @@ int main( int argc, char **argv )
           cpp << enumType(*itEntry);
         else
           cpp << param( t );
-        cpp << " v )";
+        cpp << " v )" << endl;
         // function body inline only if not using dpointer
         // for BC mode
         cpp << "{" << endl;
@@ -1986,6 +2156,8 @@ int main( int argc, char **argv )
       cpp << indent(memberAccessorBody( *itEntry ), 2);
       cpp << "}" << endl << endl;
 
+      // Default value Accessor -- written by the loop below
+
       // Item accessor
       if ( itemAccessors )
       {
@@ -2005,12 +2177,32 @@ int main( int argc, char **argv )
     }
   }
 
+  // default value getters always go in Cpp
+  for( itEntry = entries.constBegin(); itEntry != entries.constEnd(); ++itEntry ) {
+    QString n = (*itEntry)->name();
+    QString t = (*itEntry)->type();
+
+    // Default value Accessor, as "helper" function
+    if (( allDefaultGetters || defaultGetters.contains(n) ) && !(*itEntry)->defaultValue().isEmpty() ) {
+      cpp << cppType(t) << " " << getDefaultFunction(n, className) << "_helper(";
+      if ( !(*itEntry)->param().isEmpty() )
+        cpp << " " << cppType( (*itEntry)->paramType() ) <<" i ";
+      cpp << ")" << Const << endl;
+      cpp << "{" << endl;
+      cpp << memberGetDefaultBody(*itEntry) << endl;
+      cpp << "}" << endl << endl;
+    }
+  }
+
   // Destructor
   cpp << className << "::~" << className << "()" << endl;
   cpp << "{" << endl;
   if ( singleton ) {
     if ( dpointer )
       cpp << "  delete d;" << endl;
+    cpp << "  if (!s_global" << className << ".isDestroyed()) {" << endl;
+    cpp << "    s_global" << className << "->q = 0;" << endl;
+    cpp << "  }" << endl;
   }
   cpp << "}" << endl << endl;
 
