@@ -4,6 +4,7 @@
     Copyright (c) 1998-2005 Stephan Kulow <coolo@kde.org>
     Copyright (c) 1999-2004 Waldo Bastian <bastian@kde.org>
     Copyright (c) 2001-2005 Lubos Lunak <l.lunak@kde.org>
+    Copyright (C) 2008 Aaron Seigo <aseigo@kde.org>
 
     This library is free software; you can redistribute it and/or
     modify it under the terms of the GNU Library General Public
@@ -61,7 +62,10 @@ class KToolInvocationPrivate;
  * Calls to members are only allowed if there is a Q(Core)Application object created
  * If you call the members with signal/slot connections across threads, you can't use the return values
  * If a function is called from the wrong thread and it has a return value -1 is returned
- * Investigate if this is really needed or if DCOP/DBUS is threadsafe anyway
+ * Investigate if this is really needed or if DBUS is threadsafe anyway
+ *
+ * For more details see <a
+ * href="http://techbase.kde.org/Development/Architecture/KDE4/Starting_Other_Programs#KToolInvocation::startServiceByDesktopPath">techbase</a>.
  *
  */
 class KDECORE_EXPORT KToolInvocation : public QObject
@@ -91,7 +95,7 @@ public Q_SLOTS:
 
   static void invokeHelp( const QString& anchor = QString(),
                    const QString& appname = QString(),
-		   const QByteArray& startup_id = "");
+		   const QByteArray& startup_id = QByteArray());
 
   /**
    * Convenience method; invokes the standard email application.
@@ -101,7 +105,8 @@ public Q_SLOTS:
    * @param startup_id for app startup notification, "0" for none,
    *           "" ( empty string ) is the default
    */
-  static void invokeMailer( const QString &address, const QString &subject,const QByteArray& startup_id = "" );
+  static void invokeMailer( const QString &address, const QString &subject,
+          const QByteArray& startup_id = QByteArray() );
 
   /**
    * Invokes the standard email application.
@@ -112,7 +117,8 @@ public Q_SLOTS:
    * @param allowAttachments whether attachments specified in mailtoURL should be honoured.
                The default is false; do not honor requests for attachments.
    */
-  static void invokeMailer( const KUrl &mailtoURL, const QByteArray& startup_id = "" ,  bool allowAttachments = false );
+  static void invokeMailer( const KUrl &mailtoURL, const QByteArray& startup_id = QByteArray(),
+          bool allowAttachments = false );
 
   /**
    * Convenience method; invokes the standard email application.
@@ -131,19 +137,40 @@ public Q_SLOTS:
    */
   static void invokeMailer(const QString &to, const QString &cc, const QString &bcc,
                     const QString &subject, const QString &body,
-                    const QString &messageFile = QString(), const QStringList &attachURLs = QStringList(),const QByteArray& startup_id = "" );
+                    const QString &messageFile = QString(),
+                    const QStringList &attachURLs = QStringList(),
+                    const QByteArray& startup_id = QByteArray() );
 
   /**
-   * Invokes the standard browser.
+   * Invokes the user's preferred browser.
    * Note that you should only do this when you know for sure that the browser can
    * handle the URL (i.e. its mimetype). In doubt, if the URL can point to an image
-   * or anything else than directory or HTML, prefer to use new KRun( url ).
+   * or anything else than HTML, prefer to use new KRun( url ).
+   *
+   * See also <a
+   * href="http://techbase.kde.org/Development/Architecture/KDE4/Starting_Other_Programs#KToolInvocation::invokeBrowser>techbase</a>
+   * for a discussion of invokeBrowser vs KRun.
    *
    * @param url The destination address
    * @param startup_id for app startup notification, "0" for none,
    *           "" ( empty string ) is the default
    */
-  static void invokeBrowser( const QString &url, const QByteArray& startup_id = "" );
+  static void invokeBrowser( const QString &url,
+          const QByteArray& startup_id = QByteArray() );
+
+  /**
+   * Invokes the standard terminal application.
+   *
+   * @param command the command to execute, can be empty.
+   * @param workdir the initial working directory, can be empty.
+   * @param startup_id for app startup notification, "0" for none,
+   *           "" ( empty string ) is the default
+   *
+   * @since 4.1
+   */
+  static void invokeTerminal(const QString &command,
+                             const QString& workdir = QString(),
+                             const QByteArray &startup_id = "");
 
 public:
   /**
@@ -174,7 +201,7 @@ public:
    */
   static int startServiceByName( const QString& _name, const QString &URL,
                                  QString *error=0, QString *serviceName=0, int *pid=0,
-                                 const QByteArray &startup_id = "", bool noWait = false );
+                                 const QByteArray &startup_id = QByteArray(), bool noWait = false );
 
   /**
    * Starts a service based on the (translated) name of the service.
@@ -197,7 +224,8 @@ public:
    * @return an error code indicating success (== 0) or failure (> 0).
    */
   static int startServiceByName( const QString& _name, const QStringList &URLs=QStringList(),
-                QString *error=0, QString *serviceName=0, int *pid=0, const QByteArray &startup_id = "", bool noWait = false );
+                QString *error=0, QString *serviceName=0, int *pid=0,
+                const QByteArray &startup_id = QByteArray(), bool noWait = false );
 
   /**
    * Starts a service based on the desktop path of the service.
@@ -220,7 +248,8 @@ public:
    * @return an error code indicating success (== 0) or failure (> 0).
    */
   static int startServiceByDesktopPath( const QString& _name, const QString &URL,
-                QString *error=0, QString *serviceName=0, int *pid = 0, const QByteArray &startup_id = "", bool noWait = false );
+                QString *error=0, QString *serviceName=0, int *pid = 0,
+                const QByteArray &startup_id = QByteArray(), bool noWait = false );
 
   /**
    * Starts a service based on the desktop path of the service.
@@ -242,7 +271,8 @@ public:
    * @return an error code indicating success (== 0) or failure (> 0).
    */
   static int startServiceByDesktopPath( const QString& _name, const QStringList &URLs=QStringList(),
-                QString *error=0, QString *serviceName=0, int *pid = 0, const QByteArray &startup_id = "", bool noWait = false );
+                QString *error=0, QString *serviceName=0, int *pid = 0,
+                const QByteArray &startup_id = QByteArray(), bool noWait = false );
 
   /**
    * Starts a service based on the desktop name of the service.
@@ -266,7 +296,7 @@ public:
    */
   static int startServiceByDesktopName( const QString& _name, const QString &URL,
                                         QString *error=0, QString *serviceName=0, int *pid = 0,
-                                        const QByteArray &startup_id = "", bool noWait = false );
+                                        const QByteArray &startup_id = QByteArray(), bool noWait = false );
 
   /**
    * Starts a service based on the desktop name of the service.
@@ -290,7 +320,7 @@ public:
    */
   static int startServiceByDesktopName( const QString& _name, const QStringList &URLs=QStringList(),
                                         QString *error=0, QString *serviceName=0, int *pid = 0,
-                                        const QByteArray &startup_id = "", bool noWait = false );
+                                        const QByteArray &startup_id = QByteArray(), bool noWait = false );
 
   /**
    * Starts a program via kdeinit.
@@ -310,7 +340,7 @@ public:
    * @return an error code indicating success (== 0) or failure (> 0).
    */
   static int kdeinitExec( const QString& name, const QStringList &args=QStringList(),
-                QString *error=0, int *pid = 0, const QByteArray& startup_id = "" );
+                QString *error=0, int *pid = 0, const QByteArray& startup_id = QByteArray() );
 
   /**
    * Starts a program via kdeinit and wait for it to finish.
@@ -330,7 +360,7 @@ public:
    * @return an error code indicating success (== 0) or failure (> 0).
    */
   static int kdeinitExecWait( const QString& name, const QStringList &args=QStringList(),
-                QString *error=0, int *pid = 0, const QByteArray& startup_id = "" );
+                QString *error=0, int *pid = 0, const QByteArray& startup_id = QByteArray() );
 
 Q_SIGNALS:
   /**
@@ -348,7 +378,8 @@ private:
   int startServiceInternal(const char *_function,
                            const QString& _name, const QStringList &URLs,
                            QString *error, QString *serviceName, int *pid,
-                           const QByteArray& startup_id, bool noWait);
+                           const QByteArray& startup_id, bool noWait,
+                           const QString& workdir = QString());
   static bool isMainThreadActive(QString* error = 0);
 
   KToolInvocationPrivate * const d;
