@@ -24,6 +24,7 @@
 #include <kglobal.h>
 #include <kdebug.h>
 #include <klocale.h>
+#include <klocale_p.h>
 #include <kcomponentdata.h>
 #include <klibrary.h>
 #include <kstandarddirs.h>
@@ -32,6 +33,7 @@
 #include <kuitsemantics_p.h>
 #include "kcatalogname_p.h"
 
+#include <QMutexLocker>
 #include <QStringList>
 #include <QByteArray>
 #include <QChar>
@@ -81,11 +83,13 @@ class KLocalizedStringPrivate
                         const QString &ctxt) const;
     QString substituteTranscript (const QString &trans,
                                   const QString &lang,
+                                  const QString &ctry,
                                   const QString &lscr,
                                   const QString &final,
                                   bool &fallback) const;
     int resolveInterpolation (const QString &trans, int pos,
                               const QString &lang,
+                              const QString &ctry,
                               const QString &lscr,
                               const QString &final,
                               QString &result,
@@ -93,6 +97,7 @@ class KLocalizedStringPrivate
     QVariant segmentToValue (const QString &arg) const;
     QString postTranscript (const QString &pcall,
                             const QString &lang,
+                            const QString &ctry,
                             const QString &lscr,
                             const QString &final) const;
 
@@ -208,6 +213,7 @@ QString KLocalizedString::toString (const KLocale *locale) const
 QString KLocalizedStringPrivate::toString (const KLocale *locale) const
 {
     KLocalizedStringPrivateStatics *s = staticsKLSP;
+    QMutexLocker lock(kLocaleMutex());
 
     // Assure the message has been supplied.
     if (msg.isEmpty())
@@ -226,24 +232,24 @@ QString KLocalizedStringPrivate::toString (const KLocale *locale) const
                               .arg(shortenMessage(QString::fromUtf8(msg)));
 
     // Get raw translation.
-    QString lang, rawtrans, lscr;
-    if (locale != NULL)
-    {
-        if (!ctxt.isEmpty() && !plural.isEmpty())
-            locale->translateRaw(ctxt, msg, plural, number, &lang, &rawtrans);
-        else if (!plural.isEmpty())
-            locale->translateRaw(msg, plural, number, &lang, &rawtrans);
-        else if (!ctxt.isEmpty())
-            locale->translateRaw(ctxt, msg, &lang, &rawtrans);
-        else
-            locale->translateRaw(msg, &lang, &rawtrans);
-
-        // Find any higher priority writing script for the current language.
-        lscr = KTranslit::higherPriorityScript(lang, locale);
-    }
-    else
-    {
+    QString rawlang, rawtrans, lang, lscr, ctry;
+    if (locale != NULL) {
+        if (!ctxt.isEmpty() && !plural.isEmpty()) {
+            locale->translateRaw(ctxt, msg, plural, number, &rawlang, &rawtrans);
+        } else if (!plural.isEmpty()) {
+            locale->translateRaw(msg, plural, number, &rawlang, &rawtrans);
+        } else if (!ctxt.isEmpty()) {
+            locale->translateRaw(ctxt, msg, &rawlang, &rawtrans);
+        } else {
+            locale->translateRaw(msg, &rawlang, &rawtrans);
+        }
+        ctry = locale->country();
+        // Split language into base language and script,
+        // if such split is defined for this language.
+        KTranslit::splitToBaseAndScript(rawlang, locale, lang, lscr);
+    } else {
         lang = KLocale::defaultLanguage();
+        ctry = 'C';
         rawtrans = selectForEnglish();
     }
 
@@ -293,7 +299,7 @@ QString KLocalizedStringPrivate::toString (const KLocale *locale) const
     if (!strans.isEmpty()) {
         // Evaluate scripted translation.
         bool fallback;
-        QString sfinal = substituteTranscript(strans, lang, lscr, final, fallback);
+        QString sfinal = substituteTranscript(strans, lang, ctry, lscr, final, fallback);
 
         // If any translation produced and no fallback requested.
         if (!sfinal.isEmpty() && !fallback) {
@@ -307,7 +313,7 @@ QString KLocalizedStringPrivate::toString (const KLocale *locale) const
     {
         QStringList pcalls = s->ktrs->postCalls(lang);
         foreach(const QString &pcall, pcalls)
-            postTranscript(pcall, lang, lscr, final);
+            postTranscript(pcall, lang, ctry, lscr, final);
     }
 
     return final;
@@ -462,6 +468,7 @@ QString KLocalizedStringPrivate::postFormat (const QString &text,
                                              const QString &ctxt) const
 {
     KLocalizedStringPrivateStatics *s = staticsKLSP;
+    QMutexLocker lock(kLocaleMutex());
 
     QString final = text;
 
@@ -480,11 +487,13 @@ QString KLocalizedStringPrivate::postFormat (const QString &text,
 
 QString KLocalizedStringPrivate::substituteTranscript (const QString &strans,
                                                        const QString &lang,
+                                                       const QString &ctry,
                                                        const QString &lscr,
                                                        const QString &final,
                                                        bool &fallback) const
 {
     KLocalizedStringPrivateStatics *s = staticsKLSP;
+    QMutexLocker lock(kLocaleMutex());
 
     if (s->ktrs == NULL)
         // Scripting engine not available.
@@ -505,7 +514,7 @@ QString KLocalizedStringPrivate::substituteTranscript (const QString &strans,
         // Resolve interpolation.
         QString result;
         bool fallbackLocal;
-        tpos = resolveInterpolation(strans, tpos, lang, lscr, final,
+        tpos = resolveInterpolation(strans, tpos, lang, ctry, lscr, final,
                                     result, fallbackLocal);
 
         // If there was a problem in parsing the interpolation, cannot proceed
@@ -536,6 +545,7 @@ QString KLocalizedStringPrivate::substituteTranscript (const QString &strans,
 int KLocalizedStringPrivate::resolveInterpolation (const QString &strans,
                                                    int pos,
                                                    const QString &lang,
+                                                   const QString &ctry,
                                                    const QString &lscr,
                                                    const QString &final,
                                                    QString &result,
@@ -548,8 +558,9 @@ int KLocalizedStringPrivate::resolveInterpolation (const QString &strans,
     // fallback is set to true if Transcript evaluation requested so.
 
     KLocalizedStringPrivateStatics *s = staticsKLSP;
+    QMutexLocker lock(kLocaleMutex());
 
-    result = QString();
+    result.clear();
     fallback = false;
 
     // Split interpolation into arguments.
@@ -609,7 +620,7 @@ int KLocalizedStringPrivate::resolveInterpolation (const QString &strans,
             else if (strans.mid(tpos, islen) == s->startInterp) { // sub-interpolation
                 QString resultLocal;
                 bool fallbackLocal;
-                tpos = resolveInterpolation(strans, tpos, lang, lscr, final,
+                tpos = resolveInterpolation(strans, tpos, lang, ctry, lscr, final,
                                             resultLocal, fallbackLocal);
                 if (tpos < 0) { // unrecoverable problem in sub-interpolation
                     // Error reported in the subcall.
@@ -676,7 +687,8 @@ int KLocalizedStringPrivate::resolveInterpolation (const QString &strans,
     QString msgid = QString::fromUtf8(msg);
     QString scriptError;
     bool fallbackLocal;
-    result = s->ktrs->eval(iargs, lang, lscr, msgctxt, dynctxt, msgid,
+    result = s->ktrs->eval(iargs, lang, ctry, lscr,
+                           msgctxt, dynctxt, msgid,
                            args, vals, final, s->scriptModulesToLoad,
                            scriptError, fallbackLocal);
     // s->scriptModulesToLoad will be cleared during the call.
@@ -698,6 +710,7 @@ int KLocalizedStringPrivate::resolveInterpolation (const QString &strans,
 QVariant KLocalizedStringPrivate::segmentToValue (const QString &seg) const
 {
     KLocalizedStringPrivateStatics *s = staticsKLSP;
+    QMutexLocker lock(kLocaleMutex());
 
     // Return invalid variant if segment is either not a proper
     // value reference, or the reference is out of bounds.
@@ -727,10 +740,12 @@ QVariant KLocalizedStringPrivate::segmentToValue (const QString &seg) const
 
 QString KLocalizedStringPrivate::postTranscript (const QString &pcall,
                                                  const QString &lang,
+                                                 const QString &ctry,
                                                  const QString &lscr,
                                                  const QString &final) const
 {
     KLocalizedStringPrivateStatics *s = staticsKLSP;
+    QMutexLocker lock(kLocaleMutex());
 
     if (s->ktrs == NULL)
         // Scripting engine not available.
@@ -744,7 +759,8 @@ QString KLocalizedStringPrivate::postTranscript (const QString &pcall,
     QString msgid = QString::fromUtf8(msg);
     QString scriptError;
     bool fallback;
-    QString dummy = s->ktrs->eval(iargs, lang, lscr, msgctxt, dynctxt, msgid,
+    QString dummy = s->ktrs->eval(iargs, lang, ctry, lscr,
+                                  msgctxt, dynctxt, msgid,
                                   args, vals, final, s->scriptModulesToLoad,
                                   scriptError, fallback);
     // s->scriptModulesToLoad will be cleared during the call.
@@ -760,27 +776,32 @@ QString KLocalizedStringPrivate::postTranscript (const QString &pcall,
     return final;
 }
 
-static QString wrapInt (const QString &numstr)
+static QString wrapNum (const QString &tag, const QString &numstr,
+                        int fieldWidth, const QChar &fillChar)
 {
-    return "<"KUIT_NUMINTG">" + numstr + "</"KUIT_NUMINTG">"; //krazy:exclude=doublequote_chars
-}
-
-static QString wrapReal (const QString &numstr)
-{
-    return "<"KUIT_NUMREAL">" + numstr + "</"KUIT_NUMREAL">"; //krazy:exclude=doublequote_chars
+    QString optag;
+    if (fieldWidth != 0) {
+        QString fillString = KuitSemantics::escape(fillChar);
+        optag = QString("<%1 width='%2' fill='%3'>")
+                       .arg(tag, QString::number(fieldWidth), fillString);
+    } else {
+        optag = QString("<%1>").arg(tag);
+    }
+    QString cltag = QString("</%1>").arg(tag);
+    return optag + numstr + cltag;
 }
 
 KLocalizedString KLocalizedString::subs (int a, int fieldWidth, int base,
                                          const QChar &fillChar) const
 {
-    if (!d->plural.isEmpty() && !d->numberSet)
-    {
-        d->number = static_cast<pluraln>(abs(a));
-        d->numberSet = true;
-        d->numberOrd = d->args.size();
-    }
     KLocalizedString kls(*this);
-    kls.d->args.append(wrapInt(QString("%1").arg(a, fieldWidth, base, fillChar)));
+    if (!kls.d->plural.isEmpty() && !kls.d->numberSet) {
+        kls.d->number = static_cast<pluraln>(abs(a));
+        kls.d->numberSet = true;
+        kls.d->numberOrd = d->args.size();
+    }
+    kls.d->args.append(wrapNum(KUIT_NUMINTG, QString::number(a, base),
+                               fieldWidth, fillChar));
     kls.d->vals.append(static_cast<intn>(a));
     return kls;
 }
@@ -788,14 +809,14 @@ KLocalizedString KLocalizedString::subs (int a, int fieldWidth, int base,
 KLocalizedString KLocalizedString::subs (uint a, int fieldWidth, int base,
                                          const QChar &fillChar) const
 {
-    if (!d->plural.isEmpty() && !d->numberSet)
-    {
-        d->number = static_cast<pluraln>(a);
-        d->numberSet = true;
-        d->numberOrd = d->args.size();
-    }
     KLocalizedString kls(*this);
-    kls.d->args.append(wrapInt(QString("%1").arg(a, fieldWidth, base, fillChar)));
+    if (!kls.d->plural.isEmpty() && !kls.d->numberSet) {
+        kls.d->number = static_cast<pluraln>(a);
+        kls.d->numberSet = true;
+        kls.d->numberOrd = d->args.size();
+    }
+    kls.d->args.append(wrapNum(KUIT_NUMINTG, QString::number(a, base),
+                               fieldWidth, fillChar));
     kls.d->vals.append(static_cast<uintn>(a));
     return kls;
 }
@@ -803,14 +824,14 @@ KLocalizedString KLocalizedString::subs (uint a, int fieldWidth, int base,
 KLocalizedString KLocalizedString::subs (long a, int fieldWidth, int base,
                                          const QChar &fillChar) const
 {
-    if (!d->plural.isEmpty() && !d->numberSet)
-    {
-        d->number = static_cast<pluraln>(abs(a));
-        d->numberSet = true;
-        d->numberOrd = d->args.size();
-    }
     KLocalizedString kls(*this);
-    kls.d->args.append(wrapInt(QString("%1").arg(a, fieldWidth, base, fillChar)));
+    if (!kls.d->plural.isEmpty() && !kls.d->numberSet) {
+        kls.d->number = static_cast<pluraln>(abs(a));
+        kls.d->numberSet = true;
+        kls.d->numberOrd = d->args.size();
+    }
+    kls.d->args.append(wrapNum(KUIT_NUMINTG, QString::number(a, base),
+                               fieldWidth, fillChar));
     kls.d->vals.append(static_cast<intn>(a));
     return kls;
 }
@@ -818,14 +839,14 @@ KLocalizedString KLocalizedString::subs (long a, int fieldWidth, int base,
 KLocalizedString KLocalizedString::subs (ulong a, int fieldWidth, int base,
                                          const QChar &fillChar) const
 {
-    if (!d->plural.isEmpty() && !d->numberSet)
-    {
-        d->number = static_cast<pluraln>(a);
-        d->numberSet = true;
-        d->numberOrd = d->args.size();
-    }
     KLocalizedString kls(*this);
-    kls.d->args.append(wrapInt(QString("%1").arg(a, fieldWidth, base, fillChar)));
+    if (!kls.d->plural.isEmpty() && !kls.d->numberSet) {
+        kls.d->number = static_cast<pluraln>(a);
+        kls.d->numberSet = true;
+        kls.d->numberOrd = d->args.size();
+    }
+    kls.d->args.append(wrapNum(KUIT_NUMINTG, QString::number(a, base),
+                               fieldWidth, fillChar));
     kls.d->vals.append(static_cast<uintn>(a));
     return kls;
 }
@@ -833,14 +854,14 @@ KLocalizedString KLocalizedString::subs (ulong a, int fieldWidth, int base,
 KLocalizedString KLocalizedString::subs (qlonglong a, int fieldWidth, int base,
                                          const QChar &fillChar) const
 {
-    if (!d->plural.isEmpty() && !d->numberSet)
-    {
-        d->number = static_cast<pluraln>(qAbs(a));
-        d->numberSet = true;
-        d->numberOrd = d->args.size();
-    }
     KLocalizedString kls(*this);
-    kls.d->args.append(wrapInt(QString("%1").arg(a, fieldWidth, base, fillChar)));
+    if (!kls.d->plural.isEmpty() && !kls.d->numberSet) {
+        kls.d->number = static_cast<pluraln>(qAbs(a));
+        kls.d->numberSet = true;
+        kls.d->numberOrd = d->args.size();
+    }
+    kls.d->args.append(wrapNum(KUIT_NUMINTG, QString::number(a, base),
+                               fieldWidth, fillChar));
     kls.d->vals.append(static_cast<intn>(a));
     return kls;
 }
@@ -848,14 +869,14 @@ KLocalizedString KLocalizedString::subs (qlonglong a, int fieldWidth, int base,
 KLocalizedString KLocalizedString::subs (qulonglong a, int fieldWidth, int base,
                                          const QChar &fillChar) const
 {
-    if (!d->plural.isEmpty() && !d->numberSet)
-    {
-        d->number = static_cast<pluraln>(a);
-        d->numberSet = true;
-        d->numberOrd = d->args.size();
-    }
     KLocalizedString kls(*this);
-    kls.d->args.append(wrapInt(QString("%1").arg(a, fieldWidth, base, fillChar)));
+    if (!kls.d->plural.isEmpty() && !kls.d->numberSet) {
+        kls.d->number = static_cast<pluraln>(a);
+        kls.d->numberSet = true;
+        kls.d->numberOrd = d->args.size();
+    }
+    kls.d->args.append(wrapNum(KUIT_NUMINTG, QString::number(a, base),
+                               fieldWidth, fillChar));
     kls.d->vals.append(static_cast<uintn>(a));
     return kls;
 }
@@ -865,7 +886,9 @@ KLocalizedString KLocalizedString::subs (double a, int fieldWidth,
                                          const QChar &fillChar) const
 {
     KLocalizedString kls(*this);
-    kls.d->args.append(wrapReal(QString("%1").arg(a, fieldWidth, format, precision, fillChar)));
+    kls.d->args.append(wrapNum(KUIT_NUMREAL,
+                               QString::number(a, format, precision),
+                               fieldWidth, fillChar));
     kls.d->vals.append(static_cast<realn>(a));
     return kls;
 }
@@ -929,6 +952,7 @@ extern "C"
 void KLocalizedStringPrivate::loadTranscript ()
 {
     KLocalizedStringPrivateStatics *s = staticsKLSP;
+    QMutexLocker lock(kLocaleMutex());
 
     s->loadTranscriptCalled = true;
     s->ktrs = NULL; // null indicates that Transcript is not available
@@ -962,6 +986,8 @@ void KLocalizedStringPrivate::notifyCatalogsUpdated (const QStringList &language
         return;
     }
     KLocalizedStringPrivateStatics *s = staticsKLSP;
+    // Very important: do not the mutex here.
+    //QMutexLocker lock(kLocaleMutex());
 
     // Find script modules for all included language/catalogs that have them,
     // and remember their paths.
