@@ -1,5 +1,5 @@
 /* This file is part of the KDE libraries
-    Copyright (C) 2007 Andreas Hartmetz <ahartmetz@gmail.com>
+    Copyright (C) 2007, 2008 Andreas Hartmetz <ahartmetz@gmail.com>
 
     This library is free software; you can redistribute it and/or
     modify it under the terms of the GNU Library General Public
@@ -18,11 +18,14 @@
 */
 
 #include "ktcpsocket.h"
+#include "ktcpsocket_p.h"
 
 #include <kdebug.h>
 #include <kurl.h>
 #include <kglobal.h>
+#include <ksslcertificatemanager.h>
 #include <kstandarddirs.h>
+#include <klocale.h>
 
 #include <QtCore/QMutex>
 #include <QtCore/QStringList>
@@ -30,31 +33,6 @@
 #include <QtNetwork/QSslCipher>
 #include <QtNetwork/QNetworkProxy>
 
-K_GLOBAL_STATIC(QMutex, ksslsocketInitMutex)
-static QList<QSslCertificate> *kdeCaCertificateList;
-
-static void initKSslSocket()
-{
-    static bool initialized = false;
-    QMutexLocker locker(ksslsocketInitMutex);
-    if (!initialized) {
-        if (!kdeCaCertificateList) {
-            kdeCaCertificateList = new QList<QSslCertificate>;
-            QSslSocket::setDefaultCaCertificates(*kdeCaCertificateList); // set Qt's set to empty
-        }
-
-        if (!KGlobal::hasMainComponent())
-            return;                 // we need KGlobal::dirs() available
-        initialized = true;
-
-        // set default CAs from KDE's own bundle
-        QStringList bundles = KGlobal::dirs()->findAllResources("data", "kssl/ca-bundle.crt");
-        foreach (const QString &bundle, bundles) {
-            *kdeCaCertificateList += QSslCertificate::fromPath(bundle);
-        }
-        //kDebug(7029) << "Loading" << kdeCaCertificateList->count() << "CA certificates from" << bundles;
-    }
-}
 
 static KTcpSocket::SslVersion kSslVersionFromQ(QSsl::SslProtocol protocol)
 {
@@ -80,7 +58,7 @@ static QSsl::SslProtocol qSslProtocolFromK(KTcpSocket::SslVersion sslVersion)
         return QSsl::AnyProtocol;
     }
     //does it contain any valid protocol?
-    if (!(sslVersion & (KTcpSocket::SslV2 | KTcpSocket::SslV2 | KTcpSocket::TlsV1))) {
+    if (!(sslVersion & (KTcpSocket::SslV2 | KTcpSocket::SslV3 | KTcpSocket::TlsV1))) {
         return QSsl::UnknownProtocol;
     }
 
@@ -122,14 +100,14 @@ private:
 class KSslErrorPrivate
 {
 public:
-    KSslError::Error errorFromQSslError(QSslError::SslError e)
+    static KSslError::Error errorFromQSslError(QSslError::SslError e)
     {
         switch (e) {
         case QSslError::NoError:
             return KSslError::NoError;
         case QSslError::UnableToGetLocalIssuerCertificate:
         case QSslError::InvalidCaCertificate:
-            return KSslError::InvalidCertificateAuthority;
+            return KSslError::InvalidCertificateAuthorityCertificate;
         case QSslError::InvalidNotBeforeField:
         case QSslError::InvalidNotAfterField:
         case QSslError::CertificateNotYetValid:
@@ -168,8 +146,42 @@ public:
         }
     }
 
+    static QString errorString(KSslError::Error e)
+    {
+        switch (e) {
+        case KSslError::NoError:
+            return i18nc("SSL error","No error");
+        case KSslError::InvalidCertificateAuthorityCertificate:
+            return i18nc("SSL error","The certificate authority's certificate is invalid");
+        case KSslError::ExpiredCertificate:
+            return i18nc("SSL error","The certificate has expired");
+        case KSslError::InvalidCertificate:
+            return i18nc("SSL error","The certificate is invalid");
+        case KSslError::SelfSignedCertificate:
+            return i18nc("SSL error","The certificate is not signed by any trusted certificate authority");
+        case KSslError::RevokedCertificate:
+            return i18nc("SSL error","The certificate has been revoked");
+        case KSslError::InvalidCertificatePurpose:
+            return i18nc("SSL error","The certificate is unsuitable for this purpose");
+        case KSslError::UntrustedCertificate:
+            return i18nc("SSL error","The root certificate authority's certificate is not trusted for this purpose");
+        case KSslError::RejectedCertificate:
+            return i18nc("SSL error","The certificate authority's certificate is marked to reject this certificate's purpose");
+        case KSslError::NoPeerCertificate:
+            return i18nc("SSL error","The peer did not present any certificate");
+        case KSslError::HostNameMismatch:
+            return i18nc("SSL error","The certificate does not apply to the given host");
+        case KSslError::CertificateSignatureFailed:
+            return i18nc("SSL error","The certificate cannot be verified for internal reasons");
+        case KSslError::PathLengthExceeded:
+            return i18nc("SSL error","The certificate chain is too long");
+        case KSslError::UnknownError:
+        default:
+            return i18nc("SSL error","Unknown error");
+        }
+    }
+
     KSslError::Error error;
-    QString errorString;
     QSslCertificate certificate;
 };
 
@@ -179,15 +191,13 @@ KSslError::KSslError(Error errorCode, const QSslCertificate &certificate)
 {
     d->error = errorCode;
     d->certificate = certificate;
-    //TODO do *something* about the error string
 }
 
 
 KSslError::KSslError(const QSslError &other)
  : d(new KSslErrorPrivate())
 {
-    d->error = d->errorFromQSslError(other.error());
-    d->errorString = other.errorString();
+    d->error = KSslErrorPrivate::errorFromQSslError(other.error());
     d->certificate = other.certificate();
 }
 
@@ -220,7 +230,7 @@ KSslError::Error KSslError::error() const
 
 QString KSslError::errorString() const
 {
-    return d->errorString;
+    return KSslErrorPrivate::errorString(d->error);
 }
 
 
@@ -235,12 +245,11 @@ class KTcpSocketPrivate
 public:
     KTcpSocketPrivate(KTcpSocket *qq)
      : q(qq),
+       certificatesLoaded(false),
        emittedReadyRead(false)
     {
-        initKSslSocket();
-
-        Q_ASSERT(kdeCaCertificateList);
-        sock.setCaCertificates(*kdeCaCertificateList);
+        // create the instance, which sets Qt's static internal cert set to empty.
+        KSslCertificateManager::self();
     }
 
     KTcpSocket::State state(QAbstractSocket::SocketState s)
@@ -317,7 +326,11 @@ public:
     void reemitSslErrors(const QList<QSslError> &errors)
     {
         q->showSslErrors(); //H4X
-        emit q->sslErrors(errors);
+        QList<KSslError> kErrors;
+        foreach (const QSslError &e, errors) {
+            kErrors.append(KSslError(e));
+        }
+        emit q->sslErrors(kErrors);
     }
 
     void reemitStateChanged(QAbstractSocket::SocketState s)
@@ -342,7 +355,16 @@ public:
         }
     }
 
+    void maybeLoadCertificates()
+    {
+        if (!certificatesLoaded) {
+            sock.setCaCertificates(KSslCertificateManager::self()->rootCertificates());
+            certificatesLoaded = true;
+        }
+    }
+
     KTcpSocket *const q;
+    bool certificatesLoaded;
     bool emittedReadyRead;
     QSslSocket sock;
     QList<KSslCipher> ciphers;
@@ -361,7 +383,10 @@ KTcpSocket::KTcpSocket(QObject *parent)
     connect(&d->sock, SIGNAL(bytesWritten(qint64)), this, SIGNAL(bytesWritten(qint64)));
     connect(&d->sock, SIGNAL(readyRead()), this, SLOT(reemitReadyRead()));
     connect(&d->sock, SIGNAL(connected()), this, SIGNAL(connected()));
+    connect(&d->sock, SIGNAL(encrypted()), this, SIGNAL(encrypted()));
     connect(&d->sock, SIGNAL(disconnected()), this, SIGNAL(disconnected()));
+    connect(&d->sock, SIGNAL(proxyAuthenticationRequired(const QNetworkProxy &, QAuthenticator *)),
+            this, SIGNAL(proxyAuthenticationRequired(const QNetworkProxy &, QAuthenticator *)));
     connect(&d->sock, SIGNAL(error(QAbstractSocket::SocketError)),
             this, SLOT(reemitSocketError(QAbstractSocket::SocketError)));
     connect(&d->sock, SIGNAL(sslErrors(const QList<QSslError> &)),
@@ -421,7 +446,7 @@ bool KTcpSocket::isSequential() const
 bool KTcpSocket::open(QIODevice::OpenMode open)
 {
     bool ret = d->sock.open(open);
-    setOpenMode(d->sock.openMode());
+    setOpenMode(d->sock.openMode() | QIODevice::Unbuffered);
     return ret;
 }
 
@@ -463,7 +488,13 @@ void KTcpSocket::connectToHost(const QString &hostName, quint16 port, ProxyPolic
         //###
     }
     d->sock.connectToHost(hostName, port);
-    setOpenMode(d->sock.openMode());
+    // there are enough layers of buffers between us and the network, and there is a quirk
+    // in QIODevice that can make it try to readData() twice per read() call if buffered and
+    // reaData() does not deliver enough data the first time. like when the other side is
+    // simply not sending any more data...
+    // this can *apparently* lead to long delays sometimes which stalls applications.
+    // do not want.
+    setOpenMode(d->sock.openMode() | QIODevice::Unbuffered);
 }
 
 
@@ -473,7 +504,7 @@ void KTcpSocket::connectToHost(const QHostAddress &hostAddress, quint16 port, Pr
         //###
     }
     d->sock.connectToHost(hostAddress, port);
-    setOpenMode(d->sock.openMode());
+    setOpenMode(d->sock.openMode() | QIODevice::Unbuffered);
 }
 
 
@@ -483,14 +514,14 @@ void KTcpSocket::connectToHost(const KUrl &url, ProxyPolicy policy)
         //###
     }
     d->sock.connectToHost(url.host(), url.port());
-    setOpenMode(d->sock.openMode());
+    setOpenMode(d->sock.openMode() | QIODevice::Unbuffered);
 }
 
 
 void KTcpSocket::disconnectFromHost()
 {
     d->sock.disconnectFromHost();
-    setOpenMode(d->sock.openMode());
+    setOpenMode(d->sock.openMode() | QIODevice::Unbuffered);
 }
 
 
@@ -581,7 +612,9 @@ KTcpSocket::State KTcpSocket::state() const
 bool KTcpSocket::waitForConnected(int msecs)
 {
     bool ret = d->sock.waitForConnected(msecs);
-    setOpenMode(d->sock.openMode());
+    if (!ret)
+        setErrorString(d->sock.errorString());
+    setOpenMode(d->sock.openMode() | QIODevice::Unbuffered);
     return ret;
 }
 
@@ -589,7 +622,9 @@ bool KTcpSocket::waitForConnected(int msecs)
 bool KTcpSocket::waitForDisconnected(int msecs)
 {
     bool ret = d->sock.waitForDisconnected(msecs);
-    setOpenMode(d->sock.openMode());
+    if (!ret)
+        setErrorString(d->sock.errorString());
+    setOpenMode(d->sock.openMode() | QIODevice::Unbuffered);
     return ret;
 }
 
@@ -618,6 +653,7 @@ void KTcpSocket::addCaCertificates(const QList<QSslCertificate> &certificates)
 
 QList<QSslCertificate> KTcpSocket::caCertificates() const
 {
+    d->maybeLoadCertificates();
     return d->sock.caCertificates();
 }
 
@@ -630,9 +666,10 @@ QList<KSslCipher> KTcpSocket::ciphers() const
 
 void KTcpSocket::connectToHostEncrypted(const QString &hostName, quint16 port, OpenMode openMode)
 {
+    d->maybeLoadCertificates();
     d->sock.setProtocol(qSslProtocolFromK(d->advertisedSslVersion));
     d->sock.connectToHostEncrypted(hostName, port, openMode);
-    setOpenMode(d->sock.openMode());
+    setOpenMode(d->sock.openMode() | QIODevice::Unbuffered);
 }
 
 
@@ -663,6 +700,7 @@ KSslCipher KTcpSocket::sessionCipher() const
 void KTcpSocket::setCaCertificates(const QList<QSslCertificate> &certificates)
 {
     d->sock.setCaCertificates(certificates);
+    d->certificatesLoaded = true;
 }
 
 
@@ -691,6 +729,7 @@ void KTcpSocket::setLocalCertificate(const QString &fileName, QSsl::EncodingForm
 //TODO
 void KTcpSocket::setPrivateKey(const KSslKey &key)
 {
+    Q_UNUSED(key)
 }
 
 
@@ -698,6 +737,10 @@ void KTcpSocket::setPrivateKey(const KSslKey &key)
 void KTcpSocket::setPrivateKey(const QString &fileName, KSslKey::Algorithm algorithm,
                                QSsl::EncodingFormat format, const QByteArray &passPhrase)
 {
+    Q_UNUSED(fileName)
+    Q_UNUSED(algorithm)
+    Q_UNUSED(format)
+    Q_UNUSED(passPhrase)
 }
 
 
@@ -723,6 +766,7 @@ void KTcpSocket::ignoreSslErrors()
 //slot
 void KTcpSocket::startClientEncryption()
 {
+    d->maybeLoadCertificates();
     d->sock.setProtocol(qSslProtocolFromK(d->advertisedSslVersion));
     d->sock.startClientEncryption();
 }
@@ -942,9 +986,9 @@ QString KSslCipher::digestMethod() const
 {
     //### This is not really backend neutral. It works for OpenSSL and
     //    for RFC compliant names, though.
-    if (d->name.endsWith("SHA"))
+    if (d->name.endsWith(QLatin1String("SHA")))
         return "SHA-1";
-    else if (d->name.endsWith("MD5"))
+    else if (d->name.endsWith(QLatin1String("MD5")))
         return "MD5";
     else
         return "";
@@ -969,7 +1013,7 @@ int KSslCipher::usedBits() const
 }
 
 
-//static 
+//static
 QList<KSslCipher> KSslCipher::supportedCiphers()
 {
     QList<KSslCipher> ret;
@@ -981,4 +1025,38 @@ QList<KSslCipher> KSslCipher::supportedCiphers()
 }
 
 
+KSslErrorUiData::KSslErrorUiData()
+ : d(new Private())
+{
+    d->usedBits = 0;
+    d->bits = 0;
+}
+
+
+KSslErrorUiData::KSslErrorUiData(const KTcpSocket *socket)
+ : d(new Private())
+{
+    d->certificateChain = socket->peerCertificateChain();
+    d->sslErrors = socket->sslErrors();
+    d->ip = socket->peerAddress().toString();
+    d->host = socket->peerName();
+    d->sslProtocol = socket->negotiatedSslVersionName();
+    d->cipher = socket->sessionCipher().name();
+    d->usedBits = socket->sessionCipher().usedBits();
+    d->bits = socket->sessionCipher().supportedBits();
+}
+
+
+KSslErrorUiData::KSslErrorUiData(const KSslErrorUiData &other)
+ : d(new Private(*other.d))
+{}
+
+
+KSslErrorUiData &KSslErrorUiData::operator=(const KSslErrorUiData &other)
+{
+    *d = *other.d;
+    return *this;
+}
+
+
 #include "ktcpsocket.moc"
