@@ -78,6 +78,13 @@ static KService::List mimeTypeSycocaServiceOffers(const QString& mimeType)
     return lst;
 }
 
+#define CHECK_SERVICETYPE(genericServiceTypePtr) \
+    if (!genericServiceTypePtr) { \
+        kError(7014) << "KMimeTypeTrader: couldn't find service type" << genericServiceType << \
+            "\nPlease ensure that the .desktop file for it is installed; then run kbuildsycoca4."; \
+        return; \
+    }
+
 /**
  * Filter the offers for the requested mime type for the genericServiceType.
  *
@@ -87,7 +94,7 @@ static KService::List mimeTypeSycocaServiceOffers(const QString& mimeType)
 static void filterMimeTypeOffers(KServiceOfferList& list, const QString& genericServiceType)
 {
     KServiceType::Ptr genericServiceTypePtr = KServiceType::serviceType(genericServiceType);
-    Q_ASSERT(genericServiceTypePtr);
+    CHECK_SERVICETYPE(genericServiceTypePtr);
 
     QMutableListIterator<KServiceOffer> it(list);
     while(it.hasNext()) {
@@ -104,7 +111,7 @@ static void filterMimeTypeOffers(KServiceOfferList& list, const QString& generic
 static void filterMimeTypeOffers(KService::List& list, const QString& genericServiceType)
 {
     KServiceType::Ptr genericServiceTypePtr = KServiceType::serviceType(genericServiceType);
-    Q_ASSERT(genericServiceTypePtr);
+    CHECK_SERVICETYPE(genericServiceTypePtr);
 
     QMutableListIterator<KService::Ptr> it(list);
     while(it.hasNext()) {
@@ -118,6 +125,8 @@ static void filterMimeTypeOffers(KService::List& list, const QString& genericSer
     }
 }
 
+#undef CHECK_SERVICETYPE
+
 KService::List KMimeTypeTrader::query( const QString& mimeType,
                                        const QString& genericServiceType,
                                        const QString& constraint ) const
@@ -142,11 +151,11 @@ KService::Ptr KMimeTypeTrader::preferredService( const QString & mimeType, const
     Q_ASSERT(!genericServiceType.isEmpty());
     filterMimeTypeOffers(offers, genericServiceType);
 
-    KServiceOfferList::const_iterator itOff = offers.begin();
+    KServiceOfferList::const_iterator itOff = offers.constBegin();
     // Look for the first one that is allowed as default.
     // Since the allowed-as-default are first anyway, we only have
     // to look at the first one to know.
-    if( itOff != offers.end() && (*itOff).allowAsDefault() )
+    if( itOff != offers.constEnd() && (*itOff).allowAsDefault() )
         return (*itOff).service();
 
     //kDebug(7014) << "No offers, or none allowed as default";
