@@ -65,9 +65,11 @@ KProtocolInfo::KProtocolInfo(const QString &path)
   m_supportsListing = ( m_listing.count() > 0 );
   m_defaultMimetype = config.readEntry( "defaultMimetype" );
   m_determineMimetypeFromExtension = config.readEntry( "determineMimetypeFromExtension", true );
-  m_icon = config.readEntry( "Icon", "unknown" );
+  d->archiveMimetype = config.readEntry("archiveMimetype", QStringList());
+  m_icon = config.readEntry( "Icon" );
   m_config = config.readEntry( "config", m_name );
   m_maxSlaves = config.readEntry( "maxInstances", 1);
+  d->maxSlavesPerHost = config.readEntry( "maxInstancesPerHost", 0);
 
   QString tmp = config.readEntry( "input" );
   if ( tmp == "filesystem" )
@@ -149,7 +151,8 @@ KProtocolInfo::load( QDataStream& _str)
         >> d->extraFields >> i_showPreviews >> i_uriMode
         >> d->capabilities >> d->proxyProtocol
         >> i_canRenameFromFile >> i_canRenameToFile
-        >> i_canDeleteRecursive >> i_fileNameUsedForCopying;
+        >> i_canDeleteRecursive >> i_fileNameUsedForCopying
+        >> d->archiveMimetype >> d->maxSlavesPerHost;
 
    m_inputType = (Type) i_inputType;
    m_outputType = (Type) i_outputType;
@@ -227,7 +230,8 @@ KProtocolInfoPrivate::save( QDataStream& _str)
         << extraFields << i_showPreviews << i_uriMode
         << capabilities << proxyProtocol
         << i_canRenameFromFile << i_canRenameToFile
-        << i_canDeleteRecursive << i_fileNameUsedForCopying;
+        << i_canDeleteRecursive << i_fileNameUsedForCopying
+        << archiveMimetype << maxSlavesPerHost;
 }
 
 
@@ -255,7 +259,7 @@ QString KProtocolInfo::icon( const QString& _protocol )
   // We call the findProtocol directly (not via KProtocolManager) to bypass any proxy settings.
   KProtocolInfo::Ptr prot = KProtocolInfoFactory::self()->findProtocol(_protocol);
   if ( !prot )
-    return QLatin1String("unknown");
+    return QString();
 
   return prot->m_icon;
 }
@@ -279,6 +283,15 @@ int KProtocolInfo::maxSlaves( const QString& _protocol )
   return prot->m_maxSlaves;
 }
 
+int KProtocolInfo::maxSlavesPerHost( const QString& _protocol )
+{
+  KProtocolInfo::Ptr prot = KProtocolInfoFactory::self()->findProtocol(_protocol);
+  if ( !prot )
+    return 0;
+
+  return prot->d_func()->maxSlavesPerHost;
+}
+
 bool KProtocolInfo::determineMimetypeFromExtension( const QString &_protocol )
 {
   KProtocolInfo::Ptr prot = KProtocolInfoFactory::self()->findProtocol( _protocol );
@@ -367,6 +380,13 @@ QString KProtocolInfo::defaultMimeType() const
     return m_defaultMimetype;
 }
 
+
+QStringList KProtocolInfo::archiveMimeTypes() const
+{
+    Q_D(const KProtocolInfo);
+    return d->archiveMimetype;
+}
+
 bool KProtocolInfo::supportsListing() const
 {
     return m_supportsListing;
@@ -441,4 +461,3 @@ QDataStream& operator<<( QDataStream& s, const KProtocolInfo::ExtraField& field
   s << static_cast<int>( field.type );
   return s;
 }
-
