@@ -31,6 +31,8 @@ class QDataStream;
 class KProtocolInfoPrivate;
 
 /**
+ * \class KProtocolInfo kprotocolinfo.h <KProtocolInfo>
+ *
  * Information about I/O (Internet, etc.) protocols supported by KDE.
 
  * This class is useful if you want to know which protocols
@@ -51,6 +53,7 @@ class KDECORE_EXPORT KProtocolInfo : public KSycocaEntry
   friend class KProtocolManager;
 public:
   typedef KSharedPtr<KProtocolInfo> Ptr;
+    typedef QList<Ptr> List;
 
 public:
 
@@ -94,10 +97,14 @@ public:
 
   /**
    * Describes the type of a protocol.
+   * For instance ftp:// appears as a filesystem with folders and files,
+   * while bzip2:// appears as a single file (a stream of data),
+   * and telnet:// doesn't output anything.
+   * @see outputType
    */
-  enum Type { T_STREAM, ///< protocol returns a stream
-	      T_FILESYSTEM, ///<protocol describes location in a file system
-	      T_NONE,   ///< no information about the tyope available
+  enum Type { T_STREAM, ///< stream of data (e.g. single file)
+	      T_FILESYSTEM, ///< structured directory
+	      T_NONE,   ///< no information about the type available
 	      T_ERROR   ///< used to signal an error
   };
 
@@ -180,7 +187,7 @@ public:
    * This corresponds to the "Icon=" field in the protocol description file.
    *
    * @param protocol the protocol to check
-   * @return the icon of the protocol, or null if unknown
+   * @return the icon of the protocol, or an empty string if unknown
    */
   static QString icon( const QString& protocol );
 
@@ -193,7 +200,7 @@ public:
    * The default is the protocol name, see name()
    *
    * @param protocol the protocol to check
-   * @return the config file, or null if unknown
+   * @return the config file, or an empty string if unknown
    */
   static QString config( const QString& protocol );
 
@@ -211,6 +218,20 @@ public:
    */
   static int maxSlaves( const QString& protocol );
 
+
+  /**
+   * Returns the limit on the number of slaves for this protocol per host.
+   *
+   * This corresponds to the "maxInstancesPerHost=" field in the protocol description file.
+   * The default is 0 which means there is no per host limit.
+   *
+   * @param protocol the protocol to check
+   * @return the maximum number of slaves, or 1 if unknown
+   *
+   * @since 4.4
+   */
+  static int maxSlavesPerHost( const QString& protocol );
+
   /**
    * Returns whether mimetypes can be determined based on extension for this
    * protocol. For some protocols, e.g. http, the filename extension in the URL
@@ -230,7 +251,7 @@ public:
    * This corresponds to the "X-DocPath=" or "DocPath=" field in the protocol description file.
    *
    * @param protocol the protocol to check
-   * @return the docpath of the protocol, or null if unknown
+   * @return the docpath of the protocol, or an empty string if unknown
    */
   static QString docPath( const QString& protocol );
 
@@ -247,7 +268,7 @@ public:
    * the protocols themselves.
    *
    * @param protocol the protocol to check
-   * @return the class of the protocol, or null if unknown
+   * @return the class of the protocol, or an empty string if unknown
    */
   static QString protocolClass( const QString& protocol );
 
@@ -322,6 +343,8 @@ public:
   bool supportsListing() const;
   /// @internal. Use KProtocolManager instead.
   QString defaultMimeType() const;
+  /// @internal. Use KProtocolManager instead.
+  QStringList archiveMimeTypes() const;
 
 protected:
   QString m_name;
