@@ -18,11 +18,14 @@
 #ifndef KSERVICETEST_H
 #define KSERVICETEST_H
 
+#include <QAtomicInt>
 #include <QtCore/QObject>
 
 class KServiceTest : public QObject
 {
     Q_OBJECT
+public:
+    KServiceTest() : m_sycocaUpdateDone(0) {}
 private Q_SLOTS:
     void initTestCase();
     void testByName();
@@ -37,10 +40,18 @@ private Q_SLOTS:
     void testDefaultOffers();
     void testDeleteServiceTypeProfile();
     void testDBUSStartupType();
+    void testByStorageId();
+    void testActionsAndDataStream();
+    void testServiceGroups();
+    void testKSycocaUpdate();
+    void testReaderThreads();
+    void testThreads();
 
 private:
+    void createFakeService();
     QString m_firstOffer;
     bool m_hasKde4Konsole;
+    QAtomicInt m_sycocaUpdateDone;
 };
 
 #endif
