@@ -322,7 +322,7 @@ void KUrlTest::testSimpleMethods() // to test parsing, mostly
 
   //NOTE this test should be ran in UTF8 locale
   KUrl percentEncodedQuery( "http://mail.yandex.ru/message_part/%D0%9A%D1%80%D0%B8%D1%82%D0%B5%D1%80%D0%B8%D0%B8%20%D0%BE%D1%86%D0%B5%D0%BD%D0%B8%D0%B2%D0%B0%D0%BD%D0%B8%D1%8F%20%D0%BE%D1%80%D0%BB%D0%BE%D0%B2%D0%BE%D0%B9.rar?hid=1.1&mid=391.56424458.99241672611486679803334485488&name=%D0%9A%D1%80%D0%B8%D1%82%D0%B5%D1%80%D0%B8%D0%B8%20%D0%BE%D1%86%D0%B5%D0%BD%D0%B8%D0%B2%D0%B0%D0%BD%D0%B8%D1%8F%20%D0%BE%D1%80%D0%BB%D0%BE%D0%B2%D0%BE%D0%B9.rar" );
-  QCOMPARE( percentEncodedQuery.prettyUrl(), QString::fromUtf8("http://mail.yandex.ru/message_part/Критерии оценивания орловой.rar?hid=1.1&mid=391.56424458.99241672611486679803334485488&name=Критерии оценивания орловой.rar"));
+  QCOMPARE( percentEncodedQuery.prettyUrl(), QString::fromUtf8("http://mail.yandex.ru/message_part/Критерии оценивания орловой.rar?hid=1.1&mid=391.56424458.99241672611486679803334485488&name=%D0%9A%D1%80%D0%B8%D1%82%D0%B5%D1%80%D0%B8%D0%B8%20%D0%BE%D1%86%D0%B5%D0%BD%D0%B8%D0%B2%D0%B0%D0%BD%D0%B8%D1%8F%20%D0%BE%D1%80%D0%BB%D0%BE%D0%B2%D0%BE%D0%B9.rar"));
 
 #ifdef Q_WS_WIN
 #ifdef Q_CC_MSVC
@@ -546,6 +546,9 @@ void KUrlTest::testPathAndQuery()
 
   KUrl maelcum(QString::fromUtf8("http://a.b.c/äöu"));
   QCOMPARE(maelcum.encodedPathAndQuery(), QString("/%C3%A4%C3%B6u"));
+
+  KUrl gof("file:%2Ftmp%2Fkde-ogoffart%2Fkmail"); // weird URL, but well ;)
+  QCOMPARE(gof.path(), QString("/tmp/kde-ogoffart/kmail"));
 }
 
 void KUrlTest::testUpUrl()
@@ -630,9 +633,15 @@ void KUrlTest::testSetFileName() // and addPath
   KUrl singleFile( "foo.txt" );
   QCOMPARE( singleFile.path(), QString("foo.txt") );
   QCOMPARE( singleFile.pathOrUrl(), QString("foo.txt") );
+
+  QString pre;
+#ifdef Q_OS_WIN
+  // On Windows we explicitly prepend a slash, see KUrl::setPath
+  pre = "/";
+#endif
   singleFile.setFileName( "bar.bin" );
-  QCOMPARE( singleFile.path(), QString("bar.bin") );
-  QCOMPARE( singleFile.pathOrUrl(), QString("bar.bin") );
+  QCOMPARE( singleFile.path(), pre + QString("bar.bin") );
+  QCOMPARE( singleFile.pathOrUrl(), pre + QString("bar.bin") );
 }
 
 void KUrlTest::testDirectory()
@@ -692,6 +701,11 @@ void KUrlTest::testPrettyURL()
   QCOMPARE( url15581.prettyUrl(), QString("http://alain.knaff.linux.lu/bug-reports/kde/spaces in url.html") );
   QCOMPARE( url15581.url(), QString("http://alain.knaff.linux.lu/bug-reports/kde/spaces%20in%20url.html") );
 
+  KUrl newLineInQuery("http://localhost/?a=foo%0A%0Abar%20baz&b=foo%0Abar%21%3F");
+  QCOMPARE( newLineInQuery.prettyUrl(), QString("http://localhost/?a=foo%0A%0Abar%20baz&b=foo%0Abar%21%3F") );
+  KUrl nonUtf8Query("http://kde.org/?a=test%C2%A0foo%A0%A0%A0%A0bar");
+  QCOMPARE( nonUtf8Query.prettyUrl(), QString("http://kde.org/?a=test%C2%A0foo%A0%A0%A0%A0bar") );
+
   // KDE3 test was for parsing "percentage%in%url.html", but this is not supported; too broken.
   KUrl url15581bis("http://alain.knaff.linux.lu/bug-reports/kde/percentage%25in%25url.html");
   QCOMPARE( url15581bis.prettyUrl(), QString("http://alain.knaff.linux.lu/bug-reports/kde/percentage%25in%25url.html") );
@@ -701,6 +715,10 @@ void KUrlTest::testPrettyURL()
   QCOMPARE( urlWithPass.pass(), QString::fromLatin1( "password" ) );
   QCOMPARE( urlWithPass.prettyUrl(), QString::fromLatin1( "ftp://user@ftp.kde.org/path" ) );
 
+  KUrl urlWithPassAndNoUser("ftp://:password@ftp.kde.org/path");
+  QCOMPARE( urlWithPassAndNoUser.pass(), QString::fromLatin1( "password" ) );
+  QCOMPARE( urlWithPassAndNoUser.prettyUrl(), QString::fromLatin1( "ftp://ftp.kde.org/path" ) );
+
   KUrl xmppUri("xmpp:ogoffart@kde.org");
   QCOMPARE( xmppUri.prettyUrl(), QString::fromLatin1( "xmpp:ogoffart@kde.org" ) );
 
@@ -1088,7 +1106,7 @@ void KUrlTest::testBaseURL() // those are tests for the KUrl(base,relative) cons
   //QEXPECT_FAIL("","Issue N183630, task ID 183874", Continue); // Fixed by _setEncodedUrl
   QCOMPARE(dxOffEagle.url(), QString("http://something/newpage.html?%5B%7B%22foo:%20bar%22%7D%5D") );
 
-#if QT_VERSION >= 0x040500 // QtSw issue 243557
+  // QtSw issue 243557
   QByteArray tsdgeos("http://google.com/c?c=Translation+%C2%BB+trunk|");
   QUrl tsdgeosQUrl;
   tsdgeosQUrl.setEncodedUrl(tsdgeos, QUrl::TolerantMode);
@@ -1105,7 +1123,6 @@ void KUrlTest::testBaseURL() // those are tests for the KUrl(base,relative) cons
   pipesUrl.setEncodedUrl(pipesAgain, QUrl::TolerantMode);
   QVERIFY(pipesUrl.isValid());
   QCOMPARE(QString(pipesUrl.toEncoded()), QString("http://translate.google.com/translate_t#en%7Cuk%7Cdemo"));
-#endif
 
   // Shows up in nspluginviewer/flash
   QString flashRel = "javascript:window.location+\"__flashplugin_unique__\"";
@@ -1131,9 +1148,10 @@ void KUrlTest::testSetEncodedFragment_data()
 
 void KUrlTest::testSetEncodedFragment()
 {
-    // QtSw task number: TODO (mail sent)
     // Bug fixed in 4.5.1 by Thiago
+#if QT_VERSION < 0x040501
     QSKIP("Bug in Qt-4.4/4.5-rc1: setEncodedFragment doesn't work if the initial url has no fragment", SkipAll);
+#endif
 
     QFETCH(QByteArray, base);
     QFETCH(QByteArray, fragment);
@@ -1228,9 +1246,12 @@ void KUrlTest::testSubURL()
   QCOMPARE(url1Splitted[2].url(), QString("tar:/README"));
   const KUrl url1Rejoined = KUrl::join(url1Splitted);
   // Bug fixed in 4.5.1 by Thiago
+#if QT_VERSION < 0x040501
   QSKIP("Bug in Qt-4.4/4.5-rc1: setEncodedFragment doesn't work if the initial url has no fragment", SkipAll);
+#endif
   QCOMPARE(url1Rejoined.url(), url1.url());
   QCOMPARE(url1.upUrl().url(), QString("file:///home/dfaure/my%20tar%20file.tgz#gzip:/#tar:/"));
+
 }
 
 void KUrlTest::testSetUser()
@@ -1310,6 +1331,7 @@ void KUrlTest::testStreaming()
 {
   // Streaming operators
   KUrl origURL( "http://www.website.com/directory/?#ref" );
+  KUrl urlWithPassAndNoUser("ftp://:password@ftp.kde.org/path");
   KUrl accentuated(QString::fromUtf8("trash:/été"));
   KUrl empty( "" );
   KUrl invalid( "ptal://mlc:usb" );
@@ -1319,6 +1341,7 @@ void KUrlTest::testStreaming()
   {
       QDataStream stream( &buffer, QIODevice::WriteOnly );
       stream << origURL
+             << urlWithPassAndNoUser
              << accentuated
              << empty
              << invalid
@@ -1329,6 +1352,8 @@ void KUrlTest::testStreaming()
       KUrl restoredURL;
       stream >> restoredURL; // streaming valid url
       QCOMPARE( restoredURL.url(), origURL.url() );
+      stream >> restoredURL; // streaming url with pass an no user
+      QCOMPARE( restoredURL.url(), urlWithPassAndNoUser.url() );
       stream >> restoredURL; // streaming valid url with accents
       QCOMPARE( restoredURL.url(), accentuated.url() );
       stream >> restoredURL; // streaming empty url
@@ -1452,28 +1477,27 @@ void KUrlTest::testMoreBrokenStuff()
   weird = "http://strange<username>@strange<hostname>/";
   QVERIFY( !weird.isValid() );
 
+  {
+      QUrl url;
+      url.setUrl("http://strange<username>@hostname/", QUrl::TolerantMode);
+      QVERIFY(url.isValid());
+  }
   weird = "http://strange<username>@hostname/";
   QVERIFY( weird.isValid() ); // KDE3: was valid. Fixed by _setEncodedUrl.
   QCOMPARE( weird.host(), QString("hostname") );
 
   weird = "http://strange;hostname/";
-#if QT_VERSION < 0x040600
-  QVERIFY( weird.isValid() ); // KDE3: was invalid. bah*2.
-#endif
+  QVERIFY( !weird.isValid() );
 
   weird = "http://strange;username@strange;hostname/";
-#if QT_VERSION < 0x040600
-  QVERIFY( weird.isValid() ); // KDE3: was invalid. bah*3.
-#endif
+  QVERIFY( !weird.isValid() );
 
   weird = "http://strange;username@hostname/";
   QVERIFY( weird.isValid() );
   QCOMPARE( weird.host(), QString("hostname") );
 
   weird = "http://strange;username:password@strange;hostname/";
-#if QT_VERSION < 0x040600
-  QVERIFY( weird.isValid() ); // KDE3: was invalid
-#endif
+  QVERIFY( !weird.isValid() );
 
   weird = "http://strange;username:password@hostname/";
   QVERIFY( weird.isValid() );
@@ -1505,15 +1529,10 @@ void KUrlTest::testMoreBrokenStuff()
   KUrl broken;
   broken = "ptal://mlc:usb:PC_970";
   QVERIFY( !broken.isValid() );
-  QEXPECT_FAIL( "", "QUrl doesn't provide the initial string if it's an invalid url...", Continue );
-  QCOMPARE( broken.url(), QString("ptal://mlc:usb:PC_970") ); // FAILS - but we need it...
-
+  QVERIFY (!broken.errorString().isEmpty());
+  //QCOMPARE( broken.url(), QString("ptal://mlc:usb:PC_970") ); // QUrl doesn't provide the initial string if it's an invalid url
   QUrl brokenUrl( "ptal://mlc:usb:PC_970" );
   QVERIFY( !brokenUrl.isValid() );
-  QEXPECT_FAIL( "", "QUrl doesn't provide the initial string if it's an invalid url...", Continue );
-  QCOMPARE( brokenUrl.toString(), QString("ptal://mlc:usb:PC_970") );
-  QEXPECT_FAIL( "", "QUrl doesn't provide the initial string if it's an invalid url...", Continue );
-  QCOMPARE( brokenUrl.toEncoded(), QByteArray("ptal://mlc:usb:PC_970") );
 
   QUrl dxOffEagle( "http://something/newpage.html?[{\"foo: bar\"}]", QUrl::TolerantMode);
   QVERIFY(dxOffEagle.isValid());
@@ -1529,9 +1548,7 @@ void KUrlTest::testMoreBrokenStuff()
   QEXPECT_FAIL("","Issue N183630, task ID 183874; works with setUrl so we do that in _setEncodedUrl now", Continue);
 #endif
   QVERIFY(dxOffEagle3.isValid());
-#if QT_VERSION >= 0x040500
   QCOMPARE(dxOffEagle.toEncoded(), dxOffEagle3.toEncoded());
-#endif
 
   QUrl javascript;
   javascript.setUrl("javascript:window.location+\"__flashplugin_unique__\"", QUrl::TolerantMode);
@@ -1637,6 +1654,12 @@ void KUrlTest::testOtherProtocols()
   KUrl ptalSimpler("ptal://mlc:usb@pc123");
   QCOMPARE(ptalSimpler.url(), QString("ptal://mlc:usb@pc123"));
 #else
+  QUrl ptal_qurl;
+  ptal_qurl.setUrl("ptal://mlc:usb@PC_970", QUrl::TolerantMode);
+  QVERIFY(ptal_qurl.isValid());
+  QCOMPARE(QString::fromLatin1(ptal_qurl.toEncoded()), QString::fromLatin1("ptal://mlc:usb@pc_970"));
+  QCOMPARE( ptal_qurl.host(), QString("pc_970") );
+
   QVERIFY( ptal.isValid() );
   QCOMPARE( ptal.host(), QString("pc_970") );
   QCOMPARE( ptal.user(), QString("mlc") );
@@ -1899,3 +1922,14 @@ void KUrlTest::testUriMode()
   url1 = "mailto:User@Host.COM?subject=Hello";
   QCOMPARE( url1.path(), QString("User@Host.COM") ); // KDE3: "User@host.com". Does it matter?
 }
+
+void KUrlTest::testToLocalFile()
+{
+  const QString localFile( "/tmp/print.pdf" );
+
+  const KUrl urlWithHost( "file://localhost/tmp/print.pdf" );
+  const KUrl urlWithoutHost( "file:///tmp/print.pdf" );
+
+  QCOMPARE( urlWithHost.toLocalFile(), localFile );
+  QCOMPARE( urlWithoutHost.toLocalFile(), localFile );
+}
