@@ -192,7 +192,9 @@ QString KStringHandler::obscure( const QString &str )
   QString result;
   const QChar *unicode = str.unicode();
   for ( int i = 0; i < str.length(); ++i )
-    result += ( unicode[ i ].unicode() < 0x21 ) ? unicode[ i ] :
+    // yes, no typo. can't encode ' ' or '!' because
+    // they're the unicode BOM. stupid scrambling. stupid.
+    result += ( unicode[ i ].unicode() <= 0x21 ) ? unicode[ i ] :
         QChar( 0x1001F - unicode[ i ].unicode() );
 
   return result;
@@ -236,7 +238,7 @@ bool KStringHandler::isUtf8( const char *buf )
   };
 
   /* *ulen = 0; */
-  for (i = 0; (c = buf[i]); i++) {
+  for (i = 0; (c = buf[i]); ++i) {
     if ((c & 0x80) == 0) {        /* 0xxxxxxx is plain ASCII */
       /*
        * Even if the whole file is valid UTF-8 sequences,
@@ -264,7 +266,7 @@ bool KStringHandler::isUtf8( const char *buf )
     } else
       return false;
 
-      for (n = 0; n < following; n++) {
+      for (n = 0; n < following; ++n) {
         i++;
         if (!(c = buf[i]))
           goto done;
@@ -351,26 +353,35 @@ int KStringHandler::naturalCompare(const QString &_a, const QString &_b, Qt::Cas
         }
 
         // find sequence of characters ending at the first non-character
-        while (!currA->isNull() && !currA->isDigit()) {
+        while (!currA->isNull() && !currA->isDigit() && !currA->isPunct() && !currA->isSpace()) {
             ++currA;
         }
 
-        while (!currB->isNull() && !currB->isDigit()) {
+        while (!currB->isNull() && !currB->isDigit() && !currB->isPunct() && !currB->isSpace()) {
             ++currB;
         }
 
         // compare these sequences
-        QString subA(begSeqA, currA - begSeqA);
-        QString subB(begSeqB, currB - begSeqB);
-        const int cmp = QString::localeAwareCompare(subA, subB);
+        const QStringRef& subA(a.midRef(begSeqA - a.unicode(), currA - begSeqA));
+        const QStringRef& subB(b.midRef(begSeqB - b.unicode(), currB - begSeqB));
+        const int cmp = QStringRef::localeAwareCompare(subA, subB);
         if (cmp != 0) {
-            return cmp < 0 ? -1 : 1;
+            return cmp < 0 ? -1 : +1;
         }
 
         if (currA->isNull() || currB->isNull()) {
             break;
         }
 
+        // find sequence of characters ending at the first non-character
+        while (currA->isPunct() || currA->isSpace() || currB->isPunct() || currB->isSpace()) {
+            if (*currA != *currB) {
+                return (*currA < *currB) ? -1 : +1;
+            }
+            ++currA;
+            ++currB;
+        }
+
         // now some digits follow...
         if ((*currA == '0') || (*currB == '0')) {
             // one digit-sequence starts with 0 -> assume we are in a fraction part
@@ -379,9 +390,9 @@ int KStringHandler::naturalCompare(const QString &_a, const QString &_b, Qt::Cas
                 if (!currA->isDigit() && !currB->isDigit()) {
                     break;
                 } else if (!currA->isDigit()) {
-                    return -1;
+                    return +1;
                 } else if (!currB->isDigit()) {
-                    return + 1;
+                    return -1;
                 } else if (*currA < *currB) {
                     return -1;
                 } else if (*currA > *currB) {
@@ -398,6 +409,7 @@ int KStringHandler::naturalCompare(const QString &_a, const QString &_b, Qt::Cas
             // value wins, but we can't know that it will until we've scanned
             // both numbers to know that they have the same magnitude.
 
+            bool isFirstRun = true;
             int weight = 0;
             while (1) {
                 if (!currA->isDigit() && !currB->isDigit()) {
@@ -406,9 +418,17 @@ int KStringHandler::naturalCompare(const QString &_a, const QString &_b, Qt::Cas
                     }
                     break;
                 } else if (!currA->isDigit()) {
-                    return -1;
+                    if (isFirstRun) {
+                        return *currA < *currB ? -1 : +1;
+                    } else {
+                        return -1;
+                    }
                 } else if (!currB->isDigit()) {
-                    return + 1;
+                    if (isFirstRun) {
+                        return *currA < *currB ? -1 : +1;
+                    } else {
+                        return +1;
+                    }
                 } else if ((*currA < *currB) && (weight == 0)) {
                     weight = -1;
                 } else if ((*currA > *currB) && (weight == 0)) {
@@ -416,6 +436,7 @@ int KStringHandler::naturalCompare(const QString &_a, const QString &_b, Qt::Cas
                 }
                 ++currA;
                 ++currB;
+                isFirstRun = false;
             }
         }
 
@@ -429,3 +450,32 @@ int KStringHandler::naturalCompare(const QString &_a, const QString &_b, Qt::Cas
 
     return currA->isNull() ? -1 : + 1;
 }
+
+QString KStringHandler::preProcessWrap(const QString &text)
+{
+    const QChar zwsp(0x200b);
+
+    QString result;
+    result.reserve(text.length());
+
+    for (int i = 0; i < text.length(); i++) {
+        const QChar c = text[i];
+        bool openingParens = (c == QLatin1Char('(') || c == QLatin1Char('{') || c == QLatin1Char('['));
+        bool closingParens = (c == QLatin1Char(')') || c == QLatin1Char('}') || c == QLatin1Char(']'));
+        bool breakAfter   = (closingParens || c.isPunct() || c.isSymbol());
+        bool nextIsSpace  = (i == (text.length() - 1) || text[i + 1].isSpace());
+        bool prevIsSpace  = (i == 0 || text[i - 1].isSpace() || result[result.length() - 1] == zwsp);
+
+        // Provide a breaking opportunity before opening parenthesis
+        if (openingParens && !prevIsSpace)
+            result += zwsp;
+
+        result += c;
+
+        if (breakAfter && !openingParens && !nextIsSpace)
+            result += zwsp;
+    }
+
+    return result;
+}
+
