@@ -49,7 +49,6 @@ KInputDialogHelper::KInputDialogHelper( const QString &caption, const QString &l
     QWidget *frame = new QWidget(this);
     QVBoxLayout *layout = new QVBoxLayout(frame);
     layout->setMargin(0);
-    layout->setSpacing(spacingHint());
 
     m_label = new QLabel(label, frame);
     m_label->setWordWrap(true);
@@ -93,7 +92,6 @@ KInputDialogHelper::KInputDialogHelper( const QString &caption, const QString &l
     QWidget *frame = new QWidget(this);
     QVBoxLayout *layout = new QVBoxLayout(frame);
     layout->setMargin(0);
-    layout->setSpacing(spacingHint());
 
     m_label = new QLabel(label, frame);
     m_label->setWordWrap(true);
@@ -126,7 +124,6 @@ KInputDialogHelper::KInputDialogHelper( const QString &caption, const QString &l
 
     QWidget *frame = new QWidget(this);
     QVBoxLayout *layout = new QVBoxLayout(frame);
-    layout->setSpacing(spacingHint());
 
     m_label = new QLabel(label, frame);
     m_label->setWordWrap(true);
@@ -157,7 +154,6 @@ KInputDialogHelper::KInputDialogHelper( const QString &caption, const QString &l
 
     QWidget *frame = new QWidget(this);
     QVBoxLayout *layout = new QVBoxLayout(frame);
-    layout->setSpacing(spacingHint());
 
     m_label = new QLabel(label, frame);
     m_label->setWordWrap(true);
@@ -193,7 +189,6 @@ KInputDialogHelper::KInputDialogHelper( const QString &caption, const QString &l
 
     QWidget *frame = new QWidget(this);
     QVBoxLayout *layout = new QVBoxLayout(frame);
-    layout->setSpacing(spacingHint());
 
     m_label = new QLabel(label, frame);
     m_label->setWordWrap(true);
@@ -242,7 +237,6 @@ KInputDialogHelper::KInputDialogHelper( const QString &caption, const QString &l
 
     QWidget *frame = new QWidget(this);
     QVBoxLayout *layout = new QVBoxLayout(frame);
-    layout->setSpacing(spacingHint());
 
     m_label = new QLabel(label, frame);
     m_label->setWordWrap(true); 
@@ -256,7 +250,7 @@ KInputDialogHelper::KInputDialogHelper( const QString &caption, const QString &l
         m_listBox->setSelectionMode(QAbstractItemView::ExtendedSelection);
 
         for (QStringList::ConstIterator it = select.begin(); it != select.end(); ++it) {
-            QList<QListWidgetItem*> matches = m_listBox->findItems(*it, Qt::MatchCaseSensitive|Qt::MatchExactly);
+            const QList<QListWidgetItem*> matches = m_listBox->findItems(*it, Qt::MatchCaseSensitive|Qt::MatchExactly);
             if (!matches.isEmpty())
                 m_listBox->setCurrentItem(matches.first());
         }
@@ -266,7 +260,7 @@ KInputDialogHelper::KInputDialogHelper( const QString &caption, const QString &l
         if (!select.isEmpty()) {
             QString text = select.first();
 
-            QList<QListWidgetItem*> matches = m_listBox->findItems(text, Qt::MatchCaseSensitive|Qt::MatchExactly);
+            const QList<QListWidgetItem*> matches = m_listBox->findItems(text, Qt::MatchCaseSensitive|Qt::MatchExactly);
             if (!matches.isEmpty())
                 m_listBox->setCurrentItem(matches.first());
         }
