@@ -84,12 +84,10 @@ void KFindDialog::KFindDialogPrivate::init(bool forReplace, const QStringList &_
     q->setMainWidget(page);
 
     topLayout = new QVBoxLayout(page);
-    topLayout->setSpacing( KDialog::spacingHint() );
     topLayout->setMargin( 0 );
 
-    findGrp = new QGroupBox(i18n("Find"), page);
+    findGrp = new QGroupBox(i18nc("@title:group", "Find"), page);
     findLayout = new QGridLayout(findGrp);
-    findLayout->setSpacing( KDialog::spacingHint() );
 
     QLabel *findLabel = new QLabel(i18n("&Text to find:"), findGrp);
     find = new KHistoryComboBox(findGrp);
@@ -107,7 +105,6 @@ void KFindDialog::KFindDialogPrivate::init(bool forReplace, const QStringList &_
 
     replaceGrp = new QGroupBox( i18n("Replace With"), page);
     replaceLayout = new QGridLayout(replaceGrp);
-    replaceLayout->setSpacing( KDialog::spacingHint() );
 
     QLabel *replaceLabel = new QLabel(i18n("Replace&ment text:"), replaceGrp);
     replace = new KHistoryComboBox(replaceGrp);
@@ -125,7 +122,6 @@ void KFindDialog::KFindDialogPrivate::init(bool forReplace, const QStringList &_
 
     QGroupBox *optionGrp = new QGroupBox(i18n("Options"), page);
     optionsLayout = new QGridLayout(optionGrp);
-    optionsLayout->setSpacing( KDialog::spacingHint() );
 
     caseSensitive = new QCheckBox(i18n("C&ase sensitive"), optionGrp);
     wholeWordsOnly = new QCheckBox(i18n("&Whole words only"), optionGrp);
@@ -225,7 +221,7 @@ void KFindDialog::KFindDialogPrivate::init(bool forReplace, const QStringList &_
             "Enter a replacement string, or select a previous one from the list.") );
     backRef->setWhatsThis(i18n(
             "<qt>If enabled, any occurrence of <code><b>\\N</b></code>, where "
-            "<code><b>N</b></code> is a integer number, will be replaced with "
+            "<code><b>N</b></code> is an integer number, will be replaced with "
             "the corresponding capture (\"parenthesized substring\") from the "
             "pattern.<p>To include (a literal <code><b>\\N</b></code> in your "
             "replacement, put an extra backslash in front of it, like "
@@ -247,6 +243,7 @@ void KFindDialog::KFindDialogPrivate::init(bool forReplace, const QStringList &_
             "Ask before replacing each match found.") );
 
     q->connect(q, SIGNAL(okClicked()), q, SLOT(_k_slotOk()));
+    _k_textSearchChanged(find->lineEdit()->text());
 }
 
 void KFindDialog::KFindDialogPrivate::_k_textSearchChanged( const QString & text)
