@@ -1,6 +1,6 @@
 /*
    This file is part of the KDE libraries
-   Copyright (C) 2004-2008 Jaroslaw Staniek <js@iidea.pl>
+   Copyright (C) 2004-2008 Jarosław Staniek <staniek@kde.org>
 
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
@@ -20,12 +20,15 @@
 #include <QtGui/QApplication>
 #include <kstandarddirs.h>
 #include <klocale.h>
+#include <kwindowsystem.h>
 
 #include <QTranslator>
 #include <QLocale>
 #include <QLibraryInfo>
 #include <QLibrary>
 
+#include <stdio.h>
+
 /**
  * MS Windows-related actions for KApplication startup.
  *
@@ -33,6 +36,8 @@
  *    and other Qt-only GUIs. The "qt_<language>.qm" file should be stored
  *    in the same place as .po files for a given language.
  *
+ * - Increase the default open file limit with the stdio API from 512 to 2048
+ *   (2048 is the hard limit on Windows)
  * @internal
 */
 void KApplication_init_windows()
@@ -43,11 +48,16 @@ void KApplication_init_windows()
 	QString qt_transl_file = QString("qt_") + QLocale::system().name();
 	qt_transl_file.truncate(5);
 	QTranslator *qt_transl = new QTranslator();
-	if (qt_transl->load( qt_transl_file, 
+	if (qt_transl->load( qt_transl_file,
 		QLibraryInfo::location(QLibraryInfo::TranslationsPath)))
 		qApp->installTranslator( qt_transl );
 	else
 		delete qt_transl;
+
+    // For apps like KMail which have lots of open files, the default is too low
+	// so increase it to the maximum.
+    _setmaxstdio(2048);
+
 }
 
 // <copy of kdepim/libkdepim/utils.cpp, TODO: move to a shared helper library>
@@ -83,7 +93,7 @@ static PPERF_COUNTER_DEFINITION FirstCounter( PPERF_OBJECT_TYPE PerfObj )
 
 static PPERF_INSTANCE_DEFINITION NextInstance( PPERF_INSTANCE_DEFINITION PerfInst )
 {
-  PPERF_COUNTER_BLOCK PerfCntrBlk 
+  PPERF_COUNTER_BLOCK PerfCntrBlk
     = (PPERF_COUNTER_BLOCK)((PBYTE)PerfInst + PerfInst->ByteLength);
   return (PPERF_INSTANCE_DEFINITION)((PBYTE)PerfCntrBlk + PerfCntrBlk->ByteLength);
 }
@@ -149,7 +159,8 @@ void KApplication_getProcessesIdForName( const QString& processName, QList<int>&
     // retrieve the instances
     qDebug() << "INSTANCES: " << perfObject->NumInstances;
     for( int instance = 0; instance < perfObject->NumInstances; instance++ ) {
-      const QString foundProcessName( 
+      curCounter = perfCounter;
+      const QString foundProcessName(
         fromWChar( (wchar_t *)( (PBYTE)perfInstance + perfInstance->NameOffset ) ) );
       qDebug() << "foundProcessName: " << foundProcessName;
       if ( foundProcessName == processName ) {
@@ -219,10 +230,10 @@ BOOL CALLBACK EnumWindowsProc( HWND hwnd, LPARAM lParam )
     GetWindowThreadProcessId(hwnd, &pidwin);
     if ( pidwin == ((EnumWindowsStruct*)lParam)->pid ) {
       ((EnumWindowsStruct*)lParam)->windowId = hwnd;
-      return FALSE;
+      return false;
     }
   }
-  return TRUE;
+  return true;
 }
 
 void KApplication_activateWindowForProcess( const QString& executableName )
@@ -245,7 +256,7 @@ void KApplication_activateWindowForProcess( const QString& executableName )
   EnumWindows( EnumWindowsProc, (LPARAM)&winStruct );
   if ( winStruct.windowId == NULL )
     return;
-  SetForegroundWindow( winStruct.windowId );
+  KWindowSystem::forceActiveWindow( winStruct.windowId, 0 );
 }
 
 // </copy>
