@@ -1,5 +1,6 @@
 /* This file is part of the KDE libraries
    Copyright (C) 2000, 2006 David Faure <faure@kde.org>
+   Copyright 2008 Friedrich W. H. Kossebau <kossebau@kde.org>
 
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
@@ -29,10 +30,9 @@
 #include <kprotocolinfo.h>
 #include <kcomponentdata.h>
 #include <kcolorscheme.h>
+#include <kapplication.h>
 
-#ifdef Q_WS_X11
 #include <kstyle.h>
-#endif
 
 #include <QtGui/QColor>
 #include <QtGui/QCursor>
@@ -44,10 +44,10 @@
 #include <QtGui/QKeySequence>
 #include <QtGui/QPixmap>
 #include <QtGui/QPixmapCache>
-//#include <q3stylesheet.h> // no equivalent in Qt4
 #include <QApplication>
 #include <QtDBus/QtDBus>
 #include <QtGui/QStyleFactory>
+#include <QDesktopServices>
 
 // next two needed so we can set their palettes
 #include <QtGui/QToolTip>
@@ -74,40 +74,92 @@ static QRgb qt_colorref2qrgb(COLORREF col)
 #include <stdlib.h>
 #include <kconfiggroup.h>
 
-static QString* s_desktopPath = 0;
-static QString* s_autostartPath = 0;
-static QString* s_documentPath = 0;
-static QFont *_generalFont = 0;
-static QFont *_fixedFont = 0;
-static QFont *_toolBarFont = 0;
-static QFont *_menuFont = 0;
-static QFont *_windowTitleFont = 0;
-static QFont *_taskbarFont = 0;
-static QFont *_largeFont = 0;
-static QFont *_smallestReadableFont = 0;
-//static QColor *_buttonBackground = 0;
 
-static KGlobalSettings::KMouseSettings *s_mouseSettings = 0;
+//static QColor *_buttonBackground = 0;
+static KGlobalSettings::GraphicEffects _graphicEffects = KGlobalSettings::NoEffects;
+
+// KDE5: merge this with KGlobalSettings::Private
+// also think to make all methods static and not expose an object,
+// making KGlobalSettings rather a namespace
+class KGlobalSettingsData
+{
+  public:
+    // if adding a new type here also add an entry to DefaultFontData
+    enum FontTypes
+    {
+        GeneralFont = 0,
+        FixedFont,
+        ToolbarFont,
+        MenuFont,
+        WindowTitleFont,
+        TaskbarFont ,
+        SmallestReadableFont,
+        FontTypesCount
+    };
+
+  public:
+    KGlobalSettingsData();
+    ~KGlobalSettingsData();
+
+  public:
+    static KGlobalSettingsData* self();
+
+  public: // access, is not const due to caching
+    QFont font( FontTypes fontType );
+    QFont largeFont( const QString& text );
+    KGlobalSettings::KMouseSettings& mouseSettings();
+
+  public:
+    void dropFontSettingsCache();
+    void dropMouseSettingsCache();
+
+  protected:
+    QFont* mFonts[FontTypesCount];
+    QFont* mLargeFont;
+    KGlobalSettings::KMouseSettings* mMouseSettings;
+};
+
+KGlobalSettingsData::KGlobalSettingsData()
+  : mLargeFont( 0 ),
+    mMouseSettings( 0 )
+{
+    for( int i=0; i<FontTypesCount; ++i )
+        mFonts[i] = 0;
+}
+
+KGlobalSettingsData::~KGlobalSettingsData()
+{
+    for( int i=0; i<FontTypesCount; ++i )
+        delete mFonts[i];
+    delete mLargeFont;
+
+    delete mMouseSettings;
+}
+
+K_GLOBAL_STATIC( KGlobalSettingsData, globalSettingsDataSingleton )
+
+inline KGlobalSettingsData* KGlobalSettingsData::self()
+{
+    return globalSettingsDataSingleton;
+}
+
 
 class KGlobalSettings::Private
 {
     public:
         Private(KGlobalSettings *q)
-            : q(q)
+            : q(q), activated(false)
         {
         }
 
         void _k_slotNotifyChange(int, int);
 
-        void propagateSettings(SettingsCategory category);
+        void propagateQtSettings();
         void kdisplaySetPalette();
         void kdisplaySetStyle();
         void kdisplaySetFont();
         void applyGUIStyle();
 
-        // TODO (4.1?) - make not-private, if used in KStyle
-        static QPalette createTooltipPalette(const KSharedConfigPtr &config = KSharedConfigPtr());
-
         /**
          * @internal
          *
@@ -122,23 +174,12 @@ class KGlobalSettings::Private
         void applyCursorTheme();
 
         /**
-         * reads in all paths from kdeglobals
+         * drop cached values for settings that aren't in any of the previous groups
          */
-        static void initPaths();
-        /**
-         * drop cached values for fonts (called by KApplication)
-         */
-        static void rereadFontSettings();
-        /**
-         * drop cached values for paths (called by KApplication)
-         */
-        static void rereadPathSettings();
-        /**
-         * drop cached values for mouse settings (called by KApplication)
-         */
-        static void rereadMouseSettings();
+        static void rereadOtherSettings();
 
         KGlobalSettings *q;
+        bool activated;
 };
 
 KGlobalSettings* KGlobalSettings::self()
@@ -150,10 +191,6 @@ KGlobalSettings* KGlobalSettings::self()
 KGlobalSettings::KGlobalSettings()
     : QObject(0), d(new Private(this))
 {
-    d->kdisplaySetStyle();
-    d->kdisplaySetFont();
-    d->propagateSettings(SETTINGS_QT);
-
     QDBusConnection::sessionBus().connect( QString(), "/KGlobalSettings", "org.kde.KGlobalSettings",
                                            "notifyChange", this, SLOT(_k_slotNotifyChange(int,int)) );
 }
@@ -163,6 +200,17 @@ KGlobalSettings::~KGlobalSettings()
     delete d;
 }
 
+void KGlobalSettings::activate()
+{
+    if (!d->activated) {
+        d->activated = true;
+
+        d->kdisplaySetStyle(); // implies palette setup
+        d->kdisplaySetFont();
+        d->propagateQtSettings();
+    }
+}
+
 int KGlobalSettings::dndEventDelay()
 {
     KConfigGroup g( KGlobal::config(), "General" );
@@ -175,6 +223,12 @@ bool KGlobalSettings::singleClick()
     return g.readEntry("SingleClick", KDE_DEFAULT_SINGLECLICK );
 }
 
+bool KGlobalSettings::smoothScroll()
+{
+    KConfigGroup g( KGlobal::config(), "KDE" );
+    return g.readEntry("SmoothScroll", KDE_DEFAULT_SMOOTHSCROLL );
+}
+
 KGlobalSettings::TearOffHandle KGlobalSettings::insertTearOffHandle()
 {
     int tearoff;
@@ -229,7 +283,7 @@ int KGlobalSettings::contextMenuKey ()
         return QKeySequence()[0];
     }
 
-    QStringList shortCuts = s.split(';');
+    const QStringList shortCuts = s.split(';');
 
     if (shortCuts.count() < 1) {
         return QKeySequence()[0];
@@ -237,7 +291,7 @@ int KGlobalSettings::contextMenuKey ()
 
     s = shortCuts.at(0);
 
-    if ( s.startsWith( "default(" ) ) {
+    if ( s.startsWith( QLatin1String("default(") ) ) {
         s = s.mid( 8, s.length() - 9 );
     }
 
@@ -291,14 +345,14 @@ QColor KGlobalSettings::activeTextColor()
 int KGlobalSettings::contrast()
 {
     KConfigGroup g( KGlobal::config(), "KDE" );
-    return g.readEntry( "contrast", 7 );
+    return g.readEntry( "contrast", 5 );
 }
 
 qreal KGlobalSettings::contrastF(const KSharedConfigPtr &config)
 {
     if (config) {
         KConfigGroup g( config, "KDE" );
-        return 0.1 * g.readEntry( "contrast", 7 );
+        return 0.1 * g.readEntry( "contrast", 5 );
     }
     return 0.1 * (qreal)contrast();
 }
@@ -315,149 +369,123 @@ bool KGlobalSettings::allowDefaultBackgroundImages()
     return g.readEntry( "allowDefaultBackgroundImages", KDE_DEFAULT_ALLOW_DEFAULT_BACKGROUND_IMAGES );
 }
 
+struct KFontData
+{
+    const char* ConfigGroupKey;
+    const char* ConfigKey;
+    const char* FontName;
+    int Size;
+    int Weight;
+    QFont::StyleHint StyleHint;
+};
+
+// NOTE: keep in sync with kdebase/workspace/kcontrol/fonts/fonts.cpp
+static const char GeneralId[] =      "General";
+static const char DefaultFont[] =    "Sans Serif";
+#ifdef Q_WS_MAC
+static const char DefaultMacFont[] = "Lucida Grande";
+#endif
+
+static const KFontData DefaultFontData[KGlobalSettingsData::FontTypesCount] =
+{
+#ifdef Q_WS_MAC
+    { GeneralId, "font",        DefaultMacFont, 13, -1, QFont::SansSerif },
+    { GeneralId, "fixed",       "Monaco",       10, -1, QFont::TypeWriter },
+    { GeneralId, "toolBarFont", DefaultMacFont, 11, -1, QFont::SansSerif },
+    { GeneralId, "menuFont",    DefaultMacFont, 13, -1, QFont::SansSerif },
+#else
+    { GeneralId, "font",        DefaultFont, 9, -1, QFont::SansSerif },
+    { GeneralId, "fixed",       "Monospace", 9, -1, QFont::TypeWriter },
+    { GeneralId, "toolBarFont", DefaultFont,  8, -1, QFont::SansSerif },
+    { GeneralId, "menuFont",    DefaultFont, 9, -1, QFont::SansSerif },
+#endif
+    { "WM",      "activeFont",           DefaultFont,  8, -1, QFont::SansSerif },
+    { GeneralId, "taskbarFont",          DefaultFont, 9, -1, QFont::SansSerif },
+    { GeneralId, "smallestReadableFont", DefaultFont,  8, -1, QFont::SansSerif }
+};
+
+QFont KGlobalSettingsData::font( FontTypes fontType )
+{
+    QFont* cachedFont = mFonts[fontType];
+
+    if (!cachedFont)
+    {
+        const KFontData& fontData = DefaultFontData[fontType];
+        cachedFont = new QFont( fontData.FontName, fontData.Size, fontData.Weight );
+        cachedFont->setStyleHint( fontData.StyleHint );
+
+        const KConfigGroup configGroup( KGlobal::config(), fontData.ConfigGroupKey );
+        *cachedFont = configGroup.readEntry( fontData.ConfigKey, *cachedFont );
+
+        mFonts[fontType] = cachedFont;
+    }
+
+    return *cachedFont;
+}
+
 QFont KGlobalSettings::generalFont()
 {
-    if (_generalFont)
-        return *_generalFont;
-
-#ifdef Q_WS_MAC
-    _generalFont = new QFont("Lucida Grande", 13);
-    _generalFont->setPointSize(13);
-#else
-    // Sync default with kdebase/kcontrol/fonts/fonts.cpp
-    _generalFont = new QFont("Sans Serif", 10);
-    _generalFont->setPointSize(10);
-#endif
-    _generalFont->setStyleHint(QFont::SansSerif);
-
-    KConfigGroup g( KGlobal::config(), "General" );
-    *_generalFont = g.readEntry("font", *_generalFont);
-
-    return *_generalFont;
+    return KGlobalSettingsData::self()->font( KGlobalSettingsData::GeneralFont );
 }
-
 QFont KGlobalSettings::fixedFont()
 {
-    if (_fixedFont)
-        return *_fixedFont;
-
-#ifdef Q_WS_MAC
-    _fixedFont = new QFont("Monaco", 10);
-    _fixedFont->setPointSize(10);
-#else
-    // Sync default with kdebase/kcontrol/fonts/fonts.cpp
-    _fixedFont = new QFont("Monospace", 10);
-    _fixedFont->setPointSize(10);
-#endif
-    _fixedFont->setStyleHint(QFont::TypeWriter);
-
-    KConfigGroup g( KGlobal::config(), "General" );
-    *_fixedFont = g.readEntry("fixed", *_fixedFont);
-
-    return *_fixedFont;
+    return KGlobalSettingsData::self()->font( KGlobalSettingsData::FixedFont );
 }
-
 QFont KGlobalSettings::toolBarFont()
 {
-    if(_toolBarFont)
-        return *_toolBarFont;
-
-#ifdef Q_WS_MAC
-    _toolBarFont = new QFont("Lucida Grande", 11);
-    _toolBarFont->setPointSize(11);
-#else
-    // Sync default with kdebase/kcontrol/fonts/fonts.cpp
-    _toolBarFont = new QFont("Sans Serif", 8);
-    _toolBarFont->setPointSize(8);
-#endif
-    _toolBarFont->setStyleHint(QFont::SansSerif);
-
-    KConfigGroup g( KGlobal::config(), "General" );
-    *_toolBarFont = g.readEntry("toolBarFont", *_toolBarFont);
-
-    return *_toolBarFont;
+    return KGlobalSettingsData::self()->font( KGlobalSettingsData::ToolbarFont );
 }
-
 QFont KGlobalSettings::menuFont()
 {
-    if(_menuFont)
-        return *_menuFont;
-
-#ifdef Q_WS_MAC
-    _menuFont = new QFont("Lucida Grande", 13);
-    _menuFont->setPointSize(13);
-#else
-    // Sync default with kdebase/kcontrol/fonts/fonts.cpp
-    _menuFont = new QFont("Sans Serif", 10);
-    _menuFont->setPointSize(10);
-#endif
-    _menuFont->setStyleHint(QFont::SansSerif);
-
-    KConfigGroup g( KGlobal::config(), "General" );
-    *_menuFont = g.readEntry("menuFont", *_menuFont);
-
-    return *_menuFont;
+    return KGlobalSettingsData::self()->font( KGlobalSettingsData::MenuFont );
 }
-
 QFont KGlobalSettings::windowTitleFont()
 {
-    if(_windowTitleFont)
-        return *_windowTitleFont;
-
-    // Sync default with kdebase/kcontrol/fonts/fonts.cpp
-    _windowTitleFont = new QFont("Sans Serif", 9, QFont::Bold);
-    _windowTitleFont->setPointSize(10);
-    _windowTitleFont->setStyleHint(QFont::SansSerif);
-
-    KConfigGroup g( KGlobal::config(), "WM" );
-    *_windowTitleFont = g.readEntry("activeFont", *_windowTitleFont); // inconsistency
-
-    return *_windowTitleFont;
+    return KGlobalSettingsData::self()->font( KGlobalSettingsData::WindowTitleFont );
 }
-
 QFont KGlobalSettings::taskbarFont()
 {
-    if(_taskbarFont)
-        return *_taskbarFont;
-
-    // Sync default with kdebase/kcontrol/fonts/fonts.cpp
-    _taskbarFont = new QFont("Sans Serif", 10);
-    _taskbarFont->setPointSize(10);
-    _taskbarFont->setStyleHint(QFont::SansSerif);
-
-    KConfigGroup g( KGlobal::config(), "General" );
-    *_taskbarFont = g.readEntry("taskbarFont", *_taskbarFont);
-
-    return *_taskbarFont;
+    return KGlobalSettingsData::self()->font( KGlobalSettingsData::TaskbarFont );
+}
+QFont KGlobalSettings::smallestReadableFont()
+{
+    return KGlobalSettingsData::self()->font( KGlobalSettingsData::SmallestReadableFont );
 }
 
 
-QFont KGlobalSettings::largeFont(const QString &text)
+QFont KGlobalSettingsData::largeFont( const QString& text )
 {
     QFontDatabase db;
     QStringList fam = db.families();
 
     // Move a bunch of preferred fonts to the front.
-    if (fam.removeAll("Arial")>0)
-       fam.prepend("Arial");
-    if (fam.removeAll("Verdana")>0)
-       fam.prepend("Verdana");
-    if (fam.removeAll("Tahoma")>0)
-       fam.prepend("Tahoma");
-    if (fam.removeAll("Lucida Sans")>0)
-       fam.prepend("Lucida Sans");
-    if (fam.removeAll("Lucidux Sans")>0)
-       fam.prepend("Lucidux Sans");
-    if (fam.removeAll("Nimbus Sans")>0)
-       fam.prepend("Nimbus Sans");
-    if (fam.removeAll("Gothic I")>0)
-       fam.prepend("Gothic I");
+    // most preferred last
+    static const char* const PreferredFontNames[] =
+    {
+        "Arial",
+        "Sans Serif",
+        "Verdana",
+        "Tahoma",
+        "Lucida Sans",
+        "Lucidux Sans",
+        "Nimbus Sans",
+        "Gothic I"
+    };
+    static const unsigned int PreferredFontNamesCount = sizeof(PreferredFontNames)/sizeof(const char*);
+    for( unsigned int i=0; i<PreferredFontNamesCount; ++i )
+    {
+        const QString fontName (PreferredFontNames[i]);
+        if (fam.removeAll(fontName)>0)
+            fam.prepend(fontName);
+    }
 
-    if (_largeFont)
-        fam.prepend(_largeFont->family());
+    if (mLargeFont) {
+        fam.prepend(mLargeFont->family());
+        delete mLargeFont;
+    }
 
-    for(QStringList::ConstIterator it = fam.begin();
-        it != fam.end(); ++it)
+    for(QStringList::ConstIterator it = fam.constBegin();
+        it != fam.constEnd(); ++it)
     {
         if (db.isSmoothlyScalable(*it) && !db.isFixedPitch(*it))
         {
@@ -481,140 +509,66 @@ QFont KGlobalSettings::largeFont(const QString &text)
                 continue;
 
             font.setPointSize(48);
-            _largeFont = new QFont(font);
-            return *_largeFont;
+            mLargeFont = new QFont(font);
+            return *mLargeFont;
         }
     }
-    _largeFont = new QFont(KGlobalSettings::generalFont());
-    _largeFont->setPointSize(48);
-    return *_largeFont;
+    mLargeFont = new QFont( font(GeneralFont) );
+    mLargeFont->setPointSize(48);
+    return *mLargeFont;
+}
+QFont KGlobalSettings::largeFont( const QString& text )
+{
+    return KGlobalSettingsData::self()->largeFont( text );
 }
 
-QFont KGlobalSettings::smallestReadableFont()
+void KGlobalSettingsData::dropFontSettingsCache()
 {
-    if(_smallestReadableFont)
-        return *_smallestReadableFont;
-
-    // Sync default with kdebase/kcontrol/fonts/fonts.cpp
-    _smallestReadableFont = new QFont("Sans Serif", 8);
-    _smallestReadableFont->setPointSize(8);
-    _smallestReadableFont->setStyleHint(QFont::SansSerif);
-
-    KConfigGroup g( KGlobal::config(), "General" );
-    *_smallestReadableFont = g.readEntry("smallestReadableFont", *_smallestReadableFont);
-
-    return *_smallestReadableFont;
-}
-
-void KGlobalSettings::Private::initPaths()
-{
-    //this code is duplicated in kde_config.cpp.in
-
-    if ( s_desktopPath != 0 )
-        return;
-
-    KGlobalSettings::self(); // listen to changes
-
-    s_desktopPath = new QString();
-    s_autostartPath = new QString();
-    s_documentPath = new QString();
-
-    KConfigGroup g( KGlobal::config(), "Paths" );
-
-    // Desktop Path
-    *s_desktopPath = QDir::homePath() + "/Desktop/";
-    *s_desktopPath = g.readPathEntry( "Desktop", *s_desktopPath);
-    *s_desktopPath = QDir::cleanPath( *s_desktopPath );
-    if ( !s_desktopPath->endsWith('/') ) {
-        s_desktopPath->append( QLatin1Char( '/' ) );
-    }
-
-    // Autostart Path
-    *s_autostartPath = KGlobal::dirs()->localkdedir() + "Autostart/";
-    *s_autostartPath = g.readPathEntry( "Autostart" , *s_autostartPath);
-    *s_autostartPath = QDir::cleanPath( *s_autostartPath );
-    if ( !s_autostartPath->endsWith('/') ) {
-        s_autostartPath->append( QLatin1Char( '/' ) );
-    }
-
-    // Document Path
-    *s_documentPath = g.readPathEntry( "Documents",
-#ifdef Q_WS_WIN
-        getWin32ShellFoldersPath("Personal")
-#else
-        QDir::homePath()
-#endif
-    );
-    *s_documentPath = QDir::cleanPath( *s_documentPath );
-    if ( !s_documentPath->endsWith('/')) {
-        s_documentPath->append( QLatin1Char( '/' ) );
-    }
-}
-
-void KGlobalSettings::Private::rereadFontSettings()
-{
-    delete _generalFont;
-    _generalFont = 0L;
-    delete _fixedFont;
-    _fixedFont = 0L;
-    delete _menuFont;
-    _menuFont = 0L;
-    delete _toolBarFont;
-    _toolBarFont = 0L;
-    delete _windowTitleFont;
-    _windowTitleFont = 0L;
-    delete _taskbarFont;
-    _taskbarFont = 0L;
-    delete _smallestReadableFont;
-    _smallestReadableFont = 0L;
-}
-
-void KGlobalSettings::Private::rereadPathSettings()
-{
-    delete s_autostartPath;
-    s_autostartPath = 0L;
-    delete s_desktopPath;
-    s_desktopPath = 0L;
-    delete s_documentPath;
-    s_documentPath = 0L;
-}
-
-KGlobalSettings::KMouseSettings & KGlobalSettings::mouseSettings()
-{
-    if ( ! s_mouseSettings )
+    for( int i=0; i<FontTypesCount; ++i )
     {
-        s_mouseSettings = new KMouseSettings;
-        KMouseSettings & s = *s_mouseSettings; // for convenience
+        delete mFonts[i];
+        mFonts[i] = 0;
+    }
+    delete mLargeFont;
+    mLargeFont = 0;
+}
+
+KGlobalSettings::KMouseSettings& KGlobalSettingsData::mouseSettings()
+{
+    if (!mMouseSettings)
+    {
+        mMouseSettings = new KGlobalSettings::KMouseSettings;
+        KGlobalSettings::KMouseSettings& s = *mMouseSettings; // for convenience
 
 #ifndef Q_WS_WIN
         KConfigGroup g( KGlobal::config(), "Mouse" );
         QString setting = g.readEntry("MouseButtonMapping");
         if (setting == "RightHanded")
-            s.handed = KMouseSettings::RightHanded;
+            s.handed = KGlobalSettings::KMouseSettings::RightHanded;
         else if (setting == "LeftHanded")
-            s.handed = KMouseSettings::LeftHanded;
+            s.handed = KGlobalSettings::KMouseSettings::LeftHanded;
         else
         {
 #ifdef Q_WS_X11
             // get settings from X server
             // This is a simplified version of the code in input/mouse.cpp
             // Keep in sync !
-            s.handed = KMouseSettings::RightHanded;
+            s.handed = KGlobalSettings::KMouseSettings::RightHanded;
             unsigned char map[20];
             int num_buttons = XGetPointerMapping(QX11Info::display(), map, 20);
             if( num_buttons == 2 )
             {
                 if ( (int)map[0] == 1 && (int)map[1] == 2 )
-                    s.handed = KMouseSettings::RightHanded;
+                    s.handed = KGlobalSettings::KMouseSettings::RightHanded;
                 else if ( (int)map[0] == 2 && (int)map[1] == 1 )
-                    s.handed = KMouseSettings::LeftHanded;
+                    s.handed = KGlobalSettings::KMouseSettings::LeftHanded;
             }
             else if( num_buttons >= 3 )
             {
                 if ( (int)map[0] == 1 && (int)map[2] == 3 )
-                    s.handed = KMouseSettings::RightHanded;
+                    s.handed = KGlobalSettings::KMouseSettings::RightHanded;
                 else if ( (int)map[0] == 3 && (int)map[2] == 1 )
-                    s.handed = KMouseSettings::LeftHanded;
+                    s.handed = KGlobalSettings::KMouseSettings::LeftHanded;
             }
 #else
         // FIXME: Implement on other platforms
@@ -624,35 +578,90 @@ KGlobalSettings::KMouseSettings & KGlobalSettings::mouseSettings()
     }
 #ifdef Q_WS_WIN
     //not cached
-    s_mouseSettings->handed = (GetSystemMetrics(SM_SWAPBUTTON) ? KMouseSettings::LeftHanded : KMouseSettings::RightHanded);
+    mMouseSettings->handed = (GetSystemMetrics(SM_SWAPBUTTON) ?
+        KGlobalSettings::KMouseSettings::LeftHanded :
+        KGlobalSettings::KMouseSettings::RightHanded);
 #endif
-    return *s_mouseSettings;
+    return *mMouseSettings;
+}
+// KDE5: make this a const return?
+KGlobalSettings::KMouseSettings & KGlobalSettings::mouseSettings()
+{
+    return KGlobalSettingsData::self()->mouseSettings();
 }
 
-void KGlobalSettings::Private::rereadMouseSettings()
+void KGlobalSettingsData::dropMouseSettingsCache()
 {
 #ifndef Q_WS_WIN
-    delete s_mouseSettings;
-    s_mouseSettings = 0L;
+    delete mMouseSettings;
+    mMouseSettings = 0;
 #endif
 }
 
 QString KGlobalSettings::desktopPath()
 {
-    Private::initPaths();
-    return *s_desktopPath;
+    QString path = QDesktopServices::storageLocation( QDesktopServices::DesktopLocation );
+    return path.isEmpty() ? QDir::homePath() : path;
 }
 
+// Autostart is not a XDG path, so we have our own code for it.
 QString KGlobalSettings::autostartPath()
 {
-    Private::initPaths();
-    return *s_autostartPath;
+    QString s_autostartPath;
+    KConfigGroup g( KGlobal::config(), "Paths" );
+    s_autostartPath = KGlobal::dirs()->localkdedir() + "Autostart/";
+    s_autostartPath = g.readPathEntry( "Autostart" , s_autostartPath );
+    s_autostartPath = QDir::cleanPath( s_autostartPath );
+    if ( !s_autostartPath.endsWith( '/' ) ) {
+        s_autostartPath.append( QLatin1Char( '/' ) );
+    }
+    return s_autostartPath;
 }
 
 QString KGlobalSettings::documentPath()
 {
-    Private::initPaths();
-    return *s_documentPath;
+    QString path = QDesktopServices::storageLocation( QDesktopServices::DocumentsLocation );
+    return path.isEmpty() ? QDir::homePath() : path;
+}
+
+QString KGlobalSettings::downloadPath()
+{
+    // Qt 4.4.1 does not have DOWNLOAD, so we based on old code for now
+    QString downloadPath = QDir::homePath();
+#ifndef Q_WS_WIN
+    const QString xdgUserDirs = KGlobal::dirs()->localxdgconfdir() + QLatin1String( "user-dirs.dirs" );
+    if( QFile::exists( xdgUserDirs ) ) {
+        KConfig xdgUserConf( xdgUserDirs, KConfig::SimpleConfig );
+        KConfigGroup g( &xdgUserConf, "" );
+        downloadPath  = g.readPathEntry( "XDG_DOWNLOAD_DIR", downloadPath ).remove(  '"' );
+        if ( downloadPath.isEmpty() ) {
+            downloadPath = QDir::homePath();
+        }
+    }
+#endif
+    downloadPath = QDir::cleanPath( downloadPath );
+    if ( !downloadPath.endsWith( '/' ) ) {
+        downloadPath.append( QLatin1Char(  '/' ) );
+    }
+    return downloadPath;
+}
+
+QString KGlobalSettings::videosPath()
+{
+    QString path = QDesktopServices::storageLocation( QDesktopServices::MoviesLocation );
+    return path.isEmpty() ? QDir::homePath() : path;
+}
+
+QString KGlobalSettings::picturesPath()
+{
+    QString path = QDesktopServices::storageLocation( QDesktopServices::PicturesLocation );
+    return path.isEmpty() ? QDir::homePath() :path;
+}
+
+QString KGlobalSettings::musicPath()
+{
+    QString path = QDesktopServices::storageLocation( QDesktopServices::MusicLocation );
+    return path.isEmpty() ? QDir::homePath() : path;
 }
 
 bool KGlobalSettings::isMultiHead()
@@ -660,7 +669,7 @@ bool KGlobalSettings::isMultiHead()
 #ifdef Q_WS_WIN
     return GetSystemMetrics(SM_CMONITORS) > 1;
 #else
-    QByteArray multiHead = getenv("KDE_MULTIHEAD");
+    QByteArray multiHead = qgetenv("KDE_MULTIHEAD");
     if (!multiHead.isEmpty()) {
         return (multiHead.toLower() == "true");
     }
@@ -736,6 +745,36 @@ bool KGlobalSettings::showIconsOnPushButtons()
                        KDE_DEFAULT_ICON_ON_PUSHBUTTON);
 }
 
+bool KGlobalSettings::naturalSorting()
+{
+    KConfigGroup g( KGlobal::config(), "KDE" );
+    return g.readEntry("NaturalSorting",
+                       KDE_DEFAULT_NATURAL_SORTING);
+}
+
+KGlobalSettings::GraphicEffects KGlobalSettings::graphicEffectsLevel()
+{
+    // This variable stores whether _graphicEffects has the default value because it has not been
+    // loaded yet, or if it has been loaded from the user settings or defaults and contains a valid
+    // value.
+    static bool _graphicEffectsInitialized = false;
+
+    if (!_graphicEffectsInitialized) {
+        _graphicEffectsInitialized = true;
+        Private::rereadOtherSettings();
+    }
+
+    return _graphicEffects;
+}
+
+KGlobalSettings::GraphicEffects KGlobalSettings::graphicEffectsLevelDefault()
+{
+    // For now, let always enable animations by default. The plan is to make
+    // this code a bit smarter. (ereslibre)
+
+    return ComplexAnimationEffects;
+}
+
 bool KGlobalSettings::showFilePreview(const KUrl &url)
 {
     KConfigGroup g(KGlobal::config(), "PreviewSettings");
@@ -764,14 +803,23 @@ void KGlobalSettings::emitChange(ChangeType changeType, int arg)
     args.append(arg);
     message.setArguments(args);
     QDBusConnection::sessionBus().send(message);
+#ifdef Q_WS_X11
+    if (qApp && qApp->type() != QApplication::Tty) {
+        //notify non-kde qt applications of the change
+        extern void qt_x11_apply_settings_in_all_apps();
+        qt_x11_apply_settings_in_all_apps();
+    }
+#endif
 }
 
 void KGlobalSettings::Private::_k_slotNotifyChange(int changeType, int arg)
 {
     switch(changeType) {
     case StyleChanged:
-        KGlobal::config()->reparseConfiguration();
-        kdisplaySetStyle();
+        if (activated) {
+            KGlobal::config()->reparseConfiguration();
+            kdisplaySetStyle();
+        }
         break;
 
     case ToolbarStyleChanged:
@@ -780,25 +828,34 @@ void KGlobalSettings::Private::_k_slotNotifyChange(int changeType, int arg)
         break;
 
     case PaletteChanged:
-        KGlobal::config()->reparseConfiguration();
-        kdisplaySetPalette();
+        if (activated) {
+            KGlobal::config()->reparseConfiguration();
+            kdisplaySetPalette();
+        }
         break;
 
     case FontChanged:
         KGlobal::config()->reparseConfiguration();
-        KGlobalSettings::Private::rereadFontSettings();
-        kdisplaySetFont();
+        KGlobalSettingsData::self()->dropFontSettingsCache();
+        if (activated) {
+            kdisplaySetFont();
+        }
         break;
 
     case SettingsChanged: {
         KGlobal::config()->reparseConfiguration();
+        rereadOtherSettings();
         SettingsCategory category = static_cast<SettingsCategory>(arg);
-        if (category == SETTINGS_PATHS) {
-            KGlobalSettings::Private::rereadPathSettings();
-        } else if (category == SETTINGS_MOUSE) {
-            KGlobalSettings::Private::rereadMouseSettings();
+        if (category == SETTINGS_MOUSE) {
+            KGlobalSettingsData::self()->dropMouseSettingsCache();
+        }
+        if (category == SETTINGS_QT) {
+            if (activated) {
+                propagateQtSettings();
+            }
+        } else {
+            emit q->settingsChanged(category);
         }
-        propagateSettings(category);
         break;
     }
     case IconChanged:
@@ -817,44 +874,52 @@ void KGlobalSettings::Private::_k_slotNotifyChange(int changeType, int arg)
         emit q->blockShortcuts(arg); // see kwin
         break;
 
+    case NaturalSortingChanged:
+        emit q->naturalSortingChanged();
+        break;
+
     default:
         kWarning(101) << "Unknown type of change in KGlobalSettings::slotNotifyChange: " << changeType;
     }
 }
 
-// Set by KApplication - which is now in kdeui so this needs to be exported
-// In the long run, KGlobalSettings probably belongs to kdeui as well...
+// Set by KApplication
 QString kde_overrideStyle;
 
 void KGlobalSettings::Private::applyGUIStyle()
 {
-#ifdef Q_WS_X11
-    QString defaultStyle = KStyle::defaultStyle();
-#else
-    QString defaultStyle; // Mac, Windows: no change for style by default
-#endif
-    KConfigGroup pConfig (KGlobal::config(), "General");
-    QString styleStr = pConfig.readEntry("widgetStyle", defaultStyle);
+    const QLatin1String currentStyleName(qApp->style()->metaObject()->className());
 
     if (kde_overrideStyle.isEmpty()) {
-        if (styleStr.isEmpty())
+        const QString &defaultStyle = KStyle::defaultStyle();
+        const KConfigGroup pConfig(KGlobal::config(), "General");
+        const QString &styleStr = pConfig.readEntry("widgetStyle", defaultStyle);
+
+        if (styleStr.isEmpty() ||
+                // check whether we already use the correct style to return then
+                // (workaround for Qt misbehavior to avoid double style initialization)
+                0 == (QString(styleStr + QLatin1String("Style"))).compare(currentStyleName, Qt::CaseInsensitive) ||
+                0 == styleStr.compare(currentStyleName, Qt::CaseInsensitive)) {
             return;
-        // ### add check whether we already use the correct style to return then
-        // (workaround for Qt misbehavior to avoid double style initialization)
+        }
 
         QStyle* sp = QStyleFactory::create( styleStr );
+        if (sp && currentStyleName == sp->metaObject()->className()) {
+            delete sp;
+            return;
+        }
 
         // If there is no default style available, try falling back any available style
         if ( !sp && styleStr != defaultStyle)
             sp = QStyleFactory::create( defaultStyle );
         if ( !sp )
-            sp = QStyleFactory::create( *(QStyleFactory::keys().begin()) );
+            sp = QStyleFactory::create( QStyleFactory::keys().first() );
         qApp->setStyle(sp);
-    }
-    else
+    } else if (0 != kde_overrideStyle.compare(currentStyleName, Qt::CaseInsensitive) &&
+            0 != (QString(kde_overrideStyle + QLatin1String("Style"))).compare(currentStyleName, Qt::CaseInsensitive)) {
         qApp->setStyle(kde_overrideStyle);
-    // Reread palette from config file.
-    kdisplaySetPalette();
+    }
+    emit q->kdisplayStyleChanged();
 }
 
 QPalette KGlobalSettings::createApplicationPalette(const KSharedConfigPtr &config)
@@ -864,6 +929,9 @@ QPalette KGlobalSettings::createApplicationPalette(const KSharedConfigPtr &confi
     QPalette::ColorGroup states[3] = { QPalette::Active, QPalette::Inactive,
                                        QPalette::Disabled };
 
+    // TT thinks tooltips shouldn't use active, so we use our active colors for all states
+    KColorScheme schemeTooltip(QPalette::Active, KColorScheme::Tooltip, config);
+
     for ( int i = 0; i < 3 ; i++ ) {
         QPalette::ColorGroup state = states[i];
         KColorScheme schemeView(state, KColorScheme::View, config);
@@ -879,6 +947,8 @@ QPalette KGlobalSettings::createApplicationPalette(const KSharedConfigPtr &confi
         palette.setBrush( state, QPalette::ButtonText, schemeButton.foreground() );
         palette.setBrush( state, QPalette::Highlight, schemeSelection.background() );
         palette.setBrush( state, QPalette::HighlightedText, schemeSelection.foreground() );
+        palette.setBrush( state, QPalette::ToolTipBase, schemeTooltip.background() );
+        palette.setBrush( state, QPalette::ToolTipText, schemeTooltip.foreground() );
 
         palette.setColor( state, QPalette::Light, schemeWindow.shade( KColorScheme::LightShade ) );
         palette.setColor( state, QPalette::Midlight, schemeWindow.shade( KColorScheme::MidlightShade ) );
@@ -894,48 +964,6 @@ QPalette KGlobalSettings::createApplicationPalette(const KSharedConfigPtr &confi
     return palette;
 }
 
-
-QPalette KGlobalSettings::Private::createTooltipPalette(const KSharedConfigPtr &config)
-{
-    QPalette palette = QToolTip::palette();
-
-    QPalette::ColorGroup setStates[3] = { QPalette::Active, QPalette::Inactive,
-                                          QPalette::Disabled };
-    QPalette::ColorGroup useStates[3] = { QPalette::Active, QPalette::Active,
-                                          QPalette::Disabled };
-
-    for ( int i = 0; i < 3 ; i++ ) {
-        // tooltips are never active (arguably a bug?), but should be drawn as if they were
-        QPalette::ColorGroup state = setStates[i];
-        QPalette::ColorGroup stateUsed = useStates[i];
-        KColorScheme schemeButton(stateUsed, KColorScheme::Button, config);
-        KColorScheme schemeSelection(stateUsed, KColorScheme::Selection, config);
-        KColorScheme schemeTooltip(stateUsed, KColorScheme::Tooltip, config);
-
-        palette.setBrush( state, QPalette::WindowText, schemeTooltip.foreground() );
-        palette.setBrush( state, QPalette::Window, schemeTooltip.background() );
-        palette.setBrush( state, QPalette::Base, schemeTooltip.background() );
-        palette.setBrush( state, QPalette::Text, schemeTooltip.foreground() );
-        palette.setBrush( state, QPalette::Button, schemeButton.background() );
-        palette.setBrush( state, QPalette::ButtonText, schemeButton.foreground() );
-        palette.setBrush( state, QPalette::Highlight, schemeSelection.background() );
-        palette.setBrush( state, QPalette::HighlightedText, schemeSelection.foreground() );
-
-        palette.setColor( state, QPalette::Light, schemeTooltip.shade( KColorScheme::LightShade ) );
-        palette.setColor( state, QPalette::Midlight, schemeTooltip.shade( KColorScheme::MidlightShade ) );
-        palette.setColor( state, QPalette::Mid, schemeTooltip.shade( KColorScheme::MidShade ) );
-        palette.setColor( state, QPalette::Dark, schemeTooltip.shade( KColorScheme::DarkShade ) );
-        palette.setColor( state, QPalette::Shadow, schemeTooltip.shade( KColorScheme::ShadowShade ) );
-
-        palette.setBrush( state, QPalette::AlternateBase, schemeTooltip.background( KColorScheme::AlternateBackground) );
-        palette.setBrush( state, QPalette::Link, schemeTooltip.foreground( KColorScheme::LinkText ) );
-        palette.setBrush( state, QPalette::LinkVisited, schemeTooltip.foreground( KColorScheme::VisitedText ) );
-    }
-
-    return palette;
-}
-
-
 void KGlobalSettings::Private::kdisplaySetPalette()
 {
     // Added by Sam/Harald (TT) for Mac OS X initially, but why?
@@ -943,51 +971,59 @@ void KGlobalSettings::Private::kdisplaySetPalette()
     if (cg.readEntry("nopaletteChange", false))
         return;
 
-    if (qApp && qApp->type() == QApplication::GuiClient) {
+    if (qApp->type() == QApplication::GuiClient) {
         QApplication::setPalette( q->createApplicationPalette() );
-        QPalette ttp = createTooltipPalette();
-        QToolTip::setPalette( ttp );
-        // QWhatsThis::setPalette( ttp ); // TODO (when Qt supports it)
-        emit q->kdisplayPaletteChanged();
-        emit q->appearanceChanged();
     }
+    emit q->kdisplayPaletteChanged();
+    emit q->appearanceChanged();
 }
 
 
 void KGlobalSettings::Private::kdisplaySetFont()
 {
-    if (qApp && qApp->type() == QApplication::GuiClient) {
-        QApplication::setFont(KGlobalSettings::generalFont());
-        QApplication::setFont(KGlobalSettings::menuFont(), "QMenuBar");
-        QApplication::setFont(KGlobalSettings::menuFont(), "QPopupMenu");
-        QApplication::setFont(KGlobalSettings::menuFont(), "KPopupTitle");
-        QApplication::setFont(KGlobalSettings::toolBarFont(), "QToolBar");
+    if (qApp->type() == QApplication::GuiClient) {
+        KGlobalSettingsData* data = KGlobalSettingsData::self();
 
-#if 0
-        // "patch" standard QStyleSheet to follow our fonts
-        Q3StyleSheet* sheet = Q3StyleSheet::defaultSheet();
-        sheet->item (QLatin1String("pre"))->setFontFamily (KGlobalSettings::fixedFont().family());
-        sheet->item (QLatin1String("code"))->setFontFamily (KGlobalSettings::fixedFont().family());
-        sheet->item (QLatin1String("tt"))->setFontFamily (KGlobalSettings::fixedFont().family());
-#endif
-
-        emit q->kdisplayFontChanged();
-        emit q->appearanceChanged();
+        QApplication::setFont( data->font(KGlobalSettingsData::GeneralFont) );
+        const QFont menuFont = data->font( KGlobalSettingsData::MenuFont );
+        QApplication::setFont( menuFont, "QMenuBar" );
+        QApplication::setFont( menuFont, "QMenu" );
+        QApplication::setFont( menuFont, "KPopupTitle" );
+        QApplication::setFont( data->font(KGlobalSettingsData::ToolbarFont), "QToolBar" );
     }
+    emit q->kdisplayFontChanged();
+    emit q->appearanceChanged();
 }
 
 
 void KGlobalSettings::Private::kdisplaySetStyle()
 {
-    if (qApp && qApp->type() == QApplication::GuiClient) {
+    if (qApp->type() == QApplication::GuiClient) {
         applyGUIStyle();
-        emit q->kdisplayStyleChanged();
-        // already done by applyGUIStyle -> kdisplaySetPalette
-        //emit appearanceChanged();
+
+        // Reread palette from config file.
+        kdisplaySetPalette();
     }
 }
 
 
+void KGlobalSettings::Private::rereadOtherSettings()
+{
+    KConfigGroup g( KGlobal::config(), "KDE-Global GUI Settings" );
+
+    // Asking for hasKey we do not ask for graphicEffectsLevelDefault() that can
+    // contain some very slow code. If we can save that time, do it. (ereslibre)
+
+    if (g.hasKey("GraphicEffectsLevel")) {
+        _graphicEffects = ((GraphicEffects) g.readEntry("GraphicEffectsLevel", QVariant((int) NoEffects)).toInt());
+
+        return;
+    }
+
+    _graphicEffects = KGlobalSettings::graphicEffectsLevelDefault();
+}
+
+
 void KGlobalSettings::Private::applyCursorTheme()
 {
 #if defined(Q_WS_X11) && defined(HAVE_XCURSOR)
@@ -1017,7 +1053,7 @@ void KGlobalSettings::Private::applyCursorTheme()
 }
 
 
-void KGlobalSettings::Private::propagateSettings(SettingsCategory arg)
+void KGlobalSettings::Private::propagateQtSettings()
 {
     KConfigGroup cg( KGlobal::config(), "KDE" );
 
@@ -1036,20 +1072,8 @@ void KGlobalSettings::Private::propagateSettings(SettingsCategory arg)
     num = cg.readEntry("WheelScrollLines", QApplication::wheelScrollLines());
     QApplication::setWheelScrollLines(num);
 
-    bool b = cg.readEntry("EffectAnimateMenu", false);
-    QApplication::setEffectEnabled( Qt::UI_AnimateMenu, b);
-    b = cg.readEntry("EffectFadeMenu", false);
-    QApplication::setEffectEnabled( Qt::UI_FadeMenu, b);
-    b = cg.readEntry("EffectAnimateCombo", false);
-    QApplication::setEffectEnabled( Qt::UI_AnimateCombo, b);
-    b = cg.readEntry("EffectAnimateTooltip", false);
-    QApplication::setEffectEnabled( Qt::UI_AnimateTooltip, b);
-    b = cg.readEntry("EffectFadeTooltip", false);
-    QApplication::setEffectEnabled( Qt::UI_FadeTooltip, b);
-    //b = !cg.readEntry("EffectNoTooltip", false);
-    //QToolTip::setGloballyEnabled( b ); ###
-
-    emit q->settingsChanged(arg);
+    // KDE5: this seems fairly pointless
+    emit q->settingsChanged(SETTINGS_QT);
 }
 
 #include "kglobalsettings.moc"
