@@ -4,6 +4,8 @@
 #include <QtGui/QWidget>
 #include <QtCore/QPointer>
 
+class QLabel;
+class KRestrictedLine;
 class QString;
 class QPushButton;
 
@@ -26,14 +28,19 @@ private Q_SLOTS:
    void slotHide();
    void slotEnable( bool );
    void slotReadOnly( bool );
+   void slotPassword( bool );
    void slotReturnPressed();
    void resultOutput( const QString& );
    void slotReturnPressed( const QString& );
    void slotClickMessage(bool click);
+   void slotInvalidChar(int);
 protected:
-   QPointer<KLineEdit> m_lineedit;
+   KLineEdit* m_lineedit;
+   KRestrictedLine* m_restrictedLine;
+   QLabel* m_invalidCharLabel;
    QPushButton* m_btnExit;
    QPushButton* m_btnReadOnly;
+   QPushButton* m_btnPassword;
    QPushButton* m_btnEnable;
    QPushButton* m_btnHide;
    QPushButton* m_btnClickMessage;
