@@ -29,31 +29,25 @@
 #include "klineedit_p.h"
 #include "kdeuiwidgetsproxystyle_p.h"
 
-#include <kconfig.h>
-#include <QtGui/QToolTip>
-#include <kcursor.h>
-#include <klocale.h>
-#include <kstandardshortcut.h>
-#include <kmenu.h>
-#include <kdebug.h>
-#include <kcompletionbox.h>
-#include <kurl.h>
-#include <kiconloader.h>
-#include <kicontheme.h>
+#include <kaction.h>
 #include <kapplication.h>
 #include <kauthorized.h>
+#include <kconfig.h>
+#include <kconfiggroup.h>
+#include <kcursor.h>
+#include <kdebug.h>
+#include <kcompletionbox.h>
+#include <kicontheme.h>
 #include <kicon.h>
-#include <kaction.h>
+#include <klocale.h>
+#include <kmenu.h>
 #include <kstandardaction.h>
+#include <kstandardshortcut.h>
 
 #include <QtCore/QTimer>
 #include <QtGui/QClipboard>
-#include <QtGui/QKeyEvent>
-#include <QtGui/QLabel>
-#include <QtGui/QPainter>
-#include <QtGui/QStyle>
 #include <QtGui/QStyleOption>
-#include <kconfiggroup.h>
+#include <QtGui/QToolTip>
 
 class KLineEditPrivate
 {
@@ -73,6 +67,7 @@ public:
         drawClickMsg = false;
         enableClickMsg = false;
         threeStars = false;
+        completionRunning = false;
         if ( !initialized )
         {
             KConfigGroup config( KGlobal::config(), "General" );
@@ -85,6 +80,14 @@ public:
         clickInClear = false;
         wideEnoughForClear = true;
         overlap = 0;
+
+        // i18n: Placeholder text in line edit widgets is the text appearing
+        // before any user input, briefly explaining to the user what to type
+        // (e.g. "Enter search pattern").
+        // By default the text is set in italic, which may not be appropriate
+        // for some languages and scripts (e.g. for CJK ideographs).
+        QString metaMsg = i18nc("Italic placeholder text in line edits: 0 no, 1 yes", "1");
+        italicizePlaceholder = (metaMsg.trimmed() != QString('0'));
     }
 
     ~KLineEditPrivate()
@@ -102,7 +105,14 @@ public:
         }
     }
 
-    void doCompletion(const QString& txt);
+    void _k_updateUserText( const QString &txt)
+    {
+        if ((!completionRunning) && (txt != userText))
+	{
+	    userText = txt;
+	    emit q->userTextChanged(txt);
+	}
+    }
 
     /**
      * Checks whether we should/should not consume a key used as a shortcut.
@@ -123,11 +133,13 @@ public:
     bool handleURLDrops:1;
     bool grabReturnKeyEvents:1;
     bool enableSqueezedText:1;
+    bool completionRunning:1;
 
     int squeezedEnd;
     int squeezedStart;
     QPalette::ColorRole bgRole;
     QString squeezedText;
+    QString userText;
 
     QString clickMessage;
     bool enableClickMsg:1;
@@ -144,6 +156,8 @@ public:
 
     int overlap;
 
+    bool italicizePlaceholder:1;
+
     QAction *noCompletionAction, *shellCompletionAction, *autoCompletionAction, *popupCompletionAction, *shortAutoCompletionAction, *popupAutoCompletionAction, *defaultAction;
 
     QMap<KGlobalSettings::Completion, bool> disableCompletionMap;
@@ -170,7 +184,7 @@ QRect KLineEditStyle::subElementRect(SubElement element, const QStyleOption *opt
   {
     QRect rect = style()->subElementRect(SE_LineEditContents, option, widget);
 
-    int overlap = lineEditPrivate->overlap;
+    const int overlap = lineEditPrivate->overlap;
     if (option->direction == Qt::LeftToRight) return rect.adjusted(0, 0, -overlap, 0);
     else return rect.adjusted(overlap, 0, 0, 0);
   }
@@ -217,7 +231,7 @@ void KLineEdit::init()
 
     connect(KGlobalSettings::self(), SIGNAL(settingsChanged(int)), this, SLOT(_k_slotSettingsChanged(int)));
 
-    QPalette p = palette();
+    const QPalette p = palette();
     if ( !d->previousHighlightedTextColor.isValid() )
       d->previousHighlightedTextColor=p.color(QPalette::Normal,QPalette::HighlightedText);
     if ( !d->previousHighlightColor.isValid() )
@@ -226,6 +240,9 @@ void KLineEdit::init()
     QStyle *lineEditStyle = new KLineEditStyle(this, d);
     lineEditStyle->setParent(this);
     setStyle(lineEditStyle);
+
+    connect( this, SIGNAL(textChanged( const QString&)), this, SLOT(_k_updateUserText( const QString&)));
+
 }
 
 QString KLineEdit::clickMessage() const
@@ -290,7 +307,7 @@ void KLineEdit::updateClearButtonIcon(const QString& text)
         return;
     }
 
-    if (qApp->isLeftToRight()) {
+    if (layoutDirection() == Qt::LeftToRight) {
         d->clearButton->setPixmap(SmallIcon("edit-clear-locationbar-rtl", 0, clearButtonState));
     } else {
         d->clearButton->setPixmap(SmallIcon("edit-clear-locationbar-ltr", 0, clearButtonState));
@@ -321,7 +338,7 @@ void KLineEdit::updateClearButton()
         d->overlap = wideEnough ? buttonWidth + frameWidth : 0;
     }
 
-    if (qApp->isLeftToRight()) {
+    if (layoutDirection() == Qt::LeftToRight ) {
         d->clearButton->move(geom.width() - frameWidth - buttonWidth - 1, 0);
     } else {
         d->clearButton->move(frameWidth + 1, 0);
@@ -373,13 +390,13 @@ void KLineEdit::setCompletedText( const QString& t, bool marked )
     if ( !d->autoSuggest )
       return;
 
-    QString txt = text();
+    const QString txt = text();
 
     if ( t != txt )
     {
-        int start = marked ? txt.length() : t.length();
         setText(t);
-        setSelection(start, t.length());
+        if ( marked )
+            setSelection(t.length(), txt.length()-t.length());
         setUserSelection(false);
     }
     else
@@ -390,7 +407,7 @@ void KLineEdit::setCompletedText( const QString& t, bool marked )
 void KLineEdit::setCompletedText( const QString& text )
 {
     KGlobalSettings::Completion mode = completionMode();
-    bool marked = ( mode == KGlobalSettings::CompletionAuto ||
+    const bool marked = ( mode == KGlobalSettings::CompletionAuto ||
                     mode == KGlobalSettings::CompletionMan ||
                     mode == KGlobalSettings::CompletionPopup ||
                     mode == KGlobalSettings::CompletionPopupAuto );
@@ -460,17 +477,16 @@ void KLineEdit::makeCompletion( const QString& text )
 void KLineEdit::setReadOnly(bool readOnly)
 {
     // Do not do anything if nothing changed...
-    if (readOnly == isReadOnly ())
+    if (readOnly == isReadOnly ()) {
       return;
+    }
 
-    QLineEdit::setReadOnly (readOnly);
+    QLineEdit::setReadOnly(readOnly);
 
-    if (readOnly)
-    {
+    if (readOnly) {
         d->bgRole = backgroundRole();
         setBackgroundRole(QPalette::Window);
-        if (d->enableSqueezedText && d->squeezedText.isEmpty())
-        {
+        if (d->enableSqueezedText && d->squeezedText.isEmpty()) {
             d->squeezedText = text();
             setSqueezedText();
         }
@@ -479,21 +495,14 @@ void KLineEdit::setReadOnly(bool readOnly)
             d->clearButton->animateVisible(false);
             d->overlap = 0;
         }
-    }
-    else
-    {
-        if (!d->squeezedText.isEmpty())
-        {
+    } else {
+        if (!d->squeezedText.isEmpty()) {
            setText(d->squeezedText);
            d->squeezedText.clear();
         }
-        setBackgroundRole(d->bgRole);
 
-        if (d->clearButton && !text().isEmpty()) {
-            int buttonWidth = d->clearButton->sizeHint().width();
-            d->clearButton->animateVisible(true);
-            d->overlap = buttonWidth;
-        }
+        setBackgroundRole(d->bgRole);
+        updateClearButton();
     }
 }
 
@@ -534,10 +543,10 @@ void KLineEdit::setSqueezedText()
 {
     d->squeezedStart = 0;
     d->squeezedEnd = 0;
-    QString fullText = d->squeezedText;
-    QFontMetrics fm(fontMetrics());
-    int labelWidth = size().width() - 2*style()->pixelMetric(QStyle::PM_DefaultFrameWidth) - 2;
-    int textWidth = fm.width(fullText);
+    const QString fullText = d->squeezedText;
+    const QFontMetrics fm(fontMetrics());
+    const int labelWidth = size().width() - 2*style()->pixelMetric(QStyle::PM_DefaultFrameWidth) - 2;
+    const int textWidth = fm.width(fullText);
 
     if (textWidth > labelWidth)
     {
@@ -645,6 +654,7 @@ void KLineEdit::resizeEvent( QResizeEvent * ev )
     QLineEdit::resizeEvent(ev);
 }
 
+
 void KLineEdit::keyPressEvent( QKeyEvent *e )
 {
     const int key = e->key() | e->modifiers();
@@ -656,7 +666,11 @@ void KLineEdit::keyPressEvent( QKeyEvent *e )
     }
     else if ( KStandardShortcut::paste().contains( key ) )
     {
-        paste();
+      // TODO:
+      // we should restore the original text (not autocompleted), otherwise the paste
+      // will get into troubles Bug: 134691
+        if( !isReadOnly() )
+          paste();
         return;
     }
     else if ( KStandardShortcut::pasteSelection().contains( key ) )
@@ -669,17 +683,20 @@ void KLineEdit::keyPressEvent( QKeyEvent *e )
 
     else if ( KStandardShortcut::cut().contains( key ) )
     {
-        cut();
+        if( !isReadOnly() )
+           cut();
         return;
     }
     else if ( KStandardShortcut::undo().contains( key ) )
     {
-        undo();
+        if( !isReadOnly() )
+          undo();
         return;
     }
     else if ( KStandardShortcut::redo().contains( key ) )
     {
-        redo();
+        if( !isReadOnly() )
+           redo();
         return;
     }
     else if ( KStandardShortcut::deleteWordBack().contains( key ) )
@@ -733,8 +750,8 @@ void KLineEdit::keyPressEvent( QKeyEvent *e )
          completionMode() != KGlobalSettings::CompletionNone )
     {
         const KeyBindingMap keys = getKeyBindings();
-        KGlobalSettings::Completion mode = completionMode();
-        bool noModifier = (e->modifiers() == Qt::NoButton ||
+        const KGlobalSettings::Completion mode = completionMode();
+        const bool noModifier = (e->modifiers() == Qt::NoButton ||
                            e->modifiers() == Qt::ShiftModifier ||
                            e->modifiers() == Qt::KeypadModifier);
 
@@ -746,19 +763,22 @@ void KLineEdit::keyPressEvent( QKeyEvent *e )
                  ( e->key() == Qt::Key_Right || e->key() == Qt::Key_Left ) &&
                  e->modifiers()==Qt::NoButton )
             {
-                QString old_txt = text();
+                const QString old_txt = text();
                 d->disableRestoreSelection = true;
-                int start = selectionStart();
+                const int start = selectionStart();
 
                 deselect();
                 QLineEdit::keyPressEvent ( e );
-                int cPosition=cursorPosition();
+                const int cPosition=cursorPosition();
                 setText(old_txt);
-                setCursorPosition(cPosition);
-                if (e->key() ==Qt::Key_Right && cPosition > start )
-                    setSelection(cPosition, old_txt.length());
-                else
-                    setSelection(start, old_txt.length());
+
+                // keep cursor at cPosition
+                setSelection(old_txt.length(), cPosition - old_txt.length());
+                if (e->key() == Qt::Key_Right && cPosition > start )
+                {
+                    //the user explicitly accepted the autocompletion
+                    d->_k_updateUserText(text());
+                }
 
                 d->disableRestoreSelection = false;
                 return;
@@ -783,17 +803,17 @@ void KLineEdit::keyPressEvent( QKeyEvent *e )
         if ( (mode == KGlobalSettings::CompletionAuto ||
               mode == KGlobalSettings::CompletionMan) && noModifier )
         {
-            QString keycode = e->text();
+            const QString keycode = e->text();
             if ( !keycode.isEmpty() && (keycode.unicode()->isPrint() ||
                 e->key() == Qt::Key_Backspace || e->key() == Qt::Key_Delete ) )
             {
-                bool hasUserSelection=d->userSelection;
-                bool hadSelection=hasSelectedText();
+                const bool hasUserSelection=d->userSelection;
+                const bool hadSelection=hasSelectedText();
 
                 bool cursorNotAtEnd=false;
 
-                int start = selectionStart();
-                int cPos = cursorPosition();
+                const int start = selectionStart();
+                const int cPos = cursorPosition();
 
                 // When moving the cursor, we want to keep the autocompletion as an
                 // autocompletion, so we want to process events at the cursor position
@@ -830,7 +850,7 @@ void KLineEdit::keyPressEvent( QKeyEvent *e )
                     if (e->key() == Qt::Key_Delete )
                         d->autoSuggest=false;
 
-                    d->doCompletion(txt);
+                    doCompletion(txt);
 
                     if(  (e->key() == Qt::Key_Backspace || e->key() == Qt::Key_Delete) )
                         d->autoSuggest=true;
@@ -847,14 +867,14 @@ void KLineEdit::keyPressEvent( QKeyEvent *e )
                    mode == KGlobalSettings::CompletionPopupAuto ) &&
                    noModifier && !e->text().isEmpty() )
         {
-            QString old_txt = text();
-            bool hasUserSelection=d->userSelection;
-            bool hadSelection=hasSelectedText();
+            const QString old_txt = text();
+            const bool hasUserSelection=d->userSelection;
+            const bool hadSelection=hasSelectedText();
             bool cursorNotAtEnd=false;
 
-            int start = selectionStart();
-            int cPos = cursorPosition();
-            QString keycode = e->text();
+            const int start = selectionStart();
+            const int cPos = cursorPosition();
+            const QString keycode = e->text();
 
             // When moving the cursor, we want to keep the autocompletion as an
             // autocompletion, so we want to process events at the cursor position
@@ -869,7 +889,7 @@ void KLineEdit::keyPressEvent( QKeyEvent *e )
                 cursorNotAtEnd=true;
             }
 
-            int selectedLength=selectedText().length();
+            const int selectedLength=selectedText().length();
 
             d->disableRestoreSelection = true;
             QLineEdit::keyPressEvent ( e );
@@ -880,8 +900,7 @@ void KLineEdit::keyPressEvent( QKeyEvent *e )
 
             QString txt = text();
             int len = txt.length();
-
-            if ( txt != old_txt && len/* && ( cursorPosition() == len || force )*/ &&
+            if ( ( txt != old_txt || txt != e->text() ) && len/* && ( cursorPosition() == len || force )*/ &&
                  ( (!keycode.isEmpty() && keycode.unicode()->isPrint()) ||
                    e->key() == Qt::Key_Backspace || e->key() == Qt::Key_Delete) )
             {
@@ -904,7 +923,7 @@ void KLineEdit::keyPressEvent( QKeyEvent *e )
                 if ( d->completionBox )
                   d->completionBox->setCancelledText( txt );
 
-                d->doCompletion(txt);
+                doCompletion(txt);
 
                 if ( (e->key() == Qt::Key_Backspace || e->key() == Qt::Key_Delete ) &&
                     mode == KGlobalSettings::CompletionPopupAuto )
@@ -931,11 +950,11 @@ void KLineEdit::keyPressEvent( QKeyEvent *e )
             {
                 // Emit completion if the completion mode is CompletionShell
                 // and the cursor is at the end of the string.
-                QString txt = text();
-                int len = txt.length();
+                const QString txt = text();
+                const int len = txt.length();
                 if ( cursorPosition() == len && len != 0 )
                 {
-                    d->doCompletion(txt);
+                    doCompletion(txt);
                     return;
                 }
             }
@@ -1000,8 +1019,7 @@ void KLineEdit::keyPressEvent( QKeyEvent *e )
             }
         }
     }
-
-    int selectedLength = selectedText().length();
+    const int selectedLength = selectedText().length();
 
     // Let QLineEdit handle any other keys events.
     QLineEdit::keyPressEvent ( e );
@@ -1085,12 +1103,12 @@ QMenu* KLineEdit::createStandardContextMenu()
     if( !isReadOnly() )
     {
         // FIXME: This code depends on Qt's action ordering.
-        QList<QAction *> actionList = popup->actions();
+        const QList<QAction *> actionList = popup->actions();
         enum { UndoAct, RedoAct, Separator1, CutAct, CopyAct, PasteAct, DeleteAct, ClearAct,
                Separator2, SelectAllAct, NCountActs };
         QAction *separatorAction = 0L;
         // separator we want is right after Delete right now.
-        int idx = actionList.indexOf( actionList[DeleteAct] ) + 1;
+        const int idx = actionList.indexOf( actionList[DeleteAct] ) + 1;
         if ( idx < actionList.count() )
             separatorAction = actionList.at( idx );
         if ( separatorAction )
@@ -1126,21 +1144,30 @@ QMenu* KLineEdit::createStandardContextMenu()
 
         //subMenu->setAccel( KStandardShortcut::completion(), ShellCompletion );
 
-        d->shellCompletionAction->setCheckable( !d->disableCompletionMap[ KGlobalSettings::CompletionShell ] );
-        d->noCompletionAction->setCheckable( !d->disableCompletionMap[ KGlobalSettings::CompletionNone ] );
-        d->popupCompletionAction->setCheckable( !d->disableCompletionMap[ KGlobalSettings::CompletionPopup ] );
-        d->autoCompletionAction->setCheckable( !d->disableCompletionMap[ KGlobalSettings::CompletionAuto ] );
-        d->shortAutoCompletionAction->setCheckable( !d->disableCompletionMap[ KGlobalSettings::CompletionMan ] );
-        d->popupAutoCompletionAction->setCheckable( !d->disableCompletionMap[ KGlobalSettings::CompletionPopupAuto ] );
+        d->shellCompletionAction->setCheckable( true );
+        d->noCompletionAction->setCheckable( true );
+        d->popupCompletionAction->setCheckable( true );
+        d->autoCompletionAction->setCheckable( true );
+        d->shortAutoCompletionAction->setCheckable( true );
+        d->popupAutoCompletionAction->setCheckable( true );
 
-        KGlobalSettings::Completion mode = completionMode();
+        d->shellCompletionAction->setEnabled( !d->disableCompletionMap[ KGlobalSettings::CompletionShell ] );
+        d->noCompletionAction->setEnabled( !d->disableCompletionMap[ KGlobalSettings::CompletionNone ] );
+        d->popupCompletionAction->setEnabled( !d->disableCompletionMap[ KGlobalSettings::CompletionPopup ] );
+        d->autoCompletionAction->setEnabled( !d->disableCompletionMap[ KGlobalSettings::CompletionAuto ] );
+        d->shortAutoCompletionAction->setEnabled( !d->disableCompletionMap[ KGlobalSettings::CompletionMan ] );
+        d->popupAutoCompletionAction->setEnabled( !d->disableCompletionMap[ KGlobalSettings::CompletionPopupAuto ] );
+
+        const KGlobalSettings::Completion mode = completionMode();
         d->noCompletionAction->setChecked( mode == KGlobalSettings::CompletionNone );
         d->shellCompletionAction->setChecked( mode == KGlobalSettings::CompletionShell );
         d->popupCompletionAction->setChecked( mode == KGlobalSettings::CompletionPopup );
         d->autoCompletionAction->setChecked(  mode == KGlobalSettings::CompletionAuto );
         d->shortAutoCompletionAction->setChecked( mode == KGlobalSettings::CompletionMan );
         d->popupAutoCompletionAction->setChecked( mode == KGlobalSettings::CompletionPopupAuto );
-        if ( mode != KGlobalSettings::completionMode() )
+
+        const KGlobalSettings::Completion defaultMode = KGlobalSettings::completionMode();
+        if ( mode != defaultMode && !d->disableCompletionMap[ defaultMode ] )
         {
             subMenu->addSeparator();
             d->defaultAction = subMenu->addAction( i18nc("@item:inmenu Text Completion", "Default") );
@@ -1217,7 +1244,17 @@ void KLineEdit::dropEvent(QDropEvent *e)
         const KUrl::List urlList = KUrl::List::fromMimeData( e->mimeData() );
         if ( !urlList.isEmpty() )
         {
-            QString dropText = text();
+            // Let's replace the current text with the dropped URL(s), rather than appending.
+            // Makes more sense in general (#188129), e.g. konq location bar and kurlrequester
+            // can only hold one url anyway. OK this code supports multiple urls being dropped,
+            // but that's not the common case [and it breaks if they contain spaces... this is why
+            // kfiledialog uses double quotes around filenames in multiple-selection mode]...
+            //
+            // Anyway, if some apps prefer "append" then we should have a
+            // setUrlDropsSupport( {NoUrlDrops, SingleUrlDrops, MultipleUrlDrops} )
+            // where Single replaces and Multiple appends.
+            QString dropText;
+            //QString dropText = text();
             KUrl::List::ConstIterator it;
             for( it = urlList.begin() ; it != urlList.end() ; ++it )
             {
@@ -1255,9 +1292,9 @@ bool KLineEdit::event( QEvent* ev )
 
         if( e->key() == Qt::Key_Return || e->key() == Qt::Key_Enter )
         {
-            bool trap = d->completionBox && d->completionBox->isVisible();
+            const bool trap = d->completionBox && d->completionBox->isVisible();
 
-            bool stopEvent = trap || (d->grabReturnKeyEvents &&
+            const bool stopEvent = trap || (d->grabReturnKeyEvents &&
                                       (e->modifiers() == Qt::NoButton ||
                                        e->modifiers() == Qt::KeypadModifier));
 
@@ -1280,18 +1317,14 @@ bool KLineEdit::event( QEvent* ev )
             // Eat the event if the user asked for it, or if a completionbox was visible
             if (stopEvent)
                 return true;
-        } else if (e->key() == Qt::Key_Tab && e->modifiers() == Qt::NoButton ) {
-            // #65877: Key_Tab should complete using the first (or selected) item, and then offer completions again
-            const bool tabHandling = d->completionBox && d->completionBox->isVisible() && d->completionBox->count() > 0;
-            if (tabHandling) {
-                QListWidgetItem* currentItem = d->completionBox->currentItem();
-                const QString txt = currentItem ? currentItem->text() : d->completionBox->item(0)->text();
-                setTextWorkaround(txt);
-                d->doCompletion(txt);
-                e->accept();
-                return true;
-            }
         }
+    } else if (ev->type() == QEvent::ApplicationPaletteChange
+               || ev->type() == QEvent::PaletteChange) {
+        // Assume the widget uses the application's palette
+        QPalette p = QApplication::palette();
+        d->previousHighlightedTextColor=p.color(QPalette::Normal,QPalette::HighlightedText);
+        d->previousHighlightColor=p.color(QPalette::Normal,QPalette::Highlight);
+        setUserSelection(d->userSelection);
     }
     return QLineEdit::event( ev );
 }
@@ -1355,8 +1388,8 @@ void KLineEdit::userCancelled(const QString & cancelText)
       else
       {
         d->autoSuggest=false;
-        int start = selectionStart() ;
-        QString s=text().remove(selectionStart(), selectedText().length());
+        const int start = selectionStart() ;
+        const QString s=text().remove(selectionStart(), selectedText().length());
         setText(s);
         setCursorPosition(start);
         d->autoSuggest=true;
@@ -1368,7 +1401,7 @@ bool KLineEditPrivate::overrideShortcut(const QKeyEvent* e)
 {
     KShortcut scKey;
 
-    int key = e->key() | e->modifiers();
+    const int key = e->key() | e->modifiers();
     const KLineEdit::KeyBindingMap keys = q->getKeyBindings();
 
     if (keys[KLineEdit::TextCompletion].isEmpty())
@@ -1419,10 +1452,20 @@ bool KLineEditPrivate::overrideShortcut(const QKeyEvent* e)
     else if (KStandardShortcut::endOfLine().contains( key ))
         return true;
 
+    // Shortcut overrides for shortcuts that QLineEdit handles
+    // but doesn't dare force as "stronger than kaction shortcuts"...
+    else if (e->matches(QKeySequence::SelectAll)) {
+        return true;
+    }
+#ifdef Q_WS_X11
+    else if (key == Qt::CTRL + Qt::Key_E || key == Qt::CTRL + Qt::Key_U)
+        return true;
+#endif
+
     if (completionBox && completionBox->isVisible ())
     {
         const int key = e->key();
-        Qt::KeyboardModifiers modifiers = e->modifiers();
+        const Qt::KeyboardModifiers modifiers = e->modifiers();
         if ((key == Qt::Key_Backtab || key == Qt::Key_Tab) &&
             (modifiers == Qt::NoModifier || (modifiers & Qt::ShiftModifier)))
         {
@@ -1478,7 +1521,7 @@ void KLineEdit::setCompletedItems( const QStringList& items, bool autoSuggest )
             }
             if ( matchedItem )
             {
-                bool blocked = d->completionBox->blockSignals( true );
+                const bool blocked = d->completionBox->blockSignals( true );
                 d->completionBox->setCurrentItem( matchedItem );
                 matchedItem->setSelected(wasSelected);
                 d->completionBox->blockSignals( blocked );
@@ -1494,8 +1537,8 @@ void KLineEdit::setCompletedItems( const QStringList& items, bool autoSuggest )
 
         if ( d->autoSuggest && autoSuggest )
         {
-            int index = items.first().indexOf( txt );
-            QString newText = items.first().mid( index );
+            const int index = items.first().indexOf( txt );
+            const QString newText = items.first().mid( index );
             setUserSelection(false);
             setCompletedText(newText,true);
         }
@@ -1541,6 +1584,14 @@ void KLineEdit::create( WId id, bool initializeWindow, bool destroyOldWindow )
 
 void KLineEdit::setUserSelection(bool userSelection)
 {
+    //if !d->userSelection && userSelection we are accepting a completion,
+    //so trigger an update
+
+    if (!d->userSelection && userSelection)
+    {
+	d->_k_updateUserText(text());
+    }
+
     QPalette p = palette();
 
     if (userSelection)
@@ -1590,6 +1641,11 @@ QString KLineEdit::originalText() const
     return text();
 }
 
+QString KLineEdit::userText() const
+{
+    return d->userText;
+}
+
 bool KLineEdit::autoSuggest() const
 {
     return d->autoSuggest;
@@ -1598,35 +1654,74 @@ bool KLineEdit::autoSuggest() const
 void KLineEdit::paintEvent( QPaintEvent *ev )
 {
     if (echoMode() == Password && d->threeStars) {
-        QString oldText = text();
-        bool isModifiedState = isModified(); // save modified state because setText resets it
-        setText(text() + text() + text());
+        // ### hack alert!
+        // QLineEdit has currently no hooks to modify the displayed string.
+        // When we call setText(), an update() is triggered and we get
+        // into an infinite recursion.
+        // Qt offers the setUpdatesEnabled() method, but when we re-enable
+        // them, update() is triggered, and we get into the same recursion.
+        // To work around this problem, we set/clear the internal Qt flag which
+        // marks the updatesDisabled state manually.
+        setAttribute(Qt::WA_UpdatesDisabled, true);
+        blockSignals(true);
+        const QString oldText = text();
+        const bool isModifiedState = isModified(); // save modified state because setText resets it
+        setText(oldText + oldText + oldText);
         QLineEdit::paintEvent(ev);
         setText(oldText);
         setModified(isModifiedState);
-        return;
+        blockSignals(false);
+        setAttribute(Qt::WA_UpdatesDisabled, false);
+    } else {
+        QLineEdit::paintEvent( ev );
     }
 
-    QLineEdit::paintEvent( ev );
+    if (d->enableClickMsg && d->drawClickMsg && !hasFocus() && text().isEmpty()) {
+        QPainter p(this);
+        QFont f = font();
+        f.setItalic(d->italicizePlaceholder);
+        p.setFont(f);
 
-    if ( d->enableClickMsg && d->drawClickMsg && !hasFocus() && text().isEmpty() ) {
-        QPainter p( this );
-        QPen tmp = p.pen();
-        p.setPen( palette().color( QPalette::Disabled, QPalette::Text ) );
+        QColor color(palette().color(foregroundRole()));
+        color.setAlphaF(0.5);
+        p.setPen(color);
 
-        //FIXME: fugly alert!
-        // qlineedit uses an internal qstyleoption set to figure this out
-        // and then adds a hardcoded 2 pixel interior to that.
-        // probably requires fixes to Qt itself to do this cleanly
-        // see define horizontalMargin 2 in qlineedit.cpp
         QStyleOptionFrame opt;
-        initStyleOption( &opt );
-        QRect cr = style()->subElementRect( QStyle::SE_LineEditContents, &opt, this );
-        cr.setLeft( cr.left() + 2 );
-        cr.setRight( cr.right() - 2 );
+        initStyleOption(&opt);
+        QRect cr = style()->subElementRect(QStyle::SE_LineEditContents, &opt, this);
+
+        // this is copied/adapted from QLineEdit::paintEvent
+        const int verticalMargin(1);
+        const int horizontalMargin(2);
+
+        int left, top, right, bottom;
+        getTextMargins( &left, &top, &right, &bottom );
+        cr.adjust( left, top, -right, -bottom );
+
+        p.setClipRect(cr);
+
+        QFontMetrics fm = fontMetrics();
+        Qt::Alignment va = alignment() & Qt::AlignVertical_Mask;
+        int vscroll;
+        switch (va & Qt::AlignVertical_Mask)
+        {
+            case Qt::AlignBottom:
+            vscroll = cr.y() + cr.height() - fm.height() - verticalMargin;
+            break;
+
+            case Qt::AlignTop:
+            vscroll = cr.y() + verticalMargin;
+            break;
+
+            default:
+            vscroll = cr.y() + (cr.height() - fm.height() + 1) / 2;
+            break;
+
+        }
+
+        QRect lineRect(cr.x() + horizontalMargin, vscroll, cr.width() - 2*horizontalMargin, fm.height());
+        p.drawText(lineRect, Qt::AlignLeft|Qt::AlignVCenter, d->clickMessage);
 
-        p.drawText( cr, Qt::AlignLeft|Qt::AlignVCenter, d->clickMessage );
-        p.setPen( tmp );
     }
 }
 
@@ -1652,7 +1747,7 @@ void KLineEdit::focusOutEvent( QFocusEvent *ev )
 
 void KLineEdit::setClickMessage( const QString &msg )
 {
-    d->enableClickMsg = true;
+    d->enableClickMsg = !msg.isEmpty();
     d->clickMessage = msg;
     d->drawClickMsg = text().isEmpty();
     update();
@@ -1692,15 +1787,16 @@ bool KLineEdit::passwordMode() const
     return echoMode() == NoEcho || echoMode() == Password;
 }
 
-void KLineEditPrivate::doCompletion(const QString& txt)
+void KLineEdit::doCompletion(const QString& txt)
 {
-    if (q->emitSignals()) {
-        emit q->completion(txt); // emit when requested...
+    if (emitSignals()) {
+        emit completion(txt); // emit when requested...
     }
-
-    if (q->handleSignals()) {
-        q->makeCompletion(txt);  // handle when requested...
+    d->completionRunning = true;
+    if (handleSignals()) {
+        makeCompletion(txt);  // handle when requested...
     }
+    d->completionRunning = false;
 }
 
 #include "klineedit.moc"
