@@ -37,6 +37,11 @@
 #include <QtGui/QWidgetAction>
 
 #include <kdebug.h>
+#include <kglobal.h>
+#include <klocale.h>
+#include <kacceleratormanager.h>
+
+static const char *KMENU_TITLE = "kmenu_title";
 
 class KMenu::KMenuPrivate
 {
@@ -47,6 +52,7 @@ public:
     void resetKeyboardVars(bool noMatches = false);
     void actionHovered(QAction* action);
     void showCtxMenu(const QPoint &pos);
+    void skipTitles(QKeyEvent *event);
 
     KMenu *parent;
 
@@ -61,6 +67,7 @@ public:
     QString originalText;
 
     QAction* lastHitAction;
+    QAction* lastHoveredAction;
     Qt::MouseButtons mouseButtons;
     Qt::KeyboardModifiers keyboardModifiers;
 
@@ -112,6 +119,7 @@ KMenu::KMenuPrivate::KMenuPrivate (KMenu *_parent)
     , shortcuts(false)
     , autoExec(false)
     , lastHitAction(0L)
+    , lastHoveredAction(0L)
     , mouseButtons(Qt::NoButton)
     , keyboardModifiers(Qt::NoModifier)
     , ctxMenu(0)
@@ -119,6 +127,7 @@ KMenu::KMenuPrivate::KMenuPrivate (KMenu *_parent)
     , eventSniffer(new EventSniffer)
 {
     resetKeyboardVars();
+    KAcceleratorManager::manage(parent);
 }
 
 KMenu::KMenuPrivate::~KMenuPrivate ()
@@ -185,6 +194,7 @@ QAction* KMenu::addTitle(const QIcon &icon, const QString &text, QAction* before
     buttonAction->setIcon(icon);
 
     QWidgetAction *action = new QWidgetAction(this);
+    action->setObjectName(KMENU_TITLE);
     QToolButton *titleButton = new QToolButton(this);
     titleButton->installEventFilter(d->eventSniffer); // prevent clicks on the title of the menu
     titleButton->setDefaultAction(buttonAction);
@@ -224,6 +234,11 @@ void KMenu::keyPressEvent(QKeyEvent* e)
     if (!d->shortcuts) {
         d->keyboardModifiers = e->modifiers();
         QMenu::keyPressEvent(e);
+
+        if (e->key() == Qt::Key_Up || e->key() == Qt::Key_Down) {
+            d->skipTitles(e);
+        }
+
         return;
     }
 
@@ -243,6 +258,10 @@ void KMenu::keyPressEvent(QKeyEvent* e)
         //e->ignore();
         d->keyboardModifiers = e->modifiers();
         QMenu::keyPressEvent(e);
+
+        if (key == Qt::Key_Up || key == Qt::Key_Down) {
+            d->skipTitles(e);
+        }
         return;
     } else if ( key == Qt::Key_Shift || key == Qt::Key_Control || key == Qt::Key_Alt || key == Qt::Key_Meta )
         return QMenu::keyPressEvent(e);
@@ -291,7 +310,7 @@ void KMenu::keyPressEvent(QKeyEvent* e)
     }
 
     d->keySeq += keyString;
-    int seqLen = d->keySeq.length();
+    const int seqLen = d->keySeq.length();
 
     foreach (a, actions()) {
         // don't search disabled entries
@@ -308,7 +327,7 @@ void KMenu::keyPressEvent(QKeyEvent* e)
             thisText = a->text();
 
         // if there is an accelerator present, remove it
-        thisText = thisText.remove('&');
+        thisText = KGlobal::locale()->removeAcceleratorMarker(thisText);
 
         // chop text to the search length
         thisText = thisText.left(seqLen);
@@ -478,8 +497,9 @@ void KMenu::hideContextMenu()
     d->ctxMenu->hide();
 }
 
-void KMenu::KMenuPrivate::actionHovered(QAction* /*action*/)
+void KMenu::KMenuPrivate::actionHovered(QAction* action)
 {
+    lastHoveredAction = action;
     parent->hideContextMenu();
 }
 
@@ -515,6 +535,21 @@ void KMenu::KMenuPrivate::showCtxMenu(const QPoint &pos)
     ctxMenu->popup(parent->mapToGlobal(pos));
 }
 
+void KMenu::KMenuPrivate::skipTitles(QKeyEvent *event)
+{
+    QWidgetAction *action = qobject_cast<QWidgetAction*>(parent->activeAction());
+    QWidgetAction *firstAction = action;
+    while (action && action->objectName() == KMENU_TITLE)
+    {
+        parent->keyPressEvent(event);
+        action = qobject_cast<QWidgetAction*>(parent->activeAction());
+        if (firstAction == action) { // we looped and only found titles
+            parent->setActiveAction(0);
+            break;
+        }
+    }
+}
+
 KMenu * KMenu::contextMenuFocus( )
 {
   return qobject_cast<KMenu*>(QApplication::activePopupWidget());
@@ -523,8 +558,10 @@ KMenu * KMenu::contextMenuFocus( )
 QAction * KMenu::contextMenuFocusAction( )
 {
   if (KMenu* menu = qobject_cast<KMenu*>(QApplication::activePopupWidget())) {
-    //QVariant var = menu->menuAction()->data();  //it seems that this action is not the good one, at least once the action has been clicked
-    QVariant var = menu->activeAction()->data();
+    if (!menu->d->lastHoveredAction) {
+      return 0;
+    }
+    QVariant var = menu->d->lastHoveredAction->data();
     KMenuContext ctx = var.value<KMenuContext>();
     Q_ASSERT(ctx.menu() == menu);
     return ctx.action();
