@@ -19,12 +19,21 @@
 */
 
 #include "ktextedit.h"
+#include <ktoolinvocation.h>
+#include <kdebug.h>
 
 #include <QApplication>
 #include <QClipboard>
 #include <QKeyEvent>
+#include <QMenu>
+#include <QPainter>
 #include <QScrollBar>
 #include <QTextCursor>
+#include <QDBusInterface>
+#include <QDBusConnection>
+#include <QDBusConnectionInterface>
+
+#include <configdialog.h>
 #include <dialog.h>
 #include "backgroundchecker.h"
 #include <kaction.h>
@@ -40,10 +49,10 @@
 #include <kfinddialog.h>
 #include <kfind.h>
 #include <kreplace.h>
-#include <QMenu>
 #include <kmessagebox.h>
+#include <kmenu.h>
 #include <kwindowsystem.h>
-
+#include <QDebug>
 class KTextEdit::Private
 {
   public:
@@ -54,6 +63,18 @@ class KTextEdit::Private
         findReplaceEnabled(true),
         highlighter( 0 ), findDlg(0),find(0),repDlg(0),replace(0), findIndex(0), repIndex(0)
     {
+        //Check the default sonnet settings to see if spellchecking should be enabled.
+        sonnetKConfig = new KConfig("sonnetrc");
+        KConfigGroup group(sonnetKConfig, "Spelling");
+        checkSpellingEnabled = group.readEntry("checkerEnabledByDefault", false);
+
+        // i18n: Placeholder text in text edit widgets is the text appearing
+        // before any user input, briefly explaining to the user what to type
+        // (e.g. "Enter message").
+        // By default the text is set in italic, which may not be appropriate
+        // for some languages and scripts (e.g. for CJK ideographs).
+        QString metaMsg = i18nc("Italic placeholder text in line edits: 0 no, 1 yes", "1");
+        italicizePlaceholder = (metaMsg.trimmed() != QString('0'));
     }
 
     ~Private()
@@ -63,6 +84,7 @@ class KTextEdit::Private
       delete find;
       delete replace;
       delete repDlg;
+      delete sonnetKConfig;
     }
 
     /**
@@ -76,31 +98,43 @@ class KTextEdit::Private
      */
     bool handleShortcut(const QKeyEvent* e);
 
-    void slotSpellCheckDone( const QString &s );
-
     void spellCheckerMisspelling( const QString &text, int pos );
     void spellCheckerCorrected( const QString &, int,const QString &);
     void spellCheckerAutoCorrect(const QString&,const QString&);
     void spellCheckerCanceled();
+    void spellCheckerFinished();
+    void toggleAutoSpellCheck();
 
     void slotFindHighlight(const QString& text, int matchingIndex, int matchingLength);
     void slotReplaceText(const QString &text, int replacementIndex, int /*replacedLength*/, int matchedLength);
 
-    void spellCheckerFinished();
-    void toggleAutoSpellCheck();
+    /**
+     * Similar to QTextEdit::clear(), only that it is possible to undo this
+     * action.
+     */
+    void undoableClear();
+
     void slotAllowTab();
     void menuActivated( QAction* action );
 
+    void updateClickMessageRect();
+
+    void init();
+
     KTextEdit *parent;
+    KTextEditSpellInterface *spellInterface;
     QAction *autoSpellCheckAction;
     QAction *allowTab;
     QAction *spellCheckAction;
+    QString clickMessage;
+    bool italicizePlaceholder : 1;
     bool customPalette : 1;
 
     bool checkSpellingEnabled : 1;
     bool findReplaceEnabled: 1;
     QString originalBuffer;
-    QString spellChechingConfigFileName;
+    QString originalHtml;
+    QString spellCheckingConfigFileName;
     QString spellCheckingLanguage;
     Sonnet::Highlighter *highlighter;
     KFindDialog *findDlg;
@@ -108,12 +142,16 @@ class KTextEdit::Private
     KReplaceDialog *repDlg;
     KReplace *replace;
     int findIndex, repIndex;
+    KConfig *sonnetKConfig;
 };
 
 void KTextEdit::Private::spellCheckerCanceled()
 {
     parent->selectAll();
-    parent->setPlainText(originalBuffer);
+    if(parent->acceptRichText ())
+      parent->setHtml(originalHtml);
+    else
+      parent->setPlainText(originalBuffer);
     spellCheckerFinished();
 }
 
@@ -122,13 +160,6 @@ void KTextEdit::Private::spellCheckerAutoCorrect(const QString&,const QString&)
     //TODO
 }
 
-void KTextEdit::Private::slotSpellCheckDone( const QString &s )
-{
-//Necessary ?
-//    if ( s != text() ) // TODO: toPlainText()?? (MiB)
-//        setText( s ); // setPlainText() ?! we'd loose rich text info
-}
-
 void KTextEdit::Private::spellCheckerMisspelling( const QString &text, int pos )
 {
     //kDebug()<<"TextEdit::Private::spellCheckerMisspelling :"<<text<<" pos :"<<pos;
@@ -157,7 +188,17 @@ void KTextEdit::Private::spellCheckerFinished()
 
 void KTextEdit::Private::toggleAutoSpellCheck()
 {
-  parent->setCheckSpellingEnabled( !checkSpellingEnabled );
+  parent->setCheckSpellingEnabled( !parent->checkSpellingEnabled() );
+}
+
+void KTextEdit::Private::undoableClear()
+{
+    QTextCursor cursor = parent->textCursor();
+    cursor.beginEditBlock();
+    cursor.movePosition(QTextCursor::Start);
+    cursor.movePosition(QTextCursor::End, QTextCursor::KeepAnchor);
+    cursor.removeSelectedText();
+    cursor.endEditBlock();
 }
 
 void KTextEdit::Private::slotAllowTab()
@@ -188,30 +229,45 @@ void KTextEdit::Private::slotFindHighlight(const QString& text, int matchingInde
 }
 
 
-void KTextEdit::Private::slotReplaceText(const QString &text, int replacementIndex, int /*replacedLength*/, int matchedLength) {
-    Q_UNUSED(text)
+void KTextEdit::Private::slotReplaceText(const QString &text, int replacementIndex, int replacedLength, int matchedLength) {
     //kDebug() << "Replace: [" << text << "] ri:" << replacementIndex << " rl:" << replacedLength << " ml:" << matchedLength;
     QTextCursor tc = parent->textCursor();
     tc.setPosition(replacementIndex);
     tc.movePosition(QTextCursor::NextCharacter, QTextCursor::KeepAnchor, matchedLength);
     tc.removeSelectedText();
-    tc.insertText(repDlg->replacement());
+    tc.insertText(text.mid(replacementIndex, replacedLength));
     parent->setTextCursor(tc);
     if (replace->options() & KReplaceDialog::PromptOnReplace) {
         parent->ensureCursorVisible();
     }
 }
 
+void KTextEdit::Private::updateClickMessageRect()
+{
+    int margin = int(parent->document()->documentMargin());
+    QRect rect = parent->viewport()->rect().adjusted(margin, margin, -margin, -margin);
+    rect = parent->fontMetrics().boundingRect(rect, Qt::AlignTop | Qt::TextWordWrap, clickMessage);
+    parent->viewport()->update(rect);
+}
+
+void KTextEdit::Private::init()
+{
+    spellInterface = 0;
+    KCursor::setAutoHideCursor(parent, true, false);
+    parent->connect(parent, SIGNAL(languageChanged(const QString&)),
+                    parent, SLOT(setSpellCheckingLanguage(const QString&)));
+}
+
 KTextEdit::KTextEdit( const QString& text, QWidget *parent )
   : QTextEdit( text, parent ), d( new Private( this ) )
 {
-  KCursor::setAutoHideCursor( this, true, false );
+  d->init();
 }
 
 KTextEdit::KTextEdit( QWidget *parent )
   : QTextEdit( parent ), d( new Private( this ) )
 {
-  KCursor::setAutoHideCursor( this, true, false );
+  d->init();
 }
 
 KTextEdit::~KTextEdit()
@@ -221,12 +277,41 @@ KTextEdit::~KTextEdit()
 
 void KTextEdit::setSpellCheckingConfigFileName(const QString &_fileName)
 {
-    d->spellChechingConfigFileName = _fileName;
+    d->spellCheckingConfigFileName = _fileName;
+}
+
+const QString& KTextEdit::spellCheckingLanguage() const
+{
+    return d->spellCheckingLanguage;
 }
 
 void KTextEdit::setSpellCheckingLanguage(const QString &_language)
 {
-    d->spellCheckingLanguage = _language;
+    if (highlighter()) {
+        highlighter()->setCurrentLanguage(_language);
+        highlighter()->rehighlight();
+    }
+
+    if (_language != d->spellCheckingLanguage) {
+        d->spellCheckingLanguage = _language;
+        emit languageChanged(_language);
+    }
+    else
+        d->spellCheckingLanguage = _language;
+}
+
+void KTextEdit::showSpellConfigDialog(const QString &configFileName,
+                                      const QString &windowIcon)
+{
+    KConfig config(configFileName);
+    Sonnet::ConfigDialog dialog(&config, this);
+    if (!d->spellCheckingLanguage.isEmpty())
+        dialog.setLanguage(d->spellCheckingLanguage);
+    connect(&dialog, SIGNAL(languageChanged(const QString &)),
+            this, SLOT(setSpellCheckingLanguage(const QString &)));
+    if (!windowIcon.isEmpty())
+        dialog.setWindowIcon(KIcon(windowIcon));
+    dialog.exec();
 }
 
 bool KTextEdit::event(QEvent* ev)
@@ -255,11 +340,13 @@ bool KTextEdit::Private::handleShortcut(const QKeyEvent* event)
     parent->cut();
     return true;
   } else if ( KStandardShortcut::undo().contains( key ) ) {
-    parent->document()->undo();
-    return true;
+      if(!parent->isReadOnly())
+          parent->undo();
+      return true;
   } else if ( KStandardShortcut::redo().contains( key ) ) {
-    parent->document()->redo();
-    return true;
+      if(!parent->isReadOnly())
+         parent->redo();
+      return true;
   } else if ( KStandardShortcut::deleteWordBack().contains( key ) ) {
     parent->deleteWordBack();
     return true;
@@ -278,21 +365,39 @@ bool KTextEdit::Private::handleShortcut(const QKeyEvent* event)
     return true;
   } else if ( KStandardShortcut::next().contains( key ) ) {
     QTextCursor cursor = parent->textCursor();
-    int targetY = parent->verticalScrollBar()->value() + parent->viewport()->height();
     bool moved = false;
+    qreal lastY = parent->cursorRect(cursor).bottom();
+    qreal distance = 0;
     do {
-      moved = cursor.movePosition( QTextCursor::Down );
-      parent->setTextCursor( cursor );
-    } while ( moved && parent->verticalScrollBar()->value() < targetY );
+        qreal y = parent->cursorRect(cursor).bottom();
+        distance += qAbs(y - lastY);
+        lastY = y;
+        moved = cursor.movePosition(QTextCursor::Down);
+    } while (moved && distance < parent->viewport()->height());
+
+    if (moved) {
+        cursor.movePosition(QTextCursor::Up);
+        parent->verticalScrollBar()->triggerAction(QAbstractSlider::SliderPageStepAdd);
+    }
+    parent->setTextCursor(cursor);
     return true;
   } else if ( KStandardShortcut::prior().contains( key ) ) {
     QTextCursor cursor = parent->textCursor();
-    int targetY = parent->verticalScrollBar()->value() - parent->viewport()->height();
     bool moved = false;
+    qreal lastY = parent->cursorRect(cursor).bottom();
+    qreal distance = 0;
     do {
-      moved = cursor.movePosition( QTextCursor::Up );
-      parent->setTextCursor( cursor );
-    } while ( moved && parent->verticalScrollBar()->value() > targetY );
+        qreal y = parent->cursorRect(cursor).bottom();
+        distance += qAbs(y - lastY);
+        lastY = y;
+        moved = cursor.movePosition(QTextCursor::Up);
+    } while (moved && distance < parent->viewport()->height());
+
+    if (moved) {
+        cursor.movePosition(QTextCursor::Down);
+        parent->verticalScrollBar()->triggerAction(QAbstractSlider::SliderPageStepSub);
+    }
+    parent->setTextCursor(cursor);
     return true;
   } else if ( KStandardShortcut::begin().contains( key ) ) {
     QTextCursor cursor = parent->textCursor();
@@ -314,6 +419,18 @@ bool KTextEdit::Private::handleShortcut(const QKeyEvent* event)
     cursor.movePosition( QTextCursor::EndOfLine );
     parent->setTextCursor( cursor );
     return true;
+  } else if (KStandardShortcut::find().contains(key)) {
+      if (findReplaceEnabled)
+          parent->slotFind();
+      return true;
+  } else if (KStandardShortcut::findNext().contains(key)) {
+      if (findReplaceEnabled)
+          parent->slotFindNext();
+      return true;
+  } else if (KStandardShortcut::replace().contains(key)) {
+      if (!parent->isReadOnly() && findReplaceEnabled)
+          parent->slotReplace();
+      return true;
   } else if ( KStandardShortcut::pasteSelection().contains( key ) ) {
     QString text = QApplication::clipboard()->text( QClipboard::Selection );
     if ( !text.isEmpty() )
@@ -323,29 +440,31 @@ bool KTextEdit::Private::handleShortcut(const QKeyEvent* event)
   return false;
 }
 
+static void deleteWord(QTextCursor cursor, QTextCursor::MoveOperation op)
+{
+  cursor.clearSelection();
+  cursor.movePosition( op, QTextCursor::KeepAnchor );
+  cursor.removeSelectedText();
+}
+
 void KTextEdit::deleteWordBack()
 {
-  QTextCursor cursor = textCursor();
-  cursor.clearSelection();
-  cursor.movePosition( QTextCursor::PreviousWord, QTextCursor::KeepAnchor );
-  cursor.removeSelectedText();
+  deleteWord(textCursor(), QTextCursor::PreviousWord);
 }
 
 void KTextEdit::deleteWordForward()
 {
-  QTextCursor cursor = textCursor();
-  cursor.clearSelection();
-  cursor.movePosition( QTextCursor::EndOfWord, QTextCursor::KeepAnchor );
-  cursor.removeSelectedText();
+  deleteWord(textCursor(), QTextCursor::WordRight);
 }
 
 QMenu *KTextEdit::mousePopupMenu()
 {
   QMenu *popup = createStandardContextMenu();
+  if (!popup) return 0;
   connect( popup, SIGNAL( triggered ( QAction* ) ),
              this, SLOT( menuActivated( QAction* ) ) );
 
-  bool emptyDocument = document()->isEmpty();
+  const bool emptyDocument = document()->isEmpty();
   if( !isReadOnly() )
   {
       QList<QAction *> actionList = popup->actions();
@@ -356,7 +475,7 @@ QMenu *KTextEdit::mousePopupMenu()
           separatorAction = actionList.at( idx );
       if ( separatorAction )
       {
-          KAction *clearAllAction = KStandardAction::clear( this, SLOT( clear() ), this) ;
+          KAction *clearAllAction = KStandardAction::clear(this, SLOT(undoableClear()), popup);
           if ( emptyDocument )
               clearAllAction->setEnabled( false );
           popup->insertAction( separatorAction, clearAllAction );
@@ -375,7 +494,7 @@ QMenu *KTextEdit::mousePopupMenu()
         d->spellCheckAction->setEnabled( false );
       d->autoSpellCheckAction = popup->addAction( i18n( "Auto Spell Check" ) );
       d->autoSpellCheckAction->setCheckable( true );
-      d->autoSpellCheckAction->setChecked( d->checkSpellingEnabled );
+      d->autoSpellCheckAction->setChecked( checkSpellingEnabled() );
       popup->addSeparator();
       d->allowTab = popup->addAction( i18n("Allow Tabulations") );
       d->allowTab->setCheckable( true );
@@ -383,30 +502,164 @@ QMenu *KTextEdit::mousePopupMenu()
 
       if (d->findReplaceEnabled)
       {
-          KAction *findAction = KStandardAction::find( this, SLOT( slotFind() ), this );
-          KAction *findNextAction = KStandardAction::findNext( this, SLOT( slotFindNext() ), this );
-          KAction *replaceAction = KStandardAction::replace( this, SLOT( slotReplace() ), this );
+          KAction *findAction = KStandardAction::find(this, SLOT(slotFind()), popup);
+          KAction *findNextAction = KStandardAction::findNext(this, SLOT(slotFindNext()), popup);
+          KAction *replaceAction = KStandardAction::replace(this, SLOT(slotReplace()), popup);
           if (emptyDocument)
           {
               findAction->setEnabled(false);
-              findNextAction->setEnabled(d->find != 0 );
+              findNextAction->setEnabled(false );
               replaceAction->setEnabled(false);
           }
+	  else
+	      findNextAction->setEnabled(d->find != 0 );
           popup->addSeparator();
           popup->addAction(findAction);
           popup->addAction(findNextAction);
           popup->addAction(replaceAction);
       }
   }
+  popup->addSeparator();
+  QAction *speakAction = popup->addAction(i18n("Speak Text"));
+  speakAction->setIcon(KIcon("preferences-desktop-text-to-speech"));
+  speakAction->setEnabled(!emptyDocument );
+  connect( speakAction, SIGNAL(triggered(bool)), this, SLOT(slotSpeakText()) );
   return popup;
 }
 
-void KTextEdit::contextMenuEvent( QContextMenuEvent *event )
+void KTextEdit::slotSpeakText()
 {
-  QMenu *popup = mousePopupMenu();
-  popup->exec( event->globalPos() );
+    // If KTTSD not running, start it.
+    if (!QDBusConnection::sessionBus().interface()->isServiceRegistered("org.kde.kttsd"))
+    {
+        QString error;
+        if (KToolInvocation::startServiceByDesktopName("kttsd", QStringList(), &error))
+        {
+            KMessageBox::error(this, i18n( "Starting KTTSD Failed"), error );
+            return;
+        }
+    }
+    QDBusInterface ktts("org.kde.kttsd", "/KSpeech", "org.kde.KSpeech");
+    QString text;
+    if(textCursor().hasSelection())
+        text = textCursor().selectedText();
+    else
+        text = toPlainText();
+    ktts.asyncCall("say", text, 0);
+}
 
-  delete popup;
+void KTextEdit::contextMenuEvent(QContextMenuEvent *event)
+{
+    // Obtain the cursor at the mouse position and the current cursor
+    QTextCursor cursorAtMouse = cursorForPosition(event->pos());
+    const int mousePos = cursorAtMouse.position();
+    QTextCursor cursor = textCursor();
+
+    // Check if the user clicked a selected word
+    const bool selectedWordClicked = cursor.hasSelection() &&
+                               mousePos >= cursor.selectionStart() &&
+                               mousePos <= cursor.selectionEnd();
+
+    // Get the word under the (mouse-)cursor and see if it is misspelled.
+    // Don't include apostrophes at the start/end of the word in the selection.
+    QTextCursor wordSelectCursor(cursorAtMouse);
+    wordSelectCursor.clearSelection();
+    wordSelectCursor.select(QTextCursor::WordUnderCursor);
+    QString selectedWord = wordSelectCursor.selectedText();
+
+    bool isMouseCursorInsideWord = true;
+    if ((mousePos < wordSelectCursor.selectionStart() ||
+            mousePos >= wordSelectCursor.selectionEnd())
+                                        && (selectedWord.length() > 1)) {
+         isMouseCursorInsideWord = false;
+    }
+
+    // Clear the selection again, we re-select it below (without the apostrophes).
+    wordSelectCursor.setPosition(wordSelectCursor.position()-selectedWord.size());
+    if (selectedWord.startsWith('\'') || selectedWord.startsWith('\"')) {
+        selectedWord = selectedWord.right(selectedWord.size() - 1);
+        wordSelectCursor.movePosition(QTextCursor::NextCharacter, QTextCursor::MoveAnchor);
+    }
+    if (selectedWord.endsWith('\'') || selectedWord.endsWith('\"'))
+        selectedWord.chop(1);
+
+    wordSelectCursor.movePosition(QTextCursor::NextCharacter,
+                                  QTextCursor::KeepAnchor, selectedWord.size());
+
+    const bool wordIsMisspelled = isMouseCursorInsideWord &&
+                            checkSpellingEnabled() &&
+                            !selectedWord.isEmpty() &&
+                            highlighter() &&
+                            highlighter()->isWordMisspelled(selectedWord);
+
+    // If the user clicked a selected word, do nothing.
+    // If the user clicked somewhere else, move the cursor there.
+    // If the user clicked on a misspelled word, select that word.
+    // Same behavior as in OpenOffice Writer.
+    bool inQuote = false;
+    if (d->spellInterface &&
+        !d->spellInterface->shouldBlockBeSpellChecked(cursorAtMouse.block().text()))
+        inQuote = true;
+    if (!selectedWordClicked) {
+        if (wordIsMisspelled && !inQuote)
+            setTextCursor(wordSelectCursor);
+        else
+            setTextCursor(cursorAtMouse);
+        cursor = textCursor();
+    }
+
+    // Use standard context menu for already selected words, correctly spelled
+    // words and words inside quotes.
+    if (!wordIsMisspelled || selectedWordClicked || inQuote) {
+        QMenu *popup = mousePopupMenu();
+        if ( popup ) {
+            popup->exec( event->globalPos() );
+            delete popup;
+        }
+    }
+    else {
+        QMenu menu; //don't use KMenu here we don't want auto management accelerator
+
+        //Add the suggestions to the menu
+        const QStringList reps = highlighter()->suggestionsForWord(selectedWord);
+        if (reps.isEmpty()) {
+            QAction *suggestionsAction = menu.addAction(i18n("No suggestions for %1", selectedWord));
+            suggestionsAction->setEnabled(false);
+        }
+        else {
+            for (QStringList::const_iterator it = reps.constBegin(); it != reps.constEnd(); ++it) {
+                menu.addAction(*it);
+            }
+        }
+
+        menu.addSeparator();
+
+        QAction *ignoreAction = menu.addAction(i18n("Ignore"));
+        QAction *addToDictAction = menu.addAction(i18n("Add to Dictionary"));
+        //Execute the popup inline
+        const QAction *selectedAction = menu.exec(event->globalPos());
+
+        if (selectedAction) {
+            Q_ASSERT(cursor.selectedText() == selectedWord);
+
+            if (selectedAction == ignoreAction) {
+                highlighter()->ignoreWord(selectedWord);
+                highlighter()->rehighlight();
+            }
+            else if (selectedAction == addToDictAction) {
+                highlighter()->addWordToDictionary(selectedWord);
+                highlighter()->rehighlight();
+            }
+
+            // Other actions can only be one of the suggested words
+            else {
+                const QString replacement = selectedAction->text();
+                Q_ASSERT(reps.contains(replacement));
+                cursor.insertText(replacement);
+                setTextCursor(cursor);
+            }
+        }
+    }
 }
 
 void KTextEdit::wheelEvent( QWheelEvent *event )
@@ -419,7 +672,7 @@ void KTextEdit::wheelEvent( QWheelEvent *event )
 
 void KTextEdit::createHighlighter()
 {
-    setHighlighter(new Sonnet::Highlighter(this, d->spellChechingConfigFileName));
+    setHighlighter(new Sonnet::Highlighter(this, d->spellCheckingConfigFileName));
 }
 
 Sonnet::Highlighter* KTextEdit::highlighter() const
@@ -433,7 +686,15 @@ void KTextEdit::setHighlighter(Sonnet::Highlighter *_highLighter)
     d->highlighter = _highLighter;
 }
 
-void KTextEdit::setCheckSpellingEnabled( bool check )
+void KTextEdit::setCheckSpellingEnabled(bool check)
+{
+    if (d->spellInterface)
+        d->spellInterface->setSpellCheckingEnabled(check);
+    else
+        setCheckSpellingEnabledInternal(check);
+}
+
+void KTextEdit::setCheckSpellingEnabledInternal( bool check )
 {
   emit checkSpellingChanged( check );
   if ( check == d->checkSpellingEnabled )
@@ -448,6 +709,8 @@ void KTextEdit::setCheckSpellingEnabled( bool check )
     {
         if ( hasFocus() ) {
             createHighlighter();
+            if (!spellCheckingLanguage().isEmpty())
+                setSpellCheckingLanguage(spellCheckingLanguage());
         }
     }
     else
@@ -462,10 +725,21 @@ void KTextEdit::focusInEvent( QFocusEvent *event )
   if ( d->checkSpellingEnabled && !isReadOnly() && !d->highlighter )
     createHighlighter();
 
+  if (!d->clickMessage.isEmpty()) {
+      d->updateClickMessageRect();
+  }
   QTextEdit::focusInEvent( event );
 }
 
 bool KTextEdit::checkSpellingEnabled() const
+{
+    if (d->spellInterface)
+      return d->spellInterface->isSpellCheckingEnabled();
+    else
+      return checkSpellingEnabledInternal();
+}
+
+bool KTextEdit::checkSpellingEnabledInternal() const
 {
   return d->checkSpellingEnabled;
 }
@@ -531,6 +805,8 @@ void KTextEdit::checkSpelling()
   connect(spellDialog, SIGNAL(languageChanged(const QString &)),
           this, SIGNAL(languageChanged(const QString &)));
   d->originalBuffer = toPlainText();
+  if(acceptRichText ())
+    d->originalHtml = toHtml();
   spellDialog->setBuffer(d->originalBuffer);
   spellDialog->show();
 }
@@ -566,6 +842,13 @@ void KTextEdit::slotDoReplace()
         return;
     }
 
+    if(d->repDlg->pattern().isEmpty()) {
+	delete d->replace;
+        d->replace = 0;
+        ensureCursorVisible();
+        return;
+    }
+
     delete d->replace;
     d->replace = new KReplace(d->repDlg->pattern(), d->repDlg->replacement(), d->repDlg->options(), this);
     d->repIndex = 0;
@@ -623,7 +906,12 @@ void KTextEdit::slotDoFind()
         // Should really assert()
         return;
     }
-
+    if( d->findDlg->pattern().isEmpty())
+    {
+        delete d->find;
+        d->find = 0;
+        return;
+    }
     delete d->find;
     d->find = new KFind(d->findDlg->pattern(), d->findDlg->options(), this);
     d->findIndex = 0;
@@ -647,6 +935,13 @@ void KTextEdit::slotFindNext()
 {
     if (!d->find)
         return;
+    if(document()->isEmpty())
+    {
+        d->find->disconnect(this);
+        d->find->deleteLater(); // we are in a slot connected to m_find, don't delete right away
+        d->find = 0;
+        return;
+    }
 
     KFind::Result res = KFind::NoMatch;
     if (d->find->needData())
@@ -700,6 +995,11 @@ void KTextEdit::enableFindReplace( bool enabled )
     d->findReplaceEnabled = enabled;
 }
 
+void KTextEdit::setSpellInterface(KTextEditSpellInterface *spellInterface)
+{
+    d->spellInterface = spellInterface;
+}
+
 bool KTextEdit::Private::overrideShortcut(const QKeyEvent* event)
 {
   const int key = event->key() | event->modifiers();
@@ -736,6 +1036,14 @@ bool KTextEdit::Private::overrideShortcut(const QKeyEvent* event)
     return true;
   } else if ( KStandardShortcut::pasteSelection().contains( key ) ) {
     return true;
+  } else if (KStandardShortcut::find().contains(key)) {
+      return true;
+  } else if (KStandardShortcut::findNext().contains(key)) {
+      return true;
+  } else if (KStandardShortcut::replace().contains(key)) {
+      return true;
+  } else if (event->matches(QKeySequence::SelectAll)) { // currently missing in QTextEdit
+      return true;
   } else if (event->modifiers() == Qt::ControlModifier &&
             (event->key() == Qt::Key_Return || event->key() == Qt::Key_Enter) &&
               qobject_cast<KDialog*>(parent->window()) ) {
@@ -758,4 +1066,52 @@ void KTextEdit::keyPressEvent( QKeyEvent *event )
     }
 }
 
+void KTextEdit::setClickMessage(const QString &msg)
+{
+    if (msg != d->clickMessage) {
+        if (!d->clickMessage.isEmpty()) {
+            d->updateClickMessageRect();
+        }
+        d->clickMessage = msg;
+        if (!d->clickMessage.isEmpty()) {
+            d->updateClickMessageRect();
+        }
+    }
+}
+
+QString KTextEdit::clickMessage() const
+{
+    return d->clickMessage;
+}
+
+void KTextEdit::paintEvent(QPaintEvent *ev)
+{
+    QTextEdit::paintEvent(ev);
+
+    if (!d->clickMessage.isEmpty() && !hasFocus() && document()->isEmpty()) {
+        QPainter p(viewport());
+
+        QFont f = font();
+        f.setItalic(d->italicizePlaceholder);
+        p.setFont(f);
+
+        QColor color(palette().color(foregroundRole()));
+        color.setAlphaF(0.5);
+        p.setPen(color);
+
+        int margin = int(document()->documentMargin());
+        QRect cr = viewport()->rect().adjusted(margin, margin, -margin, -margin);
+
+        p.drawText(cr, Qt::AlignTop | Qt::TextWordWrap, d->clickMessage);
+    }
+}
+
+void KTextEdit::focusOutEvent(QFocusEvent *ev)
+{
+    if (!d->clickMessage.isEmpty()) {
+        d->updateClickMessageRect();
+    }
+    QTextEdit::focusOutEvent(ev);
+}
+
 #include "ktextedit.moc"
