@@ -49,6 +49,8 @@ using namespace KXMLGUI;
 class KXMLGUIFactoryPrivate : public BuildState
 {
 public:
+    enum ShortcutOption { SetActiveShortcut = 1, SetDefaultShortcut = 2};
+
     KXMLGUIFactoryPrivate()
     {
         static const QString &defaultMergingName = KGlobal::staticQString( "<default>" );
@@ -75,11 +77,21 @@ public:
         BuildState::operator=( m_stateStack.pop() );
     }
 
+    bool emptyState() const { return m_stateStack.isEmpty(); }
+
     QWidget *findRecursive( KXMLGUI::ContainerNode *node, bool tag );
     QList<QWidget*> findRecursive( KXMLGUI::ContainerNode *node, const QString &tagName );
-    void applyActionProperties( const QDomElement &element );
-    void configureAction( QAction *action, const QDomNamedNodeMap &attributes );
-    void configureAction( QAction *action, const QDomAttr &attribute );
+    void applyActionProperties( const QDomElement &element,
+        ShortcutOption shortcutOption = KXMLGUIFactoryPrivate::SetActiveShortcut );
+    void configureAction( QAction *action, const QDomNamedNodeMap &attributes,
+        ShortcutOption shortcutOption = KXMLGUIFactoryPrivate::SetActiveShortcut );
+    void configureAction( QAction *action, const QDomAttr &attribute,
+        ShortcutOption shortcutOption = KXMLGUIFactoryPrivate::SetActiveShortcut );
+
+    QDomDocument shortcutSchemeDoc(KXMLGUIClient *client);
+    void applyShortcutScheme(KXMLGUIClient *client, QDomDocument scheme);
+    void refreshActionProperties(KXMLGUIClient *client, QDomDocument doc);
+    void saveDefaultActionProperties(KXMLGUIClient *client);
 
     ContainerNode *m_rootNode;
 
@@ -104,22 +116,22 @@ public:
 
 QString KXMLGUIFactory::readConfigFile( const QString &filename, const KComponentData &_componentData )
 {
-    KComponentData componentData = _componentData.isValid() ? _componentData : KGlobal::mainComponent();
     QString xml_file;
 
     if (!QDir::isRelativePath(filename))
         xml_file = filename;
     else
     {
+        KComponentData componentData = _componentData.isValid() ? _componentData : KGlobal::mainComponent();
         xml_file = KStandardDirs::locate("data", componentData.componentName() + '/' + filename);
         if ( !QFile::exists( xml_file ) )
           xml_file = KStandardDirs::locate( "data", filename );
     }
 
     QFile file( xml_file );
-    if ( !file.open( QIODevice::ReadOnly ) )
+    if ( xml_file.isEmpty() || !file.open( QIODevice::ReadOnly ) )
     {
-        kError(240) << "No such XML file " << filename << endl;
+        kError(240) << "No such XML file" << filename;
         return QString();
     }
 
@@ -137,9 +149,9 @@ bool KXMLGUIFactory::saveConfigFile( const QDomDocument& doc,
         xml_file = KStandardDirs::locateLocal("data", componentData.componentName() + '/' + filename);
 
     QFile file( xml_file );
-    if ( !file.open( QIODevice::WriteOnly ) )
+    if ( xml_file.isEmpty() || !file.open( QIODevice::WriteOnly ) )
     {
-        kError(240) << "Could not write to " << filename << endl;
+        kError(240) << "Could not write to" << filename;
         return false;
     }
 
@@ -195,8 +207,6 @@ KXMLGUIFactory::~KXMLGUIFactory()
 void KXMLGUIFactory::addClient( KXMLGUIClient *client )
 {
     //kDebug(260) << client;
-    static const QString &actionPropElementName = KGlobal::staticQString( "ActionProperties" );
-
     if ( client->factory() ) {
         if ( client->factory() == this )
             return;
@@ -204,6 +214,8 @@ void KXMLGUIFactory::addClient( KXMLGUIClient *client )
             client->factory()->removeClient( client ); //just in case someone does stupid things ;-)
     }
 
+    if (d->emptyState())
+        emit makingChanges(true);
     d->pushState();
 
 //    QTime dt; dt.start();
@@ -248,14 +260,9 @@ void KXMLGUIFactory::addClient( KXMLGUIClient *client )
         d->clientBuilderCustomTags.clear();
     }
 
-    // process a possibly existing actionproperties section
-
-    QDomElement actionPropElement = docElement.namedItem( actionPropElementName ).toElement();
-    if ( actionPropElement.isNull() )
-        actionPropElement = docElement.namedItem( actionPropElementName.toLower() ).toElement();
-
-    if ( !actionPropElement.isNull() )
-        d->applyActionProperties( actionPropElement );
+    // load shortcut schemes, user-defined shortcuts and other action properties
+    d->saveDefaultActionProperties(client);
+    d->refreshActionProperties(client, doc);
 
     BuildHelper( *d, d->m_rootNode ).build( docElement );
 
@@ -281,6 +288,8 @@ void KXMLGUIFactory::addClient( KXMLGUIClient *client )
     foreach (KXMLGUIClient *child, client->childClients())
         addClient( child );
 
+    if (d->emptyState())
+        emit makingChanges(false);
 /*
     QString unaddedActions;
     foreach (KActionCollection* ac, KActionCollection::allCollections())
@@ -295,6 +304,86 @@ void KXMLGUIFactory::addClient( KXMLGUIClient *client )
 //    kDebug() << "addClient took " << dt.elapsed();
 }
 
+void KXMLGUIFactory::refreshActionProperties()
+{
+    foreach (KXMLGUIClient *client, d->m_clients)
+    {
+        d->guiClient = client;
+        QDomDocument doc = client->xmlguiBuildDocument();
+        if ( doc.documentElement().isNull() )
+        {
+            client->reloadXML();
+            doc = client->domDocument();
+        }
+        d->refreshActionProperties(client, doc);
+    }
+    d->guiClient = 0;
+}
+
+void KXMLGUIFactoryPrivate::refreshActionProperties(KXMLGUIClient *client, QDomDocument doc)
+{
+    // try to find and apply shortcuts schemes
+    QDomDocument scheme = shortcutSchemeDoc(client);
+    applyShortcutScheme(client, scheme);
+
+    // try to find and apply user-defined shortcuts
+    QDomElement actionPropElement = KXMLGUIFactory::actionPropertiesElement(doc);
+    if (actionPropElement.childNodes().isEmpty())
+        doc.documentElement().removeChild(actionPropElement);
+
+    if ( !actionPropElement.isNull() )
+        applyActionProperties( actionPropElement );
+}
+
+void KXMLGUIFactoryPrivate::saveDefaultActionProperties(KXMLGUIClient *client)
+{
+    // This method is called every time the user activated a new
+    // kxmlguiclient. We only want to execute the following code only once in
+    // the lifetime of an action.
+    foreach (QAction *action, client->actionCollection()->actions())
+    {
+        // Skip actions we have seen already.
+        if (action->property("_k_DefaultShortcut").isValid()) continue;
+
+        if (KAction* kaction = qobject_cast<KAction*>(action))
+        {
+            // Check if the default shortcut is set
+            KShortcut defaultShortcut = kaction->shortcut(KAction::DefaultShortcut);
+            KShortcut activeShortcut  = kaction->shortcut(KAction::ActiveShortcut);
+
+            // Check if we have an empty default shortcut and an non empty
+            // custom shortcut. This should only happen if a developer called
+            // QAction::setShortcut on an KAction. Print out a warning and
+            // correct the mistake
+            if ((!activeShortcut.isEmpty()) && defaultShortcut.isEmpty())
+            {
+                kError(240) << "Shortcut for KAction " << kaction->objectName() << kaction->text() << "set with QShortcut::setShortcut()! See KAction documentation.";
+                kaction->setProperty("_k_DefaultShortcut", activeShortcut);
+            }
+            else
+            {
+                kaction->setProperty("_k_DefaultShortcut", defaultShortcut);
+            }
+        }
+        else
+        {
+            // A QAction used with KXMLGUI? Set our property and ignore it.
+            kError(240) << "Attempt to use QAction" << action->objectName() << "with KXMLGUIFactory!";
+            action->setProperty("_k_DefaultShortcut", KShortcut());
+        }
+
+    }
+}
+
+void KXMLGUIFactory::changeShortcutScheme(const QString &scheme)
+{
+    kDebug(260) << "Changing shortcut scheme to" << scheme;
+    KConfigGroup cg = KGlobal::config()->group( "Shortcut Schemes" );
+    cg.writeEntry("Current Scheme", scheme);
+
+    refreshActionProperties();
+}
+
 void KXMLGUIFactory::removeClient( KXMLGUIClient *client )
 {
     //kDebug(260) << client;
@@ -303,6 +392,9 @@ void KXMLGUIFactory::removeClient( KXMLGUIClient *client )
     if ( !client || client->factory() != this )
         return;
 
+    if (d->emptyState())
+        emit makingChanges(true);
+
     // remove this client from our client list
     d->m_clients.removeAll( client );
 
@@ -344,6 +436,9 @@ void KXMLGUIFactory::removeClient( KXMLGUIClient *client )
 
     d->popState();
 
+    if (d->emptyState())
+        emit makingChanges(false);
+
     emit clientRemoved( client );
 }
 
@@ -470,7 +565,8 @@ void KXMLGUIFactory::unplugActionList( KXMLGUIClient *client, const QString &nam
     d->popState();
 }
 
-void KXMLGUIFactoryPrivate::applyActionProperties( const QDomElement &actionPropElement )
+void KXMLGUIFactoryPrivate::applyActionProperties( const QDomElement &actionPropElement,
+        ShortcutOption shortcutOption )
 {
     for (QDomNode n = actionPropElement.firstChild();
          !n.isNull(); n = n.nextSibling() )
@@ -483,11 +579,12 @@ void KXMLGUIFactoryPrivate::applyActionProperties( const QDomElement &actionProp
         if ( !action )
             continue;
 
-        configureAction( action, e.attributes() );
+        configureAction( action, e.attributes(), shortcutOption );
     }
 }
 
-void KXMLGUIFactoryPrivate::configureAction( QAction *action, const QDomNamedNodeMap &attributes )
+void KXMLGUIFactoryPrivate::configureAction( QAction *action, const QDomNamedNodeMap &attributes,
+        ShortcutOption shortcutOption )
 {
     for ( uint i = 0; i < attributes.length(); i++ )
     {
@@ -495,11 +592,12 @@ void KXMLGUIFactoryPrivate::configureAction( QAction *action, const QDomNamedNod
         if ( attr.isNull() )
             continue;
 
-        configureAction( action, attr );
+        configureAction( action, attr, shortcutOption );
     }
 }
 
-void KXMLGUIFactoryPrivate::configureAction( QAction *action, const QDomAttr &attribute )
+void KXMLGUIFactoryPrivate::configureAction( QAction *action, const QDomAttr &attribute,
+        ShortcutOption shortcutOption )
 {
     static const QString &attrShortcut = KGlobal::staticQString( "shortcut" );
 
@@ -528,16 +626,106 @@ void KXMLGUIFactoryPrivate::configureAction( QAction *action, const QDomAttr &at
     } else if ( propertyType == QVariant::UserType && action->property( attrName.toLatin1() ).userType() == qMetaTypeId<KShortcut>() ) {
         // Setting the shortcut by property also sets the default shortcut (which is incorrect), so we have to do it directly
         if (KAction* ka = qobject_cast<KAction*>(action)) {
-            ka->setShortcut(KShortcut(attribute.value()), KAction::ActiveShortcut);
+            if (attrName=="globalShortcut") {
+                ka->setGlobalShortcut(KShortcut(attribute.value()), KAction::ActiveShortcut);
+            } else {
+                ka->setShortcut(KShortcut(attribute.value()), KAction::ActiveShortcut);
+            }
+            if (shortcutOption & KXMLGUIFactoryPrivate::SetDefaultShortcut)
+                ka->setShortcut(KShortcut(attribute.value()), KAction::DefaultShortcut);
             return;
         }
         propertyValue = KShortcut( attribute.value() );
     } else {
         propertyValue = QVariant( attribute.value() );
     }
-    action->setProperty( attrName.toLatin1(), propertyValue );
+    if (!action->setProperty( attrName.toLatin1(), propertyValue )) {
+        kWarning() << "Error: Unknown action property " << attrName << " will be ignored!";
+    }
 }
 
+QDomDocument KXMLGUIFactoryPrivate::shortcutSchemeDoc(KXMLGUIClient *client)
+{
+    // Get the name of the current shorcut scheme
+    KConfigGroup cg = KGlobal::config()->group( "Shortcut Schemes" );
+    QString schemeName = cg.readEntry("Current Scheme", "Default");
+
+    QDomDocument doc;
+    if (schemeName != "Default")
+    {
+        // Find the document for the shortcut scheme using both current application path
+        // and current xmlguiclient path but making a preference to app path
+        QString schemeFileName = KStandardDirs::locateLocal("data",
+            client->componentData().componentName() + '/' +
+            client->componentData().componentName() + schemeName.toLower() + "shortcuts.rc" );
+
+        QFile schemeFile(schemeFileName);
+        if (schemeFile.open(QIODevice::ReadOnly))
+        {
+//             kDebug(260) << "Found shortcut scheme" << schemeFileName;
+            doc.setContent(&schemeFile);
+            schemeFile.close();
+        }
+    }
+    return doc;
+}
+
+void KXMLGUIFactoryPrivate::applyShortcutScheme(KXMLGUIClient *client, QDomDocument scheme)
+{
+    static const QString &actionPropElementName = KGlobal::staticQString( "ActionProperties" );
+
+    KConfigGroup cg = KGlobal::config()->group( "Shortcut Schemes" );
+    QString schemeName = cg.readEntry("Current Scheme", "Default");
+
+    //First clear all existing shortcuts
+    if (schemeName != "Default") {
+        foreach (QAction *action, client->actionCollection()->actions())
+        {
+            if (KAction *kaction = qobject_cast<KAction*>(action))
+            {
+                kaction->setShortcut(KShortcut(), KAction::ActiveShortcut);
+                // We clear the default shortcut as well because the shortcut scheme will set its own defaults
+                kaction->setShortcut(KShortcut(), KAction::DefaultShortcut);
+            }
+            else
+                action->setProperty("shortcut", KShortcut());
+        }
+    } else {
+        // apply saved default shortcuts
+        foreach (QAction *action, client->actionCollection()->actions())
+        {
+            if (KAction *kaction = qobject_cast<KAction*>(action))
+            {
+                QVariant savedDefaultShortcut = kaction->property("_k_DefaultShortcut");
+                if (savedDefaultShortcut.isValid())
+                {
+                    KShortcut shortcut = savedDefaultShortcut.value<KShortcut>();
+                    kaction->setShortcut(shortcut, KAction::ActiveShortcut);
+                    kaction->setShortcut(shortcut, KAction::DefaultShortcut);
+                    continue;
+                }
+            }
+            action->setProperty("shortcut", KShortcut());
+        }
+    }
+
+    if (scheme.isNull())
+        return;
+
+    QDomElement docElement = scheme.documentElement();
+    QDomElement actionPropElement = docElement.namedItem( actionPropElementName ).toElement();
+
+    //Check if we really have the shortcut configuration here
+    if (!actionPropElement.isNull())
+    {
+        kDebug(260) << "Applying shortcut scheme for XMLGUI client" << client->componentData().componentName();
+
+        //Apply all shortcuts we have
+        applyActionProperties(actionPropElement, KXMLGUIFactoryPrivate::SetDefaultShortcut);
+    }
+    else
+        kDebug(260) << "Invalid shortcut scheme file";
+}
 
 int KXMLGUIFactory::configureShortcuts(bool letterCutsOk , bool bSaveSettings )
 {
@@ -546,7 +734,7 @@ int KXMLGUIFactory::configureShortcuts(bool letterCutsOk , bool bSaveSettings )
          qobject_cast<QWidget*>(parent()));
 	foreach (KXMLGUIClient *client, d->m_clients)
 	{
-		if(!client->xmlFile().isEmpty())
+		if(client && !client->xmlFile().isEmpty())
 			dlg.addCollection( client->actionCollection() );
 	}
 	return dlg.configure(bSaveSettings);
@@ -554,13 +742,17 @@ int KXMLGUIFactory::configureShortcuts(bool letterCutsOk , bool bSaveSettings )
 
 QDomElement KXMLGUIFactory::actionPropertiesElement( QDomDocument& doc )
 {
+	KConfigGroup cg = KGlobal::config()->group( "Shortcut Schemes" );
+	QString schemeName = cg.readEntry("Current Scheme", "Default");
+
 	const QString tagActionProp = QLatin1String("ActionProperties");
 	// first, lets see if we have existing properties
 	QDomElement elem;
 	QDomNode it = doc.documentElement().firstChild();
 	for( ; !it.isNull(); it = it.nextSibling() ) {
 		QDomElement e = it.toElement();
-		if( e.tagName() == tagActionProp ) {
+		if( ((e.tagName() == tagActionProp) || (e.tagName() == tagActionProp.toLower()))
+				&& (e.attribute("scheme", "Default") == schemeName) ) {
 			elem = e;
 			break;
 		}
@@ -569,6 +761,7 @@ QDomElement KXMLGUIFactory::actionPropertiesElement( QDomDocument& doc )
 	// if there was none, create one
 	if( elem.isNull() ) {
 		elem = doc.createElement( tagActionProp );
+		elem.setAttribute( "scheme", schemeName );
 		doc.documentElement().appendChild( elem );
 	}
 	return elem;
