@@ -19,6 +19,7 @@
 */
 
 #include "kdiroperator.h"
+#include <kprotocolmanager.h>
 #include "kdirmodel.h"
 #include "kdiroperatordetailview_p.h"
 #include "kdirsortfilterproxymodel.h"
@@ -727,23 +728,47 @@ void KDirOperator::Private::_k_slotToggleIgnoreCase()
     d->sorting = d->fileView->sorting();*/
 }
 
+// Duplicated in libkonq's KonqOperations
+static bool confirmCreatingHiddenDir(const QString& name, QWidget* parent)
+{
+    KGuiItem continueGuiItem(KStandardGuiItem::cont());
+    continueGuiItem.setText(i18nc("@action:button", "Create directory"));
+    KGuiItem cancelGuiItem(KStandardGuiItem::cancel());
+    cancelGuiItem.setText(i18nc("@action:button", "Enter a different name"));
+    return KMessageBox::warningContinueCancel(
+        parent,
+        i18n("The name \"%1\" starts with a dot, so the directory will be hidden by default.", name),
+        i18n("Create hidden directory?"),
+        continueGuiItem,
+        cancelGuiItem,
+        "confirm_create_hidden_dir") == KMessageBox::Continue;
+}
+
 void KDirOperator::mkdir()
 {
     bool ok;
     QString where = url().pathOrUrl();
     QString name = i18n("New Folder");
-#ifdef Q_WS_WIN
-    if (url().isLocalFile() && QFileInfo(url().toLocalFile() + name).exists())
-#else
-    if (url().isLocalFile() && QFileInfo(url().path(KUrl::AddTrailingSlash) + name).exists())
-#endif
+    if (url().isLocalFile() && QFileInfo(url().toLocalFile(KUrl::AddTrailingSlash) + name).exists())
         name = KIO::RenameDialog::suggestName(url(), name);
 
-    QString folder = KInputDialog::getText(i18n("New Folder"),
-                                           i18n("Create new folder in:\n%1" ,  where),
-                                           name, &ok, this);
-    if (ok)
-        KDirOperator::mkdir(KIO::encodeFileName(folder), true);
+    bool askAgain;
+    do {
+        askAgain = false;
+        name = KInputDialog::getText(i18n("New Folder"),
+                                     i18n("Create new folder in:\n%1" ,  where),
+                                     name, &ok, this);
+        if (ok) {
+            if (name.startsWith('.') && !showHiddenFiles()) {
+                if (!confirmCreatingHiddenDir(name, this)) {
+                    askAgain = true;
+                    continue;
+                }
+            }
+            KDirOperator::mkdir(KIO::encodeFileName(name), true);
+            break;
+        }
+    } while (askAgain);
 }
 
 bool KDirOperator::mkdir(const QString& directory, bool enterDirectory)
@@ -1115,7 +1140,7 @@ void KDirOperator::rereadDir()
 
 bool KDirOperator::Private::openUrl(const KUrl& url, KDirLister::OpenUrlFlags flags)
 {
-    bool result = dirLister->openUrl(url, flags);
+    const bool result = KProtocolManager::supportsListing(url) && dirLister->openUrl(url, flags);
     if (!result)   // in that case, neither completed() nor canceled() will be emitted by KDL
         _k_slotCanceled();
 
@@ -2614,7 +2639,7 @@ bool KDirOperator::showHiddenFiles() const
     return d->actionCollection->action("show hidden")->isChecked();
 }
 
-QStyleOptionViewItem::Position KDirOperator::decorationPosition() const 
+QStyleOptionViewItem::Position KDirOperator::decorationPosition() const
 {
     return d->decorationPosition;
 }
