@@ -6,6 +6,7 @@
                   2000,2001 Carsten Pfeiffer <pfeiffer@kde.org>
                   2001 Frerich Raabe <raabe@kde.org>
                   2007 David Faure <faure@kde.org>
+                  2008 Rafael Fernández López <ereslibre@kde.org>
 
     This library is free software; you can redistribute it and/or
     modify it under the terms of the GNU Library General Public
@@ -33,6 +34,7 @@
 
 class KJob;
 class KFileItem;
+class KDirOperator;
 
 class KFILE_EXPORT KFileWidget : public QWidget, public KAbstractFileWidget
 {
@@ -42,29 +44,32 @@ public:
     /**
       * Constructs a file selector widget.
       *
-      * @param startDir This can either be
-      *         @li The URL of the directory to start in.
-      *         @li A KUrl() to start in the current working
-      *             directory, or the last directory where a file has been
-      *             selected.
-      *         @li An URL starting with 'kfiledialog:///&lt;keyword&gt;' to start in the
-      *             directory last used by a filedialog in the same application that specified
-      *             the same keyword.
-      *         @li An URL starting with 'kfiledialog:///&lt;keyword&gt;?global' to start
-      *             in the directory last used by a filedialog in any application that specified the
-      *             same keyword.
-      *
-      * @param filter A shell glob or a mime-type-filter that specifies
-      *               which files to display.
-      * See setFilter() and setMimeFilter() for details on how to use this argument.
+      * @param startDir This can either be:
+      *         @li An empty URL (KUrl()) to start in the current working directory,
+      *             or the last directory where a file has been selected.
+      *         @li The path or URL of a starting directory.
+      *         @li An initial file name to select, with the starting directory being
+      *             the current working directory or the last directory where a file
+      *             has been selected.
+      *         @li The path or URL of a file, specifying both the starting directory and
+      *             an initially selected file name.
+      *         @li A URL of the form @c kfiledialog:///&lt;keyword&gt; to start in the
+      *             directory last used by a filedialog in the same application that
+      *             specified the same keyword.
+      *         @li A URL of the form @c kfiledialog:///&lt;keyword&gt;/&lt;filename&gt;
+      *             to start in the directory last used by a filedialog in the same
+      *             application that specified the same keyword, and to initially
+      *             select the specified filename.
+      *         @li A URL of the form @c kfiledialog:///&lt;keyword&gt;?global to start
+      *             in the directory last used by a filedialog in any application that
+      *             specified the same keyword.
+      *         @li A URL of the form @c kfiledialog:///&lt;keyword&gt;/&lt;filename&gt;?global
+      *             to start in the directory last used by a filedialog in any
+      *             application that specified the same keyword, and to initially
+      *             select the specified filename.
       *
       * @param parent The parent widget of this widget
       *
-      * @param widget A widget, or a widget of widgets, for displaying custom
-      *               data in the file widget. This can be used, for example, to
-      *               display a check box with the caption "Open as read-only".
-      *               When creating this widget, you don't need to specify a parent,
-      *               since the widget's parent will be set automatically by KFileWidget.
       */
     KFileWidget(const KUrl& startDir, QWidget *parent);
 
@@ -167,7 +172,7 @@ public:
      *
      * You can set more
      * filters for the user to select separated by '\n'. Every
-     * filter entry is defined through namefilter|text to diplay.
+     * filter entry is defined through namefilter|text to display.
      * If no | is found in the expression, just the namefilter is
      * shown. Examples:
      *
@@ -345,16 +350,42 @@ public:
      * This method implements the logic to determine the user's default directory
      * to be listed. E.g. the documents directory, home directory or a recently
      * used directory.
-     * @param startDir A url, to be used. May use the 'kfiledialog:///keyword' and
-     *                 'kfiledialog:///keyword?global' syntax
-     *                 as documented in the KFileDialog() constructor.
-     * @param recentDirClass If the 'kfiledialog:///' syntax is used, recentDirClass
-     *        will contain the string to be used later for KRecentDir::dir()
+     * @param startDir A URL specifying the initial directory, or using the
+     *                 @c kfiledialog:/// syntax to specify a last used
+     *                 directory.  If this URL specifies a file name, it is
+     *                 ignored.  Refer to the KFileWidget::KFileWidget()
+     *                 documentation for the @c kfiledialog:/// URL syntax.
+     * @param recentDirClass If the @c kfiledialog:/// syntax is used, this
+     *        will return the string to be passed to KRecentDirs::dir() and
+     *        KRecentDirs::add().
      * @return The URL that should be listed by default (e.g. by KFileDialog or
      *         KDirSelectDialog).
+     * @see KFileWidget::KFileWidget()
      */
     static KUrl getStartUrl( const KUrl& startDir, QString& recentDirClass );
 
+    /**
+     * Similar to getStartUrl(const KUrl& startDir,QString& recentDirClass),
+     * but allows both the recent start directory keyword and a suggested file name
+     * to be returned.
+     * @param startDir A URL specifying the initial directory and/or filename,
+     *                 or using the @c kfiledialog:/// syntax to specify a
+     *                 last used location.
+     *                 Refer to the KFileWidget::KFileWidget()
+     *                 documentation for the @c kfiledialog:/// URL syntax.
+     * @param recentDirClass If the @c kfiledialog:/// syntax is used, this
+     *        will return the string to be passed to KRecentDirs::dir() and
+     *        KRecentDirs::add().
+     * @param fileName The suggested file name, if specified as part of the
+     *        @p StartDir URL.
+     * @return The URL that should be listed by default (e.g. by KFileDialog or
+     *         KDirSelectDialog).
+     *
+     * @see KFileWidget::KFileWidget()
+     * @since 4.3
+     */
+    static KUrl getStartUrl( const KUrl& startDir, QString& recentDirClass, QString& fileName );
+
     /**
      * @internal
      * Used by KDirSelectDialog to share the dialog's start directory.
@@ -396,7 +427,9 @@ public Q_SLOTS:
     virtual void slotCancel();
 
 protected:
+    virtual void resizeEvent(QResizeEvent* event);
     virtual void showEvent(QShowEvent* event);
+    virtual bool eventFilter(QObject* watched, QEvent* event);
 
 Q_SIGNALS:
     /**
@@ -405,13 +438,25 @@ Q_SIGNALS:
       * is to connect to the okClicked() signal inherited from KDialog
       * and call selectedFile(), selectedFiles(),
       * selectedUrl() or selectedUrls().
+      *
+      * \since 4.4
       */
-    void fileSelected(const QString&);
+    void fileSelected(const KUrl&);
 
     /**
-      * Emitted when the user highlights a file.
-      */
-    void fileHighlighted(const QString&);
+     * @deprecated, connect to fileSelected(const KUrl&) instead
+     */
+    QT_MOC_COMPAT void fileSelected(const QString&); // TODO KDE5: remove
+
+    /**
+     * Emitted when the user highlights a file.
+     * \since 4.4
+     */
+    void fileHighlighted(const KUrl&);
+    /**
+     * @deprecated, connect to fileSelected(const KUrl&) instead
+     */
+    QT_MOC_COMPAT void fileHighlighted(const QString&); // TODO KDE5: remove
 
     /**
      * Emitted when the user hilights one or more files in multiselection mode.
@@ -441,24 +486,44 @@ Q_SIGNALS:
      */
     void accepted();
 
+public:
+    /**
+     * @returns the KDirOperator used to navigate the filesystem
+     * @since 4.3
+     */
+    KDirOperator* dirOperator();
+
+    /**
+     * reads the configuration for this widget from the given config group
+     * @param group the KConfiGroup to read from
+     * @since 4.4
+     */
+    void readConfig( KConfigGroup& group );
+
 private:
     friend class KFileWidgetPrivate;
     KFileWidgetPrivate* const d;
 
-    Q_PRIVATE_SLOT( d, void _k_slotLocationChanged( const QString& ) )
-    Q_PRIVATE_SLOT( d, void _k_urlEntered( const KUrl& ) )
-    Q_PRIVATE_SLOT( d, void _k_enterUrl( const KUrl& ) )
-    Q_PRIVATE_SLOT( d, void _k_enterUrl( const QString& ) )
-    Q_PRIVATE_SLOT( d, void _k_locationAccepted( const QString& ) )
-    Q_PRIVATE_SLOT( d, void _k_slotFilterChanged() )
-    Q_PRIVATE_SLOT( d, void _k_fileHighlighted( const KFileItem& ) )
-    Q_PRIVATE_SLOT( d, void _k_fileSelected( const KFileItem& ) )
-    Q_PRIVATE_SLOT( d, void _k_slotStatResult( KJob* ) )
-    Q_PRIVATE_SLOT( d, void _k_slotLoadingFinished() )
-    Q_PRIVATE_SLOT( d, void _k_fileCompletion( const QString& ) )
-    Q_PRIVATE_SLOT( d, void _k_toggleSpeedbar( bool ) )
-    Q_PRIVATE_SLOT( d, void _k_toggleBookmarks( bool ) )
-    Q_PRIVATE_SLOT( d, void _k_slotAutoSelectExtClicked() )
+    Q_PRIVATE_SLOT(d, void _k_slotLocationChanged(const QString&))
+    Q_PRIVATE_SLOT(d, void _k_urlEntered(const KUrl&))
+    Q_PRIVATE_SLOT(d, void _k_enterUrl(const KUrl&))
+    Q_PRIVATE_SLOT(d, void _k_enterUrl(const QString&))
+    Q_PRIVATE_SLOT(d, void _k_locationAccepted(const QString&))
+    Q_PRIVATE_SLOT(d, void _k_slotFilterChanged())
+    Q_PRIVATE_SLOT(d, void _k_fileHighlighted(const KFileItem&))
+    Q_PRIVATE_SLOT(d, void _k_fileSelected(const KFileItem&))
+    Q_PRIVATE_SLOT(d, void _k_slotLoadingFinished())
+    Q_PRIVATE_SLOT(d, void _k_fileCompletion(const QString&))
+    Q_PRIVATE_SLOT(d, void _k_toggleSpeedbar(bool))
+    Q_PRIVATE_SLOT(d, void _k_toggleBookmarks(bool))
+    Q_PRIVATE_SLOT(d, void _k_slotAutoSelectExtClicked())
+    Q_PRIVATE_SLOT(d, void _k_placesViewSplitterMoved(int, int))
+    Q_PRIVATE_SLOT(d, void _k_activateUrlNavigator())
+    Q_PRIVATE_SLOT(d, void _k_zoomOutIconsSize())
+    Q_PRIVATE_SLOT(d, void _k_zoomInIconsSize())
+    Q_PRIVATE_SLOT(d, void _k_slotIconSizeSliderMoved(int))
+    Q_PRIVATE_SLOT(d, void _k_slotIconSizeChanged(int))
+    Q_PRIVATE_SLOT(d, void _k_slotViewDoubleClicked(const QModelIndex&))
 };
 
 #endif /* KABSTRACTFILEWIDGET_H */
