@@ -1,5 +1,6 @@
 /*
  * Copyright (C) 2007, 2008 Apple Inc. All rights reserved.
+ *           (C) 2009 Michael Howell <mhowell123@gmail.com>.
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions
@@ -28,9 +29,15 @@
 #include "HTMLMediaElement.h"
 #include "HTMLDocument.h"
 #include <misc/htmlhashes.h>
-#include <rendering/render_object.h>
 #include "MediaError.h"
 #include "TimeRanges.h"
+#include "css/cssstyleselector.h"
+#include "css/cssproperties.h"
+#include "css/cssvalues.h"
+#include "css/csshelper.h"
+#include <phonon/mediaobject.h>
+#include <phonon/backendcapabilities.h>
+#include <rendering/render_media.h>
 
 const double doubleMax = 999999999.8; // ### numeric_limits<double>::max()
 const double doubleInf = 999999999.0; // ### numeric_limits<double>::infinity()
@@ -42,12 +49,12 @@ namespace khtml {
 HTMLMediaElement::HTMLMediaElement(Document* doc)
     : HTMLElement(doc)
     , m_defaultPlaybackRate(1.0f)
-    , m_networkState(EMPTY)
-    , m_readyState(DATA_UNAVAILABLE)
+    , m_networkState(NETWORK_EMPTY)
+    , m_readyState(HAVE_NOTHING)
     , m_begun(false)
     , m_loadedFirstFrame(false)
     , m_autoplaying(true)
-    , m_currentLoop(0)
+    , m_autobuffer(true)
     , m_volume(0.5f)
     , m_muted(false)
     , m_paused(true)
@@ -56,13 +63,37 @@ HTMLMediaElement::HTMLMediaElement(Document* doc)
     , m_previousProgress(0)
     , m_previousProgressTime(doubleMax)
     , m_sentStalledEvent(false)
-    , m_bufferingRate(0)
-    , m_player(0)
+    , m_player(new MediaPlayer())
 {
 }
 
+void HTMLMediaElement::attach()
+{
+    assert(!attached());
+    assert(!m_render);
+    assert(parentNode());
+
+    RenderStyle* _style = document()->styleSelector()->styleForElement(this);
+    _style->ref();
+    if (parentNode()->renderer() && parentNode()->renderer()->childAllowed() &&
+        _style->display() != NONE)
+    {
+        m_render = new (document()->renderArena()) RenderMedia(this);
+        static_cast<RenderMedia*>(m_render)->setPlayer(m_player.data());
+        m_render->setStyle(_style);
+        parentNode()->renderer()->addChild(m_render, nextRenderer());
+    }
+    _style->deref();
+
+    NodeBaseImpl::attach();
+    if (m_render)
+        m_render->updateFromElement();
+    setRenderer(m_render);
+}
+
 HTMLMediaElement::~HTMLMediaElement()
 {
+    if (m_player) m_player->deleteLater();
 }
 
 void HTMLMediaElement::attributeChanged(NodeImpl::Id attrId)
@@ -72,13 +103,13 @@ void HTMLMediaElement::attributeChanged(NodeImpl::Id attrId)
     if (attrId == ATTR_SRC) {
         // 3.14.9.2.
         // change to src attribute triggers load()
-        if (inDocument() && m_networkState == EMPTY)
+        if (inDocument() && m_networkState == NETWORK_EMPTY)
             scheduleLoad();
     } if (attrId == ATTR_CONTROLS) {
-        if (!isVideo() && attached() && (controls() != (renderer() != 0))) {
+        /*if (!isVideo() && attached() && (controls() != (renderer() != 0))) {
             detach();
             attach();
-        }
+        }*/
         if (renderer())
             renderer()->updateFromElement();
     }
@@ -86,7 +117,7 @@ void HTMLMediaElement::attributeChanged(NodeImpl::Id attrId)
 
 void HTMLMediaElement::scheduleLoad()
 {
-    // ###
+    kDebug() << "not implemented";
 }
 
 String serializeTimeOffset(float time)
@@ -97,38 +128,6 @@ String serializeTimeOffset(float time)
     return timeString;
 }
 
-float parseTimeOffset(const String& timeString, bool* ok = 0)
-{
-#if 0
-    const UChar* characters = timeString.characters();
-    unsigned length = timeString.length();
-    
-    if (length && characters[length - 1] == 's')
-        length--;
-    
-    // FIXME parse time offset values (format not specified yet)
-    float val = charactersToFloat(characters, length, ok);
-    return val;
-#else
-    return timeString.string().toFloat(ok);
-#endif
-}
-
-float HTMLMediaElement::getTimeOffsetAttribute(NodeImpl::Id name, float valueOnError) const
-{
-    bool ok;
-    String timeString = getAttribute(name);
-    float result = parseTimeOffset(timeString, &ok);
-    if (ok)
-        return result;
-    return valueOnError;
-}
-
-void HTMLMediaElement::setTimeOffsetAttribute(NodeImpl::Id name, float value)
-{
-    setAttribute(name, serializeTimeOffset(value));
-}
-
 PassRefPtr<MediaError> HTMLMediaElement::error() const 
 {
     return m_error;
@@ -154,11 +153,14 @@ HTMLMediaElement::NetworkState HTMLMediaElement::networkState() const
     return m_networkState;
 }
 
-float HTMLMediaElement::bufferingRate()
+bool HTMLMediaElement::autobuffer() const
 {
-    if (!m_player)
-        return 0;
-    return m_bufferingRate;
+   return m_autobuffer;
+}
+
+void HTMLMediaElement::setAutobuffer(bool b)
+{
+    m_autobuffer = b;
 }
 
 void HTMLMediaElement::load(ExceptionCode&)
@@ -166,6 +168,27 @@ void HTMLMediaElement::load(ExceptionCode&)
     // ###
 }
 
+String HTMLMediaElement::canPlayType(String type)
+{
+    QString theType = type.string().simplified();
+    int paramsIdx = theType.indexOf(';');
+    bool hasParams = (paramsIdx > 0 );
+    // FIXME: Phonon doesn't provide the API to handle codec parameters yet
+    if (hasParams)
+        theType.truncate(paramsIdx);
+    while (1) {
+        if (Phonon::BackendCapabilities::isMimeTypeAvailable(theType))
+            return "probably";
+        if (theType == QLatin1String("audio/ogg") || theType == QLatin1String("video/ogg"))
+            theType = QLatin1String("application/ogg");
+        else
+            break;
+    }
+    if (theType == QLatin1String("application/octet-stream") && hasParams)
+        return "";
+    return "maybe";
+}
+
 void HTMLMediaElement::setReadyState(ReadyState state)
 {
     // 3.14.9.6. The ready states
@@ -202,9 +225,14 @@ void HTMLMediaElement::setCurrentTime(float time, ExceptionCode& ec)
     //    seek(time, ec);
 }
 
+float HTMLMediaElement::startTime() const
+{
+    return 0.0f;
+}
+
 float HTMLMediaElement::duration() const
 {
-    return m_player ? m_player->duration() : 0;
+    return m_player ? m_player->totalTime() : 0;
 }
 
 bool HTMLMediaElement::paused() const
@@ -231,11 +259,13 @@ void HTMLMediaElement::setDefaultPlaybackRate(float rate, ExceptionCode& ec)
 
 float HTMLMediaElement::playbackRate() const
 {
-    return m_player ? m_player->rate() : 0;
+    return 0; // stub...
 }
 
 void HTMLMediaElement::setPlaybackRate(float rate, ExceptionCode& ec)
 {
+    // stub
+    #if 0
     if (rate == 0.0f) {
         ec = DOMException::NOT_SUPPORTED_ERR;
         return;
@@ -244,6 +274,7 @@ void HTMLMediaElement::setPlaybackRate(float rate, ExceptionCode& ec)
         m_player->setRate(rate);
         // ### dispatchEventAsync(ratechangeEvent);
     }
+    #endif
 }
 
 bool HTMLMediaElement::ended() const
@@ -258,13 +289,23 @@ bool HTMLMediaElement::autoplay() const
 
 void HTMLMediaElement::setAutoplay(bool b)
 {
-    //   setBooleanAttribute(ATTR_AUTOPLAY, b);
+    setBooleanAttribute(ATTR_AUTOPLAY, b);
+}
+
+bool HTMLMediaElement::loop() const
+{
+    return hasAttribute(ATTR_LOOP);
+}
+
+void HTMLMediaElement::setLoop(bool b)
+{
+    setBooleanAttribute(ATTR_LOOP, b);
 }
 
 void HTMLMediaElement::play(ExceptionCode& ec)
 {
     // 3.14.9.7. Playing the media resource
-    if (!m_player || networkState() == EMPTY) {
+    if (!m_player || networkState() == NETWORK_EMPTY) {
         ec = 0;
         load(ec);
         if (ec)
@@ -272,7 +313,6 @@ void HTMLMediaElement::play(ExceptionCode& ec)
     }
     ExceptionCode unused;
     if (endedPlayback()) {
-        m_currentLoop = 0;
         // ### seek(effectiveStart(), unused);
     }
     setPlaybackRate(defaultPlaybackRate(), unused);
@@ -290,7 +330,7 @@ void HTMLMediaElement::play(ExceptionCode& ec)
 void HTMLMediaElement::pause(ExceptionCode& ec)
 {
     // 3.14.9.7. Playing the media resource
-    if (!m_player || networkState() == EMPTY) {
+    if (!m_player || networkState() == NETWORK_EMPTY) {
         ec = 0;
         load(ec);
         if (ec)
@@ -308,77 +348,6 @@ void HTMLMediaElement::pause(ExceptionCode& ec)
     updatePlayState();
 }
 
-unsigned HTMLMediaElement::playCount() const
-{
-    String val = getAttribute(ATTR_PLAYCOUNT);
-    int count = val.toInt();
-    return qMax(count, 1); 
-}
-
-void HTMLMediaElement::setPlayCount(unsigned count, ExceptionCode& ec)
-{
-    if (!count) {
-        ec = DOMException::INDEX_SIZE_ERR;
-        return;
-    }
-    setAttribute(ATTR_PLAYCOUNT, QString::number(count));
-    checkIfSeekNeeded();
-}
-
-float HTMLMediaElement::start() const 
-{ 
-    return getTimeOffsetAttribute(ATTR_START, 0); 
-}
-
-void HTMLMediaElement::setStart(float time) 
-{ 
-    setTimeOffsetAttribute(ATTR_START, time); 
-    checkIfSeekNeeded();
-}
-
-float HTMLMediaElement::end() const 
-{ 
-    return getTimeOffsetAttribute(ATTR_END, doubleInf); 
-}
-
-void HTMLMediaElement::setEnd(float time) 
-{ 
-    setTimeOffsetAttribute(ATTR_END, time); 
-    checkIfSeekNeeded();
-}
-
-float HTMLMediaElement::loopStart() const 
-{ 
-    return getTimeOffsetAttribute(ATTR_LOOPSTART, 0); 
-}
-
-void HTMLMediaElement::setLoopStart(float time) 
-{
-    setTimeOffsetAttribute(ATTR_LOOPSTART, time); 
-    checkIfSeekNeeded();
-}
-
-float HTMLMediaElement::loopEnd() const 
-{ 
-    return getTimeOffsetAttribute(ATTR_LOOPEND, doubleInf); 
-}
-
-void HTMLMediaElement::setLoopEnd(float time) 
-{ 
-    setTimeOffsetAttribute(ATTR_LOOPEND, time); 
-    checkIfSeekNeeded();
-}
-
-unsigned HTMLMediaElement::currentLoop() const
-{
-    return m_currentLoop;
-}
-
-void HTMLMediaElement::setCurrentLoop(unsigned currentLoop)
-{
-    m_currentLoop = currentLoop;
-}
-
 bool HTMLMediaElement::controls() const
 {
     return hasAttribute(ATTR_CONTROLS);
@@ -386,7 +355,7 @@ bool HTMLMediaElement::controls() const
 
 void HTMLMediaElement::setControls(bool b)
 {
-    // setBooleanAttribute(ATTR_CONTROLS, b);
+    setBooleanAttribute(ATTR_CONTROLS, b);
 }
 
 float HTMLMediaElement::volume() const
@@ -430,9 +399,12 @@ void HTMLMediaElement::checkIfSeekNeeded()
 PassRefPtr<TimeRanges> HTMLMediaElement::buffered() const
 {
     // FIXME real ranges support
+    #if 0
     if (!m_player || !m_player->maxTimeBuffered())
         return new TimeRanges;
     return new TimeRanges(0, m_player->maxTimeBuffered());
+    #endif
+    return new TimeRanges(0, 0.0f); // stub
 }
 
 PassRefPtr<TimeRanges> HTMLMediaElement::played() const
@@ -443,10 +415,13 @@ PassRefPtr<TimeRanges> HTMLMediaElement::played() const
 
 PassRefPtr<TimeRanges> HTMLMediaElement::seekable() const
 {
+    #if 0
     // FIXME real ranges support
     if (!m_player || !m_player->maxTimeSeekable())
         return new TimeRanges;
     return new TimeRanges(0, m_player->maxTimeSeekable());
+    #endif
+    return new TimeRanges(0, 0.0f); // stub
 }
 
 bool HTMLMediaElement::endedPlayback() const
@@ -454,7 +429,7 @@ bool HTMLMediaElement::endedPlayback() const
 #if 0
     return networkState() >= LOADED_METADATA && currentTime() >= effectiveEnd() && currentLoop() == playCount() - 1;
 #endif
-    return false;
+    return m_player->mediaObject()->remainingTime() == 0;
 }
 
 void HTMLMediaElement::updateVolume()
