@@ -30,9 +30,6 @@
 #include <time.h>
 
 #include "loader_client.h"
-#ifdef HAVE_LIBJPEG
-#include "loader_jpeg.h"
-#endif
 
 #include <stdlib.h>
 #include <QtCore/QObject>
@@ -42,7 +39,6 @@
 #include <QtCore/QTextCodec>
 #include <QtCore/QTimer>
 #include <QtCore/QSet>
-
 #include <QLinkedList>
 
 #include <kurl.h>
@@ -86,7 +82,8 @@ namespace khtml
 	    Image,
 	    CSSStyleSheet,
 	    Script,
-	    Sound
+	    Sound,
+	    Font
 	};
 
 	enum Status {
@@ -235,6 +232,7 @@ namespace khtml
         virtual bool schedule() const { return true; }
         void setCharsetHint( const QString& charset ) { m_charsetHint = charset; }
         void setCharset( const QString& charset ) { m_charset = charset; }
+        QString checkCharset(const QByteArray& buffer ) const;
 
 
     protected:
@@ -363,7 +361,6 @@ namespace khtml
 	bool typeChecked : 1;
         bool isFullyTransparent : 1;
         bool monochrome : 1;
-        KHTMLSettings::KAnimationAdvice m_showAnimations : 2;
 
         friend class Cache;
         friend class ::KHTMLPart;
@@ -392,6 +389,29 @@ namespace khtml
 	QByteArray m_sound;
     };
 
+    /**
+     * a cached font
+     */
+    class CachedFont : public CachedObject
+    {
+    public:
+	CachedFont(DocLoader* dl, const DOM::DOMString &url, KIO::CacheControl cachePolicy, const char* accept );
+
+	QByteArray font() const { return m_font; }
+
+	virtual void ref(CachedObjectClient *consumer);
+	virtual void data( QBuffer &buffer, bool eof );
+	virtual void error( int err, const char *text );
+        virtual bool schedule() const { return false; }
+
+	void checkNotify();
+
+        bool isLoaded() const { return !m_loading; }
+
+    protected:
+	QByteArray m_font;
+    };
+
     /**
      * @internal
      *
@@ -408,6 +428,8 @@ namespace khtml
 						const char *accept = "text/css", bool userSheet = false );
         CachedScript *requestScript( const DOM::DOMString &url, const QString& charset);
         CachedSound *requestSound( const DOM::DOMString &url );
+        CachedFont *requestFont( const DOM::DOMString &url );
+        bool willLoadMediaElement( const DOM::DOMString &url);
 
 	bool autoloadImages() const { return m_bautoloadImages; }
         KIO::CacheControl cachePolicy() const { return m_cachePolicy; }
@@ -474,7 +496,7 @@ namespace khtml
 	Loader();
 	~Loader();
 
-	void load(DocLoader* dl, CachedObject *object, bool incremental = true);
+	void load(DocLoader* dl, CachedObject *object, bool incremental = true, bool highPriority = false);
 
         int numRequests( DocLoader* dl ) const;
         void cancelRequests( DocLoader* dl );
@@ -495,10 +517,8 @@ namespace khtml
 
     protected:
 	QLinkedList<Request*> m_requestsPending;
+	Request* m_highPriorityRequestPending;
 	QHash<KIO::Job*,Request*> m_requestsLoading;
-#ifdef HAVE_LIBJPEG
-        // TODO KJPEGFormatType m_jpegloader;
-#endif
         QTimer m_timer;     
     };
 
