@@ -39,6 +39,7 @@
 #include "misc/loader_client.h"
 #include "misc/helper.h"
 #include "rendering/render_style.h"
+#include "rendering/render_position.h"
 #include <QtCore/QTextIStream>
 
 // svg
@@ -116,7 +117,6 @@ namespace DOM {
     class DocumentImpl;
     class ElementImpl;
     class EventImpl;
-    class Position;
     class Selection;
 }
 
@@ -134,6 +134,7 @@ namespace khtml {
     class InlineBox;
     class InlineFlowBox;
     class CounterNode;
+    class RenderPosition;
 
 /**
  * Base Class for all rendering tree objects.
@@ -156,8 +157,8 @@ public:
     virtual RenderObject *firstChild() const { return 0; }
     virtual RenderObject *lastChild() const { return 0; }
 
-    RenderObject *nextRenderer() const; 
-    RenderObject *previousRenderer() const; 
+    RenderObject *nextRenderer() const;
+    RenderObject *previousRenderer() const;
 
     RenderObject *nextEditable() const;
     RenderObject *previousEditable() const;
@@ -243,6 +244,7 @@ public:
     QString information() const;
     virtual void printTree(int indent=0) const;
     virtual void dump(QTextStream &stream, const QString &ind = QString()) const;
+    void printLineBoxTree() const;
 #endif
 
     static RenderObject *createObject(DOM::NodeImpl* node, RenderStyle* style);
@@ -294,6 +296,7 @@ public:
     virtual bool isFormElement() const { return false; }
     virtual bool isFrameSet() const { return false; }
     virtual bool isApplet() const { return false; }
+    virtual bool isMedia() const { return false; }
 
     virtual bool isEditable() const;
 
@@ -346,7 +349,7 @@ public:
     DOM::DocumentImpl* document() const;
     DOM::NodeImpl* element() const { return isAnonymous() ? 0L : m_node; }
     DOM::NodeImpl* node() const { return m_node; }
-    
+
     virtual bool handleEvent(const DOM::EventImpl&) { return false; }
 
    /**
@@ -580,7 +583,8 @@ public:
     virtual bool nodeAtPoint(NodeInfo& info, int x, int y, int tx, int ty, HitTestAction, bool inside = false);
     void setInnerNode(NodeInfo& info);
 
-    virtual DOM::Position positionForCoordinates(int x, int y);
+    // Position/Selection stuff
+    virtual RenderPosition positionForCoordinates(int x, int y);
 
     // set the style of the object.
     virtual void setStyle(RenderStyle *style);
@@ -644,12 +648,12 @@ public:
     // how much goes over the left hand side (0 or a negative number)
     virtual int overflowTop() const { return 0; }
     virtual int overflowLeft() const { return 0; }
-    
+
     /**
      * Returns the height that is effectively considered when contemplating the
      * object as a whole -- usually the overflow height, or the height if clipped.
      */
-    int effectiveHeight() const { return hasOverflowClip() ? height() + borderTopExtra() + borderBottomExtra() : 
+    int effectiveHeight() const { return hasOverflowClip() ? height() + borderTopExtra() + borderBottomExtra() :
                                          qMax(overflowHeight() - overflowTop(),  height() + borderTopExtra() + borderBottomExtra()); }
     /**
      * Returns the width that is effectively considered when contemplating the
@@ -667,8 +671,8 @@ public:
     int clientTop() const;
     short clientWidth() const;
     int clientHeight() const;
-    short scrollWidth() const;
-    int scrollHeight() const;
+    virtual short scrollWidth() const;
+    virtual int scrollHeight() const;
 
     virtual bool isSelfCollapsingBlock() const { return false; }
     short collapsedMarginTop() const { return maxTopMargin(true)-maxTopMargin(false);  }
@@ -807,7 +811,7 @@ public:
     virtual void detach( );
 
     bool documentBeingDestroyed() const { return !document()->renderer(); }
-    
+
     void setDoNotDelete(bool b) { m_doNotDelete = b; }
     bool doNotDelete() const { return m_doNotDelete; }
 
@@ -892,7 +896,7 @@ private:
 
     bool m_needsPageClear            : 1;
     bool m_containsPageBreak         : 1;
-    
+
     bool m_hasOverflowClip           : 1;
     bool m_inPosObjectList           : 1;
 
