@@ -5,6 +5,7 @@
  *           (C) 2000-2003 Dirk Mueller (mueller@kde.org)
  *           (C) 2003 Apple Computer, Inc.
  *           (C) 2004-2005 Allan Sandfeld Jensen (kde@carewolf.com)
+ *           (C) 2008 Germain Garand (germain@ebooksfrance.org)
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Library General Public
@@ -31,6 +32,7 @@
 #include "render_canvas.h"
 #include "break_lines.h"
 #include "render_arena.h"
+#include "rendering/render_position.h"
 #include <xml/dom_nodeimpl.h>
 
 #include <misc/loader.h>
@@ -244,11 +246,11 @@ void InlineTextBox::paintSelection(const Font *f, RenderText *text, QPainter *p,
    if (needClipping) {
        p->save();
 
-       int visualSelectionStart = f->width(text->str->s, text->str->l, m_start, startPos, m_start, m_start + m_len, m_toAdd);
-       int visualSelectionEnd = f->width(text->str->s, text->str->l, m_start, endPos, m_start, m_start + m_len, m_toAdd);
+       int visualSelectionStart = f->width(text->str->s, text->str->l, m_start, startPos, false, m_start, m_start + m_len, m_toAdd);
+       int visualSelectionEnd = f->width(text->str->s, text->str->l, m_start, endPos, false, m_start, m_start + m_len, m_toAdd);
        int visualSelectionWidth = visualSelectionEnd - visualSelectionStart;
        if (m_reversed) {
-           visualSelectionStart = f->width(text->str->s, text->str->l, m_start, m_len) - visualSelectionEnd;
+           visualSelectionStart = f->width(text->str->s, text->str->l, m_start, m_len, false) - visualSelectionEnd;
        }
 
        QRect selectionRect(m_x + tx + visualSelectionStart, m_y + ty, visualSelectionWidth, height());
@@ -324,11 +326,11 @@ void InlineTextBox::paintShadow(QPainter *pt, const Font *f, int _tx, int _ty, c
         const int gray = qGray(color);
         const bool inverse = (gray < 100);
         const QRgb bgColor = (inverse) ? qRgb(255,255,255) : qRgb(0,0,0);
-        QPixmap pixmap(w, h);
-        pixmap.fill(bgColor);
+        QImage img(w, h, QImage::Format_RGB32);
+        img.fill(bgColor);
         QPainter p;
 
-        p.begin(&pixmap);
+        p.begin(&img);
         p.setPen(shadow->color);
         p.setFont(pt->font());
         f->drawText(&p, thickness, thickness+m_baseline, text->str->s, text->str->l,
@@ -336,7 +338,6 @@ void InlineTextBox::paintShadow(QPainter *pt, const Font *f, int _tx, int _ty, c
                     m_reversed ? Qt::RightToLeft : Qt::LeftToRight);
 
         p.end();
-        QImage img = pixmap.toImage().convertToFormat(QImage::Format_RGB32);
 
         int md = thickness*thickness; // max-dist^2
 
@@ -364,8 +365,9 @@ void InlineTextBox::paintShadow(QPainter *pt, const Font *f, int _tx, int _ty, c
         float* amap = (float*)alloca(sizeof(float)*(h*w));
         memset(amap, 0, h*w*(sizeof(float)));
         for(int j=thickness; j<h-thickness; j++) {
+            const QRgb *line = (QRgb*)img.scanLine(j);
             for(int i=thickness; i<w-thickness; i++) {
-                QRgb col= img.pixel(i,j);
+                QRgb col= line[i];
                 if (col == bgColor) continue;
                 float g = qGray(col);
                 if (inverse)
@@ -392,10 +394,11 @@ void InlineTextBox::paintShadow(QPainter *pt, const Font *f, int _tx, int _ty, c
         factor = 1.0/factor;
 
         for(int j=0; j<h; j++) {
+            QRgb *line = (QRgb*)res.scanLine(j);
             for(int i=0; i<w; i++) {
                 int a = (int)(amap[i+j*w] * factor * 255.0);
                 if (a > 255) a = 255;
-                res.setPixel(i,j, qRgba(r,g,b,a));
+                line[i] = qRgba(r,g,b,a);
             }
         }
 
@@ -422,7 +425,7 @@ static inline int justifyWidth(int &numSpaces, int &toAdd) {
   return a;
 }
 
-FindSelectionResult InlineTextBox::checkSelectionPoint(int _x, int _y, int _tx, int _ty, const Font *f, RenderText *text, int & offset, short lineHeight)
+FindSelectionResult InlineTextBox::checkSelectionPoint(int _x, int _y, int _tx, int _ty, int & offset)
 {
 //       kDebug(6040) << "InlineTextBox::checkSelectionPoint " << this << " _x=" << _x << " _y=" << _y
 //                     << " _tx+m_x=" << _tx+m_x << " _ty+m_y=" << _ty+m_y << endl;
@@ -431,7 +434,7 @@ FindSelectionResult InlineTextBox::checkSelectionPoint(int _x, int _y, int _tx,
     if ( _y < _ty + m_y )
         return SelectionPointBefore; // above -> before
 
-    if ( _y > _ty + m_y + lineHeight ) {
+    if ( _y > _ty + m_y + m_height ) {
         // below -> after
         // Set the offset to the max
         offset = m_len;
@@ -449,10 +452,11 @@ FindSelectionResult InlineTextBox::checkSelectionPoint(int _x, int _y, int _tx,
 
     // consider spacing for justified text
     int toAdd = m_toAdd;
+    RenderText *text = static_cast<RenderText *>(object());
+    Q_ASSERT(text->isText());
     bool justified = text->style()->textAlign() == JUSTIFY && toAdd > 0;
     int numSpaces = 0;
     if (justified) {
-
         for( int i = 0; i < m_len; i++ )
             if ( text->str->s[m_start+i].category() == QChar::Separator_Space )
 	        numSpaces++;
@@ -462,10 +466,11 @@ FindSelectionResult InlineTextBox::checkSelectionPoint(int _x, int _y, int _tx,
     int delta = _x - (_tx + m_x);
     //kDebug(6040) << "InlineTextBox::checkSelectionPoint delta=" << delta;
     int pos = 0;
+    const Font *f = text->htmlFont(m_firstLine);
     if ( m_reversed ) {
 	delta -= m_width;
 	while(pos < m_len) {
-	    int w = f->width( text->str->s, text->str->l, m_start + pos);
+	    int w = f->charWidth( text->str->s, text->str->l, m_start + pos, text->isSimpleText());
 	    if (justified && text->str->s[m_start + pos].category() == QChar::Separator_Space)
 	        w += justifyWidth(numSpaces, toAdd);
 	    int w2 = w/2;
@@ -477,7 +482,7 @@ FindSelectionResult InlineTextBox::checkSelectionPoint(int _x, int _y, int _tx,
 	}
     } else {
 	while(pos < m_len) {
-	    int w = f->width( text->str->s, text->str->l, m_start + pos);
+	    int w = f->charWidth( text->str->s, text->str->l, m_start + pos, text->isSimpleText());
 	    if (justified && text->str->s[m_start + pos].category() == QChar::Separator_Space)
 	        w += justifyWidth(numSpaces, toAdd);
 	    int w2 = w/2;
@@ -493,6 +498,21 @@ FindSelectionResult InlineTextBox::checkSelectionPoint(int _x, int _y, int _tx,
     return SelectionPointInside;
 }
 
+long InlineTextBox::caretMinOffset() const
+{
+    return m_start;
+}
+
+long InlineTextBox::caretMaxOffset() const
+{
+    return m_start + m_len;
+}
+
+unsigned long InlineTextBox::caretMaxRenderedOffset() const
+{
+    return m_start + m_len;
+}
+
 int InlineTextBox::offsetForPoint(int _x, int &ax) const
 {
   // Do binary search for finding out offset, saves some time for long
@@ -518,6 +538,7 @@ int InlineTextBox::offsetForPoint(int _x, int &ax) const
 int InlineTextBox::widthFromStart(int pos) const
 {
   // gasp! sometimes pos is i < 0 which crashes Font::width
+  // kDebug() << this << pos << endl;
   pos = qMax(pos, 0);
 
   const RenderText *t = renderText();
@@ -559,7 +580,7 @@ int InlineTextBox::widthFromStart(int pos) const
 
       // check run without spaces
       if ( current > start ) {
-          w += f->width(t->str->s + m_start, m_len, start, current - start);
+          w += f->width(t->str->s + m_start, m_len, start, current - start, false);
           start = current;
       }
     }
@@ -570,20 +591,11 @@ int InlineTextBox::widthFromStart(int pos) const
 
   //kDebug(6000) << "default";
   // else use existing width function
-  return f->width(t->str->s + m_start, m_len, 0, pos);
+  // kDebug() << "result width:" << f->width(t->str->s + m_start, m_len, 0, pos, false) << endl;
+  return f->width(t->str->s + m_start, m_len, 0, pos, false);
 
 }
 
-long InlineTextBox::minOffset() const
-{
-  return m_start;
-}
-
-long InlineTextBox::maxOffset() const
-{
-  return m_start + m_len;
-}
-
 void InlineTextBox::deleteLine(RenderArena* arena)
 {
     static_cast<RenderText*>(m_object)->removeTextBox(this);
@@ -664,6 +676,7 @@ RenderText::RenderText(DOM::NodeImpl* node, DOMStringImpl *_str)
 
     m_selectionState = SelectionNone;
     m_hasReturn = true;
+    m_isSimpleText = false;
     m_firstTextBox = m_lastTextBox = 0;
 
 #ifdef DEBUG_LAYOUT
@@ -700,7 +713,7 @@ RenderText::~RenderText()
 
 void RenderText::detach()
 {
-    if (document()->renderer()) {
+    if (!documentBeingDestroyed()) {
         if (firstTextBox()) {
             if (isBR()) {
                 RootInlineBox* next = firstTextBox()->root()->nextRootBox();
@@ -794,7 +807,7 @@ DOM::DOMStringImpl* RenderText::originalString() const
     return element() ? element()->string() : 0;
 }
 
-InlineTextBox * RenderText::findInlineTextBox( int offset, int &pos, bool checkFirstLetter )
+const InlineTextBox * RenderText::findInlineTextBox( int offset, int &pos, bool checkFirstLetter ) const
 {
   Q_UNUSED( checkFirstLetter );
     // The text boxes point to parts of the rendertext's str string
@@ -812,7 +825,11 @@ InlineTextBox * RenderText::findInlineTextBox( int offset, int &pos, bool checkF
         off = s->m_start + s->m_len;
     }
     // we are now in the correct text run
-    pos = (offset > off ? s->m_len : s->m_len - (off - offset) );
+    if (offset >= s->m_start && offset < s->m_start + s->m_len) {
+        pos = offset - s->m_start;
+    } else {
+        pos = (offset > off ? s->m_len : s->m_len - (off - offset) );
+    }
     return s;
 }
 
@@ -867,13 +884,23 @@ FindSelectionResult RenderText::checkSelectionPoint(int _x, int _y, int _tx, int
     for (InlineTextBox* s = firstTextBox(); s; s = s->nextTextBox())
     {
         FindSelectionResult result;
-        const Font *f = htmlFont( s == m_firstTextBox );
-        result = s->checkSelectionPoint(_x, _y, _tx, _ty, f, this, offset, m_lineHeight);
+// #### ?
+//         result = s->checkSelectionPoint(_x, _y, _tx, _ty, offset);
+       if (_y < _ty + s->m_y) result = SelectionPointBefore;
+       else if (_y >= _ty + s->m_y + s->height()) result = SelectionPointAfterInLine;
+       else if (_x < _tx + s->m_x) result = SelectionPointBeforeInLine;
+       else if (_x >= _tx + s->m_x + s->width()) result = SelectionPointAfterInLine;
+       else {
+           int dummy;
+           result = SelectionPointInside;
+           // offsetForPoint shifts to the right: correct it
+           offset = s->offsetForPoint(_x - _tx, dummy) - 1;
+       }
 
 //         kDebug(6040) << "RenderText::checkSelectionPoint " << this << " line " << si << " result=" << result << " offset=" << offset;
         if ( result == SelectionPointInside ) // x,y is inside the textrun
         {
-            offset += s->m_start; // add the offset from the previous lines
+//            offset += s->m_start; // add the offset from the previous lines
 //             kDebug(6040) << "RenderText::checkSelectionPoint inside -> " << offset;
             node = element();
             return SelectionPointInside;
@@ -913,8 +940,142 @@ FindSelectionResult RenderText::checkSelectionPoint(int _x, int _y, int _tx, int
     return SelectionPointAfter;
 }
 
-void RenderText::caretPos(int offset, int flags, int &_x, int &_y, int &width, int &height)
+unsigned RenderText::convertToDOMPosition(unsigned position) const
 {
+    if (isBR())
+        return 0;
+    /*const */DOMStringImpl* domString = originalString();
+    /*const */DOMStringImpl* renderedString = string();
+    if (domString == renderedString) {
+        // kDebug() << "[rendered == dom]" << position << endl;
+        return position;
+    }
+    /* kDebug() << "[convert]" << position << endl
+        << DOMString(domString) << endl
+        << DOMString(renderedString) << endl;*/
+
+    if (!domString || !renderedString)
+        return position;
+
+    unsigned domLength = domString->length();
+    unsigned i = 0, j = 0;
+    for (; i < domLength && j < position;) {
+        bool isRenderedSpace = renderedString->unicode()[j].isSpace();
+        bool isDOMSpace = domString->unicode()[i].isSpace();
+        if (isRenderedSpace && isDOMSpace) {
+            ++i;
+            ++j;
+            continue;
+        }
+        if (isRenderedSpace) {
+            ++j;
+            continue;
+        }
+        if (isDOMSpace) {
+            ++i;
+            continue;
+        }
+        ++i;
+        ++j;
+    }
+    // kDebug() << "[result]" << i << endl;
+    return i;
+}
+
+unsigned RenderText::convertToRenderedPosition(unsigned position) const
+{
+    if (isBR())
+        return 0;
+    /*const */DOMStringImpl* domString = originalString();
+    /*const */DOMStringImpl* renderedString = string();
+    if (domString == renderedString) {
+        // kDebug() << "[rendered == dom]" << position << endl;
+        return position;
+    }
+    /* kDebug() << "[convert]" << position << endl
+        << DOMString(domString) << endl
+        << DOMString(renderedString) << endl;*/
+
+    if (!domString || !renderedString)
+        return position;
+
+    unsigned renderedLength = renderedString->length();
+    unsigned i = 0, j = 0;
+    for (; i < position && j < renderedLength;) {
+        bool isRenderedSpace = renderedString->unicode()[j].isSpace();
+        bool isDOMSpace = domString->unicode()[i].isSpace();
+        if (isRenderedSpace && isDOMSpace) {
+            ++i;
+            ++j;
+            continue;
+        }
+        if (isRenderedSpace) {
+            ++j;
+            continue;
+        }
+        if (isDOMSpace) {
+            ++i;
+            continue;
+        }
+        ++i;
+        ++j;
+    }
+    // kDebug() << "[result]" << j << endl;
+    return j;
+}
+
+RenderPosition RenderText::positionForCoordinates(int _x, int _y)
+{
+    // kDebug() << this << _x << _y << endl;
+    if (!firstTextBox() || stringLength() == 0)
+        return Position(element(), 0);
+
+    int absx, absy;
+    containingBlock()->absolutePosition(absx, absy);
+    // kDebug() << "absolute(" << absx << absy << ")" << endl;
+
+    if (_y < absy + firstTextBox()->root()->bottomOverflow() && _x < absx + firstTextBox()->m_x) {
+        // at the y coordinate of the first line or above
+        // and the x coordinate is to the left than the first text box left edge
+        return RenderPosition(element(), firstTextBox()->m_start);
+    }
+
+    if (_y >= absy + lastTextBox()->root()->topOverflow() && _x >= absx + lastTextBox()->m_x + lastTextBox()->m_width) {
+        // at the y coordinate of the last line or below
+        // and the x coordinate is to the right than the last text box right edge
+        return RenderPosition(element(), lastTextBox()->m_start + lastTextBox()->m_len);
+    }
+
+    for (InlineTextBox* box = firstTextBox(); box; box = box->nextTextBox()) {
+        // kDebug() << "[check box]" << box << endl;
+        if (_y >= absy + box->root()->topOverflow() && _y < absy + box->root()->bottomOverflow()) {
+            if (_x < absx + box->m_x + box->m_width) {
+                // and the x coordinate is to the left of the right edge of this box
+                // check to see if position goes in this box
+                int offset;
+                box->checkSelectionPoint(_x, absy + box->yPos(), absx, absy, offset);
+                // kDebug() << "offset" << offset << endl;
+                if (offset != -1) {
+                    // kDebug() << "return" << Position(element(), convertToDOMPosition(offset + box->m_start)) << endl;
+                    return RenderPosition(element(), offset + box->m_start);
+                }
+            }
+            else if (!box->prevOnLine() && _x < absx + box->m_x)
+                // box is first on line
+                // and the x coordinate is to the left than the first text box left edge
+                return RenderPosition(element(), box->m_start);
+            else if (!box->nextOnLine() && _x >= absx + box->m_x + box->m_width)
+                // box is last on line
+                // and the x coordinate is to the right than the last text box right edge
+                return RenderPosition(element(), box->m_start + box->m_len);
+        }
+    }
+    return RenderPosition(element(), 0);
+}
+
+void RenderText::caretPos(int offset, int flags, int &_x, int &_y, int &width, int &height) const
+{
+    // kDebug() << offset << flags << endl;
   if (!m_firstTextBox) {
     _x = _y = height = -1;
     width = 1;
@@ -922,8 +1083,8 @@ void RenderText::caretPos(int offset, int flags, int &_x, int &_y, int &width, i
   }
 
   int pos;
-  InlineTextBox * s = findInlineTextBox( offset, pos, true );
-  RenderText *t = s->renderText();
+  const InlineTextBox * s = findInlineTextBox( offset, pos, true );
+  const RenderText *t = s->renderText();
 //  kDebug(6040) << "offset="<<offset << " pos="<<pos;
 
   const QFontMetrics &fm = t->metrics( s->m_firstLine );
@@ -931,9 +1092,11 @@ void RenderText::caretPos(int offset, int flags, int &_x, int &_y, int &width, i
 
   _x = s->m_x + s->widthFromStart(pos);
   _y = s->m_y + s->baseline() - fm.ascent();
+  // kDebug() << "(" << _x << _y << ")" << endl;
   width = 1;
   if (flags & CFOverride) {
-    width = offset < maxOffset() ? fm.width(str->s[offset]) : 1;
+    width = offset < caretMaxOffset() ? fm.width(str->s[offset]) : 1;
+    // kDebug() << "CFOverride" << width << endl;
   }/*end if*/
 #if 0
   kDebug(6040) << "_x="<<_x << " s->m_x="<<s->m_x
@@ -955,7 +1118,7 @@ void RenderText::caretPos(int offset, int flags, int &_x, int &_y, int &width, i
   }
 }
 
-long RenderText::minOffset() const
+long RenderText::caretMinOffset() const
 {
   if (!m_firstTextBox) return 0;
   // FIXME: it is *not* guaranteed that the first run contains the lowest offset
@@ -966,7 +1129,7 @@ long RenderText::minOffset() const
   return m_firstTextBox->m_start;
 }
 
-long RenderText::maxOffset() const
+long RenderText::caretMaxOffset() const
 {
     InlineTextBox* box = m_lastTextBox;
     if (!box)
@@ -978,19 +1141,47 @@ long RenderText::maxOffset() const
     return maxOffset;
 }
 
+unsigned long RenderText::caretMaxRenderedOffset() const
+{
+    int l = 0;
+    // ### no guarantee that the order is ascending
+    for (InlineTextBox* box = firstTextBox(); box; box = box->nextTextBox()) {
+        l += box->m_len;
+    }
+    return l;
+}
+
+InlineBox *RenderText::inlineBox(long offset)
+{
+    // ### make educated guess about most likely position
+    for (InlineTextBox* box = firstTextBox(); box; box = box->nextTextBox()) {
+        if (offset >= box->m_start && offset <= box->m_start + box->m_len) {
+            return box;
+        }
+        else if (offset < box->m_start) {
+            // The offset we're looking for is before this node
+            // this means the offset must be in content that is
+            // not rendered.
+            return box->prevTextBox() ? box->prevTextBox() : firstTextBox();
+        }
+    }
+
+    return 0;
+}
+
 bool RenderText::absolutePosition(int &xPos, int &yPos, bool) const
 {
     return RenderObject::absolutePosition(xPos, yPos, false);
 }
 
-bool RenderText::posOfChar(int chr, int &x, int &y)
+bool RenderText::posOfChar(int chr, int &x, int &y) const
 {
     if (!parent())
         return false;
     parent()->absolutePosition( x, y, false );
 
     int pos;
-    InlineTextBox * s = findInlineTextBox( chr, pos );
+    const InlineTextBox * s = findInlineTextBox( chr, pos );
 
     if ( s ) {
         // s is the line containing the character
@@ -1002,6 +1193,47 @@ bool RenderText::posOfChar(int chr, int &x, int &y)
     return false;
 }
 
+static bool isSimpleChar( const unsigned short c )
+{
+    // Exclude ranges with many Mn/Me/Mc and the various combining diacriticals ranges.
+    // Unicode version used is 4.1.0
+
+    // General Combining Diacritical Marks
+    if (c < 0x300)  return true;
+    if (c <= 0x36F) return false;
+
+    // Cyrillic's
+    if (c < 0x483) return true;
+    if (c <= 0x489) return false;
+
+    // Hebrew's
+    if (c < 0x0591) return true;
+    if (c <= 0x05C7 && !(c == 0x05BE || c == 0x05C0 || c == 0x05C3 || c == 0x05C6)) return false;
+
+    // Unicode range 6 to 11 (Arabic to Korean Hangul)
+    if (c < 0x0600)  return true;
+    if (c <= 0x11F9) return false;
+
+    // Unicode range 17 to 1A (Tagalog to Buginese)
+    // (also excl. Ethiopic Combining Gemination Mark)
+    if (c < 0x1700 && c != 0x135F)  return true;
+    if (c <= 0x1A1F) return false;
+
+    // Combining Diacritical Marks Supplement
+    if (c < 0x1DC0) return true;
+    if (c <= 0x1DFF) return false;
+
+    // Diacritical Marks for Symbols 
+    if (c < 0x20D0)  return true;
+    if (c <= 0x20EB) return false;
+
+    // Combining Half Marks
+    if (c < 0xFE20)  return true;
+    if (c <= 0xFE2F) return false;
+
+    return true;
+}
+
 void RenderText::calcMinMaxWidth()
 {
     KHTMLAssert( !minMaxKnown() );
@@ -1015,6 +1247,7 @@ void RenderText::calcMinMaxWidth()
 
     int currMinWidth = 0;
     int currMaxWidth = 0;
+    m_isSimpleText = true;
     m_hasBreakableChar = m_hasBreak = m_hasBeginWS = m_hasEndWS = false;
 
     // ### not 100% correct for first-line
@@ -1031,9 +1264,12 @@ void RenderText::calcMinMaxWidth()
 
         // If line-breaks survive to here they are preserved
         if ( c == '\n' ) {
-            m_hasBreak = true;
-            isNewline = true;
-            isSpace = false;
+            if (style()->preserveLF()) {
+                m_hasBreak = true;
+                isNewline = true;
+                isSpace = false;
+            } else
+                isSpace = true;
         } else
             isSpace = c == ' ';
 
@@ -1047,16 +1283,15 @@ void RenderText::calcMinMaxWidth()
 
         int wordlen = 0;
         while( i+wordlen < len && (i+wordlen == 0 || str->s[i+wordlen].unicode() != SOFT_HYPHEN) &&
-               !(isBreakable( str->s, i+wordlen, str->l )) )
+               !(isBreakable( str->s, i+wordlen, str->l )) ) {
+            // check if we may use the simpler algorithm for estimating text width
+            m_isSimpleText = (m_isSimpleText && isSimpleChar(str->s[i+wordlen].unicode()));
             wordlen++;
+         }
 
         if (wordlen)
         {
-#ifndef APPLE_CHANGES
-            int w = f->width(str->s, str->l, i, wordlen);
-#else
-            int w = widthFromCache(f, i, wordlen);
-#endif
+            int w = f->width(str->s, str->l, i, wordlen, m_isSimpleText);
             currMinWidth += w;
             currMaxWidth += w;
 
@@ -1088,7 +1323,8 @@ void RenderText::calcMinMaxWidth()
             {
                 if ( firstLine ) {
                     firstLine = false;
-                    m_beginMinWidth = currMaxWidth;
+                    if (!style()->autoWrap())
+                        m_beginMinWidth = currMaxWidth;
                 }
 
                 if(currMaxWidth > m_maxWidth) m_maxWidth = currMaxWidth;
@@ -1096,7 +1332,7 @@ void RenderText::calcMinMaxWidth()
             }
             else
             {
-                currMaxWidth += f->width( str->s, str->l, i + wordlen );
+                currMaxWidth += f->charWidth( str->s, str->l, i + wordlen, m_isSimpleText);
             }
         }
     }
@@ -1284,7 +1520,7 @@ unsigned int RenderText::width(unsigned int from, unsigned int len, const Font *
     if ( f == &style()->htmlFont() && from == 0 && len == str->l )
  	 return m_maxWidth;
     
-    int w = f->width(str->s, str->l, from, len );
+    int w = f->width(str->s, str->l, from, len, m_isSimpleText );
 
     //kDebug( 6040 ) << "RenderText::width(" << from << ", " << len << ") = " << w;
     return w;
@@ -1390,7 +1626,7 @@ void RenderText::trimmedMinMaxWidth(short& beginMinW, bool& beginWS,
     if (stripFrontSpaces && (str->s[0].direction() == QChar::DirWS || (!preserveLF && str->s[0] == '\n'))) {
         const Font *f = htmlFont( false );
         QChar space[1]; space[0] = ' ';
-        int spaceWidth = f->width(space, 1, 0);
+        int spaceWidth = f->charWidth(space, 1, 0, m_isSimpleText);
         maxW -= spaceWidth;
     }
 
@@ -1414,11 +1650,7 @@ void RenderText::trimmedMinMaxWidth(short& beginMinW, bool& beginWS,
 
             if (linelen)
             {
-#ifndef APPLE_CHANGES
-                endMaxW = f->width(str->s, str->l, i, linelen);
-#else
-                endMaxW = widthFromCache(f, i, linelen);
-#endif
+                endMaxW = f->width(str->s, str->l, i, linelen, m_isSimpleText);
                 if (firstLine) {
                     firstLine = false;
                     beginMaxW = endMaxW;
@@ -1436,7 +1668,31 @@ void RenderText::trimmedMinMaxWidth(short& beginMinW, bool& beginWS,
         }
     }
 }
+ 
+bool RenderText::isPointInsideSelection(int x, int y, const Selection &) const
+{
+    RenderText *rt = const_cast<RenderText *>(this);
+    SelectionState selstate = selectionState();
+    if (selstate == SelectionInside) return true;
+    if (selstate == SelectionNone) return false;
+    if (!firstTextBox()) return false;
 
+    int absx, absy;
+    if (!rt->absolutePosition(absx, absy)) return false;
+
+    int startPos, endPos, offset;
+    rt->selectionStartEnd(startPos, endPos);
+
+    if (selstate == SelectionEnd) startPos = 0;
+    if (selstate == SelectionStart) endPos = str->l;
+
+    SelPointState sps;
+    DOM::NodeImpl *node;
+    /*FindSelectionResult res = */rt->checkSelectionPoint(x, y, absx, absy, node, offset, sps);
+
+    return offset >= startPos && offset < endPos;
+}
+ 
 #ifdef ENABLE_DUMP
 
 static QString quoteAndEscapeNonPrintables(const QString &s)
