@@ -3,6 +3,7 @@
 /* This file is part of the KDE libraries
    Copyright (C) 2000 David Faure <faure@kde.org>
    Copyright (C) 2003 Alexander Kellett <lypanov@kde.org>
+   Copyright (C) 2008 Norbert Frese <nf2@scheinwelt.at>
 
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
@@ -32,6 +33,68 @@
 #include <qdatetime.h>
 #include <qmimedata.h>
 
+#define METADATA_KDE_OWNER "http://www.kde.org"
+#define METADATA_FREEDESKTOP_OWNER "http://freedesktop.org"
+#define METADATA_MIME_OWNER "http://www.freedesktop.org/standards/shared-mime-info"
+
+////// utility functions
+
+static QDomNode cd(QDomNode node, const QString &name, bool create)
+{
+    QDomNode subnode = node.namedItem(name);
+    if (create && subnode.isNull())
+    {
+        subnode = node.ownerDocument().createElement(name);
+        node.appendChild(subnode);
+    }
+    return subnode;
+}
+
+static QDomNode cd_or_create(QDomNode node, const QString &name)
+{
+    return cd(node, name, true);
+}
+
+static QDomText get_or_create_text(QDomNode node)
+{
+    QDomNode subnode = node.firstChild();
+    if (subnode.isNull())
+    {
+        subnode = node.ownerDocument().createTextNode("");
+        node.appendChild(subnode);
+    }
+    return subnode.toText();
+}
+
+static QDomNode findMetadata(const QString & forOwner, QDomNode& parent, bool create)
+{
+    bool forOwnerIsKDE = forOwner == METADATA_KDE_OWNER;
+
+    QDomElement metadataElement;
+    for ( QDomNode _node = parent.firstChild(); !_node.isNull(); _node = _node.nextSibling() ) {
+        QDomElement elem = _node.toElement();
+        if ( !elem.isNull() && elem.tagName() == "metadata" ) {
+            const QString owner = elem.attribute( "owner" );
+            if ( owner == forOwner )
+                return elem;
+            if ( owner.isEmpty() && forOwnerIsKDE )
+                metadataElement = elem;
+        }
+    }
+    if ( create && metadataElement.isNull() ) {
+        metadataElement = parent.ownerDocument().createElement( "metadata" );
+        parent.appendChild(metadataElement);
+        metadataElement.setAttribute( "owner", forOwner );
+
+    } else if (!metadataElement.isNull() && forOwnerIsKDE) {
+        // i'm not sure if this is good, we shouln't take over foreign metatdata
+        metadataElement.setAttribute( "owner", METADATA_KDE_OWNER );
+    }
+    return metadataElement;
+}
+
+//////
+
 KBookmarkGroup::KBookmarkGroup()
  : KBookmark( QDomElement() )
 {
@@ -93,7 +156,8 @@ QDomElement KBookmarkGroup::nextKnownTag( const QDomElement &start, bool goNext
 
 KBookmarkGroup KBookmarkGroup::createNewFolder( const QString & text )
 {
-    Q_ASSERT(!element.isNull());
+    if (isNull())
+        return KBookmarkGroup();
     QDomDocument doc = element.ownerDocument();
     QDomElement groupElem = doc.createElement( "folder" );
     element.appendChild( groupElem );
@@ -106,7 +170,8 @@ KBookmarkGroup KBookmarkGroup::createNewFolder( const QString & text )
 
 KBookmark KBookmarkGroup::createNewSeparator()
 {
-    Q_ASSERT(!element.isNull());
+    if (isNull())
+        return KBookmark();
     QDomDocument doc = element.ownerDocument();
     Q_ASSERT(!doc.isNull());
     QDomElement sepElem = doc.createElement( "separator" );
@@ -150,18 +215,21 @@ KBookmark KBookmarkGroup::addBookmark( const KBookmark &bm )
 
 KBookmark KBookmarkGroup::addBookmark( const QString & text, const KUrl & url, const QString & icon )
 {
-    //kDebug(7043) << "KBookmarkGroup::addBookmark " << text << " into " << m_address;
+    if (isNull())
+        return KBookmark();
     QDomDocument doc = element.ownerDocument();
-    QDomElement elem = doc.createElement( "bookmark" );    
+    QDomElement elem = doc.createElement( "bookmark" );
     elem.setAttribute( "href", url.url() ); // gives us utf8
 
-    elem.setAttribute( "icon", icon.isEmpty() ? KMimeType::iconNameForUrl( url ) : icon );
-
     QDomElement textElem = doc.createElement( "title" );
     elem.appendChild( textElem );
     textElem.appendChild( doc.createTextNode( text ) );
 
-    return addBookmark( KBookmark( elem ) );
+    KBookmark newBookmark =  addBookmark( KBookmark( elem ) );
+
+    // as icons are moved to metadata, we have to use the KBookmark API for this
+    newBookmark.setIcon(icon.isEmpty() ? KMimeType::iconNameForUrl( url ) : icon );
+    return newBookmark;
 }
 
 void KBookmarkGroup::deleteBookmark( const KBookmark &bk )
@@ -252,7 +320,9 @@ QString KBookmark::fullText() const
     if (isSeparator())
         return i18n("--- separator ---");
 
-    return element.namedItem("title").toElement().text();
+    QString text = element.namedItem("title").toElement().text();
+    text.replace('\n', ' '); // #140673
+    return text;
 }
 
 void KBookmark::setFullText(const QString &fullText)
@@ -274,7 +344,7 @@ void KBookmark::setFullText(const QString &fullText)
 
 KUrl KBookmark::url() const
 {
-    return KUrl(element.attribute("href")); // Decodes it from utf8
+    return KUrl(element.attribute("href").toAscii()); // Decodes it from utf8
 }
 
 void KBookmark::setUrl(const KUrl &url)
@@ -284,7 +354,18 @@ void KBookmark::setUrl(const KUrl &url)
 
 QString KBookmark::icon() const
 {
-    QString icon = element.attribute("icon");
+    QDomNode metaDataNode = metaData(METADATA_FREEDESKTOP_OWNER, false);
+    QDomElement iconElement = cd(metaDataNode, "bookmark:icon", false).toElement();
+
+    QString icon = iconElement.attribute("name");
+
+    // migration code
+    if (icon.isEmpty())
+      icon = element.attribute("icon");
+    if (icon == "www") // common icon for kde3 bookmarks
+        return "internet-web-browser";
+    // end migration code
+
     if (icon == "bookmark_folder") {
         return "folder-bookmarks";
     }
@@ -297,8 +378,16 @@ QString KBookmark::icon() const
         else {
             if (isSeparator()) {
                 icon = "edit-clear"; // whatever
-            }
-            else {
+            } else {
+                // get icon from mimeType
+                QString _mimeType = mimeType();
+                if (!_mimeType.isEmpty()) {
+                    KMimeType::Ptr mime = KMimeType::mimeType(_mimeType, KMimeType::ResolveAliases);
+                    if (mime) {
+                        return mime->iconName();
+                    }
+                }
+                // get icon from URL
                 icon = KMimeType::iconNameForUrl(url());
             }
         }
@@ -308,7 +397,54 @@ QString KBookmark::icon() const
 
 void KBookmark::setIcon(const QString &icon)
 {
-    element.setAttribute("icon", icon);
+    QDomNode metaDataNode = metaData(METADATA_FREEDESKTOP_OWNER, true);
+    QDomElement iconElement = cd_or_create(metaDataNode, "bookmark:icon").toElement();
+    iconElement.setAttribute ( "name", icon );
+
+    // migration code
+    if(!element.attribute("icon").isEmpty())
+        element.removeAttribute("icon");
+}
+
+QString KBookmark::description() const
+{
+    if (isSeparator())
+        return QString();
+
+    QString description = element.namedItem("desc").toElement().text();
+    description.replace('\n', ' '); // #140673
+    return description;
+}
+
+void KBookmark::setDescription(const QString &description)
+{
+    QDomNode descNode = element.namedItem("desc");
+    if (descNode.isNull()) {
+        descNode = element.ownerDocument().createElement("desc");
+        element.appendChild(descNode);
+    }
+
+    if (descNode.firstChild().isNull()) {
+        QDomText domtext = descNode.ownerDocument().createTextNode(QString());
+        descNode.appendChild(domtext);
+    }
+
+    QDomText domtext = descNode.firstChild().toText();
+    domtext.setData(description);
+}
+
+QString KBookmark::mimeType() const
+{
+    QDomNode metaDataNode = metaData(METADATA_MIME_OWNER, false);
+    QDomElement mimeTypeElement = cd(metaDataNode, "mime:mime-type", false).toElement();
+    return mimeTypeElement.attribute("type");
+}
+
+void KBookmark::setMimeType(const QString &mimeType)
+{
+    QDomNode metaDataNode = metaData(METADATA_MIME_OWNER, true);
+    QDomElement iconElement = cd_or_create(metaDataNode, "mime:mime-type").toElement();
+    iconElement.setAttribute ( "type", mimeType );
 }
 
 bool KBookmark::showInToolbar() const
@@ -403,51 +539,6 @@ QString KBookmark::commonParent(const QString &first, const QString &second)
     return A.left(lastCommonSlash);
 }
 
-static QDomNode cd_or_create(QDomNode node, const QString &name)
-{
-    QDomNode subnode = node.namedItem(name);
-    if (subnode.isNull())
-    {
-        subnode = node.ownerDocument().createElement(name);
-        node.appendChild(subnode);
-    }
-    return subnode;
-}
-
-static QDomText get_or_create_text(QDomNode node)
-{
-    QDomNode subnode = node.firstChild();
-    if (subnode.isNull())
-    {
-        subnode = node.ownerDocument().createTextNode("");
-        node.appendChild(subnode);
-    }
-    return subnode.toText();
-}
-
-// Look for a metadata with owner="http://www.kde.org" or without any owner (for compatibility)
-static QDomNode findOrCreateMetadata( QDomNode& parent )
-{
-    static const char kdeOwner[] = "http://www.kde.org";
-    QDomElement metadataElement;
-    for ( QDomNode _node = parent.firstChild(); !_node.isNull(); _node = _node.nextSibling() ) {
-        QDomElement elem = _node.toElement();
-        if ( !elem.isNull() && elem.tagName() == "metadata" ) {
-            const QString owner = elem.attribute( "owner" );
-            if ( owner == kdeOwner )
-                return elem;
-            if ( owner.isEmpty() )
-                metadataElement = elem;
-        }
-    }
-    if ( metadataElement.isNull() ) {
-        metadataElement = parent.ownerDocument().createElement( "metadata" );
-        parent.appendChild(metadataElement);
-    }
-    metadataElement.setAttribute( "owner", kdeOwner );
-    return metadataElement;
-}
-
 void KBookmark::updateAccessMetadata()
 {
     kDebug(7043) << "KBookmark::updateAccessMetadata " << address() << " " << url().prettyUrl();
@@ -491,15 +582,18 @@ QString KBookmark::nextAddress( const QString & address )
         QString::number(positionInParent(address)+1);
 }
 
+QDomNode KBookmark::metaData(const QString &owner, bool create) const
+{
+    QDomNode infoNode = cd( internalElement(), "info", create);
+    if (infoNode.isNull()) return QDomNode();
+    return findMetadata(owner, infoNode , create);
+}
+
 QString KBookmark::metaDataItem( const QString &key ) const
 {
-    QDomNode infoNode = cd_or_create( internalElement(), "info" );
-    infoNode = findOrCreateMetadata( infoNode );
-    for ( QDomNode n = infoNode.firstChild(); !n.isNull(); n = n.nextSibling() ) {
-        if ( !n.isElement() ) {
-            continue;
-        }
-        const QDomElement e = n.toElement();
+    QDomNode metaDataNode = metaData(METADATA_KDE_OWNER, false);
+    for ( QDomElement e = metaDataNode.firstChildElement(); !e.isNull(); e = e.nextSiblingElement() )
+    {
         if ( e.tagName() == key ) {
             return e.text();
         }
@@ -509,10 +603,8 @@ QString KBookmark::metaDataItem( const QString &key ) const
 
 void KBookmark::setMetaDataItem( const QString &key, const QString &value, MetaDataOverwriteMode mode )
 {
-    QDomNode infoNode = cd_or_create( internalElement(), "info" );
-    infoNode = findOrCreateMetadata( infoNode );
-
-    QDomNode item = cd_or_create( infoNode, key );
+    QDomNode metaDataNode = metaData(METADATA_KDE_OWNER, true);
+    QDomNode item = cd_or_create( metaDataNode, key );
     QDomText text = get_or_create_text( item );
     if ( mode == DontOverwriteMetaData && !text.data().isEmpty() ) {
         return;
@@ -612,21 +704,27 @@ QStringList KBookmark::List::mimeDataTypes()
 }
 
 KBookmark::List KBookmark::List::fromMimeData( const QMimeData *mimeData )
+{
+    QDomDocument doc;
+    kWarning(7043) << "Deprecated method called, with wrong lifetime of QDomDocument, will probably crash";
+    return fromMimeData(mimeData, doc);
+}
+
+KBookmark::List KBookmark::List::fromMimeData( const QMimeData *mimeData, QDomDocument& doc )
 {
     KBookmark::List bookmarks;
     QByteArray payload = mimeData->data( "application/x-xbel" );
     if ( !payload.isEmpty() ) {
-        QDomDocument doc;
         doc.setContent( payload );
         QDomElement elem = doc.documentElement();
         QDomNodeList children = elem.childNodes();
         for ( int childno = 0; childno < children.count(); childno++)
         {
-            bookmarks.append( KBookmark( children.item(childno).cloneNode(true).toElement() ));
+            bookmarks.append( KBookmark( children.item(childno).toElement() ));
         }
         return bookmarks;
     }
-    KUrl::List urls = KUrl::List::fromMimeData( mimeData );
+    const KUrl::List urls = KUrl::List::fromMimeData( mimeData );
     if ( !urls.isEmpty() )
     {
         KUrl::List::ConstIterator uit = urls.begin();
