@@ -89,6 +89,7 @@ KFileMetaInfoWidget::KFileMetaInfoWidget(KFileMetaInfoItem item,
 
 void KFileMetaInfoWidgetPrivate::init(KFileMetaInfoItem item, KFileMetaInfoWidget::Mode mode)
 {
+    Q_UNUSED(item)
     kDebug(7033) << "*** item "  << m_item.name()
                   << " is a " << m_value.typeName() << endl;
 
@@ -108,7 +109,7 @@ void KFileMetaInfoWidgetPrivate::init(KFileMetaInfoItem item, KFileMetaInfoWidge
                 static_cast<QLabel*>(m_widget)->setPixmap(m_value.value<QPixmap>());
                 break;
             default:
-                m_widget = new QLabel(item.name(), q);
+                m_widget = new QLabel(m_value.toString(), q);
                 m_widget->setObjectName(QLatin1String("info label"));
         }
 
