@@ -14,7 +14,6 @@
 #include <kpluginloader.h>
 #include <kpluginfactory.h>
 #include <kimagefilepreview.h>
-#include <kmimetype.h>
 
 bool KFileMetaPreview::s_tryAudioPreview = true;
 
@@ -24,7 +23,6 @@ KFileMetaPreview::KFileMetaPreview( QWidget *parent )
 {
     QHBoxLayout *layout = new QHBoxLayout( this );
     layout->setMargin( 0 );
-    layout->setSpacing( 0 );
     m_stack = new QStackedWidget( this );
     layout->addWidget( m_stack );
 
@@ -49,7 +47,7 @@ void KFileMetaPreview::initPreviewProviders()
     m_stack->setCurrentWidget ( imagePreview );
     resize( imagePreview->sizeHint() );
 
-    QStringList mimeTypes = imagePreview->supportedMimeTypes();
+    const QStringList mimeTypes = imagePreview->supportedMimeTypes();
     QStringList::ConstIterator it = mimeTypes.begin();
     for ( ; it != mimeTypes.end(); ++it )
     {
@@ -58,22 +56,53 @@ void KFileMetaPreview::initPreviewProviders()
     }
 }
 
+KPreviewWidgetBase* KFileMetaPreview::findExistingProvider(const QString& mimeType, const KMimeType::Ptr& mimeInfo) const
+{
+    KPreviewWidgetBase* provider = m_previewProviders.value(mimeType);
+    if ( provider )
+        return provider;
+
+    if ( mimeInfo ) {
+        // check mime type inheritance
+        const QStringList parentMimeTypes = mimeInfo->allParentMimeTypes();
+        Q_FOREACH(const QString& parentMimeType, parentMimeTypes) {
+            provider = m_previewProviders.value(parentMimeType);
+            if (provider)
+                return provider;
+        }
+    }
+
+    // ### mimetype may be image/* for example, try that
+    const int index = mimeType.indexOf( '/' );
+    if (index > 0)
+    {
+        provider = m_previewProviders.value(mimeType.left(index + 1) + '*');
+        if (provider)
+            return provider;
+    }
+
+    return 0;
+}
+
 KPreviewWidgetBase * KFileMetaPreview::previewProviderFor( const QString& mimeType )
 {
-//     qDebug("### looking for: %s", mimeType.toLatin1().constData());
+    KMimeType::Ptr mimeInfo = KMimeType::mimeType( mimeType );
+
+    //     qDebug("### looking for: %s", mimeType.toLatin1().constData());
     // often the first highlighted item, where we can be sure, there is no plugin
     // (this "folders reflect icons" is a konq-specific thing, right?)
-    if ( mimeType == "inode/directory" )
+    if ( mimeInfo && mimeInfo->is("inode/directory") )
         return 0L;
 
-    KPreviewWidgetBase *provider = m_previewProviders.value( mimeType );
-    if ( provider )
+    KPreviewWidgetBase *provider = findExistingProvider(mimeType, mimeInfo);
+    if (provider)
         return provider;
 
 //qDebug("#### didn't find anything for: %s", mimeType.toLatin1().constData());
 
     if ( s_tryAudioPreview &&
-         !mimeType.startsWith("text/") && !mimeType.startsWith("image/") )
+        !mimeType.startsWith(QLatin1String("text/")) &&
+        !mimeType.startsWith(QLatin1String("image/")) )
     {
         if ( !haveAudioPreview )
         {
@@ -82,45 +111,23 @@ KPreviewWidgetBase * KFileMetaPreview::previewProviderFor( const QString& mimeTy
             {
                 haveAudioPreview = true;
                 (void) m_stack->addWidget( audioPreview );
-                QStringList mimeTypes = audioPreview->supportedMimeTypes();
+                const QStringList mimeTypes = audioPreview->supportedMimeTypes();
                 QStringList::ConstIterator it = mimeTypes.begin();
                 for ( ; it != mimeTypes.end(); ++it )
-                    m_previewProviders.insert( *it, audioPreview );
+                {
+                    // only add non already handled mimetypes
+                    if ( m_previewProviders.constFind( *it ) == m_previewProviders.constEnd() )
+                        m_previewProviders.insert( *it, audioPreview );
+                }
             }
         }
     }
 
     // with the new mimetypes from the audio-preview, try again
-    provider = m_previewProviders.value( mimeType );
-    if ( provider )
+    provider = findExistingProvider(mimeType, mimeInfo);
+    if (provider)
         return provider;
 
-    // ### mimetype may be image/* for example, try that
-    int index = mimeType.indexOf( '/' );
-    if ( index > 0 )
-    {
-        provider = m_previewProviders.value( mimeType.left( index + 1 ) + '*' );
-        if ( provider )
-            return provider;
-    }
-
-    KMimeType::Ptr mimeInfo = KMimeType::mimeType( mimeType );
-    if ( mimeInfo )
-    {
-        // check mime type inheritance
-        QString parentMimeType = mimeInfo->parentMimeType();
-        while ( !parentMimeType.isEmpty() )
-        {
-            provider = m_previewProviders.value( parentMimeType );
-            if ( provider )
-                return provider;
-
-            KMimeType::Ptr parentMimeInfo = KMimeType::mimeType( parentMimeType );
-            if ( !parentMimeInfo ) break;
-
-            parentMimeType = parentMimeInfo->parentMimeType();
-        }
-    }
     // The logic in this code duplicates the logic in PreviewJob.
     // But why do we need multiple KPreviewWidgetBase instances anyway?
 
