@@ -29,6 +29,8 @@
 #include <QtGui/QLabel>
 #include <QtGui/QLayout>
 #include <QtGui/QCheckBox>
+#include <QtGui/QStyle>
+#include <QtGui/QStyleOptionButton>
 
 #include <kauthorized.h>
 #include <khistorycombobox.h>
@@ -36,6 +38,7 @@
 #include <klineedit.h>
 #include <klocale.h>
 #include <kiconloader.h>
+#include <kmessagebox.h>
 #include <krun.h>
 #include <kstandarddirs.h>
 #include <kstringhandler.h>
@@ -74,7 +77,7 @@ public:
 
     QString icon;
     QString text;
-    QString relPath;
+    QString entryPath;
     QString exec;
     bool isDir;
 
@@ -117,26 +120,26 @@ public:
         delete root;
     }
 
-    void fillNode(const QString &relPath, KDEPrivate::AppNode *node);
+    void fillNode(const QString &entryPath, KDEPrivate::AppNode *node);
 
     KApplicationModel *q;
 
     KDEPrivate::AppNode *root;
 };
 
-void KApplicationModelPrivate::fillNode(const QString &_relPath, KDEPrivate::AppNode *node)
+void KApplicationModelPrivate::fillNode(const QString &_entryPath, KDEPrivate::AppNode *node)
 {
-   KServiceGroup::Ptr root = KServiceGroup::group(_relPath);
+   KServiceGroup::Ptr root = KServiceGroup::group(_entryPath);
    if (!root || !root->isValid()) return;
 
-   KServiceGroup::List list = root->entries();
+   const KServiceGroup::List list = root->entries();
 
    for( KServiceGroup::List::ConstIterator it = list.begin();
        it != list.end(); ++it)
    {
       QString icon;
       QString text;
-      QString relPath = _relPath;
+      QString entryPath;
       QString exec;
       bool isDir = false;
       const KSycocaEntry::Ptr p = (*it);
@@ -150,6 +153,7 @@ void KApplicationModelPrivate::fillNode(const QString &_relPath, KDEPrivate::App
          icon = service->icon();
          text = service->name();
          exec = service->exec();
+         entryPath = service->entryPath();
       }
       else if (p->isType(KST_KServiceGroup))
       {
@@ -160,7 +164,7 @@ void KApplicationModelPrivate::fillNode(const QString &_relPath, KDEPrivate::App
 
          icon = serviceGroup->icon();
          text = serviceGroup->caption();
-         relPath = serviceGroup->relPath();
+         entryPath = serviceGroup->entryPath();
          isDir = true;
       }
       else
@@ -172,7 +176,7 @@ void KApplicationModelPrivate::fillNode(const QString &_relPath, KDEPrivate::App
       KDEPrivate::AppNode *newnode = new KDEPrivate::AppNode();
       newnode->icon = icon;
       newnode->text = text;
-      newnode->relPath = relPath;
+      newnode->entryPath = entryPath;
       newnode->exec = exec;
       newnode->isDir = isDir;
       newnode->parent = node;
@@ -241,7 +245,7 @@ void KApplicationModel::fetchMore(const QModelIndex &parent)
         return;
 
     emit layoutAboutToBeChanged();
-    d->fillNode(node->relPath, node);
+    d->fillNode(node->entryPath, node);
     node->fetched = true;
     emit layoutChanged();
 }
@@ -311,13 +315,13 @@ int KApplicationModel::rowCount(const QModelIndex &parent) const
     return node->children.count();
 }
 
-QString KApplicationModel::nameFor(const QModelIndex &index) const
+QString KApplicationModel::entryPathFor(const QModelIndex &index) const
 {
     if (!index.isValid())
         return QString();
 
     KDEPrivate::AppNode *node = static_cast<KDEPrivate::AppNode*>(index.internalPointer());
-    return node->text;
+    return node->entryPath;
 }
 
 QString KApplicationModel::execFor(const QModelIndex &index) const
@@ -391,7 +395,7 @@ void KApplicationView::currentChanged(const QModelIndex &current, const QModelIn
     if (d->appModel && !d->appModel->isDirectory(current)) {
         QString exec = d->appModel->execFor(current);
         if (!exec.isEmpty()) {
-            emit highlighted(d->appModel->nameFor(current), exec);
+            emit highlighted(d->appModel->entryPathFor(current), exec);
         }
     }
 }
@@ -404,7 +408,7 @@ void KApplicationView::slotSelectionChanged(const QItemSelection &selected, cons
     if (indexes.count() == 1 && !d->appModel->isDirectory(indexes.at(0))) {
         QString exec = d->appModel->execFor(indexes.at(0));
         if (!exec.isEmpty()) {
-            emit this->selected(d->appModel->nameFor(indexes.at(0)), exec);
+            emit this->selected(d->appModel->entryPathFor(indexes.at(0)), exec);
         }
     }
 }
@@ -442,9 +446,19 @@ public:
      */
     void init(const QString &text, const QString &value);
 
+    /**
+     * Called by checkAccept() in order to save the history of the combobox
+     */
+    void saveComboboxHistory();
+
+    /**
+     * Process the choices made by the user, and return true if everything is OK.
+     * Called by KOpenWithDialog::accept(), i.e. when clicking on OK or typing Return.
+     */
+    bool checkAccept();
+
     // slots
     void _k_slotDbClick();
-    void _k_slotOK();
 
     bool saveNewApps;
     bool m_terminaldirty;
@@ -453,7 +467,6 @@ public:
     KUrlRequester *edit;
     QString m_command;
     QLabel *label;
-    QString qName;
     QString qMimeType;
     QCheckBox *terminal;
     QCheckBox *remember;
@@ -545,16 +558,16 @@ void KOpenWithDialogPrivate::init(const QString &_text, const QString &_value)
   bool bReadOnly = !KAuthorized::authorize("shell_access");
   m_terminaldirty = false;
     view = 0;
-  m_pService = 0L;
-    curService = 0L;
+    m_pService = 0;
+    curService = 0;
 
     q->setButtons(KDialog::Ok | KDialog::Cancel);
+    q->showButtonSeparator(true);
 
     QWidget *mainWidget = q->mainWidget();
 
   QBoxLayout *topLayout = new QVBoxLayout( mainWidget );
   topLayout->setMargin(0);
-  topLayout->setSpacing( KDialog::spacingHint() );
     label = new QLabel(_text, q);
   label->setWordWrap(true);
   topLayout->addWidget(label);
@@ -583,7 +596,7 @@ void KOpenWithDialogPrivate::init(const QString &_text, const QString &_value)
     edit->button()->hide();
   }
 
-  edit->setPath( _value );
+  edit->setText( _value );
   edit->setWhatsThis(i18n(
     "Following the command, you can have several place holders which will be replaced "
     "with the actual values when the actual program is run:\n"
@@ -605,13 +618,12 @@ void KOpenWithDialogPrivate::init(const QString &_text, const QString &_value)
     edit->comboBox()->setAutoDeleteCompletionObject( true );
   }
 
-    QObject::connect(edit, SIGNAL(returnPressed()), q, SLOT(_k_slotOK()));
-    QObject::connect(q, SIGNAL(okClicked()), q, SLOT(_k_slotOK()));
     QObject::connect(edit, SIGNAL(textChanged(QString)), q, SLOT(slotTextChanged()));
 
     view = new KApplicationView(mainWidget);
     view->setModel(new KApplicationModel(view));
     topLayout->addWidget(view);
+    topLayout->setStretchFactor(view, 1);
 
     QObject::connect(view, SIGNAL(selected(QString, QString)),
                      q, SLOT(slotSelected(QString, QString)));
@@ -627,10 +639,14 @@ void KOpenWithDialogPrivate::init(const QString &_text, const QString &_value)
 
   topLayout->addWidget(terminal);
 
+  QStyleOptionButton checkBoxOption;
+  checkBoxOption.initFrom(terminal);
+  int checkBoxIndentation = terminal->style()->pixelMetric( QStyle::PM_IndicatorWidth, &checkBoxOption, terminal );
+  checkBoxIndentation += terminal->style()->pixelMetric( QStyle::PM_CheckBoxLabelSpacing, &checkBoxOption, terminal );
+
   QBoxLayout* nocloseonexitLayout = new QHBoxLayout();
   nocloseonexitLayout->setMargin( 0 );
-  nocloseonexitLayout->setSpacing( KDialog::spacingHint() );
-  QSpacerItem* spacer = new QSpacerItem( 20, 0, QSizePolicy::Fixed, QSizePolicy::Minimum );
+  QSpacerItem* spacer = new QSpacerItem( checkBoxIndentation, 0, QSizePolicy::Fixed, QSizePolicy::Minimum );
   nocloseonexitLayout->addItem( spacer );
 
   nocloseonexit = new QCheckBox( i18n("&Do not close when command exits"), mainWidget );
@@ -657,6 +673,7 @@ void KOpenWithDialogPrivate::init(const QString &_text, const QString &_value)
   else
     remember = 0L;
 
+  q->setMinimumSize(q->minimumSizeHint());
   //edit->setText( _value );
   // This is what caused "can't click on items before clicking on Name header".
   // Probably due to the resizeEvent handler using width().
@@ -678,20 +695,17 @@ KOpenWithDialog::~KOpenWithDialog()
 
 void KOpenWithDialog::slotSelected( const QString& /*_name*/, const QString& _exec )
 {
-    kDebug(250)<<"KOpenWithDialog::slotSelected";
     KService::Ptr pService = d->curService;
-    d->edit->setPath(_exec); // calls slotTextChanged :(
+    d->edit->setText(_exec); // calls slotTextChanged :(
     d->curService = pService;
 }
 
 
 // ----------------------------------------------------------------------
 
-void KOpenWithDialog::slotHighlighted( const QString& _name, const QString& )
+void KOpenWithDialog::slotHighlighted(const QString& entryPath, const QString&)
 {
-    kDebug(250)<<"KOpenWithDialog::slotHighlighted";
-    d->qName = _name;
-    d->curService = KService::serviceByName(d->qName);
+    d->curService = KService::serviceByDesktopPath(entryPath);
     if (!d->m_terminaldirty)
     {
         // ### indicate that default value was restored
@@ -706,10 +720,9 @@ void KOpenWithDialog::slotHighlighted( const QString& _name, const QString& )
 
 void KOpenWithDialog::slotTextChanged()
 {
-    kDebug(250)<<"KOpenWithDialog::slotTextChanged" << d->edit->url();
     // Forget about the service
     d->curService = 0L;
-    enableButton(Ok, !d->edit->url().isEmpty());
+    enableButton(Ok, !d->edit->text().isEmpty());
 }
 
 // ----------------------------------------------------------------------
@@ -729,7 +742,7 @@ void KOpenWithDialogPrivate::_k_slotDbClick()
     if (view->isDirSel()) {
         return;
     }
-    _k_slotOK();
+    q->accept();
 }
 
 void KOpenWithDialog::setSaveNewApplications(bool b)
@@ -773,29 +786,32 @@ void KOpenWithDialogPrivate::addToMimeAppsList(const QString& serviceId /*menu i
     Q_ASSERT( m_pService );
 }
 
-void KOpenWithDialogPrivate::_k_slotOK()
+bool KOpenWithDialogPrivate::checkAccept()
 {
-    QString typedExec(edit->url().pathOrUrl());
+    const QString typedExec(edit->text());
     if (typedExec.isEmpty())
-        return;
+        return false;
     QString fullExec(typedExec);
 
     QString serviceName;
     QString initialServiceName;
     QString preferredTerminal;
+    QString binaryName;
     m_pService = curService;
     if (!m_pService) {
         // No service selected - check the command line
 
         // Find out the name of the service from the command line, removing args and paths
         serviceName = KRun::binaryName( typedExec, true );
-        if (serviceName.isEmpty())
-        {
-            // TODO add a KMessageBox::error here after the end of the message freeze
-            return;
+        if (serviceName.isEmpty()) {
+            KMessageBox::error(q, i18n("Could not extract executable name from '%1', please type a valid program name.", serviceName));
+            return false;
         }
         initialServiceName = serviceName;
-        kDebug(250) << "initialServiceName=" << initialServiceName;
+        // Also remember the binaryName with a path, if any, for the
+        // check that the binary exists.
+        binaryName = KRun::binaryName(typedExec, false);
+        kDebug(250) << "initialServiceName=" << initialServiceName << "binaryName=" << binaryName;
         int i = 1; // We have app, app-2, app-3... Looks better for the user.
         bool ok = false;
         // Check if there's already a service by that name, with the same Exec line
@@ -804,12 +820,20 @@ void KOpenWithDialogPrivate::_k_slotOK()
             KService::Ptr serv = KService::serviceByDesktopName( serviceName );
             ok = !serv; // ok if no such service yet
             // also ok if we find the exact same service (well, "kwrite" == "kwrite %U")
-            if (serv && serv->isApplication()) {
-                fullExec = serv->exec();
-                if (typedExec == simplifiedExecLineFromService(fullExec)) {
-                    ok = true;
-                    m_pService = serv;
-                    kDebug(250) << "OK, found identical service: " << serv->entryPath();
+            if (serv) {
+                if (serv->isApplication()) {
+                    /*kDebug(250) << "typedExec=" << typedExec
+                      << "serv->exec=" << serv->exec()
+                      << "simplifiedExecLineFromService=" << simplifiedExecLineFromService(fullExec);*/
+                    if (typedExec == simplifiedExecLineFromService(serv->exec())) {
+                        ok = true;
+                        m_pService = serv;
+                        kDebug(250) << "OK, found identical service: " << serv->entryPath();
+                    } else {
+                        kDebug(250) << "Exec line differs, service says:" << simplifiedExecLineFromService(fullExec);
+                    }
+                } else {
+                    kDebug(250) << "Found, but not an application:" << serv->entryPath();
                 }
             }
             if (!ok) { // service was found, but it was different -> keep looking
@@ -823,6 +847,12 @@ void KOpenWithDialogPrivate::_k_slotOK()
         serviceName = m_pService->name();
         initialServiceName = serviceName;
         fullExec = m_pService->exec();
+    } else {
+        // Ensure that the typed binary name actually exists (#81190)
+        if (KStandardDirs::findExe(binaryName).isEmpty()) {
+            KMessageBox::error(q, i18n("'%1' not found, please type a valid program name.", binaryName));
+            return false;
+        }
     }
 
     if (terminal->isChecked()) {
@@ -833,7 +863,7 @@ void KOpenWithDialogPrivate::_k_slotOK()
         if (preferredTerminal == "konsole" && nocloseonexit->isChecked())
             m_command += QString::fromLatin1(" --noclose");
         m_command += QString::fromLatin1(" -e ");
-        m_command += edit->url().pathOrUrl();
+        m_command += edit->text();
         kDebug(250) << "Setting m_command to" << m_command;
     }
     if ( m_pService && terminal->isChecked() != m_pService->terminal() )
@@ -864,7 +894,7 @@ void KOpenWithDialogPrivate::_k_slotOK()
 
             QString menuId;
             QString newPath = KService::newServicePath(false /* ignored argument */, serviceName, &menuId);
-            kDebug(250) << "Creating new service" << serviceName << "(" << newPath << ")";
+            kDebug(250) << "Creating new service" << serviceName << "(" << newPath << ")" << "menuId=" << menuId;
 
             KDesktopFile desktopFile(newPath);
             KConfigGroup cg = desktopFile.desktopGroup();
@@ -884,7 +914,15 @@ void KOpenWithDialogPrivate::_k_slotOK()
             addToMimeAppsList(menuId);
         }
     }
-    q->accept();
+
+    saveComboboxHistory();
+    return true;
+}
+
+void KOpenWithDialog::accept()
+{
+    if (d->checkAccept())
+        KDialog::accept();
 }
 
 QString KOpenWithDialog::text() const
@@ -892,7 +930,7 @@ QString KOpenWithDialog::text() const
     if (!d->m_command.isEmpty())
         return d->m_command;
     else
-        return d->edit->url().url();
+        return d->edit->text();
 }
 
 void KOpenWithDialog::hideNoCloseOnExit()
@@ -913,11 +951,11 @@ KService::Ptr KOpenWithDialog::service() const
     return d->m_pService;
 }
 
-void KOpenWithDialog::accept()
+void KOpenWithDialogPrivate::saveComboboxHistory()
 {
-    KHistoryComboBox *combo = static_cast<KHistoryComboBox*>(d->edit->comboBox());
-    if ( combo ) {
-        combo->addToHistory(d->edit->url().url());
+    KHistoryComboBox *combo = static_cast<KHistoryComboBox*>(edit->comboBox());
+    if (combo) {
+        combo->addToHistory(edit->text());
 
         KConfigGroup cg( KGlobal::config(), QString::fromLatin1("Open-with settings") );
         cg.writeEntry( "History", combo->historyItems() );
@@ -926,8 +964,6 @@ void KOpenWithDialog::accept()
         // executables
         cg.sync();
     }
-
-    QDialog::accept();
 }
 
 #include "kopenwithdialog.moc"
