@@ -101,6 +101,23 @@ namespace KIO {
          */
         void setDefaultPermissions( bool b );
 
+        /**
+         * Skip copying or moving any file when the destination already exists,
+         * instead of the default behavior (interactive mode: showing a dialog to the user,
+         * non-interactive mode: aborting with an error).
+         * Initially added for a unit test.
+         * \since 4.2
+         */
+        void setAutoSkip(bool autoSkip);
+
+        /**
+         * Reuse any directory that already exists, instead of the default behavior
+         * (interactive mode: showing a dialog to the user,
+         * non-interactive mode: aborting with an error).
+         * \since 4.2
+         */
+        void setWriteIntoExistingDirectories(bool overwriteAllDirs);
+
         /**
          * Reimplemented for internal reasons
          */
@@ -228,6 +245,7 @@ namespace KIO {
 
     protected:
         CopyJob(CopyJobPrivate &dd);
+        void emitResult();
 
     private:
         Q_PRIVATE_SLOT(d_func(), void slotStart())
