@@ -71,11 +71,40 @@ static inline Slave *jobSlave(SimpleJob *job)
     return SimpleJobPrivate::get(job)->m_slave;
 }
 
+// Returns true if the scheme and domain of the two urls match...
+static bool domainSchemeMatch(const KUrl& u1, const KUrl& u2)
+{
+    if (u1.scheme() != u2.scheme())
+        return false;
+
+    QStringList u1List = u1.host().split(QLatin1Char('.'), QString::SkipEmptyParts);
+    QStringList u2List = u2.host().split(QLatin1Char('.'), QString::SkipEmptyParts);
+
+    if (qMin(u1List.count(), u2List.count()) < 2)
+        return false;  // better safe than sorry...
+
+    while (u1List.count() > 2)
+        u1List.removeFirst();
+
+    while (u2List.count() > 2)
+        u2List.removeFirst();
+
+    return (u1List == u2List);
+}
+
+static void clearNonSSLMetaData(KIO::MetaData& metaData)
+{
+    QMutableMapIterator<QString, QString> it (metaData);
+    while (it.hasNext()) {
+        it.next();
+        if (!it.key().startsWith(QLatin1String("ssl_"), Qt::CaseInsensitive))
+            it.remove();
+    }
+}
+
 //this will update the report dialog with 5 Hz, I think this is fast enough, aleXXX
 #define REPORT_TIMEOUT 200
 
-#define KIO_ARGS QByteArray packedArgs; QDataStream stream( &packedArgs, QIODevice::WriteOnly ); stream
-
 Job::Job() : KCompositeJob(*new JobPrivate, 0)
 {
     setCapabilities( KJob::Killable | KJob::Suspendable );
@@ -127,39 +156,39 @@ bool Job::removeSubjob( KJob *jobBase )
 void JobPrivate::emitMoving(KIO::Job * job, const KUrl &src, const KUrl &dest)
 {
     emit job->description(job, i18nc("@title job","Moving"),
-                          qMakePair(i18nc("The source of a file operation", "Source"), src.prettyUrl()),
-                          qMakePair(i18nc("The destination of a file operation", "Destination"), dest.prettyUrl()));
+                          qMakePair(i18nc("The source of a file operation", "Source"), src.pathOrUrl()),
+                          qMakePair(i18nc("The destination of a file operation", "Destination"), dest.pathOrUrl()));
 }
 
 void JobPrivate::emitCopying(KIO::Job * job, const KUrl &src, const KUrl &dest)
 {
     emit job->description(job, i18nc("@title job","Copying"),
-                          qMakePair(i18nc("The source of a file operation", "Source"), src.prettyUrl()),
-                          qMakePair(i18nc("The destination of a file operation", "Destination"), dest.prettyUrl()));
+                          qMakePair(i18nc("The source of a file operation", "Source"), src.pathOrUrl()),
+                          qMakePair(i18nc("The destination of a file operation", "Destination"), dest.pathOrUrl()));
 }
 
 void JobPrivate::emitCreatingDir(KIO::Job * job, const KUrl &dir)
 {
     emit job->description(job, i18nc("@title job","Creating directory"),
-                          qMakePair(i18n("Directory"), dir.prettyUrl()));
+                          qMakePair(i18n("Directory"), dir.pathOrUrl()));
 }
 
 void JobPrivate::emitDeleting(KIO::Job *job, const KUrl &url)
 {
     emit job->description(job, i18nc("@title job","Deleting"),
-                          qMakePair(i18n("File"), url.prettyUrl()));
+                          qMakePair(i18n("File"), url.pathOrUrl()));
 }
 
 void JobPrivate::emitStating(KIO::Job *job, const KUrl &url)
 {
     emit job->description(job, i18nc("@title job","Examining"),
-                          qMakePair(i18n("File"), url.prettyUrl()));
+                          qMakePair(i18n("File"), url.pathOrUrl()));
 }
 
 void JobPrivate::emitTransferring(KIO::Job *job, const KUrl &url)
 {
     emit job->description(job, i18nc("@title job","Transferring"),
-                          qMakePair(i18nc("The source of a file operation", "Source"), url.prettyUrl()));
+                          qMakePair(i18nc("The source of a file operation", "Source"), url.pathOrUrl()));
 }
 
 void JobPrivate::emitMounting(KIO::Job * job, const QString &dev, const QString &point)
@@ -358,6 +387,17 @@ void SimpleJob::removeOnHold()
     Scheduler::removeSlaveOnHold();
 }
 
+bool SimpleJob::isRedirectionHandlingEnabled() const
+{
+    return d_func()->m_redirectionHandlingEnabled;
+}
+
+void SimpleJob::setRedirectionHandlingEnabled(bool handle)
+{
+    Q_D(SimpleJob);
+    d->m_redirectionHandlingEnabled = handle;
+}
+
 SimpleJob::~SimpleJob()
 {
     Q_D(SimpleJob);
@@ -592,7 +632,7 @@ void MkdirJobPrivate::slotRedirection( const KUrl &url)
      {
          kWarning(7007) << "Redirection from" << m_url << "to" << url << "REJECTED!";
          q->setError( ERR_ACCESS_DENIED );
-         q->setErrorText( url.prettyUrl() );
+         q->setErrorText( url.pathOrUrl() );
          return;
      }
      m_redirectionURL = url; // We'll remember that when the job finishes
@@ -605,29 +645,35 @@ void MkdirJobPrivate::slotRedirection( const KUrl &url)
 void MkdirJob::slotFinished()
 {
     Q_D(MkdirJob);
-    if ( d->m_redirectionURL.isEmpty() || !d->m_redirectionURL.isValid())
+
+    if (!d->m_redirectionURL.isEmpty() && d->m_redirectionURL.isValid())
     {
-        // Return slave to the scheduler
-        SimpleJob::slotFinished();
-    } else {
         //kDebug(7007) << "MkdirJob: Redirection to " << m_redirectionURL;
         if (queryMetaData("permanent-redirect")=="true")
             emit permanentRedirection(this, d->m_url, d->m_redirectionURL);
-        KUrl dummyUrl;
-        int permissions;
-        QDataStream istream( d->m_packedArgs );
-        istream >> dummyUrl >> permissions;
 
-        d->m_url = d->m_redirectionURL;
-        d->m_redirectionURL = KUrl();
-        d->m_packedArgs.truncate(0);
-        QDataStream stream( &d->m_packedArgs, QIODevice::WriteOnly );
-        stream << d->m_url << permissions;
+        if ( d->m_redirectionHandlingEnabled )
+        {
+            KUrl dummyUrl;
+            int permissions;
+            QDataStream istream( d->m_packedArgs );
+            istream >> dummyUrl >> permissions;
 
-        // Return slave to the scheduler
-        d->slaveDone();
-        Scheduler::doJob(this);
+            d->m_url = d->m_redirectionURL;
+            d->m_redirectionURL = KUrl();
+            d->m_packedArgs.truncate(0);
+            QDataStream stream( &d->m_packedArgs, QIODevice::WriteOnly );
+            stream << d->m_url << permissions;
+
+            // Return slave to the scheduler
+            d->slaveDone();
+            Scheduler::doJob(this);
+            return;
+        }
     }
+
+    // Return slave to the scheduler
+    SimpleJob::slotFinished();
 }
 
 SimpleJob *KIO::mkdir( const KUrl& url, int permissions )
@@ -775,6 +821,17 @@ const UDSEntry & StatJob::statResult() const
     return d_func()->m_statResult;
 }
 
+KUrl StatJob::mostLocalUrl() const
+{
+    if (!url().isLocalFile()) {
+        const UDSEntry& udsEntry = d_func()->m_statResult;
+        const QString path = udsEntry.stringValue( KIO::UDSEntry::UDS_LOCAL_PATH );
+        if (!path.isEmpty())
+            return KUrl(path);
+    }
+    return url();
+}
+
 void StatJobPrivate::start(Slave *slave)
 {
     Q_Q(StatJob);
@@ -804,7 +861,7 @@ void StatJobPrivate::slotRedirection( const KUrl &url)
      {
        kWarning(7007) << "Redirection from " << m_url << " to " << url << " REJECTED!";
        q->setError( ERR_ACCESS_DENIED );
-       q->setErrorText( url.prettyUrl() );
+       q->setErrorText( url.pathOrUrl() );
        return;
      }
      m_redirectionURL = url; // We'll remember that when the job finishes
@@ -817,24 +874,30 @@ void StatJobPrivate::slotRedirection( const KUrl &url)
 void StatJob::slotFinished()
 {
     Q_D(StatJob);
-    if ( d->m_redirectionURL.isEmpty() || !d->m_redirectionURL.isValid())
+
+    if ( !d->m_redirectionURL.isEmpty() && d->m_redirectionURL.isValid() )
     {
-        // Return slave to the scheduler
-        SimpleJob::slotFinished();
-    } else {
         //kDebug(7007) << "StatJob: Redirection to " << m_redirectionURL;
         if (queryMetaData("permanent-redirect")=="true")
             emit permanentRedirection(this, d->m_url, d->m_redirectionURL);
-        d->m_url = d->m_redirectionURL;
-        d->m_redirectionURL = KUrl();
-        d->m_packedArgs.truncate(0);
-        QDataStream stream( &d->m_packedArgs, QIODevice::WriteOnly );
-        stream << d->m_url;
 
-        // Return slave to the scheduler
-        d->slaveDone();
-        Scheduler::doJob(this);
+        if ( d->m_redirectionHandlingEnabled )
+        {
+            d->m_url = d->m_redirectionURL;
+            d->m_redirectionURL = KUrl();
+            d->m_packedArgs.truncate(0);
+            QDataStream stream( &d->m_packedArgs, QIODevice::WriteOnly );
+            stream << d->m_url;
+
+            // Return slave to the scheduler
+            d->slaveDone();
+            Scheduler::doJob(this);
+            return;
+        }
     }
+
+    // Return slave to the scheduler
+    SimpleJob::slotFinished();
 }
 
 void StatJob::slotMetaData( const KIO::MetaData &_metaData)
@@ -850,6 +913,17 @@ StatJob *KIO::stat(const KUrl& url, JobFlags flags)
     return stat( url, StatJob::SourceSide, 2, flags );
 }
 
+StatJob *KIO::mostLocalUrl(const KUrl& url, JobFlags flags)
+{
+    StatJob* job = stat( url, StatJob::SourceSide, 2, flags );
+    if (url.isLocalFile()) {
+        QTimer::singleShot(0, job, SLOT(slotFinished()));
+        Scheduler::cancelJob( job ); // deletes the slave if not 0
+        //job->d_func()->m_slave = 0; // -> set to 0
+    }
+    return job;
+}
+
 StatJob *KIO::stat(const KUrl& url, bool sideIsSource, short int details, JobFlags flags )
 {
     //kDebug(7007) << "stat" << url;
@@ -899,6 +973,13 @@ TransferJob::~TransferJob()
 void TransferJob::slotData( const QByteArray &_data)
 {
     Q_D(TransferJob);
+    if (d->m_command == CMD_GET && !d->m_isMimetypeEmitted) {
+        kWarning(7007) << "mimetype() not emitted when sending first data!; job URL ="
+                       << d->m_url << "data size =" << _data.size();
+    }
+    // shut up the warning, HACK: downside is that it changes the meaning of the variable
+    d->m_isMimetypeEmitted = true;
+
     if(d->m_redirectionURL.isEmpty() || !d->m_redirectionURL.isValid() || error())
       emit data( this, _data);
 }
@@ -926,7 +1007,7 @@ void TransferJob::slotRedirection( const KUrl &url)
     {
        kDebug(7007) << "CYCLIC REDIRECTION!";
        setError( ERR_CYCLIC_LINK );
-       setErrorText( d->m_url.prettyUrl() );
+       setErrorText( d->m_url.pathOrUrl() );
     }
     else
     {
@@ -944,62 +1025,76 @@ void TransferJob::slotFinished()
 {
     Q_D(TransferJob);
     //kDebug(7007) << this << "," << m_url;
-    if (d->m_redirectionURL.isEmpty() || !d->m_redirectionURL.isValid())
-        SimpleJob::slotFinished();
-    else {
+
+    if (!d->m_redirectionURL.isEmpty() && d->m_redirectionURL.isValid()) {
         //kDebug(7007) << "Redirection to" << m_redirectionURL;
         if (queryMetaData("permanent-redirect")=="true")
             emit permanentRedirection(this, d->m_url, d->m_redirectionURL);
-        // Honour the redirection
-        // We take the approach of "redirecting this same job"
-        // Another solution would be to create a subjob, but the same problem
-        // happens (unpacking+repacking)
-        d->staticData.truncate(0);
-        d->m_incomingMetaData.clear();
-        if (queryMetaData("cache") != "reload")
-            addMetaData("cache","refresh");
-        d->m_internalSuspended = false;
-        d->m_url = d->m_redirectionURL;
-        d->m_redirectionURL = KUrl();
-        // The very tricky part is the packed arguments business
-        QString dummyStr;
-        KUrl dummyUrl;
-        QDataStream istream( d->m_packedArgs );
-        switch( d->m_command ) {
-            case CMD_GET: {
-                d->m_packedArgs.truncate(0);
-                QDataStream stream( &d->m_packedArgs, QIODevice::WriteOnly );
-                stream << d->m_url;
-                break;
-            }
-            case CMD_PUT: {
-                int permissions;
-                qint8 iOverwrite, iResume;
-                istream >> dummyUrl >> iOverwrite >> iResume >> permissions;
-                d->m_packedArgs.truncate(0);
-                QDataStream stream( &d->m_packedArgs, QIODevice::WriteOnly );
-                stream << d->m_url << iOverwrite << iResume << permissions;
-                break;
-            }
-            case CMD_SPECIAL: {
-                int specialcmd;
-                istream >> specialcmd;
-                if (specialcmd == 1) // HTTP POST
-                {
-                   addMetaData("cache","reload");
-                   d->m_packedArgs.truncate(0);
-                   QDataStream stream( &d->m_packedArgs, QIODevice::WriteOnly );
-                   stream << d->m_url;
-                   d->m_command = CMD_GET;
-                }
-                break;
-            }
-        }
 
-        // Return slave to the scheduler
-        d->slaveDone();
-        Scheduler::doJob(this);
+        if ( d->m_redirectionHandlingEnabled ) {
+            // Honour the redirection
+            // We take the approach of "redirecting this same job"
+            // Another solution would be to create a subjob, but the same problem
+            // happens (unpacking+repacking)
+            d->staticData.truncate(0);            
+
+            // When appropriate, retain SSL meta-data information on redirection.
+            if (d->m_incomingMetaData.contains("ssl_in_use") &&
+                domainSchemeMatch(d->m_url, d->m_redirectionURL)) {
+                clearNonSSLMetaData(d->m_incomingMetaData);
+            } else {
+                d->m_incomingMetaData.clear();
+            }
+
+            if (queryMetaData("cache") != "reload")
+                addMetaData("cache","refresh");
+            d->m_internalSuspended = false;
+            d->m_url = d->m_redirectionURL;
+            d->m_redirectionURL = KUrl();
+            // The very tricky part is the packed arguments business
+            QString dummyStr;
+            KUrl dummyUrl;
+            QDataStream istream( d->m_packedArgs );
+            switch( d->m_command ) {
+                case CMD_GET: {
+                    d->m_packedArgs.truncate(0);
+                    QDataStream stream( &d->m_packedArgs, QIODevice::WriteOnly );
+                    stream << d->m_url;
+                    break;
+                }
+                case CMD_PUT: {
+                    int permissions;
+                    qint8 iOverwrite, iResume;
+                    istream >> dummyUrl >> iOverwrite >> iResume >> permissions;
+                    d->m_packedArgs.truncate(0);
+                    QDataStream stream( &d->m_packedArgs, QIODevice::WriteOnly );
+                    stream << d->m_url << iOverwrite << iResume << permissions;
+                    break;
+                }
+                case CMD_SPECIAL: {
+                    int specialcmd;
+                    istream >> specialcmd;
+                    if (specialcmd == 1) // HTTP POST
+                    {
+                       addMetaData("cache","reload");
+                       d->m_packedArgs.truncate(0);
+                       QDataStream stream( &d->m_packedArgs, QIODevice::WriteOnly );
+                       stream << d->m_url;
+                       d->m_command = CMD_GET;
+                    }
+                    break;
+                }
+            }
+
+            // Return slave to the scheduler
+            d->slaveDone();
+            Scheduler::doJob(this);
+            return;
+        }
     }
+
+    // Return slave to the scheduler
+    SimpleJob::slotFinished();
 }
 
 void TransferJob::setAsyncDataEnabled(bool enabled)
@@ -1090,6 +1185,11 @@ void TransferJob::slotMimetype( const QString& type )
 {
     Q_D(TransferJob);
     d->m_mimetype = type;
+    if (d->m_command == CMD_GET && d->m_isMimetypeEmitted) {
+        kWarning(7007) << "mimetype() emitted again, or after sending first data!; job URL ="
+                       << d->m_url;
+    }
+    d->m_isMimetypeEmitted = true;
     emit mimetype( this, type );
 }
 
@@ -1341,12 +1441,16 @@ static KIO::PostErrorJob* precheckHttpPost( const KUrl& url, const QByteArray& p
         6000, // x11
         6667, // irc
         0};
-    for (int cnt=0; bad_ports[cnt]; ++cnt)
-        if (url.port() == bad_ports[cnt])
-        {
-            _error = KIO::ERR_POST_DENIED;
-            break;
-        }
+    if (url.port() != 80)
+    {
+        const int port = url.port();
+        for (int cnt=0; bad_ports[cnt] && bad_ports[cnt] <= port; ++cnt)
+            if (port == bad_ports[cnt])
+            {
+                _error = KIO::ERR_POST_DENIED;
+                break;
+            }
+    }
 
     if ( _error )
     {
@@ -1378,7 +1482,7 @@ static KIO::PostErrorJob* precheckHttpPost( const KUrl& url, const QByteArray& p
     if (_error)
     {
         KIO_ARGS << (int)1 << url;
-        PostErrorJob * job = new PostErrorJob(_error, url.prettyUrl(), packedArgs, postData);
+        PostErrorJob * job = new PostErrorJob(_error, url.pathOrUrl(), packedArgs, postData);
         job->setUiDelegate(new JobUiDelegate());
         if (!(flags & HideProgressInfo)) {
             KIO::getJobTracker()->registerJob(job);
@@ -1578,26 +1682,30 @@ void MimetypeJob::slotFinished( )
         emit TransferJob::mimetype( this, d->m_mimetype );
         setError( 0 );
     }
-    if ( d->m_redirectionURL.isEmpty() || !d->m_redirectionURL.isValid() || error() )
-    {
-        // Return slave to the scheduler
-        TransferJob::slotFinished();
-    } else {
+
+    if ( !error() && !d->m_redirectionURL.isEmpty() && d->m_redirectionURL.isValid() ) {
         //kDebug(7007) << "Redirection to " << m_redirectionURL;
         if (queryMetaData("permanent-redirect")=="true")
             emit permanentRedirection(this, d->m_url, d->m_redirectionURL);
-        d->staticData.truncate(0);
-        d->m_internalSuspended = false;
-        d->m_url = d->m_redirectionURL;
-        d->m_redirectionURL = KUrl();
-        d->m_packedArgs.truncate(0);
-        QDataStream stream( &d->m_packedArgs, QIODevice::WriteOnly );
-        stream << d->m_url;
 
-        // Return slave to the scheduler
-        d->slaveDone();
-        Scheduler::doJob(this);
+        if ( d->m_redirectionHandlingEnabled ) {
+            d->staticData.truncate(0);
+            d->m_internalSuspended = false;
+            d->m_url = d->m_redirectionURL;
+            d->m_redirectionURL = KUrl();
+            d->m_packedArgs.truncate(0);
+            QDataStream stream( &d->m_packedArgs, QIODevice::WriteOnly );
+            stream << d->m_url;
+
+            // Return slave to the scheduler
+            d->slaveDone();
+            Scheduler::doJob(this);
+            return;
+        }
     }
+
+    // Return slave to the scheduler
+    TransferJob::slotFinished();
 }
 
 MimetypeJob *KIO::mimetype(const KUrl& url, JobFlags flags)
@@ -1732,7 +1840,7 @@ public:
 };
 
 /*
- * The FileCopyJob works according to the famous Bayern
+ * The FileCopyJob works according to the famous Bavarian
  * 'Alternating Bitburger Protocol': we either drink a beer or we
  * we order a beer, but never both at the same time.
  * Translated to io-slaves: We alternate between receiving a block of data
@@ -1803,6 +1911,8 @@ void FileCopyJobPrivate::startBestCopyMethod()
    else
    {
       startDataPump();
+      if (m_putJob)
+        SimpleJobPrivate::get(m_putJob)->m_pairedUrl = m_src;
    }
 }
 
@@ -2410,24 +2520,28 @@ void ListJob::slotFinished()
             }
         }
     }
-    if ( d->m_redirectionURL.isEmpty() || !d->m_redirectionURL.isValid() || error() ) {
-        // Return slave to the scheduler
-        SimpleJob::slotFinished();
-    } else {
+    if ( !d->m_redirectionURL.isEmpty() && d->m_redirectionURL.isValid() && !error() ) {
 
         //kDebug(7007) << "Redirection to " << d->m_redirectionURL;
         if (queryMetaData("permanent-redirect")=="true")
             emit permanentRedirection(this, d->m_url, d->m_redirectionURL);
-        d->m_url = d->m_redirectionURL;
-        d->m_redirectionURL = KUrl();
-        d->m_packedArgs.truncate(0);
-        QDataStream stream( &d->m_packedArgs, QIODevice::WriteOnly );
-        stream << d->m_url;
 
-        // Return slave to the scheduler
-        d->slaveDone();
-        Scheduler::doJob(this);
+        if ( d->m_redirectionHandlingEnabled ) {
+            d->m_url = d->m_redirectionURL;
+            d->m_redirectionURL = KUrl();
+            d->m_packedArgs.truncate(0);
+            QDataStream stream( &d->m_packedArgs, QIODevice::WriteOnly );
+            stream << d->m_url;
+
+            // Return slave to the scheduler
+            d->slaveDone();
+            Scheduler::doJob(this);
+            return;
+        }
     }
+
+    // Return slave to the scheduler
+    SimpleJob::slotFinished();
 }
 
 void ListJob::slotMetaData( const KIO::MetaData &_metaData)
@@ -2589,7 +2703,7 @@ void MultiGetJobPrivate::start(Slave *slave)
 
    m_url = entry.url;
 
-   if (!entry.url.protocol().startsWith("http"))
+   if (!entry.url.protocol().startsWith(QLatin1String("http")))
    {
       // Use normal get
       KIO_ARGS << entry.url;
@@ -2661,7 +2775,15 @@ void MultiGetJob::slotFinished()
   }
   d->m_redirectionURL = KUrl();
   setError( 0 );
-  d->m_incomingMetaData.clear();
+
+  // When appropriate, retain SSL meta-data information on redirection.
+  if (d->m_incomingMetaData.contains("ssl_in_use") &&
+      domainSchemeMatch(d->m_url, d->m_redirectionURL)) {
+      clearNonSSLMetaData(d->m_incomingMetaData);
+  } else {
+      d->m_incomingMetaData.clear();
+  }
+
   d->m_activeQueue.removeAll(d->m_currentEntry);
   if (d->m_activeQueue.count() == 0)
   {
