@@ -101,7 +101,9 @@ class KIO_EXPORT KDirWatch : public QObject
     * directory are watched for changes, too. Thus, the signals dirty(),
     * created(), deleted() can be emitted.
     * When @p watchModes is set to WatchSubDirs, all subdirs are watched using
-    * the same flags specified in @p watchModes .
+    * the same flags specified in @p watchModes (symlinks aren't followed).
+    * If the @p path points to a symlink to a directory, the target directory
+    * is watched instead. If you want to watch the link, use @p addFile().
     *
     * @param path the path to watch
     * @param watchModes watch modes
@@ -112,6 +114,7 @@ class KIO_EXPORT KDirWatch : public QObject
 
    /**
     * Adds a file to be watched.
+    * If it's a symlink to a directory, it watches the symlink itself.
     * @param file the file to watch
     */
    void addFile(const QString& file);
@@ -213,22 +216,6 @@ class KIO_EXPORT KDirWatch : public QObject
     */
    static void statistics();
 
-   /**
-    * Emits created().
-    * @param path the path of the file or directory
-    */
-   void setCreated( const QString &path );
-   /**
-    * Emits dirty().
-    * @param path the path of the file or directory
-    */
-   void setDirty( const QString &path );
-   /**
-    * Emits deleted().
-    * @param path the path of the file or directory
-    */
-   void setDeleted( const QString &path );
-
    enum Method { FAM, INotify, DNotify, Stat };
    /**
     * Returns the preferred internal method to
@@ -256,6 +243,26 @@ class KIO_EXPORT KDirWatch : public QObject
     */
    static bool exists();
 
+public Q_SLOTS: 
+
+   /**
+    * Emits created().
+    * @param path the path of the file or directory
+    */
+   void setCreated( const QString &path );
+    
+   /**
+    * Emits dirty().
+    * @param path the path of the file or directory
+    */
+   void setDirty( const QString &path );
+
+   /**
+    * Emits deleted().
+    * @param path the path of the file or directory
+    */
+   void setDeleted( const QString &path );
+
  Q_SIGNALS:
 
    /**
