@@ -35,6 +35,7 @@
 #include <QtCore/QDir>
 #include <QtCore/QFile>
 #include <QtCore/QMap>
+#include <QtGui/QApplication>
 #include <QTextDocument>
 
 #include <kdebug.h>
@@ -59,11 +60,11 @@ class KFileItemPrivate : public QSharedData
 public:
     KFileItemPrivate(const KIO::UDSEntry& entry,
                      mode_t mode, mode_t permissions,
-                     const KUrl& url, // item url if entry.isEmpty(), otherwise directory url
+                     const KUrl& itemOrDirUrl,
                      bool urlIsDirectory,
                      bool delayedMimeTypes)
         : m_entry( entry ),
-          m_url( url ),
+          m_url(itemOrDirUrl),
           m_strName(),
           m_strText(),
           m_iconName(),
@@ -73,7 +74,7 @@ public:
           m_permissions( permissions ),
           m_bMarked( false ),
           m_bLink( false ),
-          m_bIsLocalUrl( url.isLocalFile() ),
+          m_bIsLocalUrl(itemOrDirUrl.isLocalFile()),
           m_bMimeTypeKnown( false ),
           m_delayedMimeTypes( delayedMimeTypes ),
           m_useIconNameCache(false),
@@ -82,7 +83,8 @@ public:
         if (entry.count() != 0) {
             readUDSEntry( urlIsDirectory );
         } else {
-            m_strName = url.fileName();
+            Q_ASSERT(!urlIsDirectory);
+            m_strName = itemOrDirUrl.fileName();
             m_strText = KIO::decodeFileName( m_strName );
         }
         init();
@@ -92,8 +94,6 @@ public:
     {
     }
 
-//    KFileItemPrivate(const KFileItemPrivate& other);
-
     /**
      * Computes the text and mode from the UDSEntry
      * Called by constructor, but can be called again later
@@ -106,6 +106,8 @@ public:
     KDateTime time( KFileItem::FileTimes which ) const;
     void setTime(KFileItem::FileTimes which, long long time_t_val) const;
     bool cmp( const KFileItemPrivate & item ) const;
+    QString user() const;
+    QString group() const;
 
     /**
      * Extracts the data from the UDSEntry member and updates the KFileItem
@@ -214,14 +216,14 @@ void KFileItemPrivate::init()
              * This is the reason for the -1
              */
             KDE_struct_stat buf;
-            const QByteArray path = QFile::encodeName(m_url.path( KUrl::RemoveTrailingSlash ));
-            if ( KDE_lstat( path.data(), &buf ) == 0 )
+            const QString path = m_url.toLocalFile( KUrl::RemoveTrailingSlash );
+            if ( KDE::lstat( path, &buf ) == 0 )
             {
                 mode = buf.st_mode;
                 if ( S_ISLNK( mode ) )
                 {
                     m_bLink = true;
-                    if ( KDE_stat( path.data(), &buf ) == 0 )
+                    if ( KDE::stat( path, &buf ) == 0 )
                         mode = buf.st_mode;
                     else // link pointing to nowhere (see kio/file/file.cc)
                         mode = (S_IFMT-1) | S_IRWXU | S_IRWXG | S_IRWXO;
@@ -233,6 +235,8 @@ void KFileItemPrivate::init()
                     m_fileMode = mode & S_IFMT; // extract file type
                 if ( m_permissions == KFileItem::Unknown )
                     m_permissions = mode & 07777; // extract permissions
+            } else {
+                kDebug() << path << "does not exist anymore";
             }
         }
     }
@@ -290,7 +294,7 @@ KIO::filesize_t KFileItemPrivate::size() const
     // If not in the KIO::UDSEntry, or if UDSEntry empty, use stat() [if local URL]
     if ( m_bIsLocalUrl ) {
         KDE_struct_stat buf;
-        if ( KDE_stat( QFile::encodeName(m_url.path(KUrl::RemoveTrailingSlash)), &buf ) == 0 )
+        if ( KDE::stat( m_url.toLocalFile(KUrl::RemoveTrailingSlash), &buf ) == 0 )
             return buf.st_size;
     }
     return 0;
@@ -329,7 +333,7 @@ KDateTime KFileItemPrivate::time( KFileItem::FileTimes mappedWhich ) const
     if ( m_bIsLocalUrl )
     {
         KDE_struct_stat buf;
-        if ( KDE_stat( QFile::encodeName(m_url.path(KUrl::RemoveTrailingSlash)), &buf ) == 0 )
+        if ( KDE::stat( m_url.toLocalFile(KUrl::RemoveTrailingSlash), &buf ) == 0 )
         {
             setTime(KFileItem::ModificationTime, buf.st_mtime);
             setTime(KFileItem::AccessTime, buf.st_atime);
@@ -343,12 +347,32 @@ KDateTime KFileItemPrivate::time( KFileItem::FileTimes mappedWhich ) const
 inline //because it is used only in one place
 bool KFileItemPrivate::cmp( const KFileItemPrivate & item ) const
 {
+#if 0
+    kDebug() << "Comparing" << m_url << "and" << item.m_url;
+    kDebug() << " name" << (m_strName == item.m_strName);
+    kDebug() << " local" << (m_bIsLocalUrl == item.m_bIsLocalUrl);
+    kDebug() << " mode" << (m_fileMode == item.m_fileMode);
+    kDebug() << " perm" << (m_permissions == item.m_permissions);
+    kDebug() << " UDS_USER" << (user() == item.user());
+    kDebug() << " UDS_GROUP" << (group() == item.group());
+    kDebug() << " UDS_EXTENDED_ACL" << (m_entry.stringValue( KIO::UDSEntry::UDS_EXTENDED_ACL ) == item.m_entry.stringValue( KIO::UDSEntry::UDS_EXTENDED_ACL ));
+    kDebug() << " UDS_ACL_STRING" << (m_entry.stringValue( KIO::UDSEntry::UDS_ACL_STRING ) == item.m_entry.stringValue( KIO::UDSEntry::UDS_ACL_STRING ));
+    kDebug() << " UDS_DEFAULT_ACL_STRING" << (m_entry.stringValue( KIO::UDSEntry::UDS_DEFAULT_ACL_STRING ) == item.m_entry.stringValue( KIO::UDSEntry::UDS_DEFAULT_ACL_STRING ));
+    kDebug() << " m_bLink" << (m_bLink == item.m_bLink);
+    kDebug() << " m_hidden" << (m_hidden == item.m_hidden);
+    kDebug() << " size" << (size() == item.size());
+    kDebug() << " ModificationTime" << (time(KFileItem::ModificationTime) == item.time(KFileItem::ModificationTime));
+    kDebug() << " UDS_ICON_NAME" << (m_entry.stringValue( KIO::UDSEntry::UDS_ICON_NAME ) == item.m_entry.stringValue( KIO::UDSEntry::UDS_ICON_NAME ));
+#endif
     return ( m_strName == item.m_strName
              && m_bIsLocalUrl == item.m_bIsLocalUrl
              && m_fileMode == item.m_fileMode
              && m_permissions == item.m_permissions
-             && m_entry.stringValue( KIO::UDSEntry::UDS_USER ) == item.m_entry.stringValue( KIO::UDSEntry::UDS_USER )
-             && m_entry.stringValue( KIO::UDSEntry::UDS_GROUP ) == item.m_entry.stringValue( KIO::UDSEntry::UDS_GROUP )
+             && user() == item.user()
+             && group() == item.group()
+             && m_entry.stringValue( KIO::UDSEntry::UDS_EXTENDED_ACL ) == item.m_entry.stringValue( KIO::UDSEntry::UDS_EXTENDED_ACL )
+             && m_entry.stringValue( KIO::UDSEntry::UDS_ACL_STRING ) == item.m_entry.stringValue( KIO::UDSEntry::UDS_ACL_STRING )
+             && m_entry.stringValue( KIO::UDSEntry::UDS_DEFAULT_ACL_STRING ) == item.m_entry.stringValue( KIO::UDSEntry::UDS_DEFAULT_ACL_STRING )
              && m_bLink == item.m_bLink
              && m_hidden == item.m_hidden
              && size() == item.size()
@@ -396,12 +420,24 @@ QString KFileItemPrivate::parsePermissions(mode_t perm) const
 
     // Include the type in the first char like kde3 did; people are more used to seeing it,
     // even though it's not really part of the permissions per se.
-    if (m_fileMode != KFileItem::Unknown && S_ISDIR(m_fileMode))
-        buffer[0] = 'd';
-    else if (m_bLink)
+    if (m_bLink)
         buffer[0] = 'l';
-    else
+    else if (m_fileMode != KFileItem::Unknown) {
+        if (S_ISDIR(m_fileMode))
+            buffer[0] = 'd';
+        else if (S_ISSOCK(m_fileMode))
+            buffer[0] = 's';
+        else if (S_ISCHR(m_fileMode))
+            buffer[0] = 'c';
+        else if (S_ISBLK(m_fileMode))
+            buffer[0] = 'b';
+        else if (S_ISFIFO(m_fileMode))
+            buffer[0] = 'p';
+        else
+            buffer[0] = '-';
+    } else {
         buffer[0] = '-';
+    }
 
     buffer[1] = ((( perm & S_IRUSR ) == S_IRUSR ) ? 'r' : '-' );
     buffer[2] = ((( perm & S_IWUSR ) == S_IWUSR ) ? 'w' : '-' );
@@ -431,10 +467,10 @@ KFileItem::KFileItem()
 {
 }
 
-KFileItem::KFileItem( const KIO::UDSEntry& entry, const KUrl& directoryUrl,
+KFileItem::KFileItem( const KIO::UDSEntry& entry, const KUrl& itemOrDirUrl,
                       bool delayedMimeTypes, bool urlIsDirectory )
     : d(new KFileItemPrivate(entry, KFileItem::Unknown, KFileItem::Unknown,
-                             directoryUrl, urlIsDirectory, delayedMimeTypes))
+                             itemOrDirUrl, urlIsDirectory, delayedMimeTypes))
 {
 }
 
@@ -496,6 +532,9 @@ void KFileItem::setName( const QString& name )
 {
     d->m_strName = name;
     d->m_strText = KIO::decodeFileName( d->m_strName );
+    if (d->m_entry.contains(KIO::UDSEntry::UDS_NAME))
+        d->m_entry.insert(KIO::UDSEntry::UDS_NAME, d->m_strName); // #195385
+
 }
 
 QString KFileItem::linkDest() const
@@ -509,7 +548,7 @@ QString KFileItem::linkDest() const
     if ( d->m_bIsLocalUrl )
     {
         char buf[1000];
-        int n = readlink( QFile::encodeName(d->m_url.path( KUrl::RemoveTrailingSlash )), buf, sizeof(buf)-1 );
+        int n = readlink( QFile::encodeName(d->m_url.toLocalFile( KUrl::RemoveTrailingSlash )), buf, sizeof(buf)-1 );
         if ( n != -1 )
         {
             buf[ n ] = 0;
@@ -522,7 +561,7 @@ QString KFileItem::linkDest() const
 QString KFileItem::localPath() const
 {
   if ( d->m_bIsLocalUrl ) {
-    return d->m_url.path();
+    return d->m_url.toLocalFile();
   }
 
   // Extract the local path from the KIO::UDSEntry
@@ -582,21 +621,25 @@ time_t KFileItem::time( unsigned int which ) const
 
 QString KFileItem::user() const
 {
-    QString userName = d->m_entry.stringValue( KIO::UDSEntry::UDS_USER );
-    if ( userName.isEmpty() && d->m_bIsLocalUrl )
-    {
+    return d->user();
+}
+
+QString KFileItemPrivate::user() const
+{
+    QString userName = m_entry.stringValue(KIO::UDSEntry::UDS_USER);
+    if (userName.isEmpty() && m_bIsLocalUrl) {
 #ifdef Q_WS_WIN
-        QFileInfo a(d->m_url.path( KUrl::RemoveTrailingSlash ));
+        QFileInfo a(m_url.toLocalFile( KUrl::RemoveTrailingSlash ));
         userName = a.owner();
-        d->m_entry.insert( KIO::UDSEntry::UDS_USER, userName );
+        m_entry.insert( KIO::UDSEntry::UDS_USER, userName );
 #else
         KDE_struct_stat buff;
-        if ( KDE_lstat( QFile::encodeName(d->m_url.path( KUrl::RemoveTrailingSlash )), &buff ) == 0) // get uid/gid of the link, if it's a link
+        if ( KDE::lstat( m_url.toLocalFile( KUrl::RemoveTrailingSlash ), &buff ) == 0) // get uid/gid of the link, if it's a link
         {
             struct passwd *pwuser = getpwuid( buff.st_uid );
             if ( pwuser != 0 ) {
                 userName = QString::fromLocal8Bit(pwuser->pw_name);
-                d->m_entry.insert( KIO::UDSEntry::UDS_USER, userName );
+                m_entry.insert( KIO::UDSEntry::UDS_USER, userName );
             }
         }
 #endif
@@ -606,16 +649,21 @@ QString KFileItem::user() const
 
 QString KFileItem::group() const
 {
-    QString groupName = d->m_entry.stringValue( KIO::UDSEntry::UDS_GROUP );
-    if (groupName.isEmpty() && d->m_bIsLocalUrl )
+    return d->group();
+}
+
+QString KFileItemPrivate::group() const
+{
+    QString groupName = m_entry.stringValue( KIO::UDSEntry::UDS_GROUP );
+    if (groupName.isEmpty() && m_bIsLocalUrl )
     {
 #ifdef Q_WS_WIN
-        QFileInfo a(d->m_url.path( KUrl::RemoveTrailingSlash ));
+        QFileInfo a(m_url.toLocalFile( KUrl::RemoveTrailingSlash ));
         groupName = a.group();
-        d->m_entry.insert( KIO::UDSEntry::UDS_GROUP, groupName );
+        m_entry.insert( KIO::UDSEntry::UDS_GROUP, groupName );
 #else
         KDE_struct_stat buff;
-        if ( KDE_lstat( QFile::encodeName(d->m_url.path( KUrl::RemoveTrailingSlash )), &buff ) == 0) // get uid/gid of the link, if it's a link
+        if ( KDE::lstat( m_url.toLocalFile( KUrl::RemoveTrailingSlash ), &buff ) == 0) // get uid/gid of the link, if it's a link
         {
             struct group *ge = getgrgid( buff.st_gid );
             if ( ge != 0 ) {
@@ -625,7 +673,7 @@ QString KFileItem::group() const
             }
             else
                 groupName.sprintf("%d",buff.st_gid);
-            d->m_entry.insert( KIO::UDSEntry::UDS_GROUP, groupName );
+            m_entry.insert( KIO::UDSEntry::UDS_GROUP, groupName );
         }
 #endif
     }
@@ -647,7 +695,7 @@ KMimeType::Ptr KFileItem::determineMimeType() const
 
         d->m_pMimeType = KMimeType::findByUrl( url, d->m_fileMode, isLocalUrl );
         Q_ASSERT(d->m_pMimeType);
-        //kDebug() << "finding mimetype for " << url.url() << " : " << d->m_pMimeType->name();
+        //kDebug() << d << "finding final mimetype for" << url << ":" << d->m_pMimeType->name();
         d->m_bMimeTypeKnown = true;
     }
 
@@ -664,14 +712,20 @@ bool KFileItem::isMimeTypeKnown() const
 
 QString KFileItem::mimeComment() const
 {
+    const QString displayType = d->m_entry.stringValue( KIO::UDSEntry::UDS_DISPLAY_TYPE );
+    if (!displayType.isEmpty())
+        return displayType;
+
     KMimeType::Ptr mType = determineMimeType();
 
     bool isLocalUrl;
     KUrl url = mostLocalUrl(isLocalUrl);
 
     KMimeType::Ptr mime = mimeTypePtr();
+    // This cannot move to kio_file (with UDS_DISPLAY_TYPE) because it needs
+    // the mimetype to be determined, which is done here, and possibly delayed...
     if (isLocalUrl && mime->is("application/x-desktop")) {
-        KDesktopFile cfg( url.path() );
+        KDesktopFile cfg( url.toLocalFile() );
         QString comment = cfg.desktopGroup().readEntry( "Comment" );
         if (!comment.isEmpty())
             return comment;
@@ -708,7 +762,7 @@ static QString iconFromDesktopFile(const QString& path)
             const QString u = cfg.readUrl();
             const KUrl url( u );
             if ( url.protocol() == "trash" ) {
-                // We need to find if the trash is empty, preferrably without using a KIO job.
+                // We need to find if the trash is empty, preferably  without using a KIO job.
                 // So instead kio_trash leaves an entry in its config file for us.
                 KConfig trashConfig( "trashrc", KConfig::SimpleConfig );
                 if ( trashConfig.group("Status").readEntry( "Empty", true ) ) {
@@ -737,16 +791,19 @@ QString KFileItem::iconName() const
 
     KMimeType::Ptr mime = mimeTypePtr();
     if (isLocalUrl && mime->is("application/x-desktop")) {
-        d->m_iconName = iconFromDesktopFile(url.path());
+        d->m_iconName = iconFromDesktopFile(url.toLocalFile());
         if (!d->m_iconName.isEmpty()) {
             d->m_useIconNameCache = d->m_bMimeTypeKnown;
             return d->m_iconName;
         }
     }
 
-    //kDebug() << "finding icon for " << url.url() << " : " << mimeTypePtr()->name();
+    // KDE5: handle .directory files here too, and get rid of
+    // KFolderMimeType and the url argument in KMimeType::iconName().
+
     d->m_iconName = mime->iconName(url);
     d->m_useIconNameCache = d->m_bMimeTypeKnown;
+    //kDebug() << "finding icon for" << url << ":" << d->m_iconName;
     return d->m_iconName;
 }
 
@@ -762,6 +819,17 @@ QStringList KFileItem::overlays() const
         names.append("object-locked");
     }
 
+    if ( isDesktopFile() ) {
+        KDesktopFile cfg( localPath() );
+        const KConfigGroup group = cfg.desktopGroup();
+
+        // Add a warning emblem if this is an executable desktop file
+        // which is untrusted.
+        if ( group.hasKey( "Exec" ) && !KDesktopFile::isAuthorizedDesktopFile( localPath() ) ) {
+            names.append( "emblem-important" );
+        }
+    }
+
     if ( isHidden() ) {
         names.append("hidden");
     }
@@ -769,8 +837,8 @@ QStringList KFileItem::overlays() const
 #ifndef Q_OS_WIN
     if( S_ISDIR( d->m_fileMode ) && d->m_bIsLocalUrl)
     {
-        if (KSambaShare::instance()->isDirectoryShared( d->m_url.path() ) ||
-            KNFSShare::instance()->isDirectoryShared( d->m_url.path() ))
+        if (KSambaShare::instance()->isDirectoryShared( d->m_url.toLocalFile() ) ||
+            KNFSShare::instance()->isDirectoryShared( d->m_url.toLocalFile() ))
         {
             //kDebug() << d->m_url.path();
             names.append("network-workgroup");
@@ -786,15 +854,16 @@ QStringList KFileItem::overlays() const
     return names;
 }
 
+// ## where is this used?
 QPixmap KFileItem::pixmap( int _size, int _state ) const
 {
     const QString iconName = d->m_entry.stringValue( KIO::UDSEntry::UDS_ICON_NAME );
     if ( !iconName.isEmpty() )
         return DesktopIcon(iconName, _size, _state);
 
-    if ( !d->m_pMimeType )
-    {
-        if ( S_ISDIR( d->m_fileMode ) ) {
+    if (!d->m_pMimeType) {
+        // No mimetype determined yet, go for a fast default icon
+        if (S_ISDIR(d->m_fileMode)) {
             static const QString * defaultFolderIcon = 0;
             if ( !defaultFolderIcon ) {
                 const KMimeType::Ptr mimeType = KMimeType::mimeType( "inode/directory" );
@@ -859,7 +928,7 @@ bool KFileItem::isReadable() const
     }
 
     // Or if we can't read it [using ::access()] - not network transparent
-    if ( d->m_bIsLocalUrl && ::access( QFile::encodeName(d->m_url.path()), R_OK ) == -1 )
+    if ( d->m_bIsLocalUrl && KDE::access( d->m_url.toLocalFile(), R_OK ) == -1 )
         return false;
 
     return true;
@@ -882,7 +951,7 @@ bool KFileItem::isWritable() const
     }
 
     // Or if we can't read it [using ::access()] - not network transparent
-    if ( d->m_bIsLocalUrl && ::access( QFile::encodeName(d->m_url.path()), W_OK ) == -1 )
+    if ( d->m_bIsLocalUrl && KDE::access( d->m_url.toLocalFile(), W_OK ) == -1 )
         return false;
 
     return true;
@@ -890,31 +959,25 @@ bool KFileItem::isWritable() const
 
 bool KFileItem::isHidden() const
 {
+    // The kioslave can specify explicitly that a file is hidden or shown
     if ( d->m_hidden != KFileItemPrivate::Auto )
         return d->m_hidden == KFileItemPrivate::Hidden;
 
-    if ( !d->m_url.isEmpty() )
-        return d->m_url.fileName()[0] == '.';
-    else // should never happen
-        return d->m_strName[0] == '.';
+    // Prefer the filename that is part of the URL, in case the display name is different.
+    QString fileName = d->m_url.fileName();
+    if (fileName.isEmpty()) // e.g. "trash:/"
+        fileName = d->m_strName;
+    return fileName.length() > 1 && fileName[0] == '.';  // Just "." is current directory, not hidden.
 }
 
 bool KFileItem::isDir() const
 {
-    if ( d->m_fileMode == KFileItem::Unknown )
-    {
-        kDebug(7101) << "can't say -> false";
+    if (d->m_fileMode == KFileItem::Unknown) {
+        // Probably the file was deleted already, and KDirLister hasn't told the world yet.
+        //kDebug() << d << url() << "can't say -> false";
         return false; // can't say for sure, so no
     }
     return (S_ISDIR(d->m_fileMode));
-/*
-  if  (!S_ISDIR(d->m_fileMode)) {
-  if (d->m_url.isLocalFile()) {
-  KMimeType::Ptr ptr=KMimeType::findByUrl(d->m_url,0,true,true);
-  if ((ptr!=0) && (ptr->is("directory/inode"))) return true;
-  }
-  return false
-  } else return true;*/
 }
 
 bool KFileItem::isFile() const
@@ -937,7 +1000,7 @@ bool KFileItem::acceptsDrops() const
         return true;
 
     // Executable, shell script ... ?
-    if ( QFileInfo(d->m_url.path()).isExecutable() )
+    if ( QFileInfo(d->m_url.toLocalFile()).isExecutable() )
         return true;
 
     return false;
@@ -979,9 +1042,10 @@ QString KFileItem::getToolTipText(int maxcount) const
 
     // the font tags are a workaround for the fact that the tool tip gets
     // screwed if the color scheme uses white as default text color
-    const char* start = "<tr><td><nobr><font color=\"black\">";
-    const char* mid   = "</font></nobr></td><td><nobr><font color=\"black\">";
-    const char* end   = "</font></nobr></td></tr>";
+    const QString colorName = QApplication::palette().color(QPalette::ToolTipText).name();
+    const QString start = "<tr><td align=\"right\"><nobr><font color=\"" + colorName + "\"><b>";
+    const QString mid = "&nbsp;</b></font></nobr></td><td><nobr><font color=\"" + colorName + "\">";
+    const char* end = "</font></nobr></td></tr>";
 
     tip = "<table cellspacing=0 cellpadding=0>";
 
@@ -996,8 +1060,7 @@ QString KFileItem::getToolTipText(int maxcount) const
 
     if ( !S_ISDIR ( d->m_fileMode ) )
         tip += start + i18n("Size:") + mid +
-               QString("%1 (%2)").arg(KIO::convertSize(size()))
-               .arg(KGlobal::locale()->formatNumber(size(), 0)) +
+               QString("%1").arg(KIO::convertSize(size())) +
                end;
 
     tip += start + i18n("Modified:") + mid +
@@ -1011,13 +1074,17 @@ QString KFileItem::getToolTipText(int maxcount) const
 
     if (info.isValid())
     {
-        tip += "<tr><td colspan=2><center><s>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;</s></center></td></tr>";
         const QStringList keys = info.preferredKeys();
 
         // now the rest
         QStringList::ConstIterator it = keys.begin();
         for (int count = 0; count<maxcount && it!=keys.end() ; ++it)
         {
+            if ( count == 0 )
+            {
+                tip += "<tr><td colspan=2><center><s>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;</s></center></td></tr>";
+            }
+
             KFileMetaInfoItem item = info.item( *it );
             if ( item.isValid() )
             {
@@ -1050,7 +1117,7 @@ QString KFileItem::getToolTipText(int maxcount) const
 
 void KFileItem::run( QWidget* parentWidget ) const
 {
-    (void) new KRun( d->m_url, parentWidget, d->m_fileMode, d->m_bIsLocalUrl );
+    (void) new KRun( targetUrl(), parentWidget, d->m_fileMode, d->m_bIsLocalUrl );
 }
 
 bool KFileItem::cmp( const KFileItem & item ) const
@@ -1150,14 +1217,13 @@ void KFileItem::setMetaInfo( const KFileMetaInfo & info ) const
     d->m_metaInfo = info;
 }
 
-KFileMetaInfo KFileItem::metaInfo(bool autoget, int) const
+KFileMetaInfo KFileItem::metaInfo(bool autoget, int what) const
 {
-    if (autoget && !d->m_metaInfo.isValid())
+    if ((isRegularFile() || isDir()) && autoget && !d->m_metaInfo.isValid())
     {
         bool isLocalUrl;
         KUrl url(mostLocalUrl(isLocalUrl));
-        if (KGlobalSettings::showFilePreview(url))
-            d->m_metaInfo = KFileMetaInfo(url);//, mimetype() );
+        d->m_metaInfo = KFileMetaInfo(url.toLocalFile(), mimetype(), (KFileMetaInfo::What)what);
     }
     return d->m_metaInfo;
 }
@@ -1255,6 +1321,17 @@ KUrl KFileItem::targetUrl() const
       return url();
 }
 
+KUrl KFileItem::nepomukUri() const
+{
+    if(isLocalFile()) {
+        return url();
+    }
+    else {
+        const QString nepomukUriStr = d->m_entry.stringValue( KIO::UDSEntry::UDS_NEPOMUK_URI );
+        return KUrl(nepomukUriStr);
+    }
+}
+
 /*
  * Mimetype handling.
  *
@@ -1281,7 +1358,7 @@ KMimeType::Ptr KFileItem::mimeTypePtr() const
         // If we used the "fast mode" (no sniffing), and we didn't get a perfect (extension-based) match,
         // then determineMimeType will be able to do better.
         const bool canDoBetter = d->m_delayedMimeTypes && accuracy < 100;
-        //kDebug() << "finding mimetype for" << url() << ":" << m_pMimeType->name();
+        //kDebug() << "finding mimetype for" << url << ":" << d->m_pMimeType->name() << "canDoBetter=" << canDoBetter;
         d->m_bMimeTypeKnown = !canDoBetter;
     }
     return d->m_pMimeType;
@@ -1360,6 +1437,16 @@ KUrl::List KFileItemList::urlList() const {
     return lst;
 }
 
+KUrl::List KFileItemList::targetUrlList() const {
+    KUrl::List lst;
+    const_iterator it = begin();
+    const const_iterator itend = end();
+    for ( ; it != itend ; ++it ) {
+        lst.append( (*it).targetUrl() );
+    }
+    return lst;
+}
+
 bool KFileItem::isDesktopFile() const
 {
     // only local files
@@ -1380,3 +1467,17 @@ bool KFileItem::isDesktopFile() const
     return determineMimeType()->is("application/x-desktop");
 }
 
+bool KFileItem::isRegularFile() const
+{
+    return S_ISREG(d->m_fileMode);
+}
+
+QDebug operator<<(QDebug stream, const KFileItem& item)
+{
+    if (item.isNull()) {
+        stream << "[null KFileItem]";
+    } else {
+        stream << "[KFileItem for" << item.url() << "]";
+    }
+    return stream;
+}
