@@ -36,14 +36,19 @@ private Q_SLOTS:
     void cleanupTestCase();
 
     // Local tests (kio_file only)
-    void get();
+    void storedGet();
     void put();
+    void storedPut();
     void copyFileToSamePartition();
     void copyDirectoryToSamePartition();
     void copyDirectoryToExistingDirectory();
     void copyFileToOtherPartition();
     void copyDirectoryToOtherPartition();
     void listRecursive();
+    void listFile();
+    void killJob();
+    void killJobBeforeStart();
+    void deleteJobBeforeStart();
     void directorySize();
     void directorySizeError();
     void moveFileToSamePartition();
@@ -55,11 +60,21 @@ private Q_SLOTS:
     void moveDirectoryNoPermissions();
     void deleteFile();
     void deleteDirectory();
-    void deleteManyFiles();
+    void deleteSymlink();
+    void deleteManyDirs();
+    void deleteManyFilesIndependently();
+    void deleteManyFilesTogether();
+    void rmdirEmpty();
+    void rmdirNotEmpty();
     void stat();
+    void mostLocalUrl();
     void mimeType();
     //void newApiPerformance();
     void calculateRemainingSeconds();
+    void moveFileDestAlreadyExists();
+
+    void moveAndOverwrite();
+    void moveOverSymlinkToSelf();
 
     // Remote tests
     //void copyFileToSystem();
@@ -84,6 +99,9 @@ private:
     void moveLocalFile( const QString& src, const QString& dest );
     void moveLocalDirectory( const QString& src, const QString& dest );
     //void copyFileToSystem( bool resolve_local_urls );
+    void deleteSymlink(bool using_fast_path);
+    void deleteManyDirs(bool using_fast_path);
+    void deleteManyFilesTogether(bool using_fast_path);
 
     int m_result;
     QByteArray m_data;
