@@ -17,6 +17,8 @@
  */
 
 #include "browserrun.h"
+#include "browserrun_p.h"
+
 #include <kmessagebox.h>
 #include <kfiledialog.h>
 #include <kio/job.h>
@@ -29,7 +31,9 @@
 #include <kmimetypetrader.h>
 #include <ktemporaryfile.h>
 #include <kdebug.h>
+#include <kde_file.h>
 #include <kstandarddirs.h>
+#include "browseropenorsavequestion.h"
 #include <assert.h>
 
 using namespace KParts;
@@ -95,10 +99,10 @@ void BrowserRun::init()
       setIsLocalFile( true );
 
     if ( isLocalFile() )  {
-      struct stat buff;
-      if ( stat( QFile::encodeName(KRun::url().toLocalFile()), &buff ) == -1 )
+      KDE_struct_stat buff;
+      if ( KDE::stat( KRun::url().toLocalFile(), &buff ) == -1 )
       {
-        kDebug(1000) << "BrowserRun::init:" << KRun::url().toLocalFile() << "doesn't exist.";
+        kDebug(1000) << KRun::url().toLocalFile() << "doesn't exist.";
         redirectToError( KIO::ERR_DOES_NOT_EXIST, KRun::url().toLocalFile() );
         return;
       }
@@ -110,18 +114,18 @@ void BrowserRun::init()
 
 void BrowserRun::scanFile()
 {
-  kDebug(1000) << "BrowserRun::scanfile" << KRun::url();
+  kDebug(1000) << KRun::url();
 
   // Let's check for well-known extensions
   // Not when there is a query in the URL, in any case.
   // Optimization for http/https, findByURL doesn't trust extensions over http.
-  if ( KRun::url().query().isEmpty() && !KRun::url().protocol().startsWith("http") )
+  if ( KRun::url().query().isEmpty() && !KRun::url().protocol().startsWith(QLatin1String("http")) )
   {
     KMimeType::Ptr mime = KMimeType::findByUrl( KRun::url() );
     assert( mime );
-    if ( mime->name() != "application/octet-stream" || isLocalFile() )
+    if ( !mime->isDefault() || isLocalFile() )
     {
-      kDebug(1000) << "Scanfile: MIME TYPE is" << mime->name();
+      kDebug(1000) << "MIME TYPE is" << mime->name();
       mimeTypeDetermined( mime->name() );
       return;
     }
@@ -134,9 +138,9 @@ void BrowserRun::scanFile()
       if (proto == "https" || proto == "webdavs") {
             metaData.insert("main_frame_request", "TRUE" );
             metaData.insert("ssl_was_in_use", "TRUE" );
-            metaData.insert("ssl_activate_warnings", "TRUE" );
+           // metaData.insert("ssl_activate_warnings", "TRUE" );
       } else if (proto == "http" || proto == "webdav") {
-            metaData.insert("ssl_activate_warnings", "TRUE" );
+           // metaData.insert("ssl_activate_warnings", "TRUE" );
             metaData.insert("ssl_was_in_use", "FALSE" );
       }
 
@@ -146,7 +150,7 @@ void BrowserRun::scanFile()
   }
 
   KIO::TransferJob *job;
-    if ( d->m_browserArgs.doPost() && KRun::url().protocol().startsWith("http")) {
+    if ( d->m_browserArgs.doPost() && KRun::url().protocol().startsWith(QLatin1String("http"))) {
         job = KIO::http_post( KRun::url(), d->m_browserArgs.postData, KIO::HideProgressInfo );
         job->addMetaData( "content-type", d->m_browserArgs.contentType() );
     } else {
@@ -169,7 +173,7 @@ void BrowserRun::scanFile()
 
 void BrowserRun::slotBrowserScanFinished(KJob *job)
 {
-  kDebug(1000) << "BrowserRun::slotBrowserScanFinished";
+  kDebug(1000) << job->error();
   if ( job->error() == KIO::ERR_IS_DIRECTORY )
   {
       // It is in fact a directory. This happens when HTTP redirects to FTP.
@@ -192,7 +196,7 @@ void BrowserRun::slotBrowserScanFinished(KJob *job)
 
 void BrowserRun::slotBrowserMimetype( KIO::Job *_job, const QString &type )
 {
-    Q_ASSERT( _job == KRun::job() );
+    Q_ASSERT( _job == KRun::job() ); Q_UNUSED(_job)
     KIO::TransferJob *job = static_cast<KIO::TransferJob *>(KRun::job());
     // Update our URL in case of a redirection
     //kDebug(1000) << "old URL=" << KRun::url();
@@ -213,6 +217,13 @@ void BrowserRun::slotBrowserMimetype( KIO::Job *_job, const QString &type )
         //kDebug(1000) << "suggestedFileName=" << suggestedFileName;
         d->m_contentDisposition = job->queryMetaData("content-disposition-type");
 
+        QMapIterator<QString,QString> it (job->metaData());
+        while (it.hasNext()) {
+           it.next();
+           if (it.key().startsWith(QLatin1String("ssl_"), Qt::CaseInsensitive))
+             d->m_args.metaData().insert(it.key(), it.value());
+        }
+
         // Make a copy to avoid a dead reference
         QString _type = type;
         job->putOnHold();
@@ -232,19 +243,19 @@ BrowserRun::NonEmbeddableResult BrowserRun::handleNonEmbeddable( const QString&
     {
         if ( isTextExecutable(mimeType) )
             mimeType = QLatin1String("text/plain"); // view, don't execute
-        kDebug(1000) << "BrowserRun: ask for saving";
-        KService::Ptr offer = KMimeTypeTrader::self()->preferredService(mimeType, "Application");
         // ... -> ask whether to save
-        KParts::BrowserRun::AskSaveResult res = askSave( KRun::url(), offer, mimeType, suggestedFileName() );
-        if ( res == KParts::BrowserRun::Save ) {
+        BrowserOpenOrSaveQuestion question(d->m_window, KRun::url(), mimeType);
+        question.setSuggestedFileName(suggestedFileName());
+        BrowserOpenOrSaveQuestion::Result res = question.askOpenOrSave();
+        if (res == BrowserOpenOrSaveQuestion::Save) {
             save( KRun::url(), suggestedFileName() );
-            kDebug(1000) << "BrowserRun::handleNonEmbeddable: Save: returning Handled";
+            kDebug(1000) << "Save: returning Handled";
             setFinished( true );
             return Handled;
         }
-        else if ( res == KParts::BrowserRun::Cancel ) {
+        else if (res == BrowserOpenOrSaveQuestion::Cancel) {
             // saving done or canceled
-            kDebug(1000) << "BrowserRun::handleNonEmbeddable: Cancel: returning Handled";
+            kDebug(1000) << "Cancel: returning Handled";
             setFinished( true );
             return Handled;
         }
@@ -254,7 +265,7 @@ BrowserRun::NonEmbeddableResult BrowserRun::handleNonEmbeddable( const QString&
             // We must save the data to a tempfile first.
             if ( d->m_browserArgs.doPost() )
             {
-                kDebug(1000) << "BrowserRun: request comes from a POST, can't pass a URL to another app, need to save";
+                kDebug(1000) << "request comes from a POST, can't pass a URL to another app, need to save";
                 d->m_mimeType = mimeType;
                 QString extension;
                 QString fileName = suggestedFileName().isEmpty() ? KRun::url().fileName() : suggestedFileName();
@@ -302,97 +313,44 @@ bool BrowserRun::allowExecution( const QString &mimeType, const KUrl &url )
     i18n("Execute File?"), KGuiItem(i18n("Execute")) ) == KMessageBox::Continue );
 }
 
-static QString makeQuestion( const KUrl& url, const QString& mimeType, const QString& suggestedFileName )
-{
-    QString surl = KStringHandler::csqueeze( url.prettyUrl() );
-    KMimeType::Ptr mime = KMimeType::mimeType( mimeType );
-    QString comment = mimeType;
-
-    // Test if the mimeType is not recognize as octet-stream.
-    // If so then keep mime-type as comment
-    if (mime && mime->name() != KMimeType::defaultMimeType()) {
-        // The mime-type is known so display the comment instead of mime-type
-        comment = mime->comment();
-    }
-    // The strange order in the i18n() calls below is due to the possibility
-    // of surl containing a '%'
-    if ( suggestedFileName.isEmpty() )
-        return i18n("Open '%2'?\nType: %1", comment, surl);
-    else
-        return i18n("Open '%3'?\nName: %2\nType: %1", comment, suggestedFileName, surl);
-}
-
-//static
-// TODO should take a QWidget* parent argument
+//static, deprecated
 BrowserRun::AskSaveResult BrowserRun::askSave( const KUrl & url, KService::Ptr offer, const QString& mimeType, const QString & suggestedFileName )
 {
-    // SYNC SYNC SYNC SYNC SYNC SYNC SYNC SYNC SYNC SYNC SYNC SYNC SYNC SYNC
-    // NOTE: Keep this function in sync with kdebase/kcontrol/filetypes/filetypedetails.cpp
-    //       FileTypeDetails::updateAskSave()
-
-    QString question = makeQuestion( url, mimeType, suggestedFileName );
-
-    // Text used for the open button
-    QString openText = (offer && !offer->name().isEmpty())
-                       ? i18n("&Open with '%1'", offer->name())
-                       : i18n("&Open With...");
-
-    int choice = KMessageBox::questionYesNoCancel(
-        0, question, url.host(),
-        KStandardGuiItem::saveAs(), KGuiItem(openText), KStandardGuiItem::cancel(),
-        QLatin1String("askSave")+ mimeType ); // dontAskAgainName, KEEP IN SYNC!!!
-
-    return choice == KMessageBox::Yes ? Save : ( choice == KMessageBox::No ? Open : Cancel );
-    // SYNC SYNC SYNC SYNC SYNC SYNC SYNC SYNC SYNC SYNC SYNC SYNC SYNC SYNC
+    Q_UNUSED(offer);
+    BrowserOpenOrSaveQuestion question(0, url, mimeType);
+    question.setSuggestedFileName(suggestedFileName);
+    const BrowserOpenOrSaveQuestion::Result result = question.askOpenOrSave();
+    return result == BrowserOpenOrSaveQuestion::Save ? Save
+        : BrowserOpenOrSaveQuestion::Open ? Open
+        : Cancel;
 }
 
-//static
-// TODO should take a QWidget* parent argument
+//static, deprecated
 BrowserRun::AskSaveResult BrowserRun::askEmbedOrSave( const KUrl & url, const QString& mimeType, const QString & suggestedFileName, int flags )
 {
-    // SYNC SYNC SYNC SYNC SYNC SYNC SYNC SYNC SYNC SYNC SYNC SYNC SYNC SYNC
-    // NOTE: Keep this function in sync with kdebase/kcontrol/filetypes/filetypedetails.cpp
-    //       FileTypeDetails::updateAskSave()
-
-    KMimeType::Ptr mime = KMimeType::mimeType( mimeType );
-    // Don't ask for:
-    // - html (even new tabs would ask, due to about:blank!)
-    // - dirs obviously (though not common over HTTP :),
-    // - images (reasoning: no need to save, most of the time, because fast to see)
-    // e.g. postscript is different, because takes longer to read, so
-    // it's more likely that the user might want to save it.
-    // - multipart/* ("server push", see kmultipart)
-    // - other strange 'internal' mimetypes like print/manager...
-    // KEEP IN SYNC!!!
-    if (flags != (int)AttachmentDisposition && (
-         mime->is( "text/html" ) ||
-         mime->is( "application/xml" ) ||
-         mime->is( "inode/directory" ) ||
-         mimeType.startsWith( "image" ) ||
-         mime->is( "multipart/x-mixed-replace" ) ||
-         mime->is( "multipart/replace" ) ||
-         mimeType.startsWith( "print" ) ) )
-        return Open;
-
-    QString question = makeQuestion( url, mimeType, suggestedFileName );
-
-    // don't use KStandardGuiItem::open() here which has trailing ellipsis!
-    int choice = KMessageBox::questionYesNoCancel(
-        0, question, url.host(),
-        KStandardGuiItem::saveAs(), KGuiItem( i18n( "&Open" ), "document-open"), KStandardGuiItem::cancel(),
-        QLatin1String("askEmbedOrSave")+ mimeType ); // dontAskAgainName, KEEP IN SYNC!!!
-    return choice == KMessageBox::Yes ? Save : ( choice == KMessageBox::No ? Open : Cancel );
-    // SYNC SYNC SYNC SYNC SYNC SYNC SYNC SYNC SYNC SYNC SYNC SYNC SYNC SYNC
+    BrowserOpenOrSaveQuestion question(0, url, mimeType);
+    question.setSuggestedFileName(suggestedFileName);
+    const BrowserOpenOrSaveQuestion::Result result = question.askEmbedOrSave(flags);
+    return result == BrowserOpenOrSaveQuestion::Save ? Save
+        : BrowserOpenOrSaveQuestion::Embed ? Open
+        : Cancel;
 }
 
 // Default implementation, overridden in KHTMLRun
 void BrowserRun::save( const KUrl & url, const QString & suggestedFileName )
 {
-    simpleSave( url, suggestedFileName, d->m_window );
+    saveUrl(url, suggestedFileName, d->m_window, KParts::OpenUrlArguments());
 }
 
+// static
 void BrowserRun::simpleSave( const KUrl & url, const QString & suggestedFileName,
                              QWidget* window )
+{
+    saveUrl(url, suggestedFileName, window, KParts::OpenUrlArguments());
+}
+
+void KParts::BrowserRun::saveUrl(const KUrl & url, const QString & suggestedFileName,
+                                 QWidget* window, const KParts::OpenUrlArguments& args)
 {
     // DownloadManager <-> konqueror integration
     // find if the integration is enabled
@@ -421,9 +379,9 @@ void BrowserRun::simpleSave( const KUrl & url, const QString & suggestedFileName
                 // the duplicated code) with shiny new KDownload class for 3.2 (pfeiffer)
                 // Until the shiny new class comes about, send the suggestedFileName
                 // along with the actual URL to download. (DA)
-                cmd += " " + KShell::quoteArg(url.url());
+                cmd += ' ' + KShell::quoteArg(url.url());
                 if ( !suggestedFileName.isEmpty() )
-                    cmd +=" " + KShell::quoteArg(suggestedFileName);
+                    cmd += ' ' + KShell::quoteArg(suggestedFileName);
 
                 kDebug(1000) << "Calling command" << cmd;
                 // slave is already on hold (slotBrowserMimetype())
@@ -439,25 +397,42 @@ void BrowserRun::simpleSave( const KUrl & url, const QString & suggestedFileName
                                         window);
     dlg->setOperationMode( KFileDialog::Saving );
     dlg->setCaption(i18n("Save As"));
+    dlg->setConfirmOverwrite(true);
 
-    dlg->setSelection( suggestedFileName.isEmpty() ? url.fileName() : suggestedFileName );
+    QString name;
+    if ( !suggestedFileName.isEmpty() )
+        name = suggestedFileName;
+    else
+        name = url.fileName(KUrl::ObeyTrailingSlash); // can be empty, e.g. in case http://www.kde.org/
+
+    dlg->setSelection(name);
     if ( dlg->exec() )
     {
         KUrl destURL( dlg->selectedUrl() );
         if ( destURL.isValid() )
         {
-            KIO::Job *job = KIO::copy( url, destURL );
-            job->ui()->setWindow (window);
-            job->ui()->setAutoErrorHandlingEnabled( true );
+            saveUrlUsingKIO(url, destURL, window, args.metaData());
         }
     }
     delete dlg;
 }
 
+void BrowserRun::saveUrlUsingKIO(const KUrl & srcUrl, const KUrl& destUrl,
+                                 QWidget* window, const QMap<QString, QString> &metaData)
+{
+    KIO::FileCopyJob *job = KIO::file_copy(srcUrl, destUrl, -1, KIO::Overwrite);
+    job->setMetaData(metaData);
+    job->addMetaData("MaxCacheSize", "0"); // Don't store in http cache.
+    job->addMetaData("cache", "cache"); // Use entry from cache if available.
+    job->ui()->setWindow(window);
+    job->ui()->setAutoErrorHandlingEnabled( true );
+    new DownloadJobWatcher(job, metaData);
+}
+
 void BrowserRun::slotStatResult( KJob *job )
 {
     if ( job->error() ) {
-        kDebug(1000) << "BrowserRun::slotStatResult:" << job->errorString();
+        kDebug(1000) << job->errorString();
         handleError( job );
     } else
         KRun::slotStatResult( job );
@@ -466,17 +441,18 @@ void BrowserRun::slotStatResult( KJob *job )
 void BrowserRun::handleError( KJob * job )
 {
     if ( !job ) { // Shouldn't happen, see docu.
-        kWarning(1000) << "BrowserRun::handleError called with job=0! hideErrorDialog=" << d->m_bHideErrorDialog;
+        kWarning(1000) << "handleError called with job=0! hideErrorDialog=" << d->m_bHideErrorDialog;
         return;
     }
 
     KIO::TransferJob *tjob = qobject_cast<KIO::TransferJob *>(job);
-    if (tjob && tjob->isErrorPage()) {
+    if (tjob && tjob->isErrorPage() && !job->error()) {
         // The default handling of error pages is to show them like normal pages
         // But this is done here in handleError so that KHTMLRun can reimplement it
         tjob->putOnHold();
         setJob(0);
-        mimeTypeDetermined(d->m_mimeType);
+        if (!d->m_mimeType.isEmpty())
+            mimeTypeDetermined(d->m_mimeType);
         return;
     }
 
@@ -519,14 +495,13 @@ void BrowserRun::redirectToError( int error, const QString& errorText )
 void BrowserRun::slotCopyToTempFileResult(KJob *job)
 {
     if ( job->error() ) {
-	    job->uiDelegate()->showErrorMessage();
+        job->uiDelegate()->showErrorMessage();
     } else {
         // Same as KRun::foundMimeType but with a different URL
         (void) (KRun::runUrl( static_cast<KIO::FileCopyJob *>(job)->destUrl(), d->m_mimeType, d->m_window ));
     }
     setError( true ); // see above
     setFinished( true );
-    timer().start( 0 );
 }
 
 bool BrowserRun::isTextExecutable( const QString &mimeType )
@@ -540,10 +515,18 @@ bool BrowserRun::hideErrorDialog() const
     return d->m_bHideErrorDialog;
 }
 
-QString BrowserRun::contentDisposition() const {
+QString BrowserRun::contentDisposition() const
+{
     return d->m_contentDisposition;
 }
 
+bool BrowserRun::serverSuggestsSave() const
+{
+    // RfC 2183, section 2.8:
+    // Unrecognized disposition types should be treated as `attachment'.
+    return !contentDisposition().isEmpty() && (contentDisposition() != "inline");
+}
+
 KParts::OpenUrlArguments& KParts::BrowserRun::arguments()
 {
     return d->m_args;
@@ -555,3 +538,4 @@ KParts::BrowserArguments& KParts::BrowserRun::browserArguments()
 }
 
 #include "browserrun.moc"
+#include "browserrun_p.moc"
