@@ -38,6 +38,10 @@
 #include <ktoolinvocation.h>
 #include <kdebug.h>
 
+#include "auth/kauthaction.h"
+
+#include "kcolorscheme.h"
+
 #include "kcmoduleloader.h"
 #include "kcmoduleproxy.h"
 
@@ -70,6 +74,60 @@ void KCMultiDialogPrivate::updateButtons(KCModuleProxy *currentModule)
     Q_Q(KCMultiDialog);
     q->enableButton(KDialog::Help, currentModule->buttons() & KCModule::Help);
     q->enableButton(KDialog::Default, currentModule->buttons() & KCModule::Default);
+
+    if (q->button(KDialog::Apply))
+    {
+      q->disconnect(q, SIGNAL(applyClicked()), q, SLOT(slotApplyClicked()));
+      q->disconnect(q->button(KDialog::Apply), SIGNAL(authorized(KAuth::Action*)), q, SLOT(slotApplyClicked()));
+    }
+
+    if (q->button(KDialog::Ok))
+    {
+      q->disconnect(q, SIGNAL(okClicked()), q, SLOT(slotOkClicked()));
+      q->disconnect(q->button(KDialog::Ok), SIGNAL(authorized(KAuth::Action*)), q, SLOT(slotOkClicked()));
+    }
+
+    if (currentModule->realModule()->needsAuthorization()) {
+      if (q->button(KDialog::Apply))
+      {
+        q->button(KDialog::Apply)->setAuthAction(currentModule->realModule()->authAction());
+        q->connect(q->button(KDialog::Apply), SIGNAL(authorized(KAuth::Action*)), SLOT(slotApplyClicked()));
+      }
+
+      if (q->button(KDialog::Ok))
+      {
+        q->button(KDialog::Ok)->setAuthAction(currentModule->realModule()->authAction());
+        q->connect(q->button(KDialog::Ok), SIGNAL(authorized(KAuth::Action*)), SLOT(slotOkClicked()));
+      }
+    } else {
+
+        if (q->button(KDialog::Apply))
+        {
+          q->connect(q, SIGNAL(applyClicked()), SLOT(slotApplyClicked()));
+          q->button(KDialog::Apply)->setAuthAction(currentModule->realModule()->authAction());
+        }
+        if (q->button(KDialog::Ok))
+        {
+          q->connect(q, SIGNAL(okClicked()), SLOT(slotOkClicked()));
+          q->button(KDialog::Ok)->setAuthAction(currentModule->realModule()->authAction());
+        }
+    }
+}
+
+void KCMultiDialogPrivate::_k_updateHeader(bool use, const QString &message)
+{
+    Q_Q(KCMultiDialog);
+    KPageWidgetItem *item = q->currentPage();
+    KCModuleProxy *kcm = qobject_cast<KCModuleProxy*>(item->widget());
+
+    if (use) {
+        item->setHeader( "<b>"+kcm->moduleInfo().comment() + "</b><br><i>" +
+                         message + "</i>" );
+        item->setIcon( KIcon( kcm->moduleInfo().icon(), 0, QStringList() << "dialog-warning" ) );
+    } else {
+        item->setHeader( kcm->moduleInfo().comment() );
+        item->setIcon( KIcon( kcm->moduleInfo().icon() ) );
+    }
 }
 
 void KCMultiDialogPrivate::_k_clientChanged()
@@ -123,7 +181,7 @@ void KCMultiDialogPrivate::init()
     q->connect(q, SIGNAL(helpClicked()), SLOT(slotHelpClicked()));
     q->connect(q, SIGNAL(user1Clicked()), SLOT(slotUser1Clicked()));
 
-    q->setInitialSize(QSize(800, 600));
+    q->setInitialSize(QSize(800, 550));
 }
 
 KCMultiDialog::KCMultiDialog( QWidget *parent )
@@ -190,11 +248,10 @@ void KCMultiDialogPrivate::apply()
 
         if (proxy->changed()) {
             proxy->save();
-
             /**
-             * Add name of the components the kcm belongs to the list
-             * of updated components.
-             */
+                * Add name of the components the kcm belongs to the list
+                * of updated components.
+                */
             const QStringList componentNames = module.componentNames;
             foreach (const QString &componentName, module.componentNames) {
                 if (!updatedComponents.contains(componentName)) {
@@ -256,7 +313,7 @@ KPageWidgetItem* KCMultiDialog::addModule( const QString& path, const QStringLis
 {
   QString complete = path;
 
-  if ( !path.endsWith( ".desktop" ) )
+  if ( !path.endsWith( QLatin1String(".desktop") ) )
     complete += ".desktop";
 
   KService::Ptr service = KService::serviceByStorageId( complete );
@@ -279,8 +336,14 @@ KPageWidgetItem* KCMultiDialog::addModule( const KCModuleInfo& moduleInfo,
 
     kDebug(710) << moduleInfo.moduleName();
     KPageWidgetItem *item = new KPageWidgetItem(kcm, moduleInfo.moduleName());
-  item->setHeader( moduleInfo.comment() );
-  item->setIcon( KIcon( moduleInfo.icon() ) );
+  
+    if (kcm->useRootOnlyMessage()) {
+        item->setHeader( "<b>"+moduleInfo.comment() + "</b><br><i>" + kcm->rootOnlyMessage() + "</i>" );
+        item->setIcon( KIcon( moduleInfo.icon(), 0, QStringList() << "dialog-warning" ) );
+    } else {
+        item->setHeader( moduleInfo.comment() );
+        item->setIcon( KIcon( moduleInfo.icon() ) );
+    }
     item->setProperty("_k_weight", moduleInfo.weight());
 
     bool updateCurrentPage = false;
@@ -327,7 +390,7 @@ KPageWidgetItem* KCMultiDialog::addModule( const KCModuleInfo& moduleInfo,
     }
 
     connect(kcm, SIGNAL(changed(bool)), this, SLOT(_k_clientChanged()));
-
+    connect(kcm->realModule(), SIGNAL(rootOnlyMessageChanged(bool,QString)), this, SLOT(_k_updateHeader(bool,QString)));
 
     Q_D(KCMultiDialog);
   KCMultiDialogPrivate::CreatedModule cm;
